#include "Wconfig.h"
#include "Wxf.h"
#include "Wdomain.h"
#include "Wdriver.h"

/*
  This custom file was written for the shear connection simulation;
  see file scd2.in.

  Assumptions:
  o Essential BC's consist of prescribed displacement along the y-axis
    at the top and bottom of the beam.
*/

static double
compute_reaction_force (W_domain_t *d)
{
  G3D_box_t box = {-10000, -10000, 525, 10000, 10000, 525};
  W_node_t *n;
  G3D_vect_t p;
  double reaction_force_y = 0;
  G3D_INFLATE_BOX (box, 0.1);
  n = NULL;
  while (n = WD_get_next_node (d, n)) {
    WN_get_ref_coords (n, &p.x, &p.y, &p.z);
    if (G3D_PT_INSIDE_BOX (box, p)) {
      WN_get_force (n, &p.x, &p.y, &p.z);
      reaction_force_y += p.y; 
    }
  }
  return reaction_force_y;
}

static void 
save_reaction_force (W_domain_t *d, int final_call)
{
  double reaction_force_y = compute_reaction_force (d);
  WHR_store (d->history_writer, "reaction_force_y", WD_time (d), reaction_force_y);
}

/* This is just an empty custom setup routine.
   Write your own file, and link with run.o and wd.o
   to produce your own driver with custom setup you wish.
 */
int
wasp_custom_setup (W_domain_t *d)
{
  fprintf (stderr, "Custom setup: setting observer\n"); fflush (stderr);
  W_domain_set_response_observer_func (d, save_reaction_force);
  return TRUE;
}

