#!/bin/ksh

if [ $# -ne 2 ] ; then
    echo Usage: $(basename $0) outfile ncomput 
    exit 1
fi

function genl {
# generates a list of numbers from `low' to `high' (inclusive)
    genl_low=$1; shift
    genl_high=$1; shift
    # echo genl_low=$genl_low
    # echo genl_high=$genl_high
    genl_counter=$genl_low
    genl_s=""
    while [ $genl_counter -le $genl_high ] 
    do
      genl_ss="$genl_s $genl_counter"
      genl_s="$genl_ss"
      genl_counter=$(($genl_counter + 1))
    done
    echo $genl_s
}

function npdf {
    ncomput=$1; shift
    mean=$1; shift
    stddev=$1; shift
    echo $(expeval "randrestart($ncomput); npdf($mean, $stddev)")
}

outfile=$1; shift
ncomput=$1

rm -f $outfile

stddev_rat=0.025
E=117000
sigma_y=400
K=20
E_stddev=$(expeval "$E*$stddev_rat")
K_stddev=$(expeval "$K*$stddev_rat")
sigma_y_stddev=$(expeval "$sigma_y*$stddev_rat")

E=$(npdf $ncomput $E $E_stddev); ncomput=$(($ncomput+1));
sigma_y=$(npdf $ncomput $sigma_y $sigma_y_stddev); ncomput=$(($ncomput+1));
K=$(npdf $ncomput $K $K_stddev); ncomput=$(($ncomput+1));

# 
#  generate input file
#
cat <<EOF > $outfile
Refined model of splice; symmetric
!  ======================================================= 
0 0 0 0 0
!  ======================================================= 
OPTIONS 
!  option_tag (O, T, S)     value 
T 0.001
O 0.001
f 0.1
! H "SEQUENCE 0 1 0.002"
h 29
! number_seq
! M 120
!  END OPTIONS 
&
!  ================================================ 
MATERIAL
6
   1
   5  ! 5=halquist
   1   E $E   NU 0.35 rho 8.93e-9 sigma_y $sigma_y HARDENING_PARAM $K
!  END MATERIAL
&
!  ======================================================= 
TIME_FUNCTIONS 
  2  ! tabular 
! func#   #pairs 
   1        5
! time     val 
! Heaviside step function:
   0        0
   0.0001   1
   0.0005   1
   0.0008   0
   10000000 0
!  END TIME_FUNCTIONS 
&
!  ======================================================= 
NODES
! id       x       y       z  CONSTRAIN: X  Y  Z (0=fix)
  1   5   0  0       1   0   1
  2   6   0  0       1   0   1
  3   6.25   1  0       1   1   1
  4   5   1  0       1   1   1
  5   5   0  0.5       1   0   1
  6   6   0  0.5       1   0   1
  7   6.25   1  0.5       1   1   1
  8   5   1  0.5       1   1   1
  9   6.5   2  0       1   1   1
  10   5   2  0       1   1   1
  11   6.5   2  0.5       1   1   1
  12   5   2  0.5       1   1   1
  13   7   0  0       1   0   1
  14   7.5   1  0       1   1   1
  15   7   0  0.5       1   0   1
  16   7.5   1  0.5       1   1   1
  17   8   2  0       1   1   1
  18   8   2  0.5       1   1   1
  19   5   0  1       1   0   1
  20   6   0  1       1   0   1
  21   6.25   1  1       1   1   1
  22   5   1  1       1   1   1
  23   6.5   2  1       1   1   1
  24   5   2  1       1   1   1
  25   7   0  1       1   0   1
  26   7.5   1  1       1   1   1
  27   8   2  1       1   1   1
  28   9   2.5  0       1   1   1
  29   9   3.8125  0       1   1   1
  30   8   3.5  0       1   1   1
  31   9   2.5  0.5       1   1   1
  32   9   3.8125  0.5       1   1   1
  33   8   3.5  0.5       1   1   1
  34   9   5.125  0       1   1   1
  35   8   5  0       1   1   1
  36   9   5.125  0.5       1   1   1
  37   8   5  0.5       1   1   1
  38   10   3  0       1   1   1
  39   10   4.125  0       1   1   1
  40   10   3  0.5       1   1   1
  41   10   4.125  0.5       1   1   1
  42   10   5.25  0       1   1   1
  43   10   5.25  0.5       1   1   1
  44   9   2.5  1       1   1   1
  45   9   3.8125  1       1   1   1
  46   8   3.5  1       1   1   1
  47   9   5.125  1       1   1   1
  48   8   5  1       1   1   1
  49   10   3  1       1   1   1
  50   10   4.125  1       1   1   1
  51   10   5.25  1       1   1   1
  52   6.5   5  0       1   1   1
  53   6.5   3.5  0       1   1   1
  54   6.5   5  0.5       1   1   1
  55   6.5   3.5  0.5       1   1   1
  56   5   5  0       1   1   1
  57   5   3.5  0       1   1   1
  58   5   5  0.5       1   1   1
  59   5   3.5  0.5       1   1   1
  60   6.5   5  1       1   1   1
  61   6.5   3.5  1       1   1   1
  62   5   5  1       1   1   1
  63   5   3.5  1       1   1   1
  64   9.5   0  -0.1       1   0   1
  65   9.54   0.21  -0.1       1   1   1
  66   9.34322   0.281777  -0.075       1   1   1
  67   9.25   0  -0.075       1   0   1
  68   9.5   0  0.5       1   0   1
  69   9.54   0.21  0.5       1   1   1
  70   9.34322   0.281777  0.5       1   1   1
  71   9.25   0  0.5       1   0   1
  72   9.14644   0.353554  -0.05       1   1   1
  73   9   0  -0.05       1   0   1
  74   9.14644   0.353554  0.5       1   1   1
  75   9   0  0.5       1   0   1
  76   9.58   0.42  -0.1       1   1   1
  77   9.43644   0.563554  -0.075       1   1   1
  78   9.58   0.42  0.5       1   1   1
  79   9.43644   0.563554  0.5       1   1   1
  80   9.29289   0.707107  -0.05       1   1   1
  81   9.29289   0.707107  0.5       1   1   1
  82   9.5   0  1       1   0   1
  83   9.54   0.21  1       1   1   1
  84   9.34322   0.281777  1       1   1   1
  85   9.25   0  1       1   0   1
  86   9.14644   0.353554  1       1   1   1
  87   9   0  1       1   0   1
  88   9.58   0.42  1       1   1   1
  89   9.43644   0.563554  1       1   1   1
  90   9.29289   0.707107  1       1   1   1
  91   8.32322   0.676777  -0.025       1   1   1
  92   8   0  -0.025       1   0   1
  93   8.32322   0.676777  0.5       1   1   1
  94   8   0  0.5       1   0   1
  95   8.64644   1.35355  -0.025       1   1   1
  96   8.64644   1.35355  0.5       1   1   1
  97   8.32322   0.676777  1       1   1   1
  98   8   0  1       1   0   1
  99   8.64644   1.35355  1       1   1   1
  100   9.64644   0.853554  -0.05       1   1   1
  101   9.32322   1.67678  -0.025       1   1   1
  102   9.64644   0.853554  0.5       1   1   1
  103   9.32322   1.67678  0.5       1   1   1
  104   10   1  -0.05       1   1   1
  105   10   2  -0.025       1   1   1
  106   10   1  0.5       1   1   1
  107   10   2  0.5       1   1   1
  108   9.64644   0.853554  1       1   1   1
  109   9.32322   1.67678  1       1   1   1
  110   10   1  1       1   1   1
  111   10   2  1       1   1   1
  112   0   5  0.5       1   1   1
  113   0   4  0.5       1   1   1
  114   -1   3.75  0.5       1   1   1
  115   -1   5  0.5       1   1   1
  116   0   5  0       1   1   1
  117   0   4  0       1   1   1
  118   -1   3.75  0       1   1   1
  119   -1   5  0       1   1   1
  120   -2   3.5  0.5       1   1   1
  121   -2   5  0.5       1   1   1
  122   -2   3.5  0       1   1   1
  123   -2   5  0       1   1   1
  124   0   3  0.5       1   1   1
  125   -1   2.5  0.5       1   1   1
  126   0   3  0       1   1   1
  127   -1   2.5  0       1   1   1
  128   -2   2  0.5       1   1   1
  129   -2   2  0       1   1   1
  130   0   5  1       1   1   1
  131   0   4  1       1   1   1
  132   -1   3.75  1       1   1   1
  133   -1   5  1       1   1   1
  134   -2   3.5  1       1   1   1
  135   -2   5  1       1   1   1
  136   0   3  1       1   1   1
  137   -1   2.5  1       1   1   1
  138   -2   2  1       1   1   1
  139   -2.5   1  0.5       1   1   1
  140   -3.75   1  0.5       1   1   1
  141   -3.5   2  0.5       1   1   1
  142   -2.5   1  0       1   1   1
  143   -3.75   1  0       1   1   1
  144   -3.5   2  0       1   1   1
  145   -5   1  0.5       1   1   1
  146   -5   2  0.5       1   1   1
  147   -5   1  0       1   1   1
  148   -5   2  0       1   1   1
  149   -3   -1e-06  0.5       1   0   1
  150   -4   -1e-06  0.5       1   0   1
  151   -3   -1e-06  0       1   0   1
  152   -4   -1e-06  0       1   0   1
  153   -5   -1e-06  0.5       1   0   1
  154   -5   -1e-06  0       1   0   1
  155   -2.5   1  1       1   1   1
  156   -3.75   1  1       1   1   1
  157   -3.5   2  1       1   1   1
  158   -5   1  1       1   1   1
  159   -5   2  1       1   1   1
  160   -3   -1e-06  1       1   0   1
  161   -4   -1e-06  1       1   0   1
  162   -5   -1e-06  1       1   0   1
  163   -5   3.5  0.5       1   1   1
  164   -3.5   3.5  0.5       1   1   1
  165   -5   3.5  0       1   1   1
  166   -3.5   3.5  0       1   1   1
  167   -5   5  0.5       1   1   1
  168   -3.5   5  0.5       1   1   1
  169   -5   5  0       1   1   1
  170   -3.5   5  0       1   1   1
  171   -5   3.5  1       1   1   1
  172   -3.5   3.5  1       1   1   1
  173   -5   5  1       1   1   1
  174   -3.5   5  1       1   1   1
  175   4   0  0.5       1   0   1
  176   3.75   1  0.5       1   1   1
  177   4   0  0       1   0   1
  178   3.75   1  0       1   1   1
  179   3.5   2  0.5       1   1   1
  180   3.5   2  0       1   1   1
  181   3   0  0.5       1   0   1
  182   2.5   1  0.5       1   1   1
  183   3   0  0       1   0   1
  184   2.5   1  0       1   1   1
  185   2   2  0.5       1   1   1
  186   2   2  0       1   1   1
  187   4   0  1       1   0   1
  188   3.75   1  1       1   1   1
  189   3.5   2  1       1   1   1
  190   3   0  1       1   0   1
  191   2.5   1  1       1   1   1
  192   2   2  1       1   1   1
  193   1   2.5  0.5       1   1   1
  194   1   3.75  0.5       1   1   1
  195   2   3.5  0.5       1   1   1
  196   1   2.5  0       1   1   1
  197   1   3.75  0       1   1   1
  198   2   3.5  0       1   1   1
  199   1   5  0.5       1   1   1
  200   2   5  0.5       1   1   1
  201   1   5  0       1   1   1
  202   2   5  0       1   1   1
  203   1   2.5  1       1   1   1
  204   1   3.75  1       1   1   1
  205   2   3.5  1       1   1   1
  206   1   5  1       1   1   1
  207   2   5  1       1   1   1
  208   3.5   5  0.5       1   1   1
  209   3.5   3.5  0.5       1   1   1
  210   3.5   5  0       1   1   1
  211   3.5   3.5  0       1   1   1
  212   3.5   5  1       1   1   1
  213   3.5   3.5  1       1   1   1
  214   0   1  0.5       1   1   1
  215   -0.353553   0.853553  0.5       1   1   1
  216   -0.676777   1.67678  0.5       1   1   1
  217   0   2  0.5       1   1   1
  218   0   1  -0.05       1   1   1
  219   -0.353553   0.853553  -0.05       1   1   1
  220   -0.676777   1.67678  -0.025       1   1   1
  221   0   2  -0.025       1   1   1
  222   -0.707106   0.707106  0.5       1   1   1
  223   -1.35355   1.35355  0.5       1   1   1
  224   -0.707106   0.707106  -0.05       1   1   1
  225   -1.35355   1.35355  -0.025       1   1   1
  226   0   1  1       1   1   1
  227   -0.353553   0.853553  1       1   1   1
  228   -0.676777   1.67678  1       1   1   1
  229   0   2  1       1   1   1
  230   -0.707106   0.707106  1       1   1   1
  231   -1.35355   1.35355  1       1   1   1
  232   -0.853553   0.353552  0.5       1   1   1
  233   -1.67678   0.676776  0.5       1   1   1
  234   -0.853553   0.353552  -0.05       1   1   1
  235   -1.67678   0.676776  -0.025       1   1   1
  236   -1   -1e-06  0.5       1   0   1
  237   -2   -1e-06  0.5       1   0   1
  238   -1   -1e-06  -0.05       1   0   1
  239   -2   -1e-06  -0.025       1   0   1
  240   -0.853553   0.353552  1       1   1   1
  241   -1.67678   0.676776  1       1   1   1
  242   -1   -1e-06  1       1   0   1
  243   -2   -1e-06  1       1   0   1
  244   1   0  0.5       1   0   1
  245   0.853554   0.353553  0.5       1   1   1
  246   1.67678   0.676777  0.5       1   1   1
  247   2   0  0.5       1   0   1
  248   1   0  -0.05       1   0   1
  249   0.853554   0.353553  -0.05       1   1   1
  250   1.67678   0.676777  -0.025       1   1   1
  251   2   0  -0.025       1   0   1
  252   0.707107   0.707106  0.5       1   1   1
  253   1.35355   1.35355  0.5       1   1   1
  254   0.707107   0.707106  -0.05       1   1   1
  255   1.35355   1.35355  -0.025       1   1   1
  256   1   0  1       1   0   1
  257   0.853554   0.353553  1       1   1   1
  258   1.67678   0.676777  1       1   1   1
  259   2   0  1       1   0   1
  260   0.707107   0.707106  1       1   1   1
  261   1.35355   1.35355  1       1   1   1
  262   0.353554   0.853553  0.5       1   1   1
  263   0.676777   1.67678  0.5       1   1   1
  264   0.353554   0.853553  -0.05       1   1   1
  265   0.676777   1.67678  -0.025       1   1   1
  266   0.353554   0.853553  1       1   1   1
  267   0.676777   1.67678  1       1   1   1
  268   11   3.9375  0       1   1   1
  269   11   5.375  0       1   1   1
  270   11   3.9375  0.5       1   1   1
  271   11   5.375  0.5       1   1   1
  272   12   3.75  0       1   1   1
  273   12   5.5  0       1   1   1
  274   12   3.75  0.5       1   1   1
  275   12   5.5  0.5       1   1   1
  276   11   2.5  0       1   1   1
  277   11   2.5  0.5       1   1   1
  278   12   2  0       1   1   1
  279   12   2  0.5       1   1   1
  280   11   3.9375  1       1   1   1
  281   11   5.375  1       1   1   1
  282   12   3.75  1       1   1   1
  283   12   5.5  1       1   1   1
  284   11   2.5  1       1   1   1
  285   12   2  1       1   1   1
  286   12.5   1  0       1   1   1
  287   13.75   1  0       1   1   1
  288   13.5   2  0       1   1   1
  289   12.5   1  0.5       1   1   1
  290   13.75   1  0.5       1   1   1
  291   13.5   2  0.5       1   1   1
  292   15   1  0       1   1   1
  293   15   2  0       1   1   1
  294   15   1  0.5       1   1   1
  295   15   2  0.5       1   1   1
  296   13   0  0       1   0   1
  297   14   0  0       1   0   1
  298   13   0  0.5       1   0   1
  299   14   0  0.5       1   0   1
  300   15   0  0       1   0   1
  301   15   0  0.5       1   0   1
  302   12.5   1  1       1   1   1
  303   13.75   1  1       1   1   1
  304   13.5   2  1       1   1   1
  305   15   1  1       1   1   1
  306   15   2  1       1   1   1
  307   13   0  1       1   0   1
  308   14   0  1       1   0   1
  309   15   0  1       1   0   1
  310   15   3.75  0       1   1   1
  311   13.5   3.75  0       1   1   1
  312   15   3.75  0.5       1   1   1
  313   13.5   3.75  0.5       1   1   1
  314   15   5.5  0       1   1   1
  315   13.5   5.5  0       1   1   1
  316   15   5.5  0.5       1   1   1
  317   13.5   5.5  0.5       1   1   1
  318   15   3.75  1       1   1   1
  319   13.5   3.75  1       1   1   1
  320   15   5.5  1       1   1   1
  321   13.5   5.5  1       1   1   1
  322   16.5   3.75  0       1   1   1
  323   16.5   5.5  0       1   1   1
  324   16.5   3.75  0.5       1   1   1
  325   16.5   5.5  0.5       1   1   1
  326   18   3.75  0       1   1   0
  327   18   5.5  0       1   1   0
  328   18   3.75  0.5       1   1   0
  329   18   5.5  0.5       1   1   0
  330   16.5   2  0       1   1   1
  331   16.5   2  0.5       1   1   1
  332   18   2  0       1   1   0
  333   18   2  0.5       1   1   0
  334   16.5   3.75  1       1   1   1
  335   16.5   5.5  1       1   1   1
  336   18   3.75  1       1   1   0
  337   18   5.5  1       1   1   0
  338   16.5   2  1       1   1   1
  339   18   2  1       1   1   0
  340   16.5   1  0       1   1   1
  341   16.5   1  0.5       1   1   1
  342   18   1  0       1   1   0
  343   18   1  0.5       1   1   0
  344   16.5   0  0       1   0   1
  345   16.5   0  0.5       1   0   1
  346   18   0  0       1   0   0
  347   18   0  0.5       1   0   0
  348   16.5   1  1       1   1   1
  349   18   1  1       1   1   0
  350   16.5   0  1       1   0   1
  351   18   0  1       1   0   0
  352   10   0  -0.1       1   0   1
  353   10.25   0  -0.1       1   0   1
  354   10.23   0.23  -0.1       1   1   1
  355   10   0.25  -0.1       1   1   1
  356   10   0  0.5       1   0   1
  357   10.25   0  0.5       1   0   1
  358   10.23   0.23  0.5       1   1   1
  359   10   0.25  0.5       1   1   1
  360   10.21   0.46  -0.1       1   1   1
  361   10   0.5  -0.1       1   1   1
  362   10.21   0.46  0.5       1   1   1
  363   10   0.5  0.5       1   1   1
  364   10.5   0  -0.1       1   0   1
  365   10.46   0.21  -0.1       1   1   1
  366   10.5   0  0.5       1   0   1
  367   10.46   0.21  0.5       1   1   1
  368   10.42   0.42  -0.1       1   1   1
  369   10.42   0.42  0.5       1   1   1
  370   10   0  1       1   0   1
  371   10.25   0  1       1   0   1
  372   10.23   0.23  1       1   1   1
  373   10   0.25  1       1   1   1
  374   10.21   0.46  1       1   1   1
  375   10   0.5  1       1   1   1
  376   10.5   0  1       1   0   1
  377   10.46   0.21  1       1   1   1
  378   10.42   0.42  1       1   1   1
  379   10.75   0  -0.075       1   0   1
  380   10.6568   0.281777  -0.075       1   1   1
  381   10.75   0  0.5       1   0   1
  382   10.6568   0.281777  0.5       1   1   1
  383   10.5635   0.563554  -0.075       1   1   1
  384   10.5635   0.563554  0.5       1   1   1
  385   11   0  -0.05       1   0   1
  386   10.8536   0.353554  -0.05       1   1   1
  387   11   0  0.5       1   0   1
  388   10.8536   0.353554  0.5       1   1   1
  389   10.7071   0.707107  -0.05       1   1   1
  390   10.7071   0.707107  0.5       1   1   1
  391   10.75   0  1       1   0   1
  392   10.6568   0.281777  1       1   1   1
  393   10.5635   0.563554  1       1   1   1
  394   11   0  1       1   0   1
  395   10.8536   0.353554  1       1   1   1
  396   10.7071   0.707107  1       1   1   1
  397   10.2818   0.656777  -0.075       1   1   1
  398   10   0.75  -0.075       1   1   1
  399   10.2818   0.656777  0.5       1   1   1
  400   10   0.75  0.5       1   1   1
  401   10.3536   0.853554  -0.05       1   1   1
  402   10.3536   0.853554  0.5       1   1   1
  403   10.2818   0.656777  1       1   1   1
  404   10   0.75  1       1   1   1
  405   10.3536   0.853554  1       1   1   1
  406   10.6768   1.67678  -0.025       1   1   1
  407   10.6768   1.67678  0.5       1   1   1
  408   11.3536   1.35355  -0.025       1   1   1
  409   11.3536   1.35355  0.5       1   1   1
  410   10.6768   1.67678  1       1   1   1
  411   11.3536   1.35355  1       1   1   1
  412   11.6768   0.676777  -0.025       1   1   1
  413   11.6768   0.676777  0.5       1   1   1
  414   12   0  -0.025       1   0   1
  415   12   0  0.5       1   0   1
  416   11.6768   0.676777  1       1   1   1
  417   12   0  1       1   0   1
  418   9.77   0.23  -0.1       1   1   1
  419   9.75   0  -0.1       1   0   1
  420   9.77   0.23  0.5       1   1   1
  421   9.75   0  0.5       1   0   1
  422   9.79   0.46  -0.1       1   1   1
  423   9.79   0.46  0.5       1   1   1
  424   9.77   0.23  1       1   1   1
  425   9.75   0  1       1   0   1
  426   9.79   0.46  1       1   1   1
  427   9.71822   0.656777  -0.075       1   1   1
  428   9.71822   0.656777  0.5       1   1   1
  429   9.71822   0.656777  1       1   1   1
  430   -0.499996   -1e-06  -0.1       1   0   1
  431   -0.459996   0.209999  -0.1       1   1   1
  432   -0.656773   0.281776  -0.075       1   1   1
  433   -0.749996   -1e-06  -0.075       1   0   1
  434   -0.499996   -1e-06  0.5       1   0   1
  435   -0.459996   0.209999  0.5       1   1   1
  436   -0.656773   0.281776  0.5       1   1   1
  437   -0.749996   -1e-06  0.5       1   0   1
  438   -0.419996   0.419999  -0.1       1   1   1
  439   -0.563551   0.563553  -0.075       1   1   1
  440   -0.419996   0.419999  0.5       1   1   1
  441   -0.563551   0.563553  0.5       1   1   1
  442   -0.499996   -1e-06  1       1   0   1
  443   -0.459996   0.209999  1       1   1   1
  444   -0.656773   0.281776  1       1   1   1
  445   -0.749996   -1e-06  1       1   0   1
  446   -0.419996   0.419999  1       1   1   1
  447   -0.563551   0.563553  1       1   1   1
  448   4e-06   -1e-06  -0.1       1   0   1
  449   0.250004   -1e-06  -0.1       1   0   1
  450   0.230004   0.229999  -0.1       1   1   1
  451   4e-06   0.249999  -0.1       1   1   1
  452   4e-06   -1e-06  0.5       1   0   1
  453   0.250004   -1e-06  0.5       1   0   1
  454   0.230004   0.229999  0.5       1   1   1
  455   4e-06   0.249999  0.5       1   1   1
  456   0.210004   0.459999  -0.1       1   1   1
  457   4e-06   0.499999  -0.1       1   1   1
  458   0.210004   0.459999  0.5       1   1   1
  459   4e-06   0.499999  0.5       1   1   1
  460   0.500004   -1e-06  -0.1       1   0   1
  461   0.460004   0.209999  -0.1       1   1   1
  462   0.500004   -1e-06  0.5       1   0   1
  463   0.460004   0.209999  0.5       1   1   1
  464   0.420004   0.419999  -0.1       1   1   1
  465   0.420004   0.419999  0.5       1   1   1
  466   4e-06   -1e-06  1       1   0   1
  467   0.250004   -1e-06  1       1   0   1
  468   0.230004   0.229999  1       1   1   1
  469   4e-06   0.249999  1       1   1   1
  470   0.210004   0.459999  1       1   1   1
  471   4e-06   0.499999  1       1   1   1
  472   0.500004   -1e-06  1       1   0   1
  473   0.460004   0.209999  1       1   1   1
  474   0.420004   0.419999  1       1   1   1
  475   0.750004   -1e-06  -0.075       1   0   1
  476   0.656779   0.281776  -0.075       1   1   1
  477   0.750004   -1e-06  0.5       1   0   1
  478   0.656779   0.281776  0.5       1   1   1
  479   0.563554   0.563553  -0.075       1   1   1
  480   0.563554   0.563553  0.5       1   1   1
  481   0.750004   -1e-06  1       1   0   1
  482   0.656779   0.281776  1       1   1   1
  483   0.563554   0.563553  1       1   1   1
  484   0.281779   0.656776  -0.075       1   1   1
  485   4e-06   0.749999  -0.075       1   1   1
  486   0.281779   0.656776  0.5       1   1   1
  487   4e-06   0.749999  0.5       1   1   1
  488   0.281779   0.656776  1       1   1   1
  489   4e-06   0.749999  1       1   1   1
  490   -0.229996   0.229999  -0.1       1   1   1
  491   -0.249996   -1e-06  -0.1       1   0   1
  492   -0.229996   0.229999  0.5       1   1   1
  493   -0.249996   -1e-06  0.5       1   0   1
  494   -0.209996   0.459999  -0.1       1   1   1
  495   -0.209996   0.459999  0.5       1   1   1
  496   -0.229996   0.229999  1       1   1   1
  497   -0.249996   -1e-06  1       1   0   1
  498   -0.209996   0.459999  1       1   1   1
  499   -0.281773   0.656776  -0.075       1   1   1
  500   -0.281773   0.656776  0.5       1   1   1
  501   -0.281773   0.656776  1       1   1   1
  502   5   5.65218e-07  -0.2       1   0   1
  503   4   4.78261e-07  -0.2       1   0   1
  504   3.75   1  -0.2       1   1   1
  505   5   1  -0.2       1   1   1
  506   5   5.65218e-07  -0.7       1   0   1
  507   4   4.78261e-07  -0.7       1   0   1
  508   3.75   1  -0.7       1   1   1
  509   5   1  -0.7       1   1   1
  510   3.5   2  -0.2       1   1   1
  511   5   2  -0.2       1   1   1
  512   3.5   2  -0.7       1   1   1
  513   5   2  -0.7       1   1   1
  514   3   3.91305e-07  -0.2       1   0   1
  515   2.5   1  -0.2       1   1   1
  516   3   3.91305e-07  -0.7       1   0   1
  517   2.5   1  -0.7       1   1   1
  518   2   2  -0.2       1   1   1
  519   2   2  -0.7       1   1   1
  520   5   5.65218e-07  -1.2       1   0   1
  521   4   4.78261e-07  -1.2       1   0   1
  522   3.75   1  -1.2       1   1   1
  523   5   1  -1.2       1   1   1
  524   3.5   2  -1.2       1   1   1
  525   5   2  -1.2       1   1   1
  526   3   3.91305e-07  -1.2       1   0   1
  527   2.5   1  -1.2       1   1   1
  528   2   2  -1.2       1   1   1
  529   1   2.5  -0.2       1   1   1
  530   1   3.8125  -0.2       1   1   1
  531   2   3.5  -0.2       1   1   1
  532   1   2.5  -0.7       1   1   1
  533   1   3.8125  -0.7       1   1   1
  534   2   3.5  -0.7       1   1   1
  535   1   5.125  -0.2       1   1   1
  536   2   5  -0.2       1   1   1
  537   1   5.125  -0.7       1   1   1
  538   2   5  -0.7       1   1   1
  539   3.73913e-06   3  -0.2       1   1   1
  540   3.6413e-06   4.125  -0.2       1   1   1
  541   3.73913e-06   3  -0.7       1   1   1
  542   3.6413e-06   4.125  -0.7       1   1   1
  543   3.54348e-06   5.25  -0.2       1   1   1
  544   3.54348e-06   5.25  -0.7       1   1   1
  545   1   2.5  -1.2       1   1   1
  546   1   3.8125  -1.2       1   1   1
  547   2   3.5  -1.2       1   1   1
  548   1   5.125  -1.2       1   1   1
  549   2   5  -1.2       1   1   1
  550   3.73913e-06   3  -1.2       1   1   1
  551   3.6413e-06   4.125  -1.2       1   1   1
  552   3.54348e-06   5.25  -1.2       1   1   1
  553   3.5   5  -0.2       1   1   1
  554   3.5   3.5  -0.2       1   1   1
  555   3.5   5  -0.7       1   1   1
  556   3.5   3.5  -0.7       1   1   1
  557   5   5  -0.2       1   1   1
  558   5   3.5  -0.2       1   1   1
  559   5   5  -0.7       1   1   1
  560   5   3.5  -0.7       1   1   1
  561   3.5   5  -1.2       1   1   1
  562   3.5   3.5  -1.2       1   1   1
  563   5   5  -1.2       1   1   1
  564   5   3.5  -1.2       1   1   1
  565   0.656781   0.281777  -0.125       1   1   1
  566   0.750004   1.95653e-07  -0.125       1   0   1
  567   0.500004   1.73913e-07  -0.7       1   0   1
  568   0.460004   0.21  -0.7       1   1   1
  569   0.656781   0.281777  -0.7       1   1   1
  570   0.750004   1.95653e-07  -0.7       1   0   1
  571   0.853559   0.353554  -0.15       1   1   1
  572   1   2.17392e-07  -0.15       1   0   1
  573   0.853559   0.353554  -0.7       1   1   1
  574   1   2.17392e-07  -0.7       1   0   1
  575   0.563559   0.563554  -0.125       1   1   1
  576   0.420004   0.42  -0.7       1   1   1
  577   0.563559   0.563554  -0.7       1   1   1
  578   0.707114   0.707107  -0.15       1   1   1
  579   0.707114   0.707107  -0.7       1   1   1
  580   0.500004   1.73913e-07  -1.2       1   0   1
  581   0.460004   0.21  -1.2       1   1   1
  582   0.656781   0.281777  -1.2       1   1   1
  583   0.750004   1.95653e-07  -1.2       1   0   1
  584   0.853559   0.353554  -1.2       1   1   1
  585   1   2.17392e-07  -1.2       1   0   1
  586   0.420004   0.42  -1.2       1   1   1
  587   0.563559   0.563554  -1.2       1   1   1
  588   0.707114   0.707107  -1.2       1   1   1
  589   1.67678   0.676777  -0.175       1   1   1
  590   2   3.04348e-07  -0.175       1   0   1
  591   1.67678   0.676777  -0.7       1   1   1
  592   2   3.04348e-07  -0.7       1   0   1
  593   1.35356   1.35355  -0.175       1   1   1
  594   1.35356   1.35355  -0.7       1   1   1
  595   1.67678   0.676777  -1.2       1   1   1
  596   2   3.04348e-07  -1.2       1   0   1
  597   1.35356   1.35355  -1.2       1   1   1
  598   0.353559   0.853554  -0.15       1   1   1
  599   0.676781   1.67678  -0.175       1   1   1
  600   0.353559   0.853554  -0.7       1   1   1
  601   0.676781   1.67678  -0.7       1   1   1
  602   3.91304e-06   1  -0.15       1   1   1
  603   3.82609e-06   2  -0.175       1   1   1
  604   3.91304e-06   1  -0.7       1   1   1
  605   3.82609e-06   2  -0.7       1   1   1
  606   0.353559   0.853554  -1.2       1   1   1
  607   0.676781   1.67678  -1.2       1   1   1
  608   3.91304e-06   1  -1.2       1   1   1
  609   3.82609e-06   2  -1.2       1   1   1
  610   10   5  -0.7       1   1   1
  611   10   4  -0.7       1   1   1
  612   11   3.75  -0.7       1   1   1
  613   11   5  -0.7       1   1   1
  614   10   5  -0.2       1   1   1
  615   10   4  -0.2       1   1   1
  616   11   3.75  -0.2       1   1   1
  617   11   5  -0.2       1   1   1
  618   12   3.5  -0.7       1   1   1
  619   12   5  -0.7       1   1   1
  620   12   3.5  -0.2       1   1   1
  621   12   5  -0.2       1   1   1
  622   10   3  -0.7       1   1   1
  623   11   2.5  -0.7       1   1   1
  624   10   3  -0.2       1   1   1
  625   11   2.5  -0.2       1   1   1
  626   12   2  -0.7       1   1   1
  627   12   2  -0.2       1   1   1
  628   10   5  -1.2       1   1   1
  629   10   4  -1.2       1   1   1
  630   11   3.75  -1.2       1   1   1
  631   11   5  -1.2       1   1   1
  632   12   3.5  -1.2       1   1   1
  633   12   5  -1.2       1   1   1
  634   10   3  -1.2       1   1   1
  635   11   2.5  -1.2       1   1   1
  636   12   2  -1.2       1   1   1
  637   12.5   1  -0.7       1   1   1
  638   13.75   1  -0.7       1   1   1
  639   13.5   2  -0.7       1   1   1
  640   12.5   1  -0.2       1   1   1
  641   13.75   1  -0.2       1   1   1
  642   13.5   2  -0.2       1   1   1
  643   15   1  -0.7       1   1   1
  644   15   2  -0.7       1   1   1
  645   15   1  -0.2       1   1   1
  646   15   2  -0.2       1   1   1
  647   13   2.6087e-07  -0.7       1   0   1
  648   14   3.47826e-07  -0.7       1   0   1
  649   13   2.6087e-07  -0.2       1   0   1
  650   14   3.47826e-07  -0.2       1   0   1
  651   15   4.34783e-07  -0.7       1   0   1
  652   15   4.34783e-07  -0.2       1   0   1
  653   12.5   1  -1.2       1   1   1
  654   13.75   1  -1.2       1   1   1
  655   13.5   2  -1.2       1   1   1
  656   15   1  -1.2       1   1   1
  657   15   2  -1.2       1   1   1
  658   13   2.6087e-07  -1.2       1   0   1
  659   14   3.47826e-07  -1.2       1   0   1
  660   15   4.34783e-07  -1.2       1   0   1
  661   15   3.5  -0.7       1   1   1
  662   13.5   3.5  -0.7       1   1   1
  663   15   3.5  -0.2       1   1   1
  664   13.5   3.5  -0.2       1   1   1
  665   15   5  -0.7       1   1   1
  666   13.5   5  -0.7       1   1   1
  667   15   5  -0.2       1   1   1
  668   13.5   5  -0.2       1   1   1
  669   15   3.5  -1.2       1   1   1
  670   13.5   3.5  -1.2       1   1   1
  671   15   5  -1.2       1   1   1
  672   13.5   5  -1.2       1   1   1
  673   6   6.52174e-07  -0.7       1   0   1
  674   6.25   1  -0.7       1   1   1
  675   6   6.52174e-07  -0.2       1   0   1
  676   6.25   1  -0.2       1   1   1
  677   6.5   2  -0.7       1   1   1
  678   6.5   2  -0.2       1   1   1
  679   7   7.39131e-07  -0.7       1   0   1
  680   7.5   1  -0.7       1   1   1
  681   7   7.39131e-07  -0.2       1   0   1
  682   7.5   1  -0.2       1   1   1
  683   8   2  -0.7       1   1   1
  684   8   2  -0.2       1   1   1
  685   6   6.52174e-07  -1.2       1   0   1
  686   6.25   1  -1.2       1   1   1
  687   6.5   2  -1.2       1   1   1
  688   7   7.39131e-07  -1.2       1   0   1
  689   7.5   1  -1.2       1   1   1
  690   8   2  -1.2       1   1   1
  691   9   2.5  -0.7       1   1   1
  692   9   3.75  -0.7       1   1   1
  693   8   3.5  -0.7       1   1   1
  694   9   2.5  -0.2       1   1   1
  695   9   3.75  -0.2       1   1   1
  696   8   3.5  -0.2       1   1   1
  697   9   5  -0.7       1   1   1
  698   8   5  -0.7       1   1   1
  699   9   5  -0.2       1   1   1
  700   8   5  -0.2       1   1   1
  701   9   2.5  -1.2       1   1   1
  702   9   3.75  -1.2       1   1   1
  703   8   3.5  -1.2       1   1   1
  704   9   5  -1.2       1   1   1
  705   8   5  -1.2       1   1   1
  706   6.5   5  -0.7       1   1   1
  707   6.5   3.5  -0.7       1   1   1
  708   6.5   5  -0.2       1   1   1
  709   6.5   3.5  -0.2       1   1   1
  710   6.5   5  -1.2       1   1   1
  711   6.5   3.5  -1.2       1   1   1
  712   10   1  -0.7       1   1   1
  713   10.3536   0.853554  -0.7       1   1   1
  714   10.6768   1.67678  -0.7       1   1   1
  715   10   2  -0.7       1   1   1
  716   10   1  -0.15       1   1   1
  717   10.3536   0.853554  -0.15       1   1   1
  718   10.6768   1.67678  -0.175       1   1   1
  719   10   2  -0.175       1   1   1
  720   10.7071   0.707107  -0.7       1   1   1
  721   11.3536   1.35355  -0.7       1   1   1
  722   10.7071   0.707107  -0.15       1   1   1
  723   11.3536   1.35355  -0.175       1   1   1
  724   10   1  -1.2       1   1   1
  725   10.3536   0.853554  -1.2       1   1   1
  726   10.6768   1.67678  -1.2       1   1   1
  727   10   2  -1.2       1   1   1
  728   10.7071   0.707107  -1.2       1   1   1
  729   11.3536   1.35355  -1.2       1   1   1
  730   10.8536   0.353554  -0.7       1   1   1
  731   11.6768   0.676777  -0.7       1   1   1
  732   10.8536   0.353554  -0.15       1   1   1
  733   11.6768   0.676777  -0.175       1   1   1
  734   11   8.69569e-08  -0.7       1   0   1
  735   12   1.73913e-07  -0.7       1   0   1
  736   11   8.69569e-08  -0.15       1   0   1
  737   12   1.73913e-07  -0.175       1   0   1
  738   10.8536   0.353554  -1.2       1   1   1
  739   11.6768   0.676777  -1.2       1   1   1
  740   11   8.69569e-08  -1.2       1   0   1
  741   12   1.73913e-07  -1.2       1   0   1
  742   9   9.13044e-07  -0.7       1   0   1
  743   9.14645   0.353554  -0.7       1   1   1
  744   8.32323   0.676777  -0.7       1   1   1
  745   8   8.26087e-07  -0.7       1   0   1
  746   9   9.13044e-07  -0.15       1   0   1
  747   9.14645   0.353554  -0.15       1   1   1
  748   8.32323   0.676777  -0.175       1   1   1
  749   8   8.26087e-07  -0.175       1   0   1
  750   9.2929   0.707107  -0.7       1   1   1
  751   8.64645   1.35355  -0.7       1   1   1
  752   9.2929   0.707107  -0.15       1   1   1
  753   8.64645   1.35355  -0.175       1   1   1
  754   9   9.13044e-07  -1.2       1   0   1
  755   9.14645   0.353554  -1.2       1   1   1
  756   8.32323   0.676777  -1.2       1   1   1
  757   8   8.26087e-07  -1.2       1   0   1
  758   9.2929   0.707107  -1.2       1   1   1
  759   8.64645   1.35355  -1.2       1   1   1
  760   9.64645   0.853554  -0.7       1   1   1
  761   9.32323   1.67678  -0.7       1   1   1
  762   9.64645   0.853554  -0.15       1   1   1
  763   9.32323   1.67678  -0.175       1   1   1
  764   9.64645   0.853554  -1.2       1   1   1
  765   9.32323   1.67678  -1.2       1   1   1
  766   -0.999996   3.9375  -0.2       1   1   1
  767   -0.999996   5.375  -0.2       1   1   1
  768   -0.999996   3.9375  -0.7       1   1   1
  769   -0.999996   5.375  -0.7       1   1   1
  770   -2   3.75  -0.2       1   1   1
  771   -2   5.5  -0.2       1   1   1
  772   -2   3.75  -0.7       1   1   1
  773   -2   5.5  -0.7       1   1   1
  774   -0.999996   2.5  -0.2       1   1   1
  775   -0.999996   2.5  -0.7       1   1   1
  776   -2   2  -0.2       1   1   1
  777   -2   2  -0.7       1   1   1
  778   -0.999996   3.9375  -1.2       1   1   1
  779   -0.999996   5.375  -1.2       1   1   1
  780   -2   3.75  -1.2       1   1   1
  781   -2   5.5  -1.2       1   1   1
  782   -0.999996   2.5  -1.2       1   1   1
  783   -2   2  -1.2       1   1   1
  784   -2.5   1  -0.2       1   1   1
  785   -3.75   1  -0.2       1   1   1
  786   -3.5   2  -0.2       1   1   1
  787   -2.5   1  -0.7       1   1   1
  788   -3.75   1  -0.7       1   1   1
  789   -3.5   2  -0.7       1   1   1
  790   -5   1  -0.2       1   1   1
  791   -5   2  -0.2       1   1   1
  792   -5   1  -0.7       1   1   1
  793   -5   2  -0.7       1   1   1
  794   -3   -1.30434e-07  -0.2       1   0   1
  795   -4   -2.17391e-07  -0.2       1   0   1
  796   -3   -1.30434e-07  -0.7       1   0   1
  797   -4   -2.17391e-07  -0.7       1   0   1
  798   -5   -3.04347e-07  -0.2       1   0   1
  799   -5   -3.04347e-07  -0.7       1   0   1
  800   -2.5   1  -1.2       1   1   1
  801   -3.75   1  -1.2       1   1   1
  802   -3.5   2  -1.2       1   1   1
  803   -5   1  -1.2       1   1   1
  804   -5   2  -1.2       1   1   1
  805   -3   -1.30434e-07  -1.2       1   0   1
  806   -4   -2.17391e-07  -1.2       1   0   1
  807   -5   -3.04347e-07  -1.2       1   0   1
  808   -5   3.75  -0.2       1   1   1
  809   -3.5   3.75  -0.2       1   1   1
  810   -5   3.75  -0.7       1   1   1
  811   -3.5   3.75  -0.7       1   1   1
  812   -5   5.5  -0.2       1   1   1
  813   -3.5   5.5  -0.2       1   1   1
  814   -5   5.5  -0.7       1   1   1
  815   -3.5   5.5  -0.7       1   1   1
  816   -5   3.75  -1.2       1   1   1
  817   -3.5   3.75  -1.2       1   1   1
  818   -5   5.5  -1.2       1   1   1
  819   -3.5   5.5  -1.2       1   1   1
  820   -6.5   3.75  -0.2       1   1   1
  821   -6.5   5.5  -0.2       1   1   1
  822   -6.5   3.75  -0.7       1   1   1
  823   -6.5   5.5  -0.7       1   1   1
  824   -8   3.75  -0.2       1   1   0
  825   -8   5.5  -0.2       1   1   0
  826   -8   3.75  -0.7       1   1   0
  827   -8   5.5  -0.7       1   1   0
  828   -6.5   2  -0.2       1   1   1
  829   -6.5   2  -0.7       1   1   1
  830   -8   2  -0.2       1   1   0
  831   -8   2  -0.7       1   1   0
  832   -6.5   3.75  -1.2       1   1   1
  833   -6.5   5.5  -1.2       1   1   1
  834   -8   3.75  -1.2       1   1   0
  835   -8   5.5  -1.2       1   1   0
  836   -6.5   2  -1.2       1   1   1
  837   -8   2  -1.2       1   1   0
  838   -6.5   1  -0.2       1   1   1
  839   -6.5   1  -0.7       1   1   1
  840   -8   0.999999  -0.2       1   1   0
  841   -8   0.999999  -0.7       1   1   0
  842   -6.5   -4.34782e-07  -0.2       1   0   1
  843   -6.5   -4.34782e-07  -0.7       1   0   1
  844   -8   -5.65217e-07  -0.2       1   0   0
  845   -8   -5.65217e-07  -0.7       1   0   0
  846   -6.5   1  -1.2       1   1   1
  847   -8   0.999999  -1.2       1   1   0
  848   -6.5   -4.34782e-07  -1.2       1   0   1
  849   -8   -5.65217e-07  -1.2       1   0   0
  850   4e-06   1.30435e-07  -0.7       1   0   1
  851   -0.249996   1.08696e-07  -0.7       1   0   1
  852   -0.229996   0.23  -0.7       1   1   1
  853   3.97826e-06   0.25  -0.7       1   1   1
  854   -0.209996   0.46  -0.7       1   1   1
  855   3.95652e-06   0.5  -0.7       1   1   1
  856   -0.499996   8.69569e-08  -0.7       1   0   1
  857   -0.459996   0.21  -0.7       1   1   1
  858   -0.419996   0.42  -0.7       1   1   1
  859   4e-06   1.30435e-07  -1.2       1   0   1
  860   -0.249996   1.08696e-07  -1.2       1   0   1
  861   -0.229996   0.23  -1.2       1   1   1
  862   3.97826e-06   0.25  -1.2       1   1   1
  863   -0.209996   0.46  -1.2       1   1   1
  864   3.95652e-06   0.5  -1.2       1   1   1
  865   -0.499996   8.69569e-08  -1.2       1   0   1
  866   -0.459996   0.21  -1.2       1   1   1
  867   -0.419996   0.42  -1.2       1   1   1
  868   -0.749996   6.52177e-08  -0.125       1   0   1
  869   -0.656771   0.281777  -0.125       1   1   1
  870   -0.749996   6.52177e-08  -0.7       1   0   1
  871   -0.656771   0.281777  -0.7       1   1   1
  872   -0.563546   0.563554  -0.125       1   1   1
  873   -0.563546   0.563554  -0.7       1   1   1
  874   -0.999996   4.34786e-08  -0.15       1   0   1
  875   -0.853546   0.353554  -0.15       1   1   1
  876   -0.999996   4.34786e-08  -0.7       1   0   1
  877   -0.853546   0.353554  -0.7       1   1   1
  878   -0.707096   0.707107  -0.15       1   1   1
  879   -0.707096   0.707107  -0.7       1   1   1
  880   -0.749996   6.52177e-08  -1.2       1   0   1
  881   -0.656771   0.281777  -1.2       1   1   1
  882   -0.563546   0.563554  -1.2       1   1   1
  883   -0.999996   4.34786e-08  -1.2       1   0   1
  884   -0.853546   0.353554  -1.2       1   1   1
  885   -0.707096   0.707107  -1.2       1   1   1
  886   -0.281771   0.656777  -0.125       1   1   1
  887   3.93478e-06   0.75  -0.125       1   1   1
  888   -0.281771   0.656777  -0.7       1   1   1
  889   3.93478e-06   0.75  -0.7       1   1   1
  890   -0.353546   0.853554  -0.15       1   1   1
  891   -0.353546   0.853554  -0.7       1   1   1
  892   -0.281771   0.656777  -1.2       1   1   1
  893   3.93478e-06   0.75  -1.2       1   1   1
  894   -0.353546   0.853554  -1.2       1   1   1
  895   -0.676771   1.67678  -0.175       1   1   1
  896   -0.676771   1.67678  -0.7       1   1   1
  897   -1.35355   1.35355  -0.175       1   1   1
  898   -1.35355   1.35355  -0.7       1   1   1
  899   -0.676771   1.67678  -1.2       1   1   1
  900   -1.35355   1.35355  -1.2       1   1   1
  901   -1.67677   0.676777  -0.175       1   1   1
  902   -1.67677   0.676777  -0.7       1   1   1
  903   -2   -4.34779e-08  -0.175       1   0   1
  904   -2   -4.34779e-08  -0.7       1   0   1
  905   -1.67677   0.676777  -1.2       1   1   1
  906   -2   -4.34779e-08  -1.2       1   0   1
  907   0.230004   0.23  -0.7       1   1   1
  908   0.250004   1.52174e-07  -0.7       1   0   1
  909   0.210004   0.46  -0.7       1   1   1
  910   0.230004   0.23  -1.2       1   1   1
  911   0.250004   1.52174e-07  -1.2       1   0   1
  912   0.210004   0.46  -1.2       1   1   1
  913   0.281781   0.656777  -0.125       1   1   1
  914   0.281781   0.656777  -0.7       1   1   1
  915   0.281781   0.656777  -1.2       1   1   1
  916   10.6568   0.281777  -0.125       1   1   1
  917   10.75   6.52174e-08  -0.125       1   0   1
  918   10.5   4.34783e-08  -0.7       1   0   1
  919   10.46   0.21  -0.7       1   1   1
  920   10.6568   0.281777  -0.7       1   1   1
  921   10.75   6.52174e-08  -0.7       1   0   1
  922   10.5636   0.563554  -0.125       1   1   1
  923   10.42   0.42  -0.7       1   1   1
  924   10.5636   0.563554  -0.7       1   1   1
  925   10.5   4.34783e-08  -1.2       1   0   1
  926   10.46   0.21  -1.2       1   1   1
  927   10.6568   0.281777  -1.2       1   1   1
  928   10.75   6.52174e-08  -1.2       1   0   1
  929   10.42   0.42  -1.2       1   1   1
  930   10.5636   0.563554  -1.2       1   1   1
  931   10   0  -0.7       1   0   1
  932   9.75   -2.17391e-08  -0.7       1   0   1
  933   9.77   0.23  -0.7       1   1   1
  934   10   0.25  -0.7       1   1   1
  935   9.79   0.46  -0.7       1   1   1
  936   10   0.5  -0.7       1   1   1
  937   9.5   -4.34783e-08  -0.7       1   0   1
  938   9.54   0.21  -0.7       1   1   1
  939   9.58   0.42  -0.7       1   1   1
  940   10   0  -1.2       1   0   1
  941   9.75   -2.17391e-08  -1.2       1   0   1
  942   9.77   0.23  -1.2       1   1   1
  943   10   0.25  -1.2       1   1   1
  944   9.79   0.46  -1.2       1   1   1
  945   10   0.5  -1.2       1   1   1
  946   9.5   -4.34783e-08  -1.2       1   0   1
  947   9.54   0.21  -1.2       1   1   1
  948   9.58   0.42  -1.2       1   1   1
  949   9.25   -6.52174e-08  -0.125       1   0   1
  950   9.34322   0.281777  -0.125       1   1   1
  951   9.25   -6.52174e-08  -0.7       1   0   1
  952   9.34322   0.281777  -0.7       1   1   1
  953   9.43645   0.563553  -0.125       1   1   1
  954   9.43645   0.563553  -0.7       1   1   1
  955   9.25   -6.52174e-08  -1.2       1   0   1
  956   9.34322   0.281777  -1.2       1   1   1
  957   9.43645   0.563553  -1.2       1   1   1
  958   9.71822   0.656777  -0.125       1   1   1
  959   10   0.75  -0.125       1   1   1
  960   9.71822   0.656777  -0.7       1   1   1
  961   10   0.75  -0.7       1   1   1
  962   9.71822   0.656777  -1.2       1   1   1
  963   10   0.75  -1.2       1   1   1
  964   10.23   0.23  -0.7       1   1   1
  965   10.25   2.17391e-08  -0.7       1   0   1
  966   10.21   0.46  -0.7       1   1   1
  967   10.23   0.23  -1.2       1   1   1
  968   10.25   2.17391e-08  -1.2       1   0   1
  969   10.21   0.46  -1.2       1   1   1
  970   10.2818   0.656777  -0.125       1   1   1
  971   10.2818   0.656777  -0.7       1   1   1
  972   10.2818   0.656777  -1.2       1   1   1
!  END NODES 
&
!  ======================================================
ELEM
   6  ! 2pt and 8pt isoparametric hexahedrons
!  id    n1 n2 n3 n4     n5 n6 n7 n8    material 
1 1 2 3 4 5 6 7 8 1
2 4 3 9 10 8 7 11 12 1
3 2 13 14 3 6 15 16 7 1
4 3 14 17 9 7 16 18 11 1
5 5 6 7 8 19 20 21 22 1
6 8 7 11 12 22 21 23 24 1
7 6 15 16 7 20 25 26 21 1
8 7 16 18 11 21 26 27 23 1
9 17 28 29 30 18 31 32 33 1
10 30 29 34 35 33 32 36 37 1
11 28 38 39 29 31 40 41 32 1
12 29 39 42 34 32 41 43 36 1
13 18 31 32 33 27 44 45 46 1
14 33 32 36 37 46 45 47 48 1
15 31 40 41 32 44 49 50 45 1
16 32 41 43 36 45 50 51 47 1
17 35 52 53 30 37 54 55 33 1
18 30 53 9 17 33 55 11 18 1
19 52 56 57 53 54 58 59 55 1
20 53 57 10 9 55 59 12 11 1
21 37 54 55 33 48 60 61 46 1
22 33 55 11 18 46 61 23 27 1
23 54 58 59 55 60 62 63 61 1
24 55 59 12 11 61 63 24 23 1
25 64 65 66 67 68 69 70 71 1
26 67 66 72 73 71 70 74 75 1
27 65 76 77 66 69 78 79 70 1
28 66 77 80 72 70 79 81 74 1
29 68 69 70 71 82 83 84 85 1
30 71 70 74 75 85 84 86 87 1
31 69 78 79 70 83 88 89 84 1
32 70 79 81 74 84 89 90 86 1
33 73 72 91 92 75 74 93 94 1
34 92 91 14 13 94 93 16 15 1
35 72 80 95 91 74 81 96 93 1
36 91 95 17 14 93 96 18 16 1
37 75 74 93 94 87 86 97 98 1
38 94 93 16 15 98 97 26 25 1
39 74 81 96 93 86 90 99 97 1
40 93 96 18 16 97 99 27 26 1
41 80 100 101 95 81 102 103 96 1
42 95 101 28 17 96 103 31 18 1
43 100 104 105 101 102 106 107 103 1
44 101 105 38 28 103 107 40 31 1
45 81 102 103 96 90 108 109 99 1
46 96 103 31 18 99 109 44 27 1
47 102 106 107 103 108 110 111 109 1
48 103 107 40 31 109 111 49 44 1
49 112 113 114 115 116 117 118 119 1
50 115 114 120 121 119 118 122 123 1
51 113 124 125 114 117 126 127 118 1
52 114 125 128 120 118 127 129 122 1
53 130 131 132 133 112 113 114 115 1
54 133 132 134 135 115 114 120 121 1
55 131 136 137 132 113 124 125 114 1
56 132 137 138 134 114 125 128 120 1
57 128 139 140 141 129 142 143 144 1
58 141 140 145 146 144 143 147 148 1
59 139 149 150 140 142 151 152 143 1
60 140 150 153 145 143 152 154 147 1
61 138 155 156 157 128 139 140 141 1
62 157 156 158 159 141 140 145 146 1
63 155 160 161 156 139 149 150 140 1
64 156 161 162 158 140 150 153 145 1
65 146 163 164 141 148 165 166 144 1
66 141 164 120 128 144 166 122 129 1
67 163 167 168 164 165 169 170 166 1
68 164 168 121 120 166 170 123 122 1
69 159 171 172 157 146 163 164 141 1
70 157 172 134 138 141 164 120 128 1
71 171 173 174 172 163 167 168 164 1
72 172 174 135 134 164 168 121 120 1
73 5 175 176 8 1 177 178 4 1
74 8 176 179 12 4 178 180 10 1
75 175 181 182 176 177 183 184 178 1
76 176 182 185 179 178 184 186 180 1
77 19 187 188 22 5 175 176 8 1
78 22 188 189 24 8 176 179 12 1
79 187 190 191 188 175 181 182 176 1
80 188 191 192 189 176 182 185 179 1
81 185 193 194 195 186 196 197 198 1
82 195 194 199 200 198 197 201 202 1
83 193 124 113 194 196 126 117 197 1
84 194 113 112 199 197 117 116 201 1
85 192 203 204 205 185 193 194 195 1
86 205 204 206 207 195 194 199 200 1
87 203 136 131 204 193 124 113 194 1
88 204 131 130 206 194 113 112 199 1
89 200 208 209 195 202 210 211 198 1
90 195 209 179 185 198 211 180 186 1
91 208 58 59 209 210 56 57 211 1
92 209 59 12 179 211 57 10 180 1
93 207 212 213 205 200 208 209 195 1
94 205 213 189 192 195 209 179 185 1
95 212 62 63 213 208 58 59 209 1
96 213 63 24 189 209 59 12 179 1
97 214 215 216 217 218 219 220 221 1
98 217 216 125 124 221 220 127 126 1
99 215 222 223 216 219 224 225 220 1
100 216 223 128 125 220 225 129 127 1
101 226 227 228 229 214 215 216 217 1
102 229 228 137 136 217 216 125 124 1
103 227 230 231 228 215 222 223 216 1
104 228 231 138 137 216 223 128 125 1
105 222 232 233 223 224 234 235 225 1
106 223 233 139 128 225 235 142 129 1
107 232 236 237 233 234 238 239 235 1
108 233 237 149 139 235 239 151 142 1
109 230 240 241 231 222 232 233 223 1
110 231 241 155 138 223 233 139 128 1
111 240 242 243 241 232 236 237 233 1
112 241 243 160 155 233 237 149 139 1
113 244 245 246 247 248 249 250 251 1
114 247 246 182 181 251 250 184 183 1
115 245 252 253 246 249 254 255 250 1
116 246 253 185 182 250 255 186 184 1
117 256 257 258 259 244 245 246 247 1
118 259 258 191 190 247 246 182 181 1
119 257 260 261 258 245 252 253 246 1
120 258 261 192 191 246 253 185 182 1
121 252 262 263 253 254 264 265 255 1
122 253 263 193 185 255 265 196 186 1
123 262 214 217 263 264 218 221 265 1
124 263 217 124 193 265 221 126 196 1
125 260 266 267 261 252 262 263 253 1
126 261 267 203 192 253 263 193 185 1
127 266 226 229 267 262 214 217 263 1
128 267 229 136 203 263 217 124 193 1
129 42 39 268 269 43 41 270 271 1
130 269 268 272 273 271 270 274 275 1
131 39 38 276 268 41 40 277 270 1
132 268 276 278 272 270 277 279 274 1
133 43 41 270 271 51 50 280 281 1
134 271 270 274 275 281 280 282 283 1
135 41 40 277 270 50 49 284 280 1
136 270 277 279 274 280 284 285 282 1
137 278 286 287 288 279 289 290 291 1
138 288 287 292 293 291 290 294 295 1
139 286 296 297 287 289 298 299 290 1
140 287 297 300 292 290 299 301 294 1
141 279 289 290 291 285 302 303 304 1
142 291 290 294 295 304 303 305 306 1
143 289 298 299 290 302 307 308 303 1
144 290 299 301 294 303 308 309 305 1
145 293 310 311 288 295 312 313 291 1
146 288 311 272 278 291 313 274 279 1
147 310 314 315 311 312 316 317 313 1
148 311 315 273 272 313 317 275 274 1
149 295 312 313 291 306 318 319 304 1
150 291 313 274 279 304 319 282 285 1
151 312 316 317 313 318 320 321 319 1
152 313 317 275 274 319 321 283 282 1
153 314 310 322 323 316 312 324 325 1
154 323 322 326 327 325 324 328 329 1
155 310 293 330 322 312 295 331 324 1
156 322 330 332 326 324 331 333 328 1
157 316 312 324 325 320 318 334 335 1
158 325 324 328 329 335 334 336 337 1
159 312 295 331 324 318 306 338 334 1
160 324 331 333 328 334 338 339 336 1
161 293 292 340 330 295 294 341 331 1
162 330 340 342 332 331 341 343 333 1
163 292 300 344 340 294 301 345 341 1
164 340 344 346 342 341 345 347 343 1
165 295 294 341 331 306 305 348 338 1
166 331 341 343 333 338 348 349 339 1
167 294 301 345 341 305 309 350 348 1
168 341 345 347 343 348 350 351 349 1
169 352 353 354 355 356 357 358 359 1
170 355 354 360 361 359 358 362 363 1
171 353 364 365 354 357 366 367 358 1
172 354 365 368 360 358 367 369 362 1
173 356 357 358 359 370 371 372 373 1
174 359 358 362 363 373 372 374 375 1
175 357 366 367 358 371 376 377 372 1
176 358 367 369 362 372 377 378 374 1
177 364 379 380 365 366 381 382 367 1
178 365 380 383 368 367 382 384 369 1
179 379 385 386 380 381 387 388 382 1
180 380 386 389 383 382 388 390 384 1
181 366 381 382 367 376 391 392 377 1
182 367 382 384 369 377 392 393 378 1
183 381 387 388 382 391 394 395 392 1
184 382 388 390 384 392 395 396 393 1
185 361 360 397 398 363 362 399 400 1
186 398 397 401 104 400 399 402 106 1
187 360 368 383 397 362 369 384 399 1
188 397 383 389 401 399 384 390 402 1
189 363 362 399 400 375 374 403 404 1
190 400 399 402 106 404 403 405 110 1
191 362 369 384 399 374 378 393 403 1
192 399 384 390 402 403 393 396 405 1
193 104 401 406 105 106 402 407 107 1
194 105 406 276 38 107 407 277 40 1
195 401 389 408 406 402 390 409 407 1
196 406 408 278 276 407 409 279 277 1
197 106 402 407 107 110 405 410 111 1
198 107 407 277 40 111 410 284 49 1
199 402 390 409 407 405 396 411 410 1
200 407 409 279 277 410 411 285 284 1
201 389 386 412 408 390 388 413 409 1
202 408 412 286 278 409 413 289 279 1
203 386 385 414 412 388 387 415 413 1
204 412 414 296 286 413 415 298 289 1
205 390 388 413 409 396 395 416 411 1
206 409 413 289 279 411 416 302 285 1
207 388 387 415 413 395 394 417 416 1
208 413 415 298 289 416 417 307 302 1
209 352 355 418 419 356 359 420 421 1
210 419 418 65 64 421 420 69 68 1
211 355 361 422 418 359 363 423 420 1
212 418 422 76 65 420 423 78 69 1
213 356 359 420 421 370 373 424 425 1
214 421 420 69 68 425 424 83 82 1
215 359 363 423 420 373 375 426 424 1
216 420 423 78 69 424 426 88 83 1
217 361 398 427 422 363 400 428 423 1
218 422 427 77 76 423 428 79 78 1
219 398 104 100 427 400 106 102 428 1
220 427 100 80 77 428 102 81 79 1
221 363 400 428 423 375 404 429 426 1
222 423 428 79 78 426 429 89 88 1
223 400 106 102 428 404 110 108 429 1
224 428 102 81 79 429 108 90 89 1
225 430 431 432 433 434 435 436 437 1
226 433 432 234 238 437 436 232 236 1
227 431 438 439 432 435 440 441 436 1
228 432 439 224 234 436 441 222 232 1
229 434 435 436 437 442 443 444 445 1
230 437 436 232 236 445 444 240 242 1
231 435 440 441 436 443 446 447 444 1
232 436 441 222 232 444 447 230 240 1
233 448 449 450 451 452 453 454 455 1
234 451 450 456 457 455 454 458 459 1
235 449 460 461 450 453 462 463 454 1
236 450 461 464 456 454 463 465 458 1
237 452 453 454 455 466 467 468 469 1
238 455 454 458 459 469 468 470 471 1
239 453 462 463 454 467 472 473 468 1
240 454 463 465 458 468 473 474 470 1
241 460 475 476 461 462 477 478 463 1
242 461 476 479 464 463 478 480 465 1
243 475 248 249 476 477 244 245 478 1
244 476 249 254 479 478 245 252 480 1
245 462 477 478 463 472 481 482 473 1
246 463 478 480 465 473 482 483 474 1
247 477 244 245 478 481 256 257 482 1
248 478 245 252 480 482 257 260 483 1
249 457 456 484 485 459 458 486 487 1
250 485 484 264 218 487 486 262 214 1
251 456 464 479 484 458 465 480 486 1
252 484 479 254 264 486 480 252 262 1
253 459 458 486 487 471 470 488 489 1
254 487 486 262 214 489 488 266 226 1
255 458 465 480 486 470 474 483 488 1
256 486 480 252 262 488 483 260 266 1
257 448 451 490 491 452 455 492 493 1
258 491 490 431 430 493 492 435 434 1
259 451 457 494 490 455 459 495 492 1
260 490 494 438 431 492 495 440 435 1
261 452 455 492 493 466 469 496 497 1
262 493 492 435 434 497 496 443 442 1
263 455 459 495 492 469 471 498 496 1
264 492 495 440 435 496 498 446 443 1
265 457 485 499 494 459 487 500 495 1
266 494 499 439 438 495 500 441 440 1
267 485 218 219 499 487 214 215 500 1
268 499 219 224 439 500 215 222 441 1
269 459 487 500 495 471 489 501 498 1
270 495 500 441 440 498 501 447 446 1
271 487 214 215 500 489 226 227 501 1
272 500 215 222 441 501 227 230 447 1
273 502 503 504 505 506 507 508 509 1
274 505 504 510 511 509 508 512 513 1
275 503 514 515 504 507 516 517 508 1
276 504 515 518 510 508 517 519 512 1
277 506 507 508 509 520 521 522 523 1
278 509 508 512 513 523 522 524 525 1
279 507 516 517 508 521 526 527 522 1
280 508 517 519 512 522 527 528 524 1
281 518 529 530 531 519 532 533 534 1
282 531 530 535 536 534 533 537 538 1
283 529 539 540 530 532 541 542 533 1
284 530 540 543 535 533 542 544 537 1
285 519 532 533 534 528 545 546 547 1
286 534 533 537 538 547 546 548 549 1
287 532 541 542 533 545 550 551 546 1
288 533 542 544 537 546 551 552 548 1
289 536 553 554 531 538 555 556 534 1
290 531 554 510 518 534 556 512 519 1
291 553 557 558 554 555 559 560 556 1
292 554 558 511 510 556 560 513 512 1
293 538 555 556 534 549 561 562 547 1
294 534 556 512 519 547 562 524 528 1
295 555 559 560 556 561 563 564 562 1
296 556 560 513 512 562 564 525 524 1
297 460 461 565 566 567 568 569 570 1
298 566 565 571 572 570 569 573 574 1
299 461 464 575 565 568 576 577 569 1
300 565 575 578 571 569 577 579 573 1
301 567 568 569 570 580 581 582 583 1
302 570 569 573 574 583 582 584 585 1
303 568 576 577 569 581 586 587 582 1
304 569 577 579 573 582 587 588 584 1
305 572 571 589 590 574 573 591 592 1
306 590 589 515 514 592 591 517 516 1
307 571 578 593 589 573 579 594 591 1
308 589 593 518 515 591 594 519 517 1
309 574 573 591 592 585 584 595 596 1
310 592 591 517 516 596 595 527 526 1
311 573 579 594 591 584 588 597 595 1
312 591 594 519 517 595 597 528 527 1
313 578 598 599 593 579 600 601 594 1
314 593 599 529 518 594 601 532 519 1
315 598 602 603 599 600 604 605 601 1
316 599 603 539 529 601 605 541 532 1
317 579 600 601 594 588 606 607 597 1
318 594 601 532 519 597 607 545 528 1
319 600 604 605 601 606 608 609 607 1
320 601 605 541 532 607 609 550 545 1
321 610 611 612 613 614 615 616 617 1
322 613 612 618 619 617 616 620 621 1
323 611 622 623 612 615 624 625 616 1
324 612 623 626 618 616 625 627 620 1
325 628 629 630 631 610 611 612 613 1
326 631 630 632 633 613 612 618 619 1
327 629 634 635 630 611 622 623 612 1
328 630 635 636 632 612 623 626 618 1
329 626 637 638 639 627 640 641 642 1
330 639 638 643 644 642 641 645 646 1
331 637 647 648 638 640 649 650 641 1
332 638 648 651 643 641 650 652 645 1
333 636 653 654 655 626 637 638 639 1
334 655 654 656 657 639 638 643 644 1
335 653 658 659 654 637 647 648 638 1
336 654 659 660 656 638 648 651 643 1
337 644 661 662 639 646 663 664 642 1
338 639 662 618 626 642 664 620 627 1
339 661 665 666 662 663 667 668 664 1
340 662 666 619 618 664 668 621 620 1
341 657 669 670 655 644 661 662 639 1
342 655 670 632 636 639 662 618 626 1
343 669 671 672 670 661 665 666 662 1
344 670 672 633 632 662 666 619 618 1
345 506 673 674 509 502 675 676 505 1
346 509 674 677 513 505 676 678 511 1
347 673 679 680 674 675 681 682 676 1
348 674 680 683 677 676 682 684 678 1
349 520 685 686 523 506 673 674 509 1
350 523 686 687 525 509 674 677 513 1
351 685 688 689 686 673 679 680 674 1
352 686 689 690 687 674 680 683 677 1
353 683 691 692 693 684 694 695 696 1
354 693 692 697 698 696 695 699 700 1
355 691 622 611 692 694 624 615 695 1
356 692 611 610 697 695 615 614 699 1
357 690 701 702 703 683 691 692 693 1
358 703 702 704 705 693 692 697 698 1
359 701 634 629 702 691 622 611 692 1
360 702 629 628 704 692 611 610 697 1
361 698 706 707 693 700 708 709 696 1
362 693 707 677 683 696 709 678 684 1
363 706 559 560 707 708 557 558 709 1
364 707 560 513 677 709 558 511 678 1
365 705 710 711 703 698 706 707 693 1
366 703 711 687 690 693 707 677 683 1
367 710 563 564 711 706 559 560 707 1
368 711 564 525 687 707 560 513 677 1
369 712 713 714 715 716 717 718 719 1
370 715 714 623 622 719 718 625 624 1
371 713 720 721 714 717 722 723 718 1
372 714 721 626 623 718 723 627 625 1
373 724 725 726 727 712 713 714 715 1
374 727 726 635 634 715 714 623 622 1
375 725 728 729 726 713 720 721 714 1
376 726 729 636 635 714 721 626 623 1
377 720 730 731 721 722 732 733 723 1
378 721 731 637 626 723 733 640 627 1
379 730 734 735 731 732 736 737 733 1
380 731 735 647 637 733 737 649 640 1
381 728 738 739 729 720 730 731 721 1
382 729 739 653 636 721 731 637 626 1
383 738 740 741 739 730 734 735 731 1
384 739 741 658 653 731 735 647 637 1
385 742 743 744 745 746 747 748 749 1
386 745 744 680 679 749 748 682 681 1
387 743 750 751 744 747 752 753 748 1
388 744 751 683 680 748 753 684 682 1
389 754 755 756 757 742 743 744 745 1
390 757 756 689 688 745 744 680 679 1
391 755 758 759 756 743 750 751 744 1
392 756 759 690 689 744 751 683 680 1
393 750 760 761 751 752 762 763 753 1
394 751 761 691 683 753 763 694 684 1
395 760 712 715 761 762 716 719 763 1
396 761 715 622 691 763 719 624 694 1
397 758 764 765 759 750 760 761 751 1
398 759 765 701 690 751 761 691 683 1
399 764 724 727 765 760 712 715 761 1
400 765 727 634 701 761 715 622 691 1
401 543 540 766 767 544 542 768 769 1
402 767 766 770 771 769 768 772 773 1
403 540 539 774 766 542 541 775 768 1
404 766 774 776 770 768 775 777 772 1
405 544 542 768 769 552 551 778 779 1
406 769 768 772 773 779 778 780 781 1
407 542 541 775 768 551 550 782 778 1
408 768 775 777 772 778 782 783 780 1
409 776 784 785 786 777 787 788 789 1
410 786 785 790 791 789 788 792 793 1
411 784 794 795 785 787 796 797 788 1
412 785 795 798 790 788 797 799 792 1
413 777 787 788 789 783 800 801 802 1
414 789 788 792 793 802 801 803 804 1
415 787 796 797 788 800 805 806 801 1
416 788 797 799 792 801 806 807 803 1
417 791 808 809 786 793 810 811 789 1
418 786 809 770 776 789 811 772 777 1
419 808 812 813 809 810 814 815 811 1
420 809 813 771 770 811 815 773 772 1
421 793 810 811 789 804 816 817 802 1
422 789 811 772 777 802 817 780 783 1
423 810 814 815 811 816 818 819 817 1
424 811 815 773 772 817 819 781 780 1
425 812 808 820 821 814 810 822 823 1
426 821 820 824 825 823 822 826 827 1
427 808 791 828 820 810 793 829 822 1
428 820 828 830 824 822 829 831 826 1
429 814 810 822 823 818 816 832 833 1
430 823 822 826 827 833 832 834 835 1
431 810 793 829 822 816 804 836 832 1
432 822 829 831 826 832 836 837 834 1
433 791 790 838 828 793 792 839 829 1
434 828 838 840 830 829 839 841 831 1
435 790 798 842 838 792 799 843 839 1
436 838 842 844 840 839 843 845 841 1
437 793 792 839 829 804 803 846 836 1
438 829 839 841 831 836 846 847 837 1
439 792 799 843 839 803 807 848 846 1
440 839 843 845 841 846 848 849 847 1
441 448 491 490 451 850 851 852 853 1
442 451 490 494 457 853 852 854 855 1
443 491 430 431 490 851 856 857 852 1
444 490 431 438 494 852 857 858 854 1
445 850 851 852 853 859 860 861 862 1
446 853 852 854 855 862 861 863 864 1
447 851 856 857 852 860 865 866 861 1
448 852 857 858 854 861 866 867 863 1
449 430 868 869 431 856 870 871 857 1
450 431 869 872 438 857 871 873 858 1
451 868 874 875 869 870 876 877 871 1
452 869 875 878 872 871 877 879 873 1
453 856 870 871 857 865 880 881 866 1
454 857 871 873 858 866 881 882 867 1
455 870 876 877 871 880 883 884 881 1
456 871 877 879 873 881 884 885 882 1
457 457 494 886 887 855 854 888 889 1
458 887 886 890 602 889 888 891 604 1
459 494 438 872 886 854 858 873 888 1
460 886 872 878 890 888 873 879 891 1
461 855 854 888 889 864 863 892 893 1
462 889 888 891 604 893 892 894 608 1
463 854 858 873 888 863 867 882 892 1
464 888 873 879 891 892 882 885 894 1
465 602 890 895 603 604 891 896 605 1
466 603 895 774 539 605 896 775 541 1
467 890 878 897 895 891 879 898 896 1
468 895 897 776 774 896 898 777 775 1
469 604 891 896 605 608 894 899 609 1
470 605 896 775 541 609 899 782 550 1
471 891 879 898 896 894 885 900 899 1
472 896 898 777 775 899 900 783 782 1
473 878 875 901 897 879 877 902 898 1
474 897 901 784 776 898 902 787 777 1
475 875 874 903 901 877 876 904 902 1
476 901 903 794 784 902 904 796 787 1
477 879 877 902 898 885 884 905 900 1
478 898 902 787 777 900 905 800 783 1
479 877 876 904 902 884 883 906 905 1
480 902 904 796 787 905 906 805 800 1
481 448 451 450 449 850 853 907 908 1
482 449 450 461 460 908 907 568 567 1
483 451 457 456 450 853 855 909 907 1
484 450 456 464 461 907 909 576 568 1
485 850 853 907 908 859 862 910 911 1
486 908 907 568 567 911 910 581 580 1
487 853 855 909 907 862 864 912 910 1
488 907 909 576 568 910 912 586 581 1
489 457 887 913 456 855 889 914 909 1
490 456 913 575 464 909 914 577 576 1
491 887 602 598 913 889 604 600 914 1
492 913 598 578 575 914 600 579 577 1
493 855 889 914 909 864 893 915 912 1
494 909 914 577 576 912 915 587 586 1
495 889 604 600 914 893 608 606 915 1
496 914 600 579 577 915 606 588 587 1
497 364 365 916 917 918 919 920 921 1
498 917 916 732 736 921 920 730 734 1
499 365 368 922 916 919 923 924 920 1
500 916 922 722 732 920 924 720 730 1
501 918 919 920 921 925 926 927 928 1
502 921 920 730 734 928 927 738 740 1
503 919 923 924 920 926 929 930 927 1
504 920 924 720 730 927 930 728 738 1
505 352 419 418 355 931 932 933 934 1
506 355 418 422 361 934 933 935 936 1
507 419 64 65 418 932 937 938 933 1
508 418 65 76 422 933 938 939 935 1
509 931 932 933 934 940 941 942 943 1
510 934 933 935 936 943 942 944 945 1
511 932 937 938 933 941 946 947 942 1
512 933 938 939 935 942 947 948 944 1
513 64 949 950 65 937 951 952 938 1
514 65 950 953 76 938 952 954 939 1
515 949 746 747 950 951 742 743 952 1
516 950 747 752 953 952 743 750 954 1
517 937 951 952 938 946 955 956 947 1
518 938 952 954 939 947 956 957 948 1
519 951 742 743 952 955 754 755 956 1
520 952 743 750 954 956 755 758 957 1
521 361 422 958 959 936 935 960 961 1
522 959 958 762 716 961 960 760 712 1
523 422 76 953 958 935 939 954 960 1
524 958 953 752 762 960 954 750 760 1
525 936 935 960 961 945 944 962 963 1
526 961 960 760 712 963 962 764 724 1
527 935 939 954 960 944 948 957 962 1
528 960 954 750 760 962 957 758 764 1
529 352 355 354 353 931 934 964 965 1
530 353 354 365 364 965 964 919 918 1
531 355 361 360 354 934 936 966 964 1
532 354 360 368 365 964 966 923 919 1
533 931 934 964 965 940 943 967 968 1
534 965 964 919 918 968 967 926 925 1
535 934 936 966 964 943 945 969 967 1
536 964 966 923 919 967 969 929 926 1
537 361 959 970 360 936 961 971 966 1
538 360 970 922 368 966 971 924 923 1
539 959 716 717 970 961 712 713 971 1
540 970 717 722 922 971 713 720 924 1
541 936 961 971 966 945 963 972 969 1
542 966 971 924 923 969 972 930 929 1
543 961 712 713 971 963 724 725 972 1
544 971 713 720 924 972 725 728 930 1
!  END ELEM 
&
!  ======================================================= 
LOAD 
3
TRAC DX 380 1
18.000000  0 0 18 5.5 1  inflate 0.01
!  END LOADS 
&
LOAD 
3
TRAC DX -380 1
-8 0 -1.2 -8 5.50 -0.20000  inflate 0.01
!  END LOADS 
&
!  END LOADS 
!  ======================================================= 
HISTORY 
!  client_tag (N, E, W)     entity# (0 for energy (W)) 
! W
E 530 364  10.5   0  -0.1 
!  END HISTORY 
&
!  ======================================================= 
REDUCTION 
  collect
     dt 0.02
  end
  use
     nmodes 6
     file spliceref.pod
  end
&
!  END DATA 
EOF

echo $ncomput
