// -*- Mode: c++ -*-
// $Id: BoundaryStencil.h,v 1.1.1.1 2001/07/24 02:41:51 pkrysl Exp $
// $Source: /home/cvs/WASP/problems/FE/subdiv/Coke/Loop/BoundaryStencil.h,v $

#ifndef __LOOP_BOUNDARYSTENCIL_H__
#define __LOOP_BOUNDARYSTENCIL_H__

#include "Loop/Topology.h"
#include "Util/cvec.h"
#include "Util/util.h"
#include "Loop/Permutation.h"

/**
 * BoundaryStencil Class
 */
template <class Stencil>
class BoundaryStencil {
private:
  const Stencil& m_stencil;

private:
  BoundaryStencil& operator=(const BoundaryStencil& instance) {
    die();
    return (*this);
  }

public:
  BoundaryStencil(const Stencil& stencil) : m_stencil(stencil) { }

  BoundaryStencil(const BoundaryStencil& instance)
   : m_stencil(instance.m_stencil) { }

  ~BoundaryStencil(void) { }

  template <class TriangleMr, class Accessor, class Accumulator>
  void even(TriangleMr* triangle, VertexLabel v, Accessor accessor, Accumulator &result) const {
    if (isBoundaryVertex(triangle, v)) {
      typename TriangleMr::VertexMr* vertex = triangle->vertex(v);
      int depth = triangle->depth();
      result += accessor(vertex, depth);
      return;
    } else {
      m_stencil.even(triangle, v, accessor, result);
      return;
    }
  }

#pragma set woff 1478

  template <class TriangleMr, class Accessor, class Stencil2>
  CVec limit(TriangleMr* triangle,
	     VertexLabel v,
	     Accessor accessor,
	     const Stencil2& stencil) const
  {
    if (isBoundaryVertex(triangle, v)) {
      typename TriangleMr::VertexMr* vertex = triangle->vertex(v);
      int depth = triangle->depth();
      return (accessor(vertex, depth));
    } else {
      return (m_stencil.limit(triangle, v, accessor, stencil));
    }
  }

#pragma reset woff 1478

  template <class TriangleMr, class Accessor, class Accumulator>
  void odd(TriangleMr* triangle, EdgeLabel e, Accessor accessor, Accumulator &result) const {
    Permutation p;
    if (triangle->adjacentTriangle(e, p) == 0) {
      int depth = triangle->depth();
      result += 0.5 * accessor(triangle->vertex(CCWVertex(e)), depth);
      result += 0.5 * accessor(triangle->vertex(CWVertex(e)), depth);
      return;
    } else {
      m_stencil.odd(triangle, e, accessor, result);
      return;
    }
  }

private:
  template <class TriangleMr>
  bool isBoundaryVertex(TriangleMr* triangle, VertexLabel v) const {
    if (triangle->isRoot()) {
      return (triangle->triangle()->vertex(v)->isBoundaryVertex());
    }

    TriangleLabel t = triangle->label();
    if (t == T0 || v != ParentVertex(t)) {
      EdgeLabel e = OppositeEdge(v);
      Permutation p;

      if (triangle->parent()->adjacentTriangle(e, p) == 0)
	return (true);
      else
	return (false);
    }

    VertexLabel pivot = v;
    VertexLabel vertex = CCWVertex(pivot);
    TriangleMr* next = triangle->parent();
    Permutation p;
    
    do {
      next = next->adjacentTriangle(OppositeEdge(vertex), p);
      if (next == 0) {
	return (true);
      }

      pivot = p.vertex(pivot);
      vertex = OppositeVertex(p.vertex(vertex), pivot);
    } while (next != triangle->parent());

    return (false);
  }
};

#endif  /* __LOOP_BOUNDARYSTENCIL_H__ */
