// -*- Mode: c++ -*-
// $Id: ControlTriangle.h,v 1.1.1.1 2001/07/24 02:41:53 pkrysl Exp $
// $Source: /home/cvs/WASP/problems/FE/subdiv/Coke/Loop/ControlTriangle.h,v $

#ifndef __LOOP_CONTROLTRIANGLE_H__
#define __LOOP_CONTROLTRIANGLE_H__

#include "Loop/Permutation.h"
#include "Mesh/Accessor.h"
#include "Mesh/TriangleTp.h"
#include "Util/util.h"

#include "Loop/LoopTypes.h"

/**
 * ControlTriangle Class
 */
template <class T>
class ControlTriangle : public TriangleTp<T> {
public:
  DECLARE_LOOP_TYPES( T )

  template <class Mesh> friend class Accessor;

private:
  /**
   * The encoding of the permutation mapping for the triangles adjacent to
   * this triangle.
   */
  short m_perm1:4, m_perm2:4, m_perm3:4;

  /** The root of the subdivision hierarchy at this triangle. */
  TriangleMr* m_root;

private:
  /**
   * Assignment operator: undefined operation.
   */
  ControlTriangle& operator=(const ControlTriangle& instance) {
    assert(false);
    return (*this);
  }

protected:
  void setAdjacentTriangle(const EdgeLabel e, Triangle* triangle) {
    TriangleTp<T>::setAdjacentTriangle(e, triangle);

    switch (e) {
    case E1:
      m_perm1 = Permutation::encode(this, e);
      break;
    case E2:
      m_perm2 = Permutation::encode(this, e);
      break;
    case E3:
      m_perm3 = Permutation::encode(this, e);
      break;
    }
  }

public:
  /**
   * Constructs an uninitialized triangle.  Initially the triangle does not
   * belong to the topology of a mesh.
   */
  ControlTriangle(void)
   : TriangleTp<T>(),
     m_perm1(0), m_perm2(0), m_perm3(0),
     m_root(0) { }

  /**
   * Copy constructor.
   */
  ControlTriangle(const ControlTriangle& instance)
   : TriangleTp<T>(instance),
     m_perm1(0), m_perm2(0), m_perm3(0),
     m_root(0) { }

#pragma set woff 1388
  
  /**
   * Destructor.
   */
  ~ControlTriangle(void) {
    if (m_root)
      delete (m_root);
  }
  
#pragma reset woff 1388

  /**
   * Returns the permutation mapping for the adjacent triangle corresponding
   * to the specified edge.
   * @param e  triangle edge
   * @return permutation mapping for adjacent triangle across edge
   */
  Permutation permutation(const EdgeLabel e) const {
    switch (e) {
    case E1:
      return Permutation(m_perm1);
    case E2:
      return Permutation(m_perm2);
    case E3:
      return Permutation(m_perm3);
    }

    die();
    return Permutation();  // prevent compiler warning
  }

  /**
   * Returns an immutable reference to the root of the subdivision hierarchy
   * at this triangle.
   * @return immutable reference to root of subdivision hierarchy at this
   *         triangle
   */
  TriangleMr* root(void) const {
    return (m_root);
  }

  /**
   * Defines the vertices of this triangle, in counter-clockwise orientation.
   * The triangle must be non-degenerate: each vertex must be unique.  This
   * method must be called exactly once for each triangle.
   * @param v1  first vertex
   * @param v2  second vertex
   * @param v3  third vertex
   */
  void setVertices(Vertex* v1, Vertex* v2, Vertex* v3) {
    TriangleTp<T>::setVertices(v1, v2, v3);

    assert( m_root == NULL );
    m_root = new TriangleMr();
    assert(m_root);
    m_root->initialize((Triangle*)this);
  }
};


#endif  /* __LOOP_CONTROLTRIANGLE_H__ */
