#include "Util/forall.h"

#include "Permutation.h"

template <class T>
void LoopTesselator<T>::tesselate(int depth) {
  assert(depth >= 0);

  typename Mesh::const_TriangleIt ti = m_loop.triangleBegin();
  typename Mesh::const_TriangleIt te = m_loop.triangleEnd();
  for (; ti != te; ++ti) {
    TriangleMr* triangle = (*ti)->root();
    m_stack.push(triangle);

    while (!m_stack.empty()) {
      triangle = m_stack.top();
      m_stack.pop();

      if (triangle->isLeaf()) {
	tesselate(triangle, depth);
      } else if (triangle->depth() == depth) {
	tesselate(triangle, depth);
      } else {
	int below = triangle->depth() + 1;
	TriangleMr* child = triangle->child(T0);
	if (!triangle->vertex(V1)->isValid(below) ||
	    !triangle->vertex(V2)->isValid(below) ||
	    !triangle->vertex(V3)->isValid(below) ||
	    !child->vertex(V1)->isValid(below) ||
	    !child->vertex(V2)->isValid(below) ||
	    !child->vertex(V3)->isValid(below))
	  {
	    tesselate(triangle, depth);
	  } else {
	    for (int t = T0; t <= T3; t += 1) {
	      m_stack.push(triangle->child(TriangleLabel(t)));
	    }
	  }
      }
    }
  }
}


template <class T>
void LoopTesselator<T>::tesselate(TriangleMr* triangle, int depth) {
  assert(depth >= 0);

  m_stack.push(triangle);

  while (!m_stack.empty()) {
    triangle = m_stack.top();
    m_stack.pop();

    if (triangle->isLeaf()) {
      tesselateTriangle(triangle, depth);
    } else if (triangle->depth() == depth) {
      tesselateTriangle(triangle, depth);
    } else {
      int below = triangle->depth() + 1;
      TriangleMr* child = triangle->child(T0);
      if (!triangle->vertex(V1)->isValid(below) ||
	  !triangle->vertex(V2)->isValid(below) ||
	  !triangle->vertex(V3)->isValid(below) ||
	  !child->vertex(V1)->isValid(below) ||
	  !child->vertex(V2)->isValid(below) ||
	  !child->vertex(V3)->isValid(below))
	{
	  tesselateTriangle(triangle, depth);
	} else {
	  for (int t = T0; t <= T3; t += 1) {
	    m_stack.push(triangle->child(TriangleLabel(t)));
	  }
	}
    }
  }
}


template <class T>
void LoopTesselator<T>::tesselateTriangle(TriangleMr* triangle, int depth) {
  Orientation o = triangle->orientation();
  TesselatedVertex vertices[6];

  for (int i = V1; i <= V3; i += 1) {
    vertices[i] = TesselatedVertex( triangle, VertexLabel(i) );
  }

  int mask = 0;
  Permutation p;
  for (int e = E1; e <= E3; e += 1) {
    TriangleMr* adjacent = triangle->adjacentTriangle(EdgeLabel(e), p);
    if (!adjacent || adjacent->isLeaf() || adjacent->depth() == depth)
      continue;

    VertexLabel v = p.vertex(OppositeVertex(EdgeLabel(e)));
    if (!adjacent->child(T0)->vertex(v)->isValid(triangle->depth() + 1))
      continue;

    assert(adjacent && adjacent->child(T0) && adjacent->child(T0)->vertex(v));
    vertices[e + 3] = TesselatedVertex( adjacent->child(T0), v );
    mask |= 1 << e;
  }

  switch (mask) {
  case 0:
    addTriangle(vertices[0], vertices[1], vertices[2], o);
    break;
      
  case 1:
    addTriangle(vertices[0], vertices[3], vertices[2], o);
    addTriangle(vertices[0], vertices[1], vertices[3], o);
    break;
      
  case 2:
    addTriangle(vertices[0], vertices[1], vertices[4], o);
    addTriangle(vertices[1], vertices[2], vertices[4], o);
    break;
      
  case 3:
    addTriangle(vertices[0], vertices[1], vertices[3], o);
    addTriangle(vertices[0], vertices[3], vertices[4], o);
    addTriangle(vertices[2], vertices[4], vertices[3], o);
    break;
      
  case 4:
    addTriangle(vertices[0], vertices[5], vertices[2], o);
    addTriangle(vertices[1], vertices[2], vertices[5], o);
    break;
      
  case 5:
    addTriangle(vertices[0], vertices[5], vertices[2], o);
    addTriangle(vertices[1], vertices[3], vertices[5], o);
    addTriangle(vertices[2], vertices[5], vertices[3], o);
    break;
      
  case 6:
    addTriangle(vertices[0], vertices[5], vertices[4], o);
    addTriangle(vertices[1], vertices[2], vertices[4], o);
    addTriangle(vertices[1], vertices[4], vertices[5], o);
    break;
      
  case 7:
    addTriangle(vertices[0], vertices[5], vertices[4], o);
    addTriangle(vertices[1], vertices[3], vertices[5], o);
    addTriangle(vertices[2], vertices[4], vertices[3], o);
    addTriangle(vertices[3], vertices[4], vertices[5], o);
    break;
  }
}


template <class T>
void LoopTesselator<T>::addTriangle(const TesselatedVertex &v1,
				    const TesselatedVertex &v2,
				    const TesselatedVertex &v3,
				    int orientation)
{
  if (Orientation(orientation) == TriangleMr::CLOCKWISE)
    m_triangles.push_back( make_triplet( v1, v3, v2 ) );
  else
    m_triangles.push_back( make_triplet( v1, v2, v3 ) );
}


#if 0
Vertex* vertex(TriangleMr* triangle, VertexLabel v) {
  VertexMr* vertex = triangle->vertex(v);
  if (m_table.find(vertex) != m_table.end())
    return (m_assembler->getVertex(m_table[vertex]));

  Triangle* domain = triangle->triangle();
  TesselatedVertexTag<Triangle>* tag;
  if (triangle->vertex(v)->isControlVertex()) {
    tag = new TesselatedControlVertexTag<Triangle>(domain, v);
  } else {
    CVec coords(0.0f, 0.0f, 0.0f);
    coords(v) = 1.0f;

    TriangleMr* next = triangle;
    while (!next->isRoot()) {
      switch (next->label()) {
      case T0:
	coords = coords.x() * CVec(0.0f, 0.5f, 0.5f) +
	  coords.y() * CVec(0.5f, 0.0f, 0.5f) +
	  coords.z() * CVec(0.5f, 0.5f, 0.0f);
	break;

      case T1:
	coords = coords.x() * CVec(1.0f, 0.0f, 0.0f) +
	  coords.y() * CVec(0.5f, 0.0f, 0.5f) +
	  coords.z() * CVec(0.5f, 0.5f, 0.0f);
	break;

      case T2:
	coords = coords.x() * CVec(0.0f, 0.5f, 0.5f) +
	  coords.y() * CVec(0.0f, 1.0f, 0.0f) +
	  coords.z() * CVec(0.5f, 0.5f, 0.0f);
	break;

      case T3:
	coords = coords.x() * CVec(0.0f, 0.5f, 0.5f) +
	  coords.y() * CVec(0.5f, 0.0f, 0.5f) +
	  coords.z() * CVec(0.0f, 0.0f, 1.0f);
	break;
      }

      next = next->parent();
    }

    if (coords.x() == 0.0f)
      tag = new TesselatedEdgeVertexTag<Triangle>(domain, E1, coords);
    else if (coords.y() == 0.0f)
      tag = new TesselatedEdgeVertexTag<Triangle>(domain, E2, coords);
    else if (coords.z() == 0.0f)
      tag = new TesselatedEdgeVertexTag<Triangle>(domain, E3, coords);
    else
      tag = new TesselatedInteriorVertexTag<Triangle>(domain, coords);
  }
  assert(tag);

  Vertex* newVertex = new Vertex(tag, triangle->vertex(v));
  assert(newVertex);
  if (m_limitType == LIMIT) {
    Accessor accessor;
    m_loop->limit(triangle, v);
    newVertex->setPoint(accessor.limit(vertex));
  } else if (m_limitType == NORMAL) {
    assert(triangle->vertex(v) == vertex);
    newVertex->setPoint(m_loop->normal(triangle, v));
  } else {
    Accessor accessor;
    newVertex->setPoint(accessor(vertex, triangle->depth()));
  }

  int index = m_assembler->addVertex(newVertex);
  m_table[vertex] = index;

  // NJL: detail temperature hack
  m_vertexMap[newVertex] = vertex;
  return (newVertex);				
}
#endif


template <class T>
CVec LoopTesselator<T>::evalPoint(const TesselatedVertex& v, LimitType limitType) {

  assert( v.t != NULL );

  if ( limitType == LIMIT ) {

    m_loop.limit( v.t, v.v );
    return v.t->vertex(v.v)->limit();
  } 
  else if ( limitType == NORMAL ) {

    return m_loop.normal( v.t, v.v );
  } 
  else {

    return v.t->vertex(v.v)->point( v.t->depth() );
  }
}


template <class T>
template <class MT>
typename MT::Mesh* LoopTesselator<T>::generateMesh(const MT& meshtype, LimitType limitType) {

  typedef typename MT::Mesh     MT_Mesh;
  typedef typename MT::Vertex   MT_Vertex;
  typedef typename MT::Triangle MT_Triangle;

  MT_Mesh* mesh = new MT_Mesh;

  hash_map< VertexMr*, MT_Vertex*, typename VertexMr::hash > h;

  Assembler<MT> assembler(*mesh);

  forall(TesselatedTriangleList::const_iterator, t, m_triangles.begin(), m_triangles.end())

    MT_Triangle* triangle = new MT_Triangle;
    VertexMr* vertex;

    assert( t.a.t != NULL );
    vertex = t.a.t->vertex( t.a.v );
    assert( vertex != NULL );
    MT_Vertex* va = h[ vertex ];

    if (va == NULL) {

      va = new MT_Vertex;
      assert(va);

      va->setPoint( evalPoint( t.a, limitType ) );

      assembler.addVertex(va);
      h[ vertex ] = va;
    }

    assert( t.b.t != NULL );
    vertex = t.b.t->vertex( t.b.v );
    assert( vertex != NULL );
    MT_Vertex* vb = h[ vertex ];

    if (vb == NULL) {

      vb = new MT_Vertex;
      assert(vb);

      vb->setPoint( evalPoint( t.b, limitType ) );

      assembler.addVertex(vb);
      h[ vertex ] = vb;
    }

    assert( t.c.t != NULL );
    vertex = t.c.t->vertex( t.c.v );
    assert( vertex != NULL );
    MT_Vertex* vc = h[ vertex ];

    if (vc == NULL) {

      vc = new MT_Vertex;
      assert(vc);

      vc->setPoint( evalPoint( t.c, limitType ) );

      assembler.addVertex(vc);
      h[ vertex ] = vc;
    }

    assert( va != NULL  &&  vb != NULL  &&  vc != NULL );
    triangle->setVertices( va, vb, vc );

    assembler.addTriangle( triangle );

  endforall;

  assembler.assembleMesh();

  return mesh;
}
