// -*- Mode: c++ -*-
// $Id: LoopTesselator.h,v 1.1.1.1 2001/07/24 02:41:51 pkrysl Exp $
// $Source: /home/cvs/WASP/problems/FE/subdiv/Coke/Loop/LoopTesselator.h,v $

#ifndef __LOOP_LOOPTESSELATOR_H__
#define __LOOP_LOOPTESSELATOR_H__

#include <hash_map.h>
#include <stack.h>
#include "Loop/Topology.h"
#include "Mesh/MeshTp.h"
#include "Mesh/PointVertex.h"
#include "Mesh/TriangleTp.h"

#include "Loop/LoopTypes.h"

template <class A, class B, class C> 
struct triplet {
  triplet(const A& p_a, const B& p_b, const C& p_c) 
    : a(p_a), b(p_b), c(p_c) { }

  A a;
  B b;
  C c;
};

template <class A, class B, class C> 
triplet<A,B,C> make_triplet(const A& p_a, const B& p_b, const C& p_c) {
  return triplet<A,B,C>(p_a, p_b, p_c);
}

/**
 * LoopTesselator Class
 */
template <class T>
class LoopTesselator {
public:
  DECLARE_LOOP_TYPES( T )

  typedef typename TriangleMr::Orientation Orientation;

  //typedef hash_map<Vertex*, VertexMr*, typename Vertex::hash> ;

  enum LimitType{NONE, LIMIT, NORMAL};

private:
  //Assembler* m_assembler;

  Mesh& m_loop;

  stack<TriangleMr*> m_stack;

  //hash_map<VertexMr*, , VertexMr::hash> m_table;

  struct TesselatedVertex {

    TesselatedVertex() { }

    TesselatedVertex(TriangleMr* p_t, VertexLabel p_v)
      : t(p_t), v(p_v) { }

    TriangleMr *t;
    VertexLabel v;
  };

  typedef triplet<TesselatedVertex, TesselatedVertex, TesselatedVertex> TesselatedTriangle;

  typedef vector<TesselatedTriangle> TesselatedTriangleList;

  TesselatedTriangleList m_triangles;

  //LimitType m_limitType;

  // NJL: detail temperature hack
  //VertexMap m_vertexMap;

public:
  LoopTesselator(Mesh& loopMesh) : m_loop(loopMesh) { }

  ~LoopTesselator(void) { }

  // NJL: detail temperature hack
  //VertexMap& getVertexMap(void) { return (m_vertexMap); }

  void tesselate(int depth = __MAX_DEPTH);

  void tesselate(TriangleMr* triangle, int depth = __MAX_DEPTH);

  template <class MT>
  typename MT::Mesh* generateMesh(const MT& meshtype, LimitType limitType = LIMIT);

private:

  void tesselateTriangle(TriangleMr* triangle, int depth);

  void addTriangle(const TesselatedVertex &v1,
		   const TesselatedVertex &v2,
		   const TesselatedVertex &v3,
		   int orientation);

  CVec evalPoint(const TesselatedVertex& v, LimitType limitType);
};

#include "Loop/LoopTesselator.cc"

#endif  /* __LOOP_LOOPTESSELATOR_H__ */
