// -*- Mode: c++ -*-
// $Id: MeshTp.h,v 1.1.1.1 2001/07/24 02:41:34 pkrysl Exp $
// $Source: /home/cvs/WASP/problems/FE/subdiv/Coke/Mesh/MeshTp.h,v $

#ifndef __MESH_MESHTP_H__
#define __MESH_MESHTP_H__

#include <algo.h>
#include <vector.h>
#include <iostream.h>
#include "Mesh/Accessor.h"
#include "Mesh/SMF.h"
#include "Util/util.h"
#include "Mesh/EdgeTp.h"

#include "Mesh/MeshTypes.h"

/**
 * MeshTp Class
 */
template <class T>
class MeshTp {

public:
  DECLARE_MESH_TYPES( T )

  friend class Accessor<T>;
  friend class Assembler<T>;

  typedef vector<Triangle*>                  TriangleCt;      
  typedef TriangleCt::iterator               TriangleIt;      
  typedef TriangleCt::const_iterator         const_TriangleIt;
  typedef vector<Vertex*>                    VertexCt;        
  typedef VertexCt::iterator                 VertexIt;        
  typedef VertexCt::const_iterator           const_VertexIt;  

protected:
  /** The set of triangles in this mesh. */
  TriangleCt m_triangles;
  
  /** The set of vertices in this mesh. */
  VertexCt m_vertices;

private:
  /**
   * Assignment operator: undefined operation.
   */
  MeshTp& operator=(const MeshTp&) {
    die();
    return (*this);
  }

public:
  /**
   * Constructs an empty mesh.
   */
  MeshTp(void) { }

  /**
   * Copy constructor.
   */
  MeshTp(const MeshTp&) { }

  /**
   * Destructor.
   */
  ~MeshTp(void) {
    for_each(triangleBegin(), triangleEnd(), Delete<Triangle>());
    for_each(vertexBegin(), vertexEnd(), Delete<Vertex>());
  }

  /**
   * Returns a constant iterator over the triangles in this mesh, positioned
   * at the start of the triangle list.
   * @return constant iterator over triangles in this mesh, positioned at start
   *	     of list
   */
  const_TriangleIt triangleBegin(void) const {
    return (m_triangles.begin());
  }

  /**
   * Returns an iterator over the triangles in this mesh, positioned at the
   * start of the triangle list.
   * @return iterator over triangles in this mesh, positioned at start of list
   */
  TriangleIt triangleBegin(void) {
    return (m_triangles.begin());
  }

  /**
   * Returns the number of triangles in this mesh.
   * @return number of triangles in this mesh
   */
  int triangleCount(void) const {
    return (m_triangles.size());
  }

  /**
   * Returns a constant iterator over the triangles in this mesh, positioned
   * at the end of the triangle list.
   * @return constant iterator over triangles in this mesh, positioned at end
   *	     of list
   */
  const_TriangleIt triangleEnd(void) const {
    return (m_triangles.end());
  }

  /**
   * Returns an iterator over the triangles in this mesh, positioned at the
   * end of the triangle list.
   * @return iterator over triangles in this mesh, positioned at end of list
   */
  TriangleIt triangleEnd(void) {
    return (m_triangles.end());
  }

  /**
   * Returns a constant iterator over the vertices in this mesh, positioned
   * at the start of the vertex list.
   * @return constant iterator over vertices in this mesh, positioned at start
   *	     of list
   */
  const_VertexIt vertexBegin(void) const {
    return (m_vertices.begin());
  }

  /**
   * Returns an iterator over the vertices in this mesh, positioned at the
   * start of the vertex list.
   * @return iterator over vertices in this mesh, positioned at start of list
   */
  VertexIt vertexBegin(void) {
    return (m_vertices.begin());
  }

  /**
   * Returns the number of vertices in this mesh.
   * @return number of vertices in this mesh
   */
  int vertexCount(void) const {
    return (m_vertices.size());
  }

  /**
   * Returns a constant iterator over the vertices in this mesh, positioned
   * at the end of the vertex list.
   * @return constant iterator over vertices in this mesh, positioned at end
   *         of list
   */
  const_VertexIt vertexEnd(void) const {
    return (m_vertices.end());
  }

  /**
   * Returns an iterator over the vertices in this mesh, positioned at the
   * end of the vertex list.
   * @return iterator over vertices in this mesh, positioned at end of list
   */
  VertexIt vertexEnd(void) {
    return (m_vertices.end());
  }

  /* Methods for backwards compatibility. */
  int triangleSize(void) const {
    return (m_triangles.size());
  }

  int vertexSize(void) const {
    return (m_vertices.size());
  }
};

template <class T>
istream& operator>>(istream& is, MeshTp<T>& mesh)
{
    SMF::readMesh(is, mesh);
    return (is);
}

template <class T>
ostream& operator<<(ostream& os,
		    const MeshTp<T>& mesh)
{
    SMF::writeMesh(os, mesh);
    return (os);
}


#define MESHTYPE_INSTANTIATE_TYPES(TYPE_STRUCT, MTYPE, Mesh_CLASS, Triangle_CLASS, Vertex_CLASS) \
typedef Mesh_CLASS<TYPE_STRUCT> MTYPE;                       \
typedef Triangle_CLASS<TYPE_STRUCT> MTYPE##_Triangle;        \
typedef Vertex_CLASS<TYPE_STRUCT> MTYPE##_Vertex;            \
typedef EdgeTp<TYPE_STRUCT> MTYPE##_Edge;                    

#define MESHTYPE_DEFINE_TYPE_STRUCT(TYPE_STRUCT, MTYPE, Mesh_CLASS, Triangle_CLASS, Vertex_CLASS) \
struct TYPE_STRUCT {                                          \
  typedef MTYPE##_Types                      Types;           \
  typedef ::MTYPE                            Mesh;            \
  typedef MTYPE##_Triangle                   Triangle;        \
  typedef MTYPE##_Vertex                     Vertex;          \
  typedef MTYPE##_Edge                       Edge;            \
};

#define MESHTYPE(MTYPE, Mesh_CLASS, Triangle_CLASS, Vertex_CLASS)                            \
struct MTYPE##_Types;                                                                        \
MESHTYPE_INSTANTIATE_TYPES(  MTYPE##_Types, MTYPE, Mesh_CLASS, Triangle_CLASS, Vertex_CLASS) \
MESHTYPE_DEFINE_TYPE_STRUCT( MTYPE##_Types, MTYPE, Mesh_CLASS, Triangle_CLASS, Vertex_CLASS) \

#endif  /* __MESH_MESHTP_H__ */
