// -*- Mode: c++ -*-
// $Id: Accessor.h,v 1.1.1.1 2001/07/24 02:41:34 pkrysl Exp $
// $Source: /home/cvs/WASP/problems/FE/subdiv/Coke/Mesh/bak/Accessor.h,v $

#ifndef __ACCESSOR_H__
#define __ACCESSOR_H__

#include "Mesh/Labels.h"

template <class TriangleIf, class VertexIf>
class MeshTp;

/**
 * Accessor Class
 */
template <class TriangleIf, class VertexIf>
class Accessor {
protected:
  typedef MeshTp<TriangleIf, VertexIf> Mesh;
  typedef Mesh::Triangle Triangle;
  typedef Mesh::Vertex Vertex;

private:
  /** The mesh being accessed. */
  Mesh& m_mesh;

public:
  /**
   * Destructor.
   */
  ~Accessor(void) { }
  
protected:
  /**
   * Constructs an accessor for a specified mesh.
   * @param mesh  mesh to access
   */
  Accessor(Mesh& mesh) : m_mesh(mesh) { }

  /**
   * Returns the mesh being accessed.
   * @return mesh being accessed
   */
  Mesh& mesh(void) {
    return (m_mesh);
  }

  /**
   * Returns the list of triangles for the mesh.
   * @return list of triangles for the mesh
   */
  Mesh::TriangleCt& getTriangles(void) const {
    return (m_mesh.m_triangles);
  }
  
  /**
   * Returns the list of vertices for the mesh.
   * @return list of vertices for the mesh
   */
  Mesh::VertexCt& getVertices(void) const {
    return (m_mesh.m_vertices);
  }
  
  /**
   * Defines the triangle adjacent to a specified triangle along a common
   * edge.  It is the responsibility of the caller to maintain the
   * reciprocal relation in the adjacent triangle if not null.
   * @param triangle  triangle
   * @param e  edge label
   * @param adjacent  adjacent triangle
   */
  void setAdjacentTriangle(Triangle* triangle,
			   const EdgeLabel e,
			   Triangle* adjacent) const
  {
    triangle->setAdjacentTriangle(e, adjacent);
  }
  
  /**
   * Defines the triangle incident to a specified vertex.  It is the
   * responsibility of the caller to respect the clockwise-most triangle
   * convention for the assignment.
   * @param vertex  vertex
   * @param triangle  incident triangle
   */
  void setIncidentTriangle(Vertex* vertex, Triangle* triangle) const {
    vertex->m_triangle = triangle;
  }
  
  /**
   * Defines the vertex incident to a specified triangle at a given corner.
   * It is the responsibility of the caller to propagate this definition to
   * the adjacent triangles, and to refer the vertex to its clockwise-most
   * incident triangle.
   * @param triangle  triangle
   * @param v  label of incident vertex
   * @param vertex  incident vertex
   */
  void setIncidentVertex(Triangle* triangle,
			 const VertexLabel v,
			 Vertex* vertex) const
  {
    triangle->setIncidentVertex(v, vertex);
  }
};

#endif  /* __ACCESSOR_H__ */
