// -*- Mode: c++ -*-
// $Id: MeshTp.h,v 1.1.1.1 2001/07/24 02:41:34 pkrysl Exp $
// $Source: /home/cvs/WASP/problems/FE/subdiv/Coke/Mesh/bak/MeshTp.h,v $

#ifndef __MESHTP_H__
#define __MESHTP_H__

#include <algo.h>
#include <vector.h>
#include <iostream.h>
#include "Mesh/SMF.h"
#include "Util/util.h"

/**
 * MeshTp Class
 */
template <class TriangleIf, class VertexIf>
class MeshTp {
  friend class Accessor<TriangleIf, VertexIf>;

public:
  typedef TriangleIf Triangle;
  typedef VertexIf Vertex;

  typedef vector<Triangle*> TriangleCt;
  typedef TriangleCt::iterator TriangleIt;
  typedef TriangleCt::const_iterator const_TriangleIt;
  typedef vector<Vertex*> VertexCt;
  typedef VertexCt::iterator VertexIt;
  typedef VertexCt::const_iterator const_VertexIt;
  typedef MeshTp Mesh;

protected:
  /** The set of triangles in this mesh. */
  TriangleCt m_triangles;
  
  /** The set of vertices in this mesh. */
  VertexCt m_vertices;

public:
  /**
   * Constructs an empty mesh.
   */
  MeshTp(void) { }

  /**
   * Destructor.
   */
  ~MeshTp(void) {
    for_each(triangleBegin(), triangleEnd(), Delete<Triangle>());
    for_each(vertexBegin(), vertexEnd(), Delete<Vertex>());
  }

  /**
   * Returns a constant iterator over the triangles in this mesh, positioned
   * at the start of the triangle list.
   * @return constant iterator over triangles in this mesh, positioned at start
   *	     of list
   */
  const_TriangleIt triangleBegin(void) const {
    return (m_triangles.begin());
  }

  /**
   * Returns an iterator over the triangles in this mesh, positioned at the
   * start of the triangle list.
   * @return iterator over triangles in this mesh, positioned at start of list
   */
  TriangleIt triangleBegin(void) {
    return (m_triangles.begin());
  }

  /**
   * Returns the number of triangles in this mesh.
   * @return number of triangles in this mesh
   */
  int triangleCount(void) const {
    return (m_triangles.size());
  }

  /**
   * Returns a constant iterator over the triangles in this mesh, positioned
   * at the end of the triangle list.
   * @return constant iterator over triangles in this mesh, positioned at end
   *	     of list
   */
  const_TriangleIt triangleEnd(void) const {
    return (m_triangles.end());
  }

  /**
   * Returns an iterator over the triangles in this mesh, positioned at the
   * end of the triangle list.
   * @return iterator over triangles in this mesh, positioned at end of list
   */
  TriangleIt triangleEnd(void) {
    return (m_triangles.end());
  }

  /**
   * Returns a constant iterator over the vertices in this mesh, positioned
   * at the start of the vertex list.
   * @return constant iterator over vertices in this mesh, positioned at start
   *	     of list
   */
  const_VertexIt vertexBegin(void) const {
    return (m_vertices.begin());
  }

  /**
   * Returns an iterator over the vertices in this mesh, positioned at the
   * start of the vertex list.
   * @return iterator over vertices in this mesh, positioned at start of list
   */
  VertexIt vertexBegin(void) {
    return (m_vertices.begin());
  }

  /**
   * Returns the number of vertices in this mesh.
   * @return number of vertices in this mesh
   */
  int vertexCount(void) const {
    return (m_vertices.size());
  }

  /**
   * Returns a constant iterator over the vertices in this mesh, positioned
   * at the end of the vertex list.
   * @return constant iterator over vertices in this mesh, positioned at end
   *         of list
   */
  const_VertexIt vertexEnd(void) const {
    return (m_vertices.end());
  }

  /**
   * Returns an iterator over the vertices in this mesh, positioned at the
   * end of the vertex list.
   * @return iterator over vertices in this mesh, positioned at end of list
   */
  VertexIt vertexEnd(void) {
    return (m_vertices.end());
  }

  /* Methods for backwards compatibility. */
  int triangleSize(void) const {
    return (m_triangles.size());
  }

  int vertexSize(void) const {
    return (m_vertices.size());
  }

private:
  /**
   * Copy constructor: undefined operation.
   */
  MeshTp(const MeshTp&) {
    die();
  }

  /**
   * Assignment operator: undefined operation.
   */
  MeshTp& operator=(const MeshTp&) {
    die();
    return (*this);
  }
};

template <class TriangleIf, class VertexIf>
istream& operator>>(istream& is, MeshTp<TriangleIf, VertexIf>& mesh) {
    SMF::readMesh(is, mesh);
    return (is);
}

template <class TriangleIf, class VertexIf>
ostream& operator<<(ostream& os, const MeshTp<TriangleIf, VertexIf>& mesh) {
    SMF::writeMesh(os, mesh);
    return (os);
}

#define MESHTYPE(MTYPE, MCLASS, TCLASS, VCLASS)                               \
    class MTYPE##_Triangle;                                                   \
    class MTYPE##_Vertex;                                                     \
                                                                              \
    class MTYPE##_Triangle : public TCLASS<MTYPE##_Triangle, MTYPE##_Vertex> {\
      public:                                                                 \
        MTYPE##_Triangle(void)                                                \
	 : TCLASS<MTYPE##_Triangle, MTYPE##_Vertex>() { }                     \
        ~MTYPE##_Triangle(void) { }                                           \
    };                                                                        \
                                                                              \
    class MTYPE##_Vertex : public VCLASS<MTYPE##_Triangle, MTYPE##_Vertex> {  \
      public:                                                                 \
        MTYPE##_Vertex(void)                                                  \
         : VCLASS<MTYPE##_Triangle, MTYPE##_Vertex>() { }                     \
        ~MTYPE##_Vertex(void) { }                                             \
    };                                                                        \
                                                                              \
    typedef MCLASS<MTYPE##_Triangle, MTYPE##_Vertex> MTYPE;                   \

#endif  /* __MESHTP_H__ */
