// -*- Mode: c++ -*-
// $Id: iv2smf.cc,v 1.1.1.1 2001/07/24 02:41:34 pkrysl Exp $
// $Source: /home/cvs/WASP/problems/FE/subdiv/Coke/Mesh/iv2smf.cc,v $

#include <iostream.h>
#include <fstream.h>
#include <vector.h>
#include <Inventor/actions/SoSearchAction.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoIndexedFaceSet.h>
#include <Inventor/nodes/SoSeparator.h>
#include "Mesh/Assembler.h"
#include "Mesh/MeshTp.h"
#include "Mesh/PointVertex.h"
#include "Mesh/TriangleTp.h"

MESHTYPE(Mesh, MeshTp, TriangleTp, PointVertex)

int main(int argc, char* argv[]) {
  if (argc < 2) {
    cerr << "Usage: " << argv[0] << " <input.iv> [<output.smf>]" << endl;
    exit(0);
  }

  // Read input file.
  SoDB::init();
  SoInput input;
  assert(input.openFile(argv[1]));
  SoSeparator* root = SoDB::readAll(&input);
  assert(root);
  input.closeFile();
  root->ref();

  // Locate first face set node.
  SoSearchAction searchFaces;
  searchFaces.setInterest(SoSearchAction::FIRST);
  searchFaces.setType(SoIndexedFaceSet::getClassTypeId());
  searchFaces.apply(root);
  SoPath* pathFaces = searchFaces.getPath();
  pathFaces->ref();
  SoIndexedFaceSet* faces = (SoIndexedFaceSet*)pathFaces->getTail();
  pathFaces->unref();

  // Obtain coordinate vector.
  SoMFVec3f* coords;
  SoVertexProperty* vertexProperty;
  vertexProperty = (SoVertexProperty*)faces->vertexProperty.getValue();
  if (vertexProperty) {
    coords = &(vertexProperty->vertex);
  } else {
    // Locate coordinate vector.
    SoSearchAction searchCoords;
    searchCoords.setInterest(SoSearchAction::LAST);
    searchCoords.setType(SoCoordinate3::getClassTypeId());
    searchCoords.apply(pathFaces);
    SoPath* pathCoords = searchCoords.getPath();
    pathCoords->ref();
    SoCoordinate3* coordinate3 = (SoCoordinate3*)pathCoords->getTail();
    coords = &(coordinate3->point);
    pathCoords->unref();
  }

  // Construct mesh.
  Mesh mesh;
  Assembler<Mesh> assembler(mesh);

  int n = coords->getNum();
  for (int i = 0; i < n; ++i) {
    SbVec3f point = (*coords)[i];
    Mesh::Vertex* vertex = new Mesh::Vertex();
    vertex->setPoint(CVec(point[0], point[1], point[2]));
    assembler.addVertex(vertex);
  }

  int m = faces->coordIndex.getNum() / 4;
  for (int j = 0; j < m; ++j) {
    int index = j * 4;
    Mesh::Vertex* vertex1 = assembler.getVertex(faces->coordIndex[index + 0]);
    Mesh::Vertex* vertex2 = assembler.getVertex(faces->coordIndex[index + 1]);
    Mesh::Vertex* vertex3 = assembler.getVertex(faces->coordIndex[index + 2]);
    assert(faces->coordIndex[index + 3] == SO_END_FACE_INDEX);
    Mesh::Triangle* triangle = new Mesh::Triangle();
    triangle->setVertices(vertex1, vertex2, vertex3);
    assembler.addTriangle(triangle);
  }

  assembler.assembleMesh();

  // Output mesh.
  if (argc > 2) {
    ofstream out(argv[2]);
    out << mesh;
    out.close();
  } else {
    cout << mesh;
  }
}
