#ifndef	__FEMTRIANGLE_H__
#define	__FEMTRIANGLE_H__

// Do not change anything above this line

// include files
#include "Mesh/Labels.h"
#include "Mesh/TriangleTp.h"

#include "Simulator/SimTypes.h"

template <class T>
class ShellDOF;

// class definition
template <class T>
class FEMTriangle : public TriangleTp<T>{

DECLARE_SIM_TYPES(T)

 public:

  FEMTriangle( void )
    : TriangleTp<T>(),
      m_statee1(NOTHING), m_statee2(NOTHING), m_statee3(NOTHING)
    { m_d[E1] = m_d[E2] = m_d[E3] = 0; }
  FEMTriangle( Vertex *a, Vertex *b, Vertex *c )
    : TriangleTp<T>( a, b, c ),
      m_statee1(NOTHING), m_statee2(NOTHING), m_statee3(NOTHING)
    { m_d[E1] = m_d[E2] = m_d[E3] = 0; }

  ~FEMTriangle( void );

  typedef Vertex* VertexP;
  
  bool Crease( const EdgeLabel e ) const{
    return e == E1 ? m_statee1 == CREASE :
      ( e == E2 ? m_statee2 == CREASE :	m_statee3 == CREASE );
  }
  bool Convex( const EdgeLabel e ) const{
    return e == E1 ? m_statee1 == CONVEX :
      ( e == E2 ? m_statee2 == CONVEX : m_statee3 == CONVEX );
  }
  bool Concave( const EdgeLabel e ) const{
    return e == E1 ? m_statee1 == CONCAVE :
      ( e == E2 ? m_statee2 == CONCAVE : m_statee3 == CONCAVE );
  }
  bool Crease( const VertexP v1, const VertexP v2 ) const{ return Crease( edge(v1,v2) ); }
  bool Concave( const VertexP v1, const VertexP v2 ) const{ return Concave( edge(v1,v2) ); }
  bool Convex( const VertexP v1, const VertexP v2 ) const{ return Convex( edge(v1,v2) ); }

  void AttachGaussPoints( void );

  void setCrease( const EdgeLabel e ){
    switch(e){
    case E1: m_statee1 = CREASE; break;
    case E2: m_statee2 = CREASE; break;
    case E3: m_statee3 = CREASE; break;
    default: die();
    }
  }
  void setConvex( const EdgeLabel e ){
    switch(e){
    case E1: m_statee1 = CONVEX; break;
    case E2: m_statee2 = CONVEX; break;
    case E3: m_statee3 = CONVEX; break;
    default: die();
    }
  }
  void setConcave( const EdgeLabel e ){
    switch(e){
    case E1: m_statee1 = CONCAVE; break;
    case E2: m_statee2 = CONCAVE; break;
    case E3: m_statee3 = CONCAVE; break;
    default: die();
    }
  }
  void unsetTags( const EdgeLabel e ){
    switch(e){
    case E1: m_statee1 = NOTHING; break;
    case E2: m_statee2 = NOTHING; break;
    case E3: m_statee3 = NOTHING; break;
    default: die();
    }
  }

  friend ostream& operator<<( ostream& os, const FEMTriangle<T>& t )
    {
      t.print( os ); return os;
    }

  GaussPoint *getGaussPoint( const EdgeLabel e ){
    assert(e == E1 || e == E2 || e == E3);
    return m_d[e];
  }  

 protected:
  void print( ostream& os ) const;
 private:
  GaussPoint *m_d[3];

  enum EdgeState{
    NOTHING = 0,
    CREASE = 1,
    CONVEX = 2,
    CONCAVE = 3
  };
  // an edge can be nothing special, crease, convex, or concave
  EdgeState m_statee1:2, m_statee2:2, m_statee3:2;
};

// extern defs for non-member outline functions go here
// externally visible constant defs go here

#include "FEMTriangle.cc"

#endif	/* __FEMTRIANGLE_H__ */
