// -*- Mode: c++ -*-
// $Id: testquad.cc,v 1.1.1.1 2001/07/24 02:41:34 pkrysl Exp $
// $Source: /home/cvs/WASP/problems/FE/subdiv/Coke/Quadrature/test/testquad.cc,v $

#include <iostream.h>
#include <fstream.h>
#include <vector.h>

#include "FEMMesh.h"
#include "FEMTriangle.h"
#include "ShellDOF.h"

#include <Inventor/SoInput.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoIndexedFaceSet.h>
#include <Inventor/nodes/SoIndexedLineSet.h>
#include <Inventor/nodes/SoComplexity.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoLineSet.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/actions/SoWriteAction.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoTransformSeparator.h>
#include <Inventor/nodes/SoSphere.h>

#include "Loop/BoundaryStencil.h"
#include "Loop/ControlTriangle.h"
#include "Loop/ControlVertex.h"
#include "Loop/LoopMesh.h"
#include "Loop/MultiresVertex.h"
#include "Loop/WarrenStencil.h"

MESHTYPE(FEMMESH, FEMMesh, FEMTriangle, ShellDOF)

LOOPTYPE(LOOPMESH, LoopMesh, ControlTriangle, ControlVertex, MultiresVertex,
	 BoundaryStencil<WarrenStencil>)

template <class T>
class hash<T *> {
public:
  size_t operator()(const T *v) const {
    return hash<char*>()((char*)v);
  }
};

hash_map<FEMMESH::Triangle *, LOOPMESH::Triangle *> FEM_to_Loop;


void testTolerance( char *name, CVec quadValue, CVec approxValue, double tolerance, double &maxError ) {
  double mag = (quadValue - approxValue).l2();
  maxError = max(maxError, mag);
  if (mag > tolerance) {
    for (int i=0; i<90; ++i) cerr << "\b";
    cerr << name << ": error=" << mag << " quad: <" << quadValue <<  "> |" << quadValue.l2() 
	 << "|    approx: <" << approxValue << "> |" << approxValue.l2() << "|\n";
  }
}


void
main(int argc, char* argv[])
{
  WarrenStencil warren;
  BoundaryStencil<WarrenStencil> stencil(warren);
  LOOPMESH loopMesh(stencil);
  FEMMESH femMesh;
  int subdivisionLevels = 2;
  double tolerance = 1.0e-6;
  double maxError = 0;

  if (argc < 2) {
    cerr << "testquad mesh.smf [subdivision level] [tolerance]" << endl;
    return;
  }

  ifstream fin1(argv[1]), fin2(argv[1]);

  if (argc >= 3) 
    subdivisionLevels = atoi(argv[2]);

  assert(subdivisionLevels > 1 && subdivisionLevels < 15);

  if (argc >=4)
    tolerance = atof(argv[3]);

  assert(tolerance > 0.0 && tolerance < 1.0 );

  fin1 >> femMesh;

  cerr << "Done reading FEM mesh." << endl;

  fin2 >> loopMesh;

  cerr << "Done reading loop mesh." << endl;

  {
    FEMMESH::VertexIt vb = femMesh.vertexBegin(), ve = femMesh.vertexEnd();
    for( ; vb != ve; ++vb ) (*vb)->InitOneRing();
    FEMMESH::TriangleIt tb = femMesh.triangleBegin(), te = femMesh.triangleEnd();
    for( ; tb != te; ++tb ) (*tb)->AttachDiamonds();
    tb = femMesh.triangleBegin();
  }

  cerr << "Done initializing FEM mesh." << endl;

  {
    int count = 0;
    FEMMESH::TriangleIt femT = femMesh.triangleBegin(), femTEnd = femMesh.triangleEnd();
    for(; femT != femTEnd; ++femT) {
      ++count;
      if (count % 100 == 0) {
	for (int i=0; i<90; ++i) cerr << "\b";
	cerr << count << " triangles matched";
      }
      LOOPMESH::TriangleIt loopT = loopMesh.triangleBegin(), loopTEnd = loopMesh.triangleEnd();
      for(; loopT != loopTEnd; ++loopT) {
	if (    (*femT)->vertex(V1)->point() ==  (*loopT)->vertex(V1)->point()
	     && (*femT)->vertex(V2)->point() ==  (*loopT)->vertex(V2)->point()
	     && (*femT)->vertex(V3)->point() ==  (*loopT)->vertex(V3)->point() ) {
	  FEM_to_Loop[*femT] = (*loopT);
	  break;
	}
      }
      if (loopT == loopTEnd) {
	cerr << "Could not match up the meshes" << endl;
	return;
      }
    }
    for (int i=0; i<90; ++i) cerr << "\b";
    cerr << count << " triangles matched\n";
  }  

  cerr << "Done matching the meshes." <<  endl;

  loopMesh.subdivide(subdivisionLevels);

  cerr << "Done subdividing to depth " << subdivisionLevels << ".\n";

  {
    int count = 0;
    maxError = 0;
    hash_map<Diamond<FEMMESH::Triangle,FEMMESH::Vertex> *, bool> diamondProcessed;
    FEMMESH::TriangleIt femT = femMesh.triangleBegin(), femTEnd = femMesh.triangleEnd();
    for(; femT != femTEnd; ++femT) {
      ++count;
      if (count % 100 == 0) {
	for (int i=0; i<90; ++i) cerr << "\b";
	cerr << count << " triangles matched";
      }

      LOOPMESH::Triangle *loopControlT = FEM_to_Loop[*femT];
      assert(loopControlT);
      LOOPMESH::TriangleMr *loopT = loopControlT->root();
      assert(loopT);

      for (int edgeNum=E1; edgeNum<=E3; ++edgeNum) {
	Diamond<FEMMESH::Triangle, FEMMESH::Vertex> *diamond = (*femT)->getDiamond(EdgeLabel(edgeNum));
	if (diamondProcessed[diamond]) continue;
	diamondProcessed[diamond] = true;
	LOOPMESH::TriangleMr *tri;
	
	CVec pos = diamond->Evalp();

	tri = loopT->child(T0);
	assert(tri);

	VertexLabel lMidpointVertex = OppositeVertex(EdgeLabel(edgeNum));

	CVec pOrigin = loopMesh.limit(tri, lMidpointVertex);
	double mag = (pos-pOrigin).l2();

	maxError = max(maxError, mag);
	if (mag > tolerance) {
	  for (int i=0; i<90; ++i) cerr << "\b";
	  cerr <<  mag << " <" << pos <<  "> <" << pOrigin << ">\n";
	}

	double scale = 2.0;
	tri = loopT->child(T0);

	while (!tri->isLeaf()) {
	  tri = tri->child(ChildTriangle(lMidpointVertex));
	  scale *= 2.0;
	}

	assert(tri->orientation() == LOOPMESH::TriangleMr::CLOCKWISE);

	LOOPMESH::TriangleMr::VertexIt v = tri->vertexBegin(lMidpointVertex), 
	                              ve = tri->vertexEnd(lMidpointVertex);

	assert( (*v) == tri->vertex(CWVertex(lMidpointVertex)) );

	CVec approxDx(0.0), approxDy(0.0);
	CVec approxDxdx = -32.0 * tri->vertex(lMidpointVertex)->point(tri->depth());
	assert(approxDx == CVec(0,0,0));
	assert(approxDy == CVec(0,0,0));
	double   dxStencil[6] = {2.0,  4.0,  2.0, -2.0, -4.0, -2.0};
	double   dyStencil[6] = {2.0,  0.0, -2.0, -2.0,  0.0,  2.0};
	double dxdxStencil[6] = {0.0, 16.0,  0.0, 16.0,  0.0,  0.0};
	int idx = 0;

	for ( ; v != ve; ++v, ++idx) {
	  if (idx == 5)
	    assert( (*v) == tri->vertex(CCWVertex(lMidpointVertex)) );
	  approxDx   +=   dxStencil[idx] * (*v)->point(tri->depth());
	  approxDy   +=   dyStencil[idx] * (*v)->point(tri->depth());
	  approxDxdx += dxdxStencil[idx] * (*v)->point(tri->depth());
	}

	assert(idx == 6);

	approxDx   = approxDx   * scale / 3.0;
	approxDy   = approxDy   * scale / sqrt(2.0);
	approxDxdx = approxDxdx * scale;

	approxDx   /= -4.0;
	approxDy   /= -4.0;
	approxDxdx /= -8.0;

	CVec quadDx    = diamond->Evaldp1();
	CVec quadDy    = diamond->Evaldp2();
	CVec quadDxdx = diamond->Evaldp11();

        testTolerance("dx", quadDx, approxDx, tolerance, maxError);
        testTolerance("dy", quadDy, approxDy, tolerance, maxError);
        testTolerance("dxdx", quadDxdx, approxDxdx, tolerance, maxError);

	cerr << "quad dx.dy = " << quadDx.dot(quadDy) << "   approx dx.dy = " << approxDx.dot(approxDy) << endl;
	cerr << "ang<ap-dx,q-dx> = " << 180*acos(approxDx.dot(quadDx)/(approxDx.l2()*quadDx.l2()))/M_PI << "   ";
	cerr << "ang<ap-dx,q-dy> = " << 180*acos(approxDx.dot(quadDy)/(approxDx.l2()*quadDy.l2()))/M_PI << endl;
	cerr << "ang<ap-dy,q-dx> = " << 180*acos(approxDy.dot(quadDx)/(approxDy.l2()*quadDx.l2()))/M_PI << "   ";
	cerr << "ang<ap-dy,q-dy> = " << 180*acos(approxDy.dot(quadDy)/(approxDy.l2()*quadDy.l2()))/M_PI << endl;
	cerr << "ang<ap-dxdx,q-dxdx> = " << 180*acos(approxDxdx.dot(quadDxdx)/(approxDxdx.l2()*quadDxdx.l2()))/M_PI << endl;
	cerr << endl;
      }
    }
    for (int i=0; i<90; ++i) cerr << "\b";
    cerr << count << " triangles verified\n";
  }  

  cerr << "Maximum error: " << maxError << endl;

  cerr << "Done verifying quadrature code\n";

  //  for( ; tb != te; ++tb ) cout << **tb;
  //  ProduceInventorFile(mesh);
}






class BuildMesh {
public:
  template <class Mesh, int N=2>
  Mesh::Triangle *operator()(Mesh &mesh) {
    Assembler<Mesh::Triangle, Mesh::Vertex> a(mesh);
    int x, y;

    Mesh::Vertex *grid[N+1][N+1];

    for (y=0; y < N+1; ++y)
      for (x=0; x < N+1; ++x) {
	double X = x;
	double Y = y;
	grid[x][y] = new Mesh::Vertex();
	if (y%2 == 0)
	  grid[x][y]->point() = CVec(X+0.5, Y*sqrt(3.0)/2.0, 0.0);
	else 
	  grid[x][y]->point() = CVec(X, Y*sqrt(3.0)/2.0, 0.0);
	a.addVertex(grid[x][y]);
      }

    for (y=1; y < N+1; y+=2)
      for (x=1; x < N; ++x) {
	Mesh::Triangle *t = new MESH::Triangle;
	t->setVertices( a.getVertex(grid[x  ][y  ]),
			a.getVertex(grid[x-1][y-1]),
			a.getVertex(grid[x  ][y-1]));
	a.addTriangle(t);
      }


    for (y=2; y < N+1; y+=2)
      for (x=0; x < N; ++x) {
	Mesh::Triangle *t = new MESH::Triangle;
	t->setVertices( a.getVertex(grid[x  ][y  ]),
			a.getVertex(grid[x  ][y-1]),
			a.getVertex(grid[x+1][y-1]));
	a.addTriangle(t);
      }


    for (y=1; y < N; y+=2)
      for (x=1; x < N; ++x) {
	Mesh::Triangle *t = new MESH::Triangle;
	t->setVertices( a.getVertex(grid[x  ][y  ]),
			a.getVertex(grid[x  ][y+1]),
			a.getVertex(grid[x-1][y+1]));
	a.addTriangle(t);
      }

    for (y=0; y < N; y+=2)
      for (x=0; x < N; ++x) {
	Mesh::Triangle *t = new MESH::Triangle;
	t->setVertices( a.getVertex(grid[x  ][y  ]),
			a.getVertex(grid[x+1][y+1]),
			a.getVertex(grid[x  ][y+1]));
	a.addTriangle(t);
      }

    assert(a.assembleMesh());

#if 0
    // NJL: this belongs in an assembler object
    // EG: I agree! Please put it there.
    Mesh::VertexIt vi = mesh.vertexBegin();
    Mesh::VertexIt ve = mesh.vertexEnd();

    for (; vi != ve; ++vi) {
      Mesh::Vertex* vertex = *vi;
      vertex->vertex()->point(0) = vertex->point();
      vertex->vertex()->setValid(0);
    }
#endif
  }
};

