#include <math.h>

// #include "shell.h"

#include "Util/Real.h"

// svmrtr.c, sa3der0, sa3der1



void sinv3 (Real a[3][3]) {

  /* inversion of a unsymmetric 3x3 matrix a */

  Real b11,b12,b13,b21,b22,b23,b31,b32,b33, det;
  int i,j;

  b11 = a[0][0];
  b12 = a[0][1];
  b13 = a[0][2];
  b21 = a[1][0];
  b22 = a[1][1];
  b23 = a[1][2];
  b31 = a[2][0];
  b32 = a[2][1];
  b33 = a[2][2];

  a[0][0] =   b22*b33 - b32*b23;
  a[1][0] = - b21*b33 + b31*b23;
  a[2][0] =   b21*b32 - b31*b22;
  a[0][1] = - b12*b33 + b32*b13;
  a[1][1] =   b11*b33 - b31*b13;
  a[2][1] = - b11*b32 + b31*b12;
  a[0][2] =   b12*b23 - b22*b13;
  a[1][2] = - b11*b23 + b21*b13;
  a[2][2] =   b11*b22 - b21*b12;

  det = b11*a[0][0] + b12*a[1][0] + b13*a[2][0];
  for (i=0; i<3; i++) 
    for (j=0; j<3; j++)
      a[i][j] = a[i][j]/det;
                        
  return;
}



void greenlagrstrain (Real gs[3][3], Real gmkovr[3][3], 
		      Real gmkovc[3][3])
/* compute green lagrange strains                        */
{

  gs[0][0] = 0.5*(gmkovc[0][0]-gmkovr[0][0]);
  gs[0][1] = 0.5*(gmkovc[0][1]-gmkovr[0][1]);
  gs[0][2] = 0.5*(gmkovc[0][2]-gmkovr[0][2]);
  gs[1][1] = 0.5*(gmkovc[1][1]-gmkovr[1][1]);
  gs[1][2] = 0.5*(gmkovc[1][2]-gmkovr[1][2]);
  gs[2][2] = 0.5*(gmkovc[2][2]-gmkovr[2][2]);
  gs[1][0] = gs[0][1];
  gs[2][0] = gs[0][2];
  gs[2][1] = gs[1][2];

  return;
}




void kirchhoffmater (Real emod, Real nue, Real gmkonr[3][3], 
		     Real gs[3][3], Real pk2[3][3])
/* material tensor for elasticity                        */
/* input : deformation gradient                          */
/* output : 1.PK stresses                                */
{

  int i,j,k,l;
  Real xl1, xl2, c[3][3][3][3], cc[6][6]; 


  /* compute constitutive tensor */

  xl1= emod*nue/(1.0-nue*nue);   /* plane stress */

  xl2= emod/2.0/(1.0+nue);

  for (i=0; i<3; i++)
    for (j=0; j<3; j++)
      for (k=0; k<3; k++)
	for (l=0; l<3; l++)
	  c[i][j][k][l] = xl1*gmkonr[i][j]*gmkonr[k][l] 
	    + xl2* (gmkonr[i][k]*gmkonr[j][l]+gmkonr[i][l]*gmkonr[k][j]);

  cc[0][0]=c[0][0][0][0];  cc[0][1]=c[0][0][1][0];  
  cc[0][2]=c[0][0][2][0];  cc[0][3]=c[0][0][1][1];  
  cc[0][4]=c[0][0][2][1];  cc[0][5]=c[0][0][2][2];
  
  cc[1][0]=c[1][0][0][0];  cc[1][1]=c[1][0][1][0];  
  cc[1][2]=c[1][0][2][0];  cc[1][3]=c[1][0][1][1];  
  cc[1][4]=c[1][0][2][1];  cc[1][5]=c[1][0][2][2];
      
  cc[2][0]=c[2][0][0][0];  cc[2][1]=c[2][0][1][0];  
  cc[2][2]=c[2][0][2][0];  cc[2][3]=c[2][0][1][1];  
  cc[2][4]=c[2][0][2][1];  cc[2][5]=c[2][0][2][2];
      
  cc[3][0]=c[1][1][0][0];  cc[3][1]=c[1][1][1][0];  
  cc[3][2]=c[1][1][2][0];  cc[3][3]=c[1][1][1][1];
  cc[3][4]=c[1][1][2][1];  cc[3][5]=c[1][1][2][2];
  
  cc[4][0]=c[2][1][0][0];  cc[4][1]=c[2][1][1][0];  
  cc[4][2]=c[2][1][2][0];  cc[4][3]=c[2][1][1][1];  
  cc[4][4]=c[2][1][2][1];  cc[4][5]=c[2][1][2][2];
  
  cc[5][0]=c[2][2][0][0];  cc[5][1]=c[2][2][1][0];  
  cc[5][2]=c[2][2][2][0];  cc[5][3]=c[2][2][1][1];  
  cc[5][4]=c[2][2][2][1];  cc[5][5]=c[2][2][2][2];
 
  cc[5][5] = 0.0;
  
  /* compute the second PK stresses */  
  pk2[0][0] = cc[0][0]*gs[0][0]+2.0*cc[0][1]*gs[0][1]+2.0*cc[0][2]*gs[0][2]
            + cc[0][3]*gs[1][1]+2.0*cc[0][4]*gs[1][2]+cc[0][5]*gs[2][2];
  pk2[1][0] = cc[1][0]*gs[0][0]+2.0*cc[1][1]*gs[0][1]+2.0*cc[1][2]*gs[0][2]
            + cc[1][3]*gs[1][1]+2.0*cc[1][4]*gs[1][2]+cc[1][5]*gs[2][2];
  pk2[2][0] = cc[2][0]*gs[0][0]+2.0*cc[2][1]*gs[0][1]+2.0*cc[2][2]*gs[0][2]
            + cc[2][3]*gs[1][1]+2.0*cc[2][4]*gs[1][2]+cc[2][5]*gs[2][2];
  pk2[1][1] = cc[3][0]*gs[0][0]+2.0*cc[3][1]*gs[0][1]+2.0*cc[3][2]*gs[0][2]
            + cc[3][3]*gs[1][1]+2.0*cc[3][4]*gs[1][2]+cc[3][5]*gs[2][2];
  pk2[2][1] = cc[4][0]*gs[0][0]+2.0*cc[4][1]*gs[0][1]+2.0*cc[4][2]*gs[0][2] 
            + cc[4][3]*gs[1][1]+2.0*cc[4][4]*gs[1][2]+cc[4][5]*gs[2][2];

#if 0
  cerr << "Stress calc temp:\n";

  cerr << cc[5][0]*gs[0][0] << endl;
  cerr << 2.0*cc[5][1]*gs[0][1] << endl;
  cerr << 2.0*cc[5][2]*gs[0][2] << endl; 
  cerr <<  cc[5][3]*gs[1][1] << endl;
  cerr << 2.0*cc[5][4]*gs[1][2] << endl;
  cerr << cc[5][5]*gs[2][2] << endl;
#endif

  pk2[2][2] = cc[5][0]*gs[0][0]+2.0*cc[5][1]*gs[0][1]+2.0*cc[5][2]*gs[0][2]
            + cc[5][3]*gs[1][1]+2.0*cc[5][4]*gs[1][2]+cc[5][5]*gs[2][2];
  pk2[0][1] = pk2[1][0];
  pk2[0][2] = pk2[2][0];
  pk2[1][2] = pk2[2][1];

  pk2[2][2] = 0.0;

  return;
}


void svstr (Real stress[3][3], Real gkov1[3][3], Real gkov2[3][3],
	    Real st0[3][3])
{
  int i, j, k, l;
  
  /* stress transformation from curvilinear to cartesian */
    
  for (i=0; i<3; i++) 
    for (j=0; j<3; j++) {
      st0[i][j] = 0.0;
      for (k=0; k<3; k++) 
	for (l=0; l<3; l++)
	  st0[i][j] = st0[i][j] + gkov1[i][k]*gkov2[j][l]*stress[k][l];
    }


  return;
}



void stress(Real S[3][3], 
            Real emod, Real nue, 
            Real gmkovr[3][3], 
            Real gmkovc[3][3])
{
  int i, j;
  Real gmkonr[3][3];
  Real gs[3][3];

  for (i = 0; i < 3; ++i)
    for (j = 0; j < 3; ++j)
      gmkonr[i][j] = gmkovr[i][j];

  sinv3(gmkonr);

#if 0
  cerr << endl << "gmkovr" << endl;
  print33(gmkovr);
  cerr << endl;

  cerr << endl << "gmkovc" << endl;
  print33(gmkovc);
  cerr << endl;

  cerr << endl << "gmkonr" << endl;
  print33(gmkonr);
  cerr << endl;
#endif

  greenlagrstrain(gs, gmkovr, gmkovc);

#if 0
  cerr << endl << "gs" << endl;
  print33(gs);
  cerr << endl;
#endif

  kirchhoffmater(emod, nue, gmkonr, gs, S);


#if 0
  cerr << endl << "S" << endl;
  print33(S);
  cerr << endl;
#endif
}
