#ifndef __SIMMASS_H
#define __SIMMASS_H

#include "Util/forall.h"
#include "SimTypes.h"

template <class T>
class SimMass {
public:
  DECLARE_SIM_TYPES(T)

  static void Initialize(Mesh &mesh) {
    const Real rho = Parameters::rho;

    // Zero the mask of every vertex

    forall(typename Mesh::VertexIt, v, mesh.vertexBegin(), mesh.vertexEnd())
      m(v) = 0;
    endforall;

    // For every gauss point,
    //    For non-zero basis function on that point,
    //       lump the mass contribution of that basis function
    //       onto the associated vertex.

    forall(typename Mesh::GaussPointIt, p, mesh.gaussPointBegin(), mesh.gaussPointEnd())

      forall(typename GaussPoint::SupportIt, s1, p->supportBegin(), p->supportEnd())
      
        Vertex *v1 = (Vertex*) s1.first;
        Real w1 = s1.second->p();

	m(v1) += rho * w1 * p->JacobianR() * p->hR() * p->W();

      endforall;
    endforall;
  } 

  static void Update(Mesh &mesh) {
  }

  static Real m(const Vertex *v) {
    return v->massData.m_m;
  }

  static void SetM(Vertex *v, Real mass) {
    m(v) = mass;
  }

private:
  static Real &m(Vertex *v) {
    return v->massData.m_m;
  }

  Real m_m;
};

#endif



#if 0

  static void Initialize(Mesh &mesh) {
    const Real rho = Parameters::rho;

    // Zero the mask of every vertex

    forall(typename Mesh::VertexIt, v, mesh.vertexBegin(), mesh.vertexEnd())
      m(v) = 0;
    endforall;

    // For every gauss point, for every pair of vertices "v_i" and "v_j" in the support of
    // the gauss point, compute the contribution, rho * N_i * N_j * J * h, of the pair
    // to the mass of v_i.

    forall(typename Mesh::GaussPointIt, p, mesh.gaussPointBegin(), mesh.gaussPointEnd())

      forall(GaussPoint::SupportIt, s1, p->supportBegin(), p->supportEnd())
      
        Vertex *v1 = (Vertex*) s1.first;
        Real w1 = s1.second->p();

        forall(GaussPoint::SupportIt, s2, p->supportBegin(), p->supportEnd())

	  Real w2 = s2.second->p();

	  m(v1) += rho * w1 * w2 * p->JacobianR() * p->hR() * p->W();

        endforall;
      endforall;
    endforall;
  } 

#endif /* #if 0 */








