#ifndef __SIMMESH_H
#define __SIMMESH_H

#include "Quadrature/FEMMesh.h"
#include "Util/Real.h"

#include "Util/forall.h"

//#include "Boom/Boom.h"

template <class T>
class SimMesh : public FEMMesh<T> {
public:
  DECLARE_SIM_TYPES(T)

    //  Patch *rootPatch;
    
//  friend class SimAccessor<Triangle,Vertex>;

//  Parameters parameters;

#if 0
  SimMesh() : rootPatch(NULL), FEMMesh<T>() {  
  }
#endif
  SimMesh() : FEMMesh<T>() {  
  }

  typedef vector<GaussPoint*> GaussPointCt;
  typedef GaussPointCt::iterator GaussPointIt;
  typedef GaussPointCt::const_iterator const_GaussPointIt;

  void InitGaussPoints() {
    cerr << "Initializing gauss points..." << endl;

    forall(VertexIt, v, vertexBegin(), vertexEnd())
      v->InitOneRing();
    endforall;

    forall(TriangleIt, t, triangleBegin(), triangleEnd())

      t->AttachGaussPoints();

      if (!t->triangle(E1) || t->triangle(E1) > t) {
	m_gaussPoints.push_back(t->getGaussPoint(E1));
      }
      if (!t->triangle(E2) || t->triangle(E2) > t) {
	m_gaussPoints.push_back(t->getGaussPoint(E2));
      }
      if (!t->triangle(E3) || t->triangle(E3) > t) {
	m_gaussPoints.push_back(t->getGaussPoint(E3));
      }
    endforall;
  }

  /**
   * Returns a constant iterator over the gauss points in this mesh, positioned
   * at the start of the gauss point list.
   * @return constant iterator over gauss points in this mesh, positioned at start
   *	     of list
   */
  const_GaussPointIt gaussPointBegin(void) const {
    return (m_gaussPoints.begin());
  }

  /**
   * Returns an iterator over the gauss points in this mesh, positioned at the
   * start of the gauss point list.
   * @return iterator over gauss points in this mesh, positioned at start of list
   */
  GaussPointIt gaussPointBegin(void) {
    return (m_gaussPoints.begin());
  }

  /**
   * Returns the number of gauss points in this mesh.
   * @return number of gauss points in this mesh
   */
  int gaussPointCount(void) const {
    return (m_gaussPoints.size());
  }

  /**
   * Returns a constant iterator over the gauss points in this mesh, positioned
   * at the end of the gauss point list.
   * @return constant iterator over gauss points in this mesh, positioned at end
   *	     of list
   */
  const_GaussPointIt gaussPointEnd(void) const {
    return (m_gaussPoints.end());
  }

  /**
   * Returns an iterator over the gauss points in this mesh, positioned at the
   * end of the gauss point list.
   * @return iterator over gauss points in this mesh, positioned at end of list
   */
  GaussPointIt gaussPointEnd(void) {
    return (m_gaussPoints.end());
  }


protected:
  /** The set of gauss points in this mesh. */
  GaussPointCt m_gaussPoints;
};


#define SIMMESHTYPE_INSTANTIATE_TYPES(TYPE_STRUCT, MTYPE, MCLASS, TCLASS, VCLASS, GPCLASS, MTCLASS, SFCLASS, MSCLASS, FRCLASS, GMCLASS, INCLASS, SICLASS, PACLASS)   \
MESHTYPE_INSTANTIATE_TYPES( TYPE_STRUCT, MTYPE, MCLASS, TCLASS, VCLASS )    \
typedef GPCLASS<TYPE_STRUCT> MTYPE##_GaussPoint;       \
typedef MTCLASS<TYPE_STRUCT> MTYPE##_Material;         \
typedef SFCLASS<TYPE_STRUCT> MTYPE##_Surface;          \
typedef MSCLASS<TYPE_STRUCT> MTYPE##_Mass;             \
typedef FRCLASS<TYPE_STRUCT> MTYPE##_Forces;           \
typedef GMCLASS<TYPE_STRUCT> MTYPE##_Geometry;         \
typedef INCLASS<TYPE_STRUCT> MTYPE##_Integrator;       \
typedef SICLASS<TYPE_STRUCT> MTYPE##_ShellIntegrator;  \
typedef PACLASS<TYPE_STRUCT> MTYPE##_Parameters;       

#define SIMMESHTYPE_DEFINE_TYPE_STRUCT(TYPE_STRUCT, MTYPE, MCLASS, TCLASS, VCLASS, GPCLASS, MTCLASS, SFCLASS, MSCLASS, FRCLASS, GMCLASS, INCLASS, SICLASS, PACLASS)   \
MESHTYPE_DEFINE_TYPE_STRUCT( MTYPE##_MeshTypes, MTYPE, MCLASS, TCLASS, VCLASS ) \
struct TYPE_STRUCT : public MTYPE##_MeshTypes {          \
  typedef MTYPE##_GaussPoint          GaussPoint;        \
  typedef MTYPE##_Material            Material;          \
  typedef MTYPE##_Surface             Surface;           \
  typedef MTYPE##_Mass                Mass;              \
  typedef MTYPE##_Forces              Forces;            \
  typedef MTYPE##_Geometry            Geometry;          \
  typedef MTYPE##_Integrator          Integrator;        \
  typedef MTYPE##_ShellIntegrator     ShellIntegrator;   \
  typedef MTYPE##_Parameters          Parameters;        \
};

#define SIMMESHTYPE(MTYPE, MCLASS, TCLASS, VCLASS, GPCLASS, MTCLASS, SFCLASS, MSCLASS, FRCLASS, GMCLASS, INCLASS, SICLASS, PACLASS)   \
struct MTYPE##_Types;                                                                        \
SIMMESHTYPE_INSTANTIATE_TYPES(MTYPE##_Types, MTYPE, MCLASS, TCLASS, VCLASS, GPCLASS, MTCLASS, SFCLASS, MSCLASS, FRCLASS, GMCLASS, INCLASS, SICLASS, PACLASS) \
SIMMESHTYPE_DEFINE_TYPE_STRUCT(MTYPE##_Types, MTYPE, MCLASS, TCLASS, VCLASS, GPCLASS, MTCLASS, SFCLASS, MSCLASS, FRCLASS, GMCLASS, INCLASS, SICLASS, PACLASS) 

template <class T>
istream& operator>>(istream& is, SimMesh<T> &mesh) {\
    cerr << "Reading...\n";\
    ThinShellReader<T> reader;\
    reader.readMesh(is, mesh);\
    mesh.InitGaussPoints();\
    return (is);\
}

#endif
