#ifndef __SIMSHELLINTEGRATOR_H
#define __SIMSHELLINTEGRATOR_H

#include "Quadrature/QuadCoeffs.h"

#include "Util/forall.h"
#include "SimTypes.h"

#include "iostream.h"

#define _IEEE 1

#ifndef __GNUC__
#include "nan.h"
#endif

template <class T>
class SimShellIntegrator {
public:
  DECLARE_SIM_TYPES(T)

  static void Initialize(Mesh &mesh) {
  }

  static void BeginAreaIntegral(Mesh &mesh) {
    Forces::BeginAreaIntegral(mesh);
  }

  static void BeginThicknessIntegral(const GaussPoint *p, Real dA) {
    Forces::BeginThicknessIntegral(p, dA);
  }

  static void AddVolumeElement(const GaussPoint *p, typename Surface::e2Iter e2Idx, Real dV) {
    Forces::AddVolumeElement(p, e2Idx, dV);
    Integrator::InternalEnergy_AddIntegrationPoint(p, e2Idx, dV);
  }  

  static void EndThicknessIntegral(const GaussPoint *p, Real simpsonsHeight) {
    Forces::EndThicknessIntegral(p, simpsonsHeight);
  }

  static void BeginVisitSupportVertices(const GaussPoint *p) {
    Forces::BeginVisitSupportVertices(p);
  }

  static void VisitSupportVertex(const GaussPoint *p, Vertex *v, 
				 QuadCoeffs *w, Real dA_vertex) {
    Forces::VisitSupportVertex(p, v, w, dA_vertex);
  }

  static void EndVisitSupportVertices(const GaussPoint *p) {
    Forces::EndVisitSupportVertices(p);
  }

  static void EndAreaIntegral(Mesh &mesh) {
    Forces::EndAreaIntegral(mesh);
  }

  static void Update(Mesh &mesh) {

    ShellIntegrator::BeginAreaIntegral(mesh);

    // For every gauss point p....

    forall(typename Mesh::GaussPointIt, p, mesh.gaussPointBegin(), mesh.gaussPointEnd())

      p->FillCache();

      const Real dA = p->JacobianR() * p->W();

      // Compute thickness integral

      ShellIntegrator::BeginThicknessIntegral(p, dA);

      for (typename Surface::e2Iter e2Idx = Surface::e2Begin();
	   e2Idx != Surface::e2End();
	   ++e2Idx) {

	p->FillCache(e2Idx);

	const Real dV = 
	  p->JacobianR() * p->W() * p->LayerHeight() * p->muR(e2Idx)
	  * e2Idx.SimpsonsWeight() / 3.0;

	ShellIntegrator::AddVolumeElement(p, e2Idx, dV);
      }

      Real simpsonsHeight = p->LayerHeight() / 3.0;

      ShellIntegrator::EndThicknessIntegral(p, simpsonsHeight);

      ShellIntegrator::BeginVisitSupportVertices(p);

      // For every vertex v in the support of p...

      forall(typename GaussPoint::SupportIt, s1, p->supportBegin(), p->supportEnd())
      
        Vertex *v = (Vertex*) s1.first;
        QuadCoeffs *w = s1.second;

	p->FillCache(w);

	ShellIntegrator::VisitSupportVertex(p, v, w, dA);

      endforall;

      ShellIntegrator::EndVisitSupportVertices(p);

    endforall;

    ShellIntegrator::EndAreaIntegral(mesh);
  }

private:  
};


#endif

