#ifndef __GENERALMATRIX_H
#define __GENERALMATRIX_H


#include <algorithm>
#include <algo.h>
#include <vector>

#include <fstream.h>

template <class Scalar>
class GeneralMatrix {

  typedef vector<Scalar> Row;

 public:

  GeneralMatrix( int height, int width ) 
    : m_height( height ), m_width( width ) {

    int i, j;

    m_rows.resize( height );

    //cerr << "Creating " << m_height << "x" << m_width << " matrix\n";

    for ( i = 0; i < height; ++i ) {
     
      m_rows[i].resize( m_width );

      for ( j = 0; j < m_width; ++j ){

	m_rows[i][j] = Scalar();
      }

      //cerr << "size = " << m_rows[i].size() << endl;
      assert( m_rows[i].size() == m_width );
    }
  }


  int height() const {

    return m_height;
  }


  int width() const {

    return m_width;
  }


  const Row& operator[] ( int i ) const {

    assert( i >= 0  &&  i < m_height );
    return m_rows[i];
  }


  Row& operator[] ( int i ) {

    assert( i >= 0  &&  i < m_height );
    return m_rows[i];
  }


  template <class Vector>
    Vector operator* ( const Vector& x ) const {

    int i;
    Vector result( m_height );
    typedef typename Vector::value_type result_type;

    assert( x.size() == m_width );

    for ( i = 0; i < m_height; ++ i ) {
      result[i] = inner_product( m_rows[i].begin(), m_rows[i].end(), 
				 x.begin(), result_type() );

      //cerr << "inner product of (length = " << m_rows[i].size() << ") " << m_rows[i] << " and " << x << " is " << result[i] << endl;
    }

    return result;
  }

 private:

  int m_height, m_width;
  vector<Row> m_rows;

};


template <class Scalar>
ostream&
operator<<( ostream& os, const GeneralMatrix<Scalar>& M )
{
  int i, j;

  for ( i = 0; i < M.height(); ++i ) {

    assert( M[i].size() == M.width() );
    for ( j = 0; j < M.width(); ++j ){
      os << M[i][j] << " ";
    }
    os << endl;
  }

  return os;
}


#endif /* #ifndef __GENERALMATRIX_H */
