#include "Util/Real.h"

#ifndef __INTERVAL_H
#define __INTERVAL_H

#include<algorithm>

inline Real min(Real a, Real b, Real c, Real d) {
  return min( min(a, b), min(c, d) );
}

inline Real max(Real a, Real b, Real c, Real d) {
  return max( max(a, b), max(c, d) );
}

class Interval {

 public:

  Interval(Real min, Real max) : m_min(min), m_max(max) {}

  Interval(Real a) : m_min(a), m_max(a) {}

  Interval() : m_min(0), m_max(0) {}

  Real &min() {
    return m_min;
  }

  Real min() const {
    return m_min;
  }

  Real &max() {
    return m_max;
  }

  Real max() const {
    return m_max;
  }

  Interval operator +(const Real z) const {
    return Interval(min() + z, max() + z);
  }

  Interval operator +(const Interval b) const {
    return Interval(min() + b.min(), max() + b.max());
  }

  Interval operator -(const Real z) const {
    return Interval(min() - z, max() - z);
  }

  Interval operator -(const Interval b) const {
    return Interval(min() - b.max(), max() - b.min());
  }

  Interval operator *(const Real z) const {
    if (z > 0) {
      return Interval( z * min(), z * max() );
    }
    else {
      return Interval( z * max(), z * min() );
    }
  }

  Interval operator *(const Interval b) const {
    Real z1 = min() * b.min();
    Real z2 = min() * b.max();
    Real z3 = max() * b.min();
    Real z4 = max() * b.max();

    return Interval(::min(z1,z2,z3,z4), ::max(z1,z2,z3,z4));
  }

  Interval& operator =(const Interval b) {
    min() = b.min();
    max() = b.max();
    return (*this);
  }

  Interval& operator =(const Real b) {
    min() = b;
    max() = b;
    return (*this);
  }

  Interval& operator +=(const Interval b) {
    min() += b.min();
    max() += b.max();
    return (*this);
  }

  bool contains(const Real z) const {
    return min() <= z  &&  z <= max();
  }

  Interval U(const Interval& b) const {
    return Interval( ::min( min(), b.min() ), ::max( max(), b.max() ) );
  }

 private:

  Real m_min, m_max;
};

inline ostream& operator<<( ostream& os, const Interval& a )
{
  os << "[ " << a.min() << ", " << a.max() << " ]";
  return os;
}


#endif /* #ifndef __INTERVAL_H */
