#ifndef __REAL_H
#define __REAL_H

typedef double Real_NativeType;

#include <values.h>

#include <stdlib.h>

#include <math.h>

#define INF MAXDOUBLE

class Real {

 public:

  Real(const Real_NativeType &a) 
    : m_data(a) {}

  Real() 
    : m_data(0) {}

  Real operator +(const Real &b) const {
    return m_data + b.m_data;
  }

  Real operator -(const Real &b) const {
    return m_data - b.m_data;
  }

  Real operator *(const Real &b) const {
    return m_data * b.m_data;
  }

  Real operator /(const Real &b) const {
    return m_data / b.m_data;
  }

  Real operator +=(const Real &b) {
    return m_data += b.m_data;
  }

  Real operator -=(const Real &b) {
    return m_data -= b.m_data;
  }

  Real operator *=(const Real &b) {
    return m_data *= b.m_data;
  }

  Real operator /=(const Real &b) {
    return m_data /= b.m_data;
  }

  bool operator <(const Real &b) const {
    return m_data < b.m_data;
  }

  bool operator <=(const Real &b) const {
    return m_data <= b.m_data;
  }

  bool operator >(const Real &b) const {
    return m_data > b.m_data;
  }

  bool operator >=(const Real &b) const {
    return m_data >= b.m_data;
  }

  bool operator ==(const Real &b) const {
    return m_data == b.m_data;
  }

  bool operator !=(const Real &b) const {
    return m_data != b.m_data;
  }

  Real operator +() const {
    return +m_data;
  }

  Real operator -() const {
    return -m_data;
  }

  Real_NativeType getNativeData() const {
    return m_data;
  }

  Real_NativeType& getNativeData() {
    return m_data;
  }

  Real_NativeType GetNativeData() const { // deprecated
    return m_data;
  }

  Real_NativeType& GetNativeData() { // deprecated
    return m_data;
  }

 private:
  Real_NativeType m_data;
};

inline
Real operator +( const Real_NativeType &a,
		 const Real &b ) {
  return Real(a) + b;
}


inline
Real operator -( const Real_NativeType &a,
		 const Real &b ) {
  return Real(a) - b;
}


inline
Real operator *( const Real_NativeType &a,
		 const Real &b ) {
  return Real(a) * b;
}


inline
Real operator /( const Real_NativeType &a,
		 const Real &b ) {
  return Real(a) / b;
}


extern const Real RealLarge;

extern const Real RealEPS;

inline Real abs(const Real &x) {
  return (x >= 0.0) ? x : -x;
}

inline Real acos(const Real &x) {
  return acos(x.GetNativeData());
}

inline Real asin(const Real &x) {
  return asin(x.GetNativeData());
}

inline Real atan(const Real &x) {
  return atan(x.GetNativeData());
}

inline Real atan2(const Real &x, const Real &y) {
  return atan2(x.GetNativeData(), y.GetNativeData());
}

inline Real cos(const Real &x) {
  return cos(x.GetNativeData());
}

inline Real sec(const Real &x) {
  return 1.0 / cos(x);
}

inline Real sin(const Real &x) {
  return sin(x.GetNativeData());
}

inline Real tan(const Real &x) {
  return tan(x.GetNativeData());
}

inline Real cot(const Real &x) {
  return tan(Real(1.0) / x);
}

inline Real sq(const Real &x) {
  return x*x;
}

inline Real pow(const Real &x, const Real &y) {
  return pow(x.GetNativeData(), y.GetNativeData());
}

inline Real sqrt(const Real &x) {
  return sqrt(x.GetNativeData());
}

inline Real cubed(const Real &x) {
  return x*x*x;
}

inline Real min( const Real &a, const Real &b ) {
  return (a < b) ? a : b;
}

inline Real max( const Real &a, const Real &b ) {
  return (a > b) ? a : b;
}

inline int round( const Real &x ) {
  return (int) rint( x.getNativeData() );
}

inline ostream&
operator<<( ostream &os, const Real &c )
{
  if (abs(c) < RealEPS) {
    os << 0;
  } 
  else {
    os << c.GetNativeData();
  }

  return os;
}

inline istream&
operator>>( istream &is, Real &c )
{
  return is >> c.GetNativeData();
}

#include "Util/util.h"


class Random {
public: 

  static long RandUnsignedInt() {
    return nrand48(m_xsubi);
  }

  static long RandSignedInt() {
    return jrand48(m_xsubi);
  }

  static Real RandReal() {
    return erand48(m_xsubi);
  }

  static void SaveState(ostream &fout) {
    fout.write((char*)m_xsubi, sizeof(m_xsubi));
  }

  static void SaveStateASCII(ostream &fout) {
    fout << m_xsubi[0] << " " << m_xsubi[1] << " " << m_xsubi[2] << endl;
  }

  static void LoadState(istream &fin) {
    fin.read((char*)m_xsubi, sizeof(m_xsubi));
  }

private:
  static unsigned short m_xsubi[3];
};


#define _IEEE 1

//#include "nan.h"


#endif


