#ifndef __TENSOR_H
#define __TENSOR_H

#include <assert.h>
#include <iostream.h>

const bool COVARIANT     = true;
const bool CONTRAVARIANT = false;

template <class SubTensorClass, class VectorClass, class Scalar, int Dim, bool Kind>
class Tensor;

template <class SubTensorClass, class VectorClass, class Scalar, int Dim>
class Tensor<SubTensorClass, VectorClass, Scalar, Dim, COVARIANT> {
public:
  typedef SubTensorClass SubTensor;
  typedef VectorClass Vector;

  Tensor() {}

  Tensor(const Tensor &t) {
    for (int i=0; i<Dim; ++i)
      m_r[i] = t.m_r[i];
  }

  Tensor(Scalar coeffs[Dim][Dim]) {
    Tensor &T = *this;
    int i, j;

    for (i = 0; i < Dim; ++i)
      for (j = 0; j < Dim; ++j)
	T[i][j] = coeffs[i][j];
  }

  SubTensor &operator[](unsigned int i) {
    assert(i < Dim);
    return m_r[ i ];
  }

  SubTensor operator[](unsigned int i) const {
    assert(i < Dim);
    return m_r[ i ];
  }

private:

  SubTensor m_r[Dim];
};

template <class SubTensorClass, class VectorClass, class Scalar, int Dim>
class Tensor<SubTensorClass, VectorClass, Scalar, Dim, CONTRAVARIANT> {
public:
  typedef SubTensorClass SubTensor;
  typedef VectorClass Vector;

  Tensor() {};

  Tensor(const Tensor &t) {
    for (int i=0; i<Dim; ++i)
      m_r[i] = t.m_r[i];
  }

  Tensor(Scalar coeffs[Dim][Dim]) {
    Tensor &T = *this;
    int i, j;

    for (i = 0; i < Dim; ++i)
      for (j = 0; j < Dim; ++j)
	T(i)(j) = coeffs[i][j];
  }
  
  SubTensor &operator()(unsigned int i) {
    assert(i < Dim);
    return m_r[ i ];
  }

  SubTensor operator()(unsigned int i) const {
    assert(i < Dim);
    return m_r[ i ];
  }

private:

  SubTensor m_r[Dim];
};


template <class SubTensorClass, class VectorClass, class Scalar, int Dim>
inline ostream &operator <<( ostream &o, 
                                 const Tensor<SubTensorClass,VectorClass,Scalar,Dim,COVARIANT> &T) {
  int i;
  o << "[ ";
  for (i = 0; i < Dim; ++i) {
    o << T[i] << " ";
  }
  o << "]";
  return o;
}

template <class SubTensorClass, class VectorClass, class Scalar, int Dim>
inline ostream &operator <<( ostream &o, 
                                 const Tensor<SubTensorClass,VectorClass,Scalar,Dim,CONTRAVARIANT> &T) {
  int i;
  o << "( ";
  for (i = 0; i < Dim; ++i) {
    o << T(i) << " ";
  }
  o << ")";
  return o;
}


#endif
