// -*- Mode: c++ -*-
// $Id: Vector3.h,v 1.1.1.1 2001/07/24 02:41:51 pkrysl Exp $
// $Source: /home/cvs/WASP/problems/FE/subdiv/Coke/Util/Vector3.h,v $

#ifndef	__VECTOR3_H__
#define	__VECTOR3_H__

// Do not change anything above this line
#ifdef _WIN32
#undef min
#undef max
#endif

#include <stddef.h>
#include <math.h>
#include "Util/Real.h"
#include "Util/Interval.h"
#include "Util/ProductType.h"
//#include "util.h"

#ifdef INVENTOR
#include "Inventor/SbLinear.h"
#endif

class ostream;
class istream;

template< class A, class B >
struct ProductType;

// column type 3 vectors
template <class Scalar>
class Vector3{
 public:
  static const Vector3 Zero;

  enum{
    X = 0,
    Y = 1,
    Z = 2
    };
  Vector3( void ) {
    m_v[X] = m_v[Y] = m_v[Z] = 0;
  }
  //  Vector3( const Scalar f ) { m_v[X] = m_v[Y] = m_v[Z] = f; }
  Vector3( const Vector3& c )
    { m_v[X] = c.m_v[X], m_v[Y] = c.m_v[Y], m_v[Z] = c.m_v[Z]; }
  Vector3( const Scalar a, const Scalar b, const Scalar c )
    { m_v[X] = a, m_v[Y] = b, m_v[Z] = c; }
  Vector3( const Real* a )
    { m_v[X] = a[X], m_v[Y] = a[Y], m_v[Z] = a[Z]; }
  Vector3( const float* a )
    { m_v[X] = a[X], m_v[Y] = a[Y], m_v[Z] = a[Z]; }
  Vector3( const double* a )
    { m_v[X] = a[X], m_v[Y] = a[Y], m_v[Z] = a[Z]; }
  Vector3( const long double* a )
    { m_v[X] = a[X], m_v[Y] = a[Y], m_v[Z] = a[Z]; }
  operator Scalar*( void )
    { return &m_v[X]; }
  operator const Scalar*( void ) const
    { return &m_v[X]; }
  // ~Vector3( void ) {}
 
#if 0 
  friend istream& operator>> ( istream&, Vector3& );
  friend ostream& operator<< ( ostream&, const Vector3& );
#endif
  void print( ostream& os ) const;
  bool read ( istream& is );
  
  bool operator==(const Vector3& c)const
    { return m_v[X]==c.m_v[X] && m_v[Y]==c.m_v[Y] && m_v[Z]==c.m_v[Z];}
  bool operator<(const Vector3& c)const 
    { return unsigned(this)<unsigned(&c); }
  
  Vector3& operator=( const Vector3& c )
    { m_v[X] = c.m_v[X], m_v[Y] = c.m_v[Y], m_v[Z] = c.m_v[Z]; return *this; }
  Vector3 operator+( const Vector3& c ) const
    { return Vector3( m_v[X] + c.m_v[X], m_v[Y] + c.m_v[Y], m_v[Z] + c.m_v[Z] ); }
  Vector3 operator-( const Vector3& c ) const
    { return Vector3( m_v[X] - c.m_v[X], m_v[Y] - c.m_v[Y], m_v[Z] - c.m_v[Z] ); }
  Vector3 operator/( const Scalar s ) const
    { return Vector3( m_v[X] / s, m_v[Y] / s, m_v[Z] / s ); }

  Vector3 U( const Vector3& c ) const
    { return Vector3( m_v[X]. U( c.m_v[X] ),
		      m_v[Y]. U( c.m_v[Y] ),
		      m_v[Z]. U( c.m_v[Z] ) ); }

  Vector3& operator+=( const Vector3& c )
    { m_v[X] += c.m_v[X], m_v[Y] += c.m_v[Y], m_v[Z] += c.m_v[Z]; return *this; }
  Vector3& operator-=( const Vector3& c )
    { m_v[X] -= c.m_v[X], m_v[Y] -= c.m_v[Y], m_v[Z] -= c.m_v[Z]; return *this; }
  Vector3& operator*=( const Scalar s )
    { m_v[X] *= s, m_v[Y] *= s, m_v[Z] *= s; return *this; }
  Vector3& operator/=( const Scalar s )
    { m_v[X] /= s, m_v[Y] /= s, m_v[Z] /= s; return *this; }
  Vector3 operator-( void ) const
    { return Vector3( -m_v[X], -m_v[Y], -m_v[Z] ); }
  
  //stupid but useful
  Vector3 operator*(const Vector3& c)const{
    return Vector3(m_v[X]*c.m_v[X], m_v[Y]*c.m_v[Y], m_v[Z]*c.m_v[Z]);
  }
  Vector3 operator/(const Vector3& c)const{
    return Vector3(m_v[X]/c.m_v[X], m_v[Y]/c.m_v[Y], m_v[Z]/c.m_v[Z]);
  }
  
  Scalar& x( void ) { return m_v[X]; }
  Scalar x( void ) const { return m_v[X]; }
  Scalar& y( void ) { return m_v[Y]; }
  Scalar y( void ) const { return m_v[Y]; }
  Scalar& z( void ) { return m_v[Z]; }
  Scalar z( void ) const { return m_v[Z]; }
  Scalar& operator() (const int i)      { return m_v[i]; }
  Scalar operator() (const int i) const { return m_v[i]; }
  
  Vector3 lerp( const Vector3& v1, const Scalar t ) const
    { return Vector3( m_v[X] + t * ( v1.m_v[X] - m_v[X] ),
		   m_v[Y] + t * ( v1.m_v[Y] - m_v[Y] ),
		   m_v[Z] + t * ( v1.m_v[Z] - m_v[Z] ) ); }

  Vector3 min( const Vector3& o ) const
  { Scalar a = ::Min( m_v[X], o.m_v[X] );
	Scalar b = ::Min( m_v[Y], o.m_v[Y] );
	Scalar c = ::Min( m_v[Z], o.m_v[Z] );
    return Vector3( a, b, c ); }
  Vector3 max( const Vector3& o ) const
  {		Scalar a = ::Max( m_v[X], o.m_v[X] );
      Scalar b = ::Max( m_v[Y], o.m_v[Y] );
      Scalar c = ::Max( m_v[Z], o.m_v[Z] );
      return Vector3( a, b, c ); }
  void bbox( Vector3& min, Vector3& max ){
    if( m_v[X] < min.m_v[X] ) min.m_v[X] = m_v[X];
    else if( m_v[X] > max.m_v[X] ) max.m_v[X] = m_v[X];
    if( m_v[Y] < min.m_v[Y] ) min.m_v[Y] = m_v[Y];
    else if( m_v[Y] > max.m_v[Y] ) max.m_v[Y] = m_v[Y];
    if( m_v[Z] < min.m_v[Z] ) min.m_v[Z] = m_v[Z];
    else if( m_v[Z] > max.m_v[Z] ) max.m_v[Z] = m_v[Z];
  }

  Scalar dot( const Vector3& c ) const
    { return m_v[X] * c.m_v[X] + m_v[Y] * c.m_v[Y] + m_v[Z] * c.m_v[Z]; }
  Scalar dot( void ) const
    { return m_v[X] * m_v[X] + m_v[Y] * m_v[Y] + m_v[Z] * m_v[Z]; }
  Vector3 cross( const Vector3& c ) const
    { return Vector3( m_v[Y] * c.m_v[Z] - m_v[Z] * c.m_v[Y],
		  m_v[Z] * c.m_v[X] - m_v[X] * c.m_v[Z],
		  m_v[X] * c.m_v[Y] - m_v[Y] * c.m_v[X] ); }
  Scalar l1( void ) const
    { Scalar a = abs( m_v[X] ); a += abs( m_v[Y] ); return a + abs( m_v[Z] ); }
  Scalar linfty( void ) const
    { Scalar a = abs( m_v[X] ); a = ::Max( a, abs( m_v[Y] ) );
      return ::Max( a, abs( m_v[Z] ) ); }
  Scalar l2( void ) const { return sqrt( dot() ); }
  
  Vector3 dir( void ) const;

  Scalar normalize( void )
    { Scalar mag = l2(); return (mag == 0.0) ? 0.0 : ( *this *= 1 / mag, 1 ); }
  Scalar dist( const Vector3& c ) const { return ( *this - c ).l2(); }
  int collinear( const Vector3& a, const Vector3& b ) const;
  Vector3 perp( void ) const;

#ifdef INVENTOR
  SbVec3f getSbVec3f( void ) {

    return SbVec3f( x().GetNativeData(), y().GetNativeData(), z().GetNativeData() );
  }

  Vector3& operator= ( const SbVec3f& c ) {

    m_v[0] = c[0];
    m_v[1] = c[1];
    m_v[2] = c[2];

    return (*this);
  }  
#endif

  template< class T >
  typename ProductType<Vector3,T>::T operator*( const T &s ) const
    { return typename ProductType<Vector3,T>::T( s * m_v[X], s * m_v[Y], s * m_v[Z] ); }

  template< class T >
  friend typename ProductType<T,Vector3>::T operator*( const T &s, const Vector3& c )
    { return typename ProductType<T,Vector3>::T( s * c.m_v[X], s * c.m_v[Y], s * c.m_v[Z] ); }

#if 0
  friend Vector3 operator*( const Real &s, const Vector3& c )
    { return Vector3( s * c.m_v[X], s * c.m_v[Y], s * c.m_v[Z] ); }
#endif

  void copyTo( double tgt[3] ) {

    tgt[0] = m_v[0].GetNativeData();
    tgt[1] = m_v[1].GetNativeData();
    tgt[2] = m_v[2].GetNativeData();
  }
  
 protected:
 private:
  Scalar	m_v[3];
};


// extern defs for non-member outline functions go here
// externally visible constant defs go here

template <class Scalar>
const Vector3<Scalar> Vector3<Scalar>::Zero = Vector3<Scalar>(0.0, 0.0, 0.0);


typedef Vector3<Real> Real3Vec;

extern const Real3Vec Zero3Vec;

extern const Real3Vec INF3Vec;

typedef Vector3<Interval> Interval3Vec;

PRODUCT_TYPE( Real, Real3Vec, Real3Vec )
PRODUCT_TYPE( Real3Vec, Real, Real3Vec )

PRODUCT_TYPE( Interval, Real3Vec, Interval3Vec )
PRODUCT_TYPE( Real3Vec, Interval, Interval3Vec )

PRODUCT_TYPE( float, Real3Vec, Real3Vec )
PRODUCT_TYPE( Real3Vec, float, Real3Vec )

PRODUCT_TYPE( double, Real3Vec, Real3Vec )
PRODUCT_TYPE( Real3Vec, double, Real3Vec )

PRODUCT_TYPE( int, Real3Vec, Real3Vec )
PRODUCT_TYPE( Real3Vec, int, Real3Vec )


template <class Scalar>
inline ostream&
operator<<( ostream& os, const Vector3<Scalar>& c )
{
  c.print( os );	
  return os;
}

template <class Scalar>
inline istream&
operator>>( istream& is, Vector3<Scalar>& f )
{
  return is >> f(0) >> f(1) >> f(2);
}


#include "Vector3.cc"

#endif
