#include "triangulation.h"
#include "dictP.h"

static void
dump_nodes (triangulation_t t)
{
  triangulation_node_p n;
  dict_iterator di = dict_get_iterator ();

  fprintf (stdout, "Nodes\n");

  dict_reset_iterator (di);
  while (n = tri_next_node (t, di)) {
    int id;
    double x, y;
    id = tri_node_get_id (n);
    tri_node_get_xy (n, &x, &y);
    fprintf (stdout, "%d %g %g 0.0\n", id, x, y);
  }

  fprintf (stdout, "&\n");

  dict_del_iterator (di);
}

static void
dump_elems (triangulation_t t)
{
  triangulation_triangle_p tri;
  triangulation_node_p n[3];
  dict_iterator di = dict_get_iterator ();
  fprintf (stdout, "ELEM\n");
  fprintf (stdout, "15\n");

  dict_reset_iterator (di);
  while (tri = tri_next_triangle (t, di)) {
    int j;
    fprintf (stdout, "%d ", tri_triangle_get_id (tri));
    tri_triangle_get_nodes (tri, n);
    for (j = 0; j < 3; j++) {
      fprintf (stdout, "%d ", tri_node_get_id (n[j]));
    }
    fprintf (stdout, "1 thickness\n");
  }

  fprintf (stdout, "&\n");

  dict_del_iterator (di);
}

static void
dump_to_wasp (triangulation_t t)
{
  dump_nodes (t);
  dump_elems (t);
}

int
main (int argc, char **argv)
{
   FILE *stream;
  triangulation_t t;

  if (argc < 2) {
    fprintf (stderr, "Expected argument\n");
    exit (1);
  }
  
  stream = fopen (argv[1], "r");
  if (stream == NULL) {
    fprintf (stderr, "Could not open %s\n", argv[1]);
    exit (1);
  }

  t = tri_from_afro_file (stream);
  if (t == NULL) {
    fprintf (stderr, "Error #%d\n", tri_status ());
    exit (1);
  }
  fclose (stream);

  dump_to_wasp (t);

  tri_destroy_triangulation (t);

  return 0; 
}
