#! /bin/bash

mesh_size=2

# geometrical parameters
H=100.01
H1=50
H2=50
R3=12
R7=8
W=50
R9=5
W9=20
H9=20
hole1=f
R10=$R9
W10=$W9
H10=40
hole2=f
R11=8
W11=30
H11=20
hole3=f

function usage {
cat <<EOF
Usage: $(basename $0) [ options ]

Options:

 Geometrical parameters:

 -H number   (default $H)
 -W number   (default $W)
 -H1 number  (default $H1)
 -H2 number  (default $H2)
 -R3 number  (default $R3)
 -R7 number  (default $R7)
 -R9 number  (default $R9)
 -W9 number  (default $W9)
 -H9 number  (default $H9)
 -hole1      (boolean; default $hole1)
 -R10 number (default $R10)
 -W10 number (default $W10)
 -H10 number (default $H10)
 -hole2      (boolean; default $hole2)
 -R11 number (default $R11)
 -W11 number (default $W11)
 -H11 number (default $H11)
 -hole3      (boolean; default $hole3)

 Others

 -mesh_size number (default $mesh_size)

EOF
}

if [ $# -lt 1 ] ; then
  usage; exit 1;
fi

basename=$1; shift
tin=$basename.mesh.in
tout=$basename.mesh.out
waspf=$basename.in

args=""
while [ $# -ge 1 ] 
do
   a=$1; shift
   case $a in
    -help)
        usage; exit 0; 
        ;;
    -H)
        if [ $# -ge 1 ] ; then
            H=$1; shift
        else
            usage; exit 1;
        fi
        ;;
    -H1)
        if [ $# -ge 1 ] ; then
            H1=$1; shift
        else
            usage; exit 1;
        fi
        ;;
    -H2)
        if [ $# -ge 1 ] ; then
            H2=$1; shift
        else
            usage; exit 1;
        fi
        ;;
    -R3)
        if [ $# -ge 1 ] ; then
            R3=$1; shift
        else
            usage; exit 1;
        fi
        ;;
    -R7)
        if [ $# -ge 1 ] ; then
            R7=$1; shift
        else
            usage; exit 1;
        fi
        ;;
    -W)
        if [ $# -ge 1 ] ; then
            W=$1; shift
        else
            usage; exit 1;
        fi
        ;;
    -R9)
        if [ $# -ge 1 ] ; then
            R9=$1; shift
        else
            usage; exit 1;
        fi
        ;;
    -W9)
        if [ $# -ge 1 ] ; then
            W9=$1; shift
        else
            usage; exit 1;
        fi
        ;;
    -H9)
        if [ $# -ge 1 ] ; then
            H9=$1; shift
        else
            usage; exit 1;
        fi
        ;;
    -R10)
        if [ $# -ge 1 ] ; then
            R10=$1; shift
        else
            usage; exit 1;
        fi
        ;;
    -W10)
        if [ $# -ge 1 ] ; then
            W10=$1; shift
        else
            usage; exit 1;
        fi
        ;;
    -H10)
        if [ $# -ge 1 ] ; then
            H10=$1; shift
        else
            usage; exit 1;
        fi
        ;;
    -R11)
        if [ $# -ge 1 ] ; then
            R11=$1; shift
        else
            usage; exit 1;
        fi
        ;;
    -W11)
        if [ $# -ge 1 ] ; then
            W11=$1; shift
        else
            usage; exit 1;
        fi
        ;;
    -H11)
        if [ $# -ge 1 ] ; then
            H11=$1; shift
        else
            usage; exit 1;
        fi
        ;;
    -mesh_size)
        if [ $# -ge 1 ] ; then
            mesh_size=$1; shift
        else
            usage; exit 1;
        fi
        ;;
    -hole1)
        hole1=t
        ;;
    -hole2)
        hole2=t
        ;;
    -hole3)
        hole3=t
        ;;
    *)
        ta="$args $a"
        args="$ta"
        ;;
    esac
done

# hole 1
if [ $(expeval "lt($R9,$W9)") -eq 0 ] ; then
    echo 'ERROR: Constraint violation: lt($R9,$W9): ' "lt($R9,$W9)"
    exit 1
fi
if [ $(expeval "lt($R9,$H9)") -eq 0 ] ; then
    echo 'ERROR: Constraint violation: lt($R9,$H9)'
    exit 1
fi
# hole 2
if [ $(expeval "lt(2*$R9+$H9+$R10,$H10)") -eq 0 ] ; then
    echo 'ERROR: Constraint violation: lt(2*$R9+$H9+$R10,$H10)' "lt(2*$R9+$H9+$R10,$H10)"
    exit 1
fi
# hole 3
if [ $(expeval "lt(2*$R11,$W)") -eq 0 ] ; then
    echo 'ERROR: Constraint violation: lt(2*$R11,$W)' "lt(2*$R11,$W)"
    exit 1
fi
if [ $(expeval "lt(2*$R11+$W11,$W)") -eq 0 ] ; then
    echo 'ERROR: Constraint violation: lt(2*$R11+$W11,$W)' "lt(2*$R11+$W11,$W)"
    exit 1
fi
# other parameters
thickness=5.0

if [ $hole1 = t ] ; then
   curve9def="curve 9 circle center $(expeval "$R7+$W9") $(expeval "$H-$H9") radius $R9"
   curve9ref="-9"
else
   curve9def=
   curve9ref=
fi
if [ $hole2 = t ] ; then
   curve10def="curve 10 circle center $(expeval "$R7+$W10") $(expeval "$H-$H10") radius $R10"
   curve10ref="-10"
else
   curve10def=
   curve10ref=
fi
if [ $hole3 = t ] ; then
   curve11def="curve 11 circle center $W11 $H11 radius $R11"
   curve11ref="-11"
else
   curve11def=
   curve11ref=
fi


rm -f $tin
cat <<EOF > $tin
curve 1 line 0 0 $W 0 
curve 2 line $W 0 $W $H1 $(expeval "$H-$R3-$H1")
curve 3 arc $W $H1 $(expeval "$W-$R3") $(expeval "$R3+$H1") center $(expeval "$W-0.0001") $(expeval "$R3+$H1-0.0001")
curve 4 line $(expeval "$W-$R3") $(expeval "$R3+$H1") $(expeval "$W-$R3") $H
curve 51 line $(expeval "$W-$R3") $H $(expeval "($W-$R3+$R7)/2") $H
curve 52 line $(expeval "($W-$R3+$R7)/2") $H $R7 $H
curve 6 line $R7 $H $R7 $(expeval "$H2+$R7")
curve 7 arc $R7 $(expeval "$H2+$R7") 0 $H2 center 0.0001 $(expeval "$H2+$R7-0.0001") reverse
curve 8 line 0 $H2 0 0

$curve9def
$curve10def
$curve11def

subregion 1  property 1 boundary 1 2 3 4 51 52 6 7 8 $curve9ref $curve10ref $curve11ref 

m-ctl-point constant $mesh_size

fixed-point 1 xy $R7 $H
EOF

# Run the triangulation
targe2 -i $tin 

# Now produce the WASP file

cat <<EOF > $waspf
Holy sheet: H=$H, H1=$H1, H2=$H2, R3=$R3, R7=$R7, W=$W, R9=$R9, W9=$W9, H9=$H9, hole1=$hole1, R10=$R10, W10=$W10, H10=$H10, hole2=$hole2, R11=$R11, W11=$W11, H11=$H11, hole3=$hole3
0 0 0 0 0
!  ======================================================= 
OPTIONS 
!  option_tag (O, T, S)     value 
R 500
O 1e-5
f 0.00001
x 1
T 4
h 1
M 1000 ! damping
hexvue_format binary
! H "SEQUENCE 0 0.01 0.0001" 
number_seq
!  END OPTIONS 
&
!  ======================================================= 
MATERIAL 
   4 ! bi-axial linear elastic
! id   E         nu   psscode  rho
   1   2000000   0.35 1        8.93e-9  
!  END MATERIAL 
&
!  ======================================================= 
TIME_FUNCTIONS 
  1  ! harmonic
! id freq phase
  1  1745.9 0
!  END TIME_FUNCTIONS 
&
TIME_FUNCTIONS 
  1  ! harmonic
! id freq phase
  2  3120 0.4
!  END TIME_FUNCTIONS 
&
EOF

# Convert the nodes and element definitions to the WASP format
makewaspf $tout | sed -e "s/thickness/$thickness/g" >> $waspf

cat <<EOF >> $waspf
DispBC
1 ax 0 ay 0 az "0" vx "0" vy 0 vz "0" box 0 0 0 $H 0 0 inflate 0.001
2 az 0 vz 0 box 0 0 0 $W $(expeval "$H-0.02") 0 inflate 0.001
&
LOAD 
3 
DX $(expeval "2110000*($mesh_size/$W)") 1
$(expeval "($W-$R3+$R7)/2") $H 0 $R7 $H 0 inflate 0.1
& 
LOAD 
3 
DY $(expeval "-2120000*($mesh_size/$W)") 2
$(expeval "($W-$R3+$R7)/2") $H 0 $R7 $H 0 inflate 0.1
& 
History
W
N 1
&
Reduction 
  collect
  !  save_data
  end
  use
     nmodes 2
  end
&
EOF

# Update model log
meshfz=$tout-$$
cp $tout $meshfz
gzip $meshfz &
inpfz=$waspf-$$
cp $waspf $inpfz
gzip $inpfz&
cat <<EOF >> model.log


Time:        $(date)
Mesh file:   $meshfz
Input file:  $inpfz
Mesh size:   $mesh_size
Parameters:
             H=$H
             H1=$H1
             H2=$H2
             R3=$R3
             R7=$R7
             W=$W
             R9=$R9
             W9=$W9
             H9=$H9
             hole1=$hole1
             R10=$R10
             W10=$W10
             H10=$H10
             hole2=$hole2
             R11=$R11
             W11=$W11
             H11=$H11
             hole3=$hole3
EOF