#include "Wconfig.h"
#include "Wxf.h"
#include "Wloads.h"
#include "Wdomain.h"
#include "Wnode.h"
#include "Wtfuncs.h"

#include <ctype.h>

#define SMALL_NUM 0.01
#define LARGE_NUM 100

/*
   Create a new load.
 */
W_load_t * /* new load */
WL_new_load (void)
{
  W_load_t *l;

  l = w_new(W_load_t);
  CHECK_MALLOC(l);
  
  return l;
}

static void
make_node_group_list (W_load_t *l, W_domain_t *d)
{
  W_node_group_node_t *gn;
  W_node_t *n;
  W_vect_t rc;

  l->data.node_group.nodes = make_list ();
  n = NULL;
  while ((n = WD_get_next_node (d, n)) != NULL) {
    WN_get_ref_coords (n, &rc.x, &rc.y, &rc.z);
    if (G3D_PT_INSIDE_BOX (l->data.node_group.box, rc)) {
      gn = w_new (W_node_group_node_t);
      CHECK_MALLOC (gn);
      gn->n     = n;
#ifdef X_XFACE
      gn->f_rep = NULL;
#endif
      add_to_tail (l->data.node_group.nodes, gn);
    } /* if */
  } /* while */
}

#ifdef X_XFACE

static void
assemble_mouse_anim_load(W_domain_t *d, W_node_t *n)
{ 
  WCRec wcp;
  W_coords_3d_t c, V, F;
  W_float_t tm, f, L, cos_ax, cos_ay, cos_az;

  EVLastPointerWCPos(&wcp);
  WN_get_coords (n, &c.x, &c.y, &c.z);
  V.x = wcp.x - c.x;
  V.y = wcp.y - c.y;
  V.z = wcp.z - c.z;
  L   = W_vector_3d_length (&V);
  cos_ax = V.x / L;  cos_ay = V.y / L;  cos_az = V.z / L;
  tm  = WD_total_mass(d);
  if (L != 0) {
    f = LARGE_NUM * tm * (1/SMALL_NUM - 1/(L+SMALL_NUM));
  } else
    f = 0;
  F.x = f * cos_ax;   F.y = f * cos_ay;  F.z = f * cos_az;
  WN_assemble_force (n, &F);
}

#endif



/*
   Reader of load data.
 */
W_bool_t
W_load_reader (W_domain_t *d)
{
  const char *input_line;
  W_load_t *l;
  long n_num, lcode, tf_num, dir;
  double amplitude;
  char sdir[2];

  while ((input_line = W_get_input_line(d)) != NULL) {
    if (W_is_end_of_block (input_line))
      return TRUE;
    else {
      if (sscanf(input_line, "%ld", &lcode) != 1)
        BAD_INPUT();
      switch (lcode) {
      case MOUSE_ANIM_LOAD:
        if ((input_line = W_get_input_line(d)) == NULL)
          BAD_INPUT();
        if (sscanf(input_line, "%ld", &n_num) != 1) {
          BAD_INPUT();
        } else {
          l = WL_new_load();
          l->type                     = MOUSE_ANIM_LOAD;
          l->data.nodal.node_num      = n_num;
          WD_add_load(d, l); 
        }
        break;
      case NODAL_LOAD:
        if ((input_line = W_get_input_line(d)) == NULL)
          BAD_INPUT();
        
        if (sscanf(input_line, "%ld %ld %lf %ld",
                   &n_num, &dir, &amplitude, &tf_num) != 4) {
          if (sscanf(input_line, "%ld %s %lf %ld",
                     &n_num, sdir, &amplitude, &tf_num) != 4) {
            BAD_INPUT();
          } else {
            dir = W_DX;
            if      (toupper(sdir[0]) == 'D')
              if      (toupper(sdir[1]) == 'X') dir = W_DX;
              else if (toupper(sdir[1]) == 'Y') dir = W_DY;
              else if (toupper(sdir[1]) == 'Z') dir = W_DZ;
              else  { BAD_INPUT(); }
            else if (toupper(sdir[0]) == 'R')
              if      (toupper(sdir[1]) == 'X') dir = W_RX;
              else if (toupper(sdir[1]) == 'Y') dir = W_RY;
              else if (toupper(sdir[1]) == 'Z') dir = W_RZ;
              else  { BAD_INPUT(); }
            else
              { BAD_INPUT(); }
          }
        }
        l = WL_new_load();
        l->type                     = NODAL_LOAD;
        l->data.nodal.node_num      = n_num;
        
        l->data.nodal.direction     = dir;
        l->data.nodal.amplitude     = amplitude;
        l->data.nodal.time_func_num = tf_num;
#       ifdef X_XFACE
        l->data.nodal.fg_rep        = NULL;
        l->data.nodal.mg_rep        = NULL;
#       endif
        WD_add_load(d, l); 
        break;
      case ELEM_LOAD:
        break;
      case NODE_GROUP_LOAD:
        {
          G3D_box_t box;
          double inflate;
          W_loc_t x;
          W_node_group_load_type_enum kind = W_NODE_GROUP_LOAD_FORCE;
          char *accel[3] = {NULL,NULL,NULL};
          
          if ((input_line = W_get_input_line(d)) == NULL)
            BAD_INPUT();
          TOKENS_PARSE (input_line);

          if (TOKEN_IS_STRING (1)) {
            const char *s;
            s = TOKEN_STR (1);
            if        (tokens_token_and_keyword_equiv ("BODY", s)) {
              kind = W_NODE_GROUP_LOAD_BODY;
            }
          }

          if (kind == W_NODE_GROUP_LOAD_FORCE) {
            if (TOKEN_IS_STRING (1)) {
              const char *s;
              s = TOKEN_STR (1);
              if        (tokens_token_and_keyword_equiv ("DX", s)) {
                dir = W_DX;
              } else if (tokens_token_and_keyword_equiv ("DY", s)) {
                dir = W_DY;
              } else if (tokens_token_and_keyword_equiv ("DZ", s)) {
                dir = W_DZ;
              }
            } else {
              dir = TOKEN_INT (1);
            }
            amplitude = TOKEN_DBL (2);
            tf_num    = TOKEN_INT (3);
          } else {
            const char *s;
            s = TOKEN_STR (2);
            DUPLICATE_STRING (accel[0], s);
            s = TOKEN_STR (3);
            DUPLICATE_STRING (accel[1], s);
            s = TOKEN_STR (4);
            DUPLICATE_STRING (accel[2], s);
            tf_num    = TOKEN_INT (5);
          }
            
          if ((input_line = W_get_input_line(d)) == NULL)
            BAD_INPUT();
          TOKENS_PARSE (input_line);

          x.x = TOKEN_DBL (1);
          x.y = TOKEN_DBL (2);
          x.z = TOKEN_DBL (3);
          G3D_INIT_BOX (box, x);
          x.x = TOKEN_DBL (4);
          x.y = TOKEN_DBL (5);
          x.z = TOKEN_DBL (6);
          G3D_UPDT_BOX (box, x);

          TOKENS_SET_BY_KEYWORD ("INFLATE",
                                 FALSE,
                                 {
                                   inflate = TOKEN_DBL(_kpos_+1);
                                 },
                                 {
                                   inflate = 0;
                                 });
          G3D_INFLATE_BOX (box, inflate);
          
          l = WL_new_load ();
          l->type = NODE_GROUP_LOAD;
          l->data.node_group.kind                  = kind;
          switch (l->data.node_group.kind) {
          case W_NODE_GROUP_LOAD_FORCE:
            l->data.node_group.spec.force.direction  = dir;
            l->data.node_group.spec.force.amplitude  = amplitude;
            break;
          case W_NODE_GROUP_LOAD_BODY:
            l->data.node_group.spec.body.accel[0]  = accel[0];
            l->data.node_group.spec.body.accel[1]  = accel[1];
            l->data.node_group.spec.body.accel[2]  = accel[2];
            break;
          }
          l->data.node_group.time_func_num         = tf_num;
          l->data.node_group.box                   = box;
          make_node_group_list (l, d);
          WD_add_load (d, l); 
        }
        break;
      default: 
        return FALSE;
      }
    }
  }
  return FALSE;
}

static void
eval_bodyl (W_load_t *l, W_node_t *n, W_vect_t *ampl)
{
  W_node_group_load_t *ngl = &l->data.node_group;
  int stat;
  double f;
  W_loc_t x, X, u;

  WN_get_ref_coords (n, &X.x, &X.y, &X.z);
  WN_get_displ (n, &u.x, &u.y, &u.z);
  VECMAC_3_VEC (x, =, X, +, 0 * u); /* use reference config instead */
  
  expeval_set_value ("x", x.x);
  expeval_set_value ("y", x.y);
  expeval_set_value ("z", x.z);

  f = 0;
  if (ngl->spec.body.accel[0]) {
    stat = expeval_eval (ngl->spec.body.accel[0], &f);
    if (stat != EXPEVAL_E_OK) {
      sprintf (W_buf (), "%s: \"%s\"", expeval_err_msgs[stat], ngl->spec.body.accel[0]);
      REPORT_ERROR (NULL, W_buf ());
    } 
  }
  ampl->x = f;

  f = 0;
  if (ngl->spec.body.accel[1]) {
    stat = expeval_eval (ngl->spec.body.accel[1], &f);
    if (stat != EXPEVAL_E_OK) {
      sprintf (W_buf (), "%s: \"%s\"", expeval_err_msgs[stat], ngl->spec.body.accel[1]);
      REPORT_ERROR (NULL, W_buf ());
    } 
  }
  ampl->y = f;

  f = 0;
  if (ngl->spec.body.accel[2]) {
    stat = expeval_eval (ngl->spec.body.accel[2], &f);
    if (stat != EXPEVAL_E_OK) {
      sprintf (W_buf (), "%s: \"%s\"", expeval_err_msgs[stat], ngl->spec.body.accel[2]);
      REPORT_ERROR (NULL, W_buf ());
    } 
  }
  ampl->z = f;
}


#ifdef X_XFACE
static EPixel force_color = 0x0, moment_color = 0x0;

static void
modify_vect_rep (W_node_t *n,
                 W_coords_3d_t *f, int is_moment,
                 double displ_scale,
                 GraphicObj **grep)
{
  BOOLEAN suc;
  WCRec pnts[2];
  EPixel color;
  W_loc_t x, X, u;
  int np;
  
  if ((VECMAC_NORM ((*f)) == 0)) { /* zero length vector: do not draw */
    if (*grep != NULL) {        /* and if it was added to the graphic model, remove it */
      EMDeleteGraphics (ESIModel (), *grep);
      *grep = NULL;
    }
    return;
  }

  if (force_color == 0x0) {
    force_color = ColorGetPixelFromString("yellow", &suc);
    moment_color = ColorGetPixelFromString("magenta", &suc);
  }
  WN_get_ref_coords (n, &X.x, &X.y, &X.z);
  WN_get_displ (n, &u.x, &u.y, &u.z);
  VECMAC_3_VEC (x, =, X, +, displ_scale * u);
  if (is_moment) {
    color = moment_color;
  } else {
    color = force_color;
  }
  VECMAC_2_VEC (pnts[0], =, x);
  VECMAC_2_VEC (pnts[1], =, (*f));
  if (*grep == NULL) {
    EMNoDrawAddGraphicsToModel(ESIModel (), (*grep = CreateVector3D (pnts)));
    EASValsSetColor(color);
    EGWithMaskChangeAttributes(COLOR_MASK, *grep);
  }
  EGModifyGraphicsGeometry(*grep, pnts);
}


static void
modify_node_group_load_body_grep (
                                   W_load_t *l,
                                   W_domain_t *d
                                   )
{
  W_node_group_load_t *ngl = &l->data.node_group;
  W_time_func_t *tf = WD_get_time_func_by_idx(d, ngl->time_func_num);
  double tfv = WTF_func_val(tf, WD_time(d));
  W_node_group_node_t *gn;
  W_coords_3d_t F = {0, 0, 0};
  double force_scale = WOPT_OPT (d->options, force_scale);
  double displ_scale = WOPT_OPT (d->options, displ_scale);
  double fact = force_scale * tfv;

  loop_through_list (ngl->nodes, gn, W_node_group_node_t *) {
    if (fact != 0) {
      eval_bodyl (l, gn->n, &F);
      VECMAC_SCALAR (F, *=, gn->n->mass * fact);
    } 
    modify_vect_rep (gn->n, &F, FALSE, displ_scale, &gn->f_rep);
  }
}

static void
modify_node_group_load_force_grep (
                                   W_load_t *l,
                                   W_domain_t *d
                                   )
{
  W_node_group_load_t *ngl = &l->data.node_group;
  W_time_func_t *tf = WD_get_time_func_by_idx(d, ngl->time_func_num);
  double tfv = WTF_func_val(tf, WD_time(d));
  double amplitude = ngl->spec.force.amplitude;
  double val = amplitude * tfv;
  int dir = ngl->spec.force.direction ;
  W_node_group_node_t *gn;
  W_coords_3d_t F = {0, 0, 0};
  double displ_scale = WOPT_OPT (d->options, displ_scale);
  double force_scale = WOPT_OPT (d->options, force_scale);

  F.x = F.y = F.z = 0;
  if      (dir == W_DX) F.x = force_scale * val;
  else if (dir == W_DY) F.y = force_scale * val;
  else if (dir == W_DZ) F.z = force_scale * val;
  loop_through_list (ngl->nodes, gn, W_node_group_node_t *) {
    modify_vect_rep (gn->n, &F, FALSE, displ_scale, &gn->f_rep);
  }
#if W_ROT_DOFS
  F.x = F.y = F.z = 0;
  if      (dir == W_RX) F.x = force_scale * val;
  else if (dir == W_RY) F.y = force_scale * val;
  else if (dir == W_RZ) F.z = force_scale * val;
  loop_through_list (ngl->nodes, gn, W_node_group_node_t *) {
    modify_vect_rep (gn->n, &F, TRUE, displ_scale, &gn->f_rep);
  }
#endif
}

static void
modify_node_group_load_grep (
                             W_load_t *l,
                             W_domain_t *d
                             )
{
  switch (l->data.node_group.kind) {
  case W_NODE_GROUP_LOAD_FORCE:
    modify_node_group_load_force_grep (l, d);
    break;
  case W_NODE_GROUP_LOAD_BODY:
    modify_node_group_load_body_grep (l, d);
    break;
  }
}

static void
modify_nodal_load_grep(W_load_t *l, W_domain_t *d);

static void
modify_nodal_load_grep (W_load_t *l, W_domain_t *d)
{
  W_coords_3d_t F = {0, 0, 0};
  W_coords_3d_t M = {0, 0, 0};
  W_float_t val;
  W_nodal_load_t *nl = &l->data.nodal;
  W_time_func_t *tf = WD_get_time_func_by_idx(d, nl->time_func_num);
  W_node_t *n = WD_get_node_by_idx(d, nl->node_num);
  double displ_scale = WOPT_OPT (d->options, displ_scale);
  double force_scale = WOPT_OPT (d->options, force_scale);
  
  val = nl->amplitude * WTF_func_val (tf, WD_time(d));
  if      (nl->direction == W_DX) F.x = force_scale * val;
  else if (nl->direction == W_DY) F.y = force_scale * val;
  else if (nl->direction == W_DZ) F.z = force_scale * val;
  modify_vect_rep (n, &F, FALSE, displ_scale, &nl->fg_rep);

  if      (nl->direction == W_RX) M.x = force_scale * val;
  else if (nl->direction == W_RY) M.y = force_scale * val;
  else if (nl->direction == W_RZ) M.z = force_scale * val;
  modify_vect_rep (n, &M, TRUE, displ_scale, &nl->mg_rep);
}
#endif


static void
assemble_nodal_force_or_moment(W_load_t *l, W_domain_t *d)
{
  W_coords_3d_t F = {0, 0, 0};
#if W_ROT_DOFS
  W_coords_3d_t M = {0, 0, 0};
#endif
  W_float_t val;
  W_nodal_load_t *nl = &l->data.nodal;
  W_time_func_t *tf = WD_get_time_func_by_idx(d, nl->time_func_num);
  W_node_t *n = WD_get_node_by_idx(d, nl->node_num);
  
  val = nl->amplitude * WTF_func_val(tf, WD_time(d));
  if      (nl->direction == W_DX) F.x = val;
  else if (nl->direction == W_DY) F.y = val;
  else if (nl->direction == W_DZ) F.z = val;
  if (val != 0) {
    WN_assemble_force (n, &F);
  }
#if W_ROT_DOFS
  if      (nl->direction == W_RX) M.x = val;
  else if (nl->direction == W_RY) M.y = val;
  else if (nl->direction == W_RZ) M.z = val;
  if (val != 0) {
    WN_assemble_moment (n, &M);
  }
#endif
}


static void
assemble_node_group_force_load (W_load_t *l, W_domain_t *d)
{
  W_coords_3d_t F = {0, 0, 0};
#if W_ROT_DOFS
  W_coords_3d_t M = {0, 0, 0};
#endif
  W_node_group_load_t *ngl = &l->data.node_group;
  W_time_func_t *tf = WD_get_time_func_by_idx (d, ngl->time_func_num);
  double tfv = WTF_func_val(tf, WD_time(d));
  double amplitude = ngl->spec.force.amplitude;
  double val = amplitude * tfv;
  int dir = ngl->spec.force.direction ;
  W_node_group_node_t *gn;

  if (val != 0) {
    F.x = F.y = F.z = 0;
    if      (dir == W_DX) F.x = val;
    else if (dir == W_DY) F.y = val;
    else if (dir == W_DZ) F.z = val;
    loop_through_list (ngl->nodes, gn, W_node_group_node_t *) {
      WN_assemble_force (gn->n, &F);
    }
#if W_ROT_DOFS
    M.x = M.y = M.z = 0;
    if      (dir == W_RX) M.x = val;
    else if (dir == W_RY) M.y = val;
    else if (dir == W_RZ) M.z = val;
    loop_through_list (ngl->nodes, gn, W_node_group_node_t *) {
      WN_assemble_moment (gn->n, &M);
    }
#endif
  }
}


static void
assemble_node_group_body_load (W_load_t *l, W_domain_t *d)
{
  W_coords_3d_t F = {0, 0, 0};
  W_node_group_load_t *ngl = &l->data.node_group;
  W_time_func_t *tf = WD_get_time_func_by_idx (d, ngl->time_func_num);
  double tfv = WTF_func_val(tf, WD_time(d));
  W_node_group_node_t *gn;
  double fact = tfv;

  if (fact != 0) {
    loop_through_list (ngl->nodes, gn, W_node_group_node_t *) {
      eval_bodyl (l, gn->n, &F);
      if (F.x != 0 || F.y != 0 || F.z != 0) {
        VECMAC_SCALAR (F, *=, gn->n->mass * fact);
        WN_assemble_force (gn->n, &F);
      }
    }
  }
}
static void
assemble_node_group_load (W_load_t *l, W_domain_t *d)
{
  switch (l->data.node_group.kind) {
  case W_NODE_GROUP_LOAD_FORCE:
    assemble_node_group_force_load (l, d);
    break;
  case W_NODE_GROUP_LOAD_BODY:
    assemble_node_group_body_load (l, d);
    break;
  }
}



/*
   Assemble current value of external load.
 */
void
WL_assemble_load (W_load_t *l, /* load */
                  W_domain_t *d /* domain */
                  )
{
  switch (l->type) {
#ifdef X_XFACE
  case MOUSE_ANIM_LOAD:
    {
      W_nodal_load_t *nl = &l->data.nodal;

      if (WOPT_OPT (d->options, X_xface)) 
        assemble_mouse_anim_load(d, WD_get_node_by_idx(d, nl->node_num));
    }
    break;
#endif
  case NODAL_LOAD:
    assemble_nodal_force_or_moment (l, d);
    break;
  case NODE_GROUP_LOAD:
    assemble_node_group_load (l, d);
    break;
  case ELEM_LOAD:
    break;
  default:
    /* VOID */
    break;
  }
}



static W_float_t
incr_ext_nodal_load_work (W_nodal_load_t *nl, W_domain_t *d)
{
  W_float_t val = 0, dew = 0;
  W_time_func_t *tf = WD_get_time_func_by_idx(d, nl->time_func_num);
  W_coords_3d_t incr_disp, incr_rot;
  W_node_t *n = WD_get_node_by_idx(d, nl->node_num);
  
  val = nl->amplitude * WTF_func_val(tf, WD_time(d));

  WN_get_disp_increment(n, &incr_disp, &incr_rot);
  if      (nl->direction == W_DX) dew += val * incr_disp.x;
  else if (nl->direction == W_DY) dew += val * incr_disp.y;
  else if (nl->direction == W_DZ) dew += val * incr_disp.z;
  if      (nl->direction == W_RX) dew += val * incr_rot.x;
  else if (nl->direction == W_RY) dew += val * incr_rot.y;
  else if (nl->direction == W_RZ) dew += val * incr_rot.z;

  return dew;
}


static W_float_t
incr_ext_node_group_load_work (W_node_group_load_t *ngl,
                               W_domain_t *d)
{
  W_float_t dew = 0;
  W_time_func_t *tf = WD_get_time_func_by_idx(d, ngl->time_func_num);
  W_coords_3d_t incr_disp, incr_rot;
  double tfv = WTF_func_val(tf, WD_time(d));
  double amplitude = ngl->spec.force.amplitude;
  double val = amplitude * tfv;
  int dir = ngl->spec.force.direction ;
  W_node_group_node_t *gn;

  loop_through_list (ngl->nodes, gn, W_node_group_node_t *) {
    WN_get_disp_increment (gn->n, &incr_disp, &incr_rot);
    if      (dir == W_DX) dew += val * incr_disp.x;
    else if (dir == W_DY) dew += val * incr_disp.y;
    else if (dir == W_DZ) dew += val * incr_disp.z;
    if      (dir == W_RX) dew += val * incr_rot.x;
    else if (dir == W_RY) dew += val * incr_rot.y;
    else if (dir == W_RZ) dew += val * incr_rot.z;
  }

  return dew;
}



/*
   Compute increment of work of external load.
 */
W_float_t /* increment of work in the time step */
WL_incr_ext_load_work (W_load_t *l, /* load */
                       W_domain_t *d /* domain */
                       ) 
{
  switch (l->type) {
  case NODAL_LOAD:
    return incr_ext_nodal_load_work (&l->data.nodal, d);
  case NODE_GROUP_LOAD:
    return incr_ext_node_group_load_work (&l->data.node_group, d);
  case ELEM_LOAD:
    return 0; RAW
  default:
    return 0;
  }
}


#ifdef X_XFACE
/*
   Modify load graphic representation.
 */
void
WL_modify_load_grep (W_load_t *l, /* load */
                       W_domain_t *d /* domain */ ,
                     W_bool_t immed_draw
                       ) 
{
  switch (l->type) {
  case MOUSE_ANIM_LOAD:
    /* VOID */
    break;
  case NODAL_LOAD:
    modify_nodal_load_grep (l, d);
    break;
  case NODE_GROUP_LOAD:
    modify_node_group_load_grep (l, d);
    break;
  case ELEM_LOAD:
    /* RAW -- need to update */
    break;
  }
}
#endif
