#! /bin/ksh -f


base=Truss
nmodes=9
N_max=40
up_to_time=0.001

function usage {
cat <<EOF    
Usage: $(basename $0)  [ options ]
Options:
-nmodes n (default $nmodes)
-up_to_time t (default $up_to_time)
EOF
}

args=""
while [ $# -ge 1 ] 
do
   a=$1; shift
   case $a in
    -help)
        usage; 
        $wd -help;
        exit 0;
        ;;
    -base)
        if [ $# -ge 1 ] ; then
            base=$1; shift
        else
            usage; exit 1;
        fi
        ;;
     -up_to_time)
        if [ $# -ge 1 ] ; then
            up_to_time=$1; shift
        else
            usage; exit 1;
        fi
        ;;
    -nmodes)
        if [ $# -ge 1 ] ; then
            nmodes=$1; shift
        else
            usage; exit 1;
        fi
        ;;
    *)
        ta="$args $a"
        args="$ta"
        ;;
    esac
done

function genl {
# generates a list of numbers from `low' to `high' (inclusive)
    genl_low=$1; shift
    genl_high=$1; shift
    # echo genl_low=$genl_low
    # echo genl_high=$genl_high
    genl_counter=$genl_low
    genl_s=""
    while [ $genl_counter -le $genl_high ] 
    do
      genl_ss="$genl_s $genl_counter"
      genl_s="$genl_ss"
      genl_counter=$(($genl_counter + 1))
    done
    echo $genl_s
}

full_suffix="-full-R5000"
red_suffix="-red-R5000-c0.00001-M${nmodes}-direct"

if [ ! -d WASP.HIST/${base}${red_suffix} ] ; then
    echo 0 "# not found"
    exit 1
fi

# echo "# ${input_base_name}, $nmodes modes, time interval=$up_to_time"
sum_j=0
for N in $(genl 1 $N_max)
do
    for dir in x y  
    do
        v=$(l2diff \
           -u1 WASP.HIST/${base}${full_suffix}/N${N}TD${dir}.xmgr \
           -u2 WASP.HIST/${base}${red_suffix}/N${N}TD${dir}.xmgr \
           -up_to_time $up_to_time)
        sum_j=$(expeval "$sum_j + $v")
        #echo N=$N dir=$dir v=$v sum_j=$sum_j #debug
    done
done
echo $sum_j
