import java.lang.Math.*;
import CircleCurve;

class GenerateRandStructure {
    private static double minx = -100, maxx = 100; 
    private static double miny = -100, maxy = 100; 
    private static int max_circles = 100;
    private static double radius = 5;
    private static double min_dist = 3 * radius;
    private static boolean too_close (double x, double y, double ax, double ay)
    {
        double d;
        double hmin_dist = 0.5 * min_dist;
        d = Math.sqrt ((x - ax)*(x - ax) + (y - ay)*(y - ay));
        return
            (
             (d < min_dist) ||
             (Math.abs (ax - minx) < hmin_dist) ||
             (Math.abs (ax - maxx) < hmin_dist) ||
             (Math.abs (ay - miny) < hmin_dist) ||
             (Math.abs (ay - maxy) < hmin_dist)
             );
    }
    private static boolean gen_circle (CircleCurve [] circles, int which)
    {
        double x, y;
        boolean acceptable = false;
        int num_attempts = 0;
        int max_num_attempts = max_circles * 10000;
        while (true) {
            acceptable = true;
            x = minx + (maxx - minx) * Math.random ();
            y = miny + (maxy - miny) * Math.random ();
            for (int i = 0; i < max_circles; i++) {
                if (circles[i] != null) {
                    if (too_close (circles[i].get_x (), circles[i].get_y (), x, y)) {
                        acceptable = false;
                        break;
                    }
                }
            }
            if (acceptable) {
                circles[which] = new CircleCurve (radius, x, y);
                return true;
            }
            num_attempts++;
            if (num_attempts > max_num_attempts)
                return false;
        }
    }
    public static void main (String[] args)
    {
        CircleCurve [] circles = new CircleCurve[max_circles];

        for (int i = 0; i < max_circles; i++) circles[i] = null;

        System.out.println ("curve " + (max_circles + 1) + " line " +
                            minx + " " + miny  + " " + minx + " " + maxy);
        System.out.println ("curve " + (max_circles + 2) + " line " +
                            minx + " " + maxy + " " + maxx + " " + maxy);
        System.out.println ("curve " + (max_circles + 3) + " line " +
                            maxx + " " + maxy + " " + maxx + " " + miny);
        System.out.println ("curve " + (max_circles + 4) + " line " +
                            maxx + " " + miny + " " + minx + " " + miny);
        for (int i = 0; i < max_circles; i++) {
            if (circles[i] == null)
                if (!gen_circle (circles, i))
                    break;
                System.out.println ("curve " + (i+1) + " circle center " +
                                    circles[i].get_x () + " " + circles[i].get_y () +
                                    " radius " + circles[i].get_radius ());
        }
        System.out.println ("m-ctl-point constant " +
                            ((maxx - minx) + (maxy - miny))/100);
        String s = new String ("");
        for (int i = 0; i < max_circles; i++) {
            if (circles[i] != null)
                s = s + " hole " + (-(i+1));
        }
        System.out.println ("subregion 1 property 1 boundary " +
                            -(max_circles + 1) + " " +
                            -(max_circles + 2) + " " +
                            -(max_circles + 3) + " " +
                            -(max_circles + 4) + " " + s);
    }
}
