#! /bin/ksh -f

default_A=0.013
default_M=100000
default_x_2=-2
default_z_2=0
default_x_3=-2
default_z_3=-1
default_A_1=$default_A
default_A_2=$default_A
default_A_3=$default_A
default_A_4=$default_A


function usage_and_exit {
cat <<EOF
Usage: $(basename $0) [ options ]
-A n area  (default A=$default_A)
-x n
-z n
EOF
exit 0
}

A_1=$default_A_1
A_2=$default_A_2
A_3=$default_A_3
A_4=$default_A_4

x_2=$default_x_2
z_2=$default_z_2
x_3=$default_x_3
z_3=$default_z_3

M=$default_M

args=""
while [ $# -ge 1 ] 
do
   a=$1; shift
   case $a in
    -A)
        if [ $# -ge 2 ] ; then
            n=$1; shift
            A=$1; shift
            eval A_$n=$A
        else
            usage_and_exit;
        fi
        ;;
    -x)
        if [ $# -ge 2 ] ; then
            n=$1; shift
            x=$1; shift
            eval x_$n=$x
        else
            usage_and_exit;
        fi
        ;;
    -z)
        if [ $# -ge 2 ] ; then
            n=$1; shift
            z=$1; shift
            eval z_$n=$z
        else
            usage_and_exit;
        fi
        ;;
    -M)
        if [ $# -ge 1 ] ; then
            M=$1; shift
        else
            usage_and_exit;
        fi
        ;;
    *)
        ta="$args $a"
        args="$ta"
        ;;
    esac
done

W="sd-A_1=$A_1-A_2=$A_2-A_3=$A_3-A_4=$A_4-M=$M-x_2=$x_2-z_2=$z_2-x_3=$x_3-z_3=$z_3"

cat > simpledesign.in <<EOF
Simple truss for design. A_1=$A_1, A_2=$A_2, A_3=$A_3, A_4=$A_4, M=$M, x_2=$x_2, z_2=$z_2, x_3=$x_3, z_3=$z_3
! num_of_nodes num_of_elements num_of_materials num_of_loads num_of_time_funcs
5 6 1 1 1
!
Options
M $M
R 0.5
T 1000
O 0.01
f 0.1
! x 1
M $M
&
!
Material 
! uniaxial elastic
   1
! material data 
   1  117000  0.35  8.93e-9 
&
!  ======================================================= 
TIME_FUNCTIONS 
2  ! tabular 
! func#   #pairs 
1        3
! time     val 
! Heaviside step function:
0        1
1        1
10000000 1
!  END TIME_FUNCTIONS 
&
!  ======================================================
NODES
1 0 0 0  1 0 1
2 $x_2 0 $z_2  1 0 1
3 $x_3 0 $z_3  1 0 1
4 -4 0 -2  0 0 0
5 -4 0 0  0 0 0
&
ELEM
1 ! truss
1 1 2 1 $A_1
2 1 3 1 $A_2
3 2 3 1 $A_3
4 2 5 1 $A_1
5 5 3 1 $A_4
6 3 4 1 $A_2
&
!
LOAD 
3 
!    gx  gy  gz      tfnum
DZ -10 1
0 0 -1 0 0 10 inflate 0.01 
& 
!
History 
N 1
W
&
!
Reduction 
  collect
     dt 0.02
  end
  use
     nmodes 2
     file simpledesign.pod
  end
&
EOF

wd -! -input simpledesign -O 0 -T 10000 -W "$W" $args | grep -v '#'
