#include "Wconfig.h"
#include "Wxf.h"
#include "Wdomain.h"
#include "Wdriver.h"
#include "Welem.h"

#include "design_observer.h"

static double
compute_max_max_energy_density (W_domain_t *d)
{
  double mmed = 0;
  double eed, em;
  W_elem_t *e = NULL;
  while (e = WD_get_next_element (d, e)) {
    eed = WE_deformation_energy (e);
    em  = WE_elem_total_mass (e);
    if (em == 0) REPORT_FATAL_ERROR (d->errh, "Zero element mass?");
    mmed = max (mmed, eed / em);
  }
  return mmed;
}
    

static W_node_t *n1 = NULL;
static double n1_max_x_displ = 0;
static double n1_max_y_displ = 0;
static double max_max_energy_density = 0;

static void
recalc (W_domain_t *d)
{
  W_loc_t u;
  if (n1 == NULL) {
    n1 = WD_get_node_by_idx (d, 1);
    if (!n1) REPORT_FATAL_ERROR (d->errh, "No node #1?");
  }
  WN_get_displ (n1, &u.x, &u.y, &u.z);
  n1_max_x_displ = max (n1_max_x_displ, u.x);
  n1_max_y_displ = max (n1_max_y_displ, u.y);
  max_max_energy_density = max (max_max_energy_density, compute_max_max_energy_density (d));
}

static void
print_results (W_domain_t *d)
{
  printf ("      mass  maxU maxV maxmaxE\n");
  printf ("DESIGN: %g : %g : %g : %g\n",
          WD_total_mass (d), n1_max_x_displ, n1_max_y_displ, max_max_energy_density/1.0e10);
}

void
design_observer (W_domain_t *d, int end_of_integration)
{
  if (end_of_integration) print_results (d);
  else                    recalc (d);
}
