#ifndef __BOUNDARYSTENCIL_H__
#define __BOUNDARYSTENCIL_H__

#include "Loop/Topology.h"
#include "Util/cvec.h"

/**
 * BoundaryStencil Class
 */
template <class Stencil>
class BoundaryStencil {
private:
  Stencil& m_stencil;

public:
  BoundaryStencil(Stencil& stencil) : m_stencil(stencil) { }

  BoundaryStencil(const BoundaryStencil& instance)
   : m_stencil(instance.m_stencil) { }

  ~BoundaryStencil(void) { }

  BoundaryStencil& operator=(const BoundaryStencil& instance) {
    m_stencil = instance.m_stencil;
    return (*this);
  }

  template <class TriangleMr>
  CVec& even(TriangleMr* triangle, VertexLabel v) {
    if (isBoundaryVertex(triangle, v)) {
      TriangleMr::Vertex* vertex = triangle->vertex(v);
      int depth = triangle->depth();
      vertex->point(depth + 1) = vertex->point(depth);
      vertex->setValid(depth + 1);
      return (vertex->point(depth + 1));
    }

    return (m_stencil.even(triangle, v));
  }

  template <class TriangleMr>
  CVec limit(TriangleMr* triangle, VertexLabel v) {
    if (isBoundaryVertex(triangle, v)) {
      TriangleMr::Vertex* vertex = triangle->vertex(v);
      int depth = triangle->depth();
      return (vertex->point(depth));
    }

    return (m_stencil.limit(triangle, v));
  }

  template <class TriangleMr>
  CVec& odd(TriangleMr* triangle, EdgeLabel e) {
    Permutation p;
    if (triangle->adjacentTriangle(e, p) == 0) {
      int depth = triangle->depth();
      CVec odd = 0.5f * (triangle->vertex(CCWVertex(e))->point(depth) +
			 triangle->vertex(CWVertex(e))->point(depth));

      VertexLabel v = OppositeVertex(e);
      TriangleMr::Vertex* vertex = triangle->child(T0)->vertex(v);
      vertex->point(depth + 1) = odd;
      vertex->setValid(depth + 1);
      return (vertex->point(depth + 1));
    }

    return (m_stencil.odd(triangle, e));
  }

private:
  template <class TriangleMr>
  bool isBoundaryVertex(TriangleMr* triangle, VertexLabel v) {
    TriangleLabel t = triangle->label();
    if (!triangle->isRoot() && (t == T0 || v != ParentVertex(t))) {
      EdgeLabel e = OppositeEdge(v);
      Permutation p;

      if (triangle->parent()->adjacentTriangle(e, p) == 0)
	return (true);
      else
	return (false);
    }

    VertexLabel pivot = v;
    VertexLabel vertex = CCWVertex(pivot);
    TriangleMr* next = triangle;
    Permutation p;
    
    do {
      next = next->adjacentTriangle(OppositeEdge(vertex), p);
      if (next == 0) {
	return (true);
      }

      pivot = p.vertex(pivot);
      vertex = OppositeVertex(p.vertex(vertex), pivot);
    } while (next != triangle);

    return (false);
  }
};

#endif  /* __BOUNDARYSTENCIL_H__ */
