#ifndef __CONTROLVERTEX_H__
#define __CONTROLVERTEX_H__

#include <assert.h>
#include "Mesh/PointVertex.h"
#include "Util/cvec.h"

/**
 * ControlVertex Class
 */
template <class TriangleIf, class VertexIf, class VertexMr>
class ControlVertex : public PointVertex<TriangleIf, VertexIf> {
public:
  typedef VertexMr Vertex;

protected:
  /** The multiresolution vertex associated with this geometric vertex. */
  Vertex* m_vertex;

public:
  /**
   * Constructs an uninitialized vertex.  Initially the vertex does not
   * belong to the topology of a mesh.
   */
  ControlVertex(void) : PointVertex<TriangleIf, VertexIf>() {
    m_vertex = new Vertex(0);
    assert(m_vertex);
  }
  
  /**
   * Destructor.
   */
  ~ControlVertex(void) {
    delete (m_vertex);
  }

  /**
   * Returns the multiresolution vertex associated with this geometric vertex.
   * @return multiresolution vertex at this geometric vertex
   */
  Vertex* vertex(void) const {
    return (m_vertex);
  }

private:
  /**
   * Copy constructor: undefined operation.
   */
  ControlVertex(const ControlVertex& instance) {
    die();
  }

  /**
   * Assignment operator: undefined operation.
   */
  ControlVertex& operator=(const ControlVertex& instance) {
    die();
    return (*this);
  }
};

#endif  /* __CONTROLVERTEX_H__ */
