#ifndef __PERMUTATION_H__
#define __PERMUTATION_H__

#include "Loop/Topology.h"

/**
 * Permutation Class
 */
class Permutation {
public:
  static const short IDENTITY;

  template <class TriangleIf>
  static short encode(TriangleIf* triangle, EdgeLabel edge) {
    TriangleIf* adjacent = triangle->triangle(edge);
    if (!adjacent)
      return (0);

    Permutation p = PERMUTATION[edge];
    TriangleIf::Vertex* v1 = triangle->vertex(p.vertex(V2));
    TriangleIf::Vertex* v2 = triangle->vertex(p.vertex(V3));
    if (v1 == adjacent->vertex(p.vertex(p.vertex(V2)))) {
      if (v2 == adjacent->vertex(p.vertex(p.vertex(V3))))
	return (0);
      else
	return (3);
    } else if (v1 == adjacent->vertex(p.vertex(p.vertex(V1)))) {
      if (v2 == adjacent->vertex(p.vertex(p.vertex(V2))))
	return (1);
      else
	return (4);
    } else {
      if (v2 == adjacent->vertex(p.vertex(p.vertex(V1))))
	return (2);
      else
	return (5);
    }
  }

private:
  static const Permutation PERMUTATION[6];
  
public:
  Permutation(void) : m_edgeMap(0), m_triangleMap(0), m_vertexMap(0) { }
  
  Permutation(const Permutation& instance)
    : m_edgeMap(instance.m_edgeMap),
      m_triangleMap(instance.m_triangleMap),
      m_vertexMap(instance.m_vertexMap) { }
  
  Permutation(const short index)
    : m_edgeMap(PERMUTATION[index].m_edgeMap),
      m_triangleMap(PERMUTATION[index].m_triangleMap),
      m_vertexMap(PERMUTATION[index].m_vertexMap) { }

  Permutation& operator=(const Permutation& instance) {
    m_edgeMap = instance.m_edgeMap;
    m_triangleMap = instance.m_triangleMap;
    m_vertexMap = instance.m_vertexMap;
    return (*this);
  }
  
  EdgeLabel edge(const EdgeLabel e) const {
    assert(m_edgeMap);
    return (m_edgeMap[e]);
  }
  
  void identity(void) {
    *this = PERMUTATION[IDENTITY];
  }
  
  TriangleLabel triangle(const TriangleLabel t) const {
    assert(m_triangleMap);
    return (m_triangleMap[t]);
  }
  
  VertexLabel vertex(const VertexLabel v) const {
    assert(m_vertexMap);
    return (m_vertexMap[v]);
  }
  
private:
  EdgeLabel* m_edgeMap;
  
  TriangleLabel* m_triangleMap;
  
  VertexLabel* m_vertexMap;
  
  
  Permutation(VertexLabel v1, VertexLabel v2, VertexLabel v3) {
    m_vertexMap = new VertexLabel[3];
    m_vertexMap[V1] = v1;
    m_vertexMap[V2] = v2;
    m_vertexMap[V3] = v3;
    
    m_edgeMap = new EdgeLabel[3];
    m_edgeMap[E1] = OppositeEdge(v1);
    m_edgeMap[E2] = OppositeEdge(v2);
    m_edgeMap[E3] = OppositeEdge(v3);
    
    m_triangleMap = new TriangleLabel[4];
    m_triangleMap[T0] = T0;
    m_triangleMap[T1] = ChildTriangle(v1);
    m_triangleMap[T2] = ChildTriangle(v2);
    m_triangleMap[T3] = ChildTriangle(v3);
  }
};

#endif  /* __PERMUTATION_H__ */
