#ifndef __LOOP_TOPOLOGY_H__
#define __LOOP_TOPOLOGY_H__

#include <assert.h>
#include "Mesh/Labels.h"

#define __MAX_DEPTH 20
#define __MAX_VALENCE 25

enum TriangleLabel { T0 = 0, T1 = 1, T2 = 2, T3 = 3 };

inline EdgeLabel CCWEdge(const EdgeLabel e) {
  return EdgeLabel((e + 1) % 3);
}

inline VertexLabel CCWVertex(const EdgeLabel e) {
    return VertexLabel((e + 1) % 3);
}

inline VertexLabel CCWVertex(const VertexLabel v) {
  return VertexLabel((v + 1) % 3);
}

inline EdgeLabel CWEdge(const EdgeLabel e) {
  return EdgeLabel((e + 2) % 3);
}

inline VertexLabel CWVertex(const EdgeLabel e) {
    return VertexLabel((e + 2) % 3);
}

inline VertexLabel CWVertex(const VertexLabel v) {
  return VertexLabel((v + 2) % 3);
}

inline TriangleLabel ChildTriangle(const VertexLabel v) {
    return TriangleLabel(v + 1);
}

inline EdgeLabel IncidentEdge(const VertexLabel v1, const VertexLabel v2) {
    assert(v1 != v2);
    return EdgeLabel(3 - v1 - v2);
}

inline EdgeLabel OppositeEdge(const VertexLabel v) {
    return EdgeLabel(v);
}

inline VertexLabel OppositeVertex(const EdgeLabel e) {
    return VertexLabel(e);
}

inline VertexLabel OppositeVertex(const VertexLabel v1, const VertexLabel v2) {
    assert(v1 != v2);
    return VertexLabel(3 - v1 - v2);
}

inline EdgeLabel ParentEdge(const TriangleLabel t, const EdgeLabel e) {
    return EdgeLabel(((e + 1) ^ t) - 1);
}

inline VertexLabel ParentVertex(const TriangleLabel t) {
    return VertexLabel(t - 1);
}


typedef TriangleLabel TriangleName;

inline VertexName   VT(const TriangleName t) { return VertexName(t - 1); }
inline VertexName   VE(const EdgeName e)     { return VertexName(e - 1); }
inline EdgeName     EV(const VertexName v)   { return EdgeName(v + 1); }
inline TriangleName TV(const VertexName v)   { return TriangleName(v + 1); }
inline EdgeName     ET(const TriangleName t) { return EdgeName(t); }
inline TriangleName TE(const EdgeName e)     { return TriangleName(e); }
inline VertexName   OtherV(const VertexName v1, const VertexName v2) {
    assert(v1 != v2);
    return VertexName(3 - v1 - v2);
}
inline EdgeName     EVV(const VertexName v1, const VertexName v2) {
    return EV(OtherV(v1, v2));
}
inline VertexName   HeadV(const EdgeName e) {
    return VertexName((e + 1) % 3);
}
inline VertexName   TailV(const EdgeName e) { return VertexName(e % 3); }


#endif  /* __LOOP_TOPOLOGY_H__ */
