#ifndef __WARRENSTENCIL_H__
#define __WARRENSTENCIL_H__

#include "Loop/LoopStencil.h"
#include "Util/cvec.h"

/**
 * WarrenStencil Class
 */
class WarrenStencil : public LoopStencil {
public:
  WarrenStencil(void) : LoopStencil() { }

  WarrenStencil(const WarrenStencil& instance)
   : LoopStencil(instance) { }

  ~WarrenStencil(void) { }

  WarrenStencil& operator=(const WarrenStencil& instance) {
    LoopStencil::operator=(instance);
    return (*this);
  }

  template <class TriangleMr>
  CVec& even(TriangleMr* triangle, VertexLabel v) {
    // In 9/16 of all cases, this point is incident to an edge of the
    // triangle's parent.
    TriangleLabel t = triangle->label();
    if (!triangle->isRoot() && (t == T0 || v != ParentVertex(t))) {
      return (LoopStencil::even(triangle, v));
    }

    int valence = triangle->valence(v);
    float alpha, beta;
    if (valence == 3) {
      alpha = 7.0f / 16.0f;
      beta = 3.0f / 16.0f;
    } else {
      alpha = 5.0f / 8.0f;
      beta = 3.0f / (8.0f * (float)valence);
    }

    TriangleMr::Vertex* vertex = triangle->vertex(v);
    int depth = triangle->depth();
    CVec even = alpha * vertex->point(depth);
    
    VertexLabel pivot = v;
    VertexLabel outer = CCWVertex(pivot);
    Permutation p;
    TriangleMr* next = triangle;
    
    for (int i = 0; i < valence; i += 1) {
      even += beta * next->vertex(outer)->point(depth);
      next = next->adjacentTriangle(OppositeEdge(outer), p);
      pivot = p.vertex(pivot);
      outer = OppositeVertex(p.vertex(outer), pivot);
    }

    vertex->point(depth + 1) = even;
    vertex->setValid(depth + 1);
    return (vertex->point(depth + 1));
  }
};

#endif  /* __WARRENSTENCIL_H__ */
