#include <iostream.h>
#include "Loop/BoundaryStencil.h"
#include "Loop/ControlTriangle.h"
#include "Loop/ControlVertex.h"
#include "Loop/LoopMesh.h"
#include "Loop/MultiresVertex.h"
#include "Loop/WarrenStencil.h"
#include "Mesh/MeshTp.h"

LOOPTYPE(MESH, LoopMesh, ControlTriangle, ControlVertex, MultiresVertex,
	 BoundaryStencil<WarrenStencil>)

void print(MESH::TriangleMr* triangle) {
  cout << triangle->parent()->vertex(V1)->point(0).x() <<
          triangle->parent()->vertex(V2)->point(0).x() <<
          triangle->parent()->vertex(V3)->point(0).x() <<
          't' << triangle->label();
}

int main(int argc, char* argv[]) {
  WarrenStencil warren;
  BoundaryStencil<WarrenStencil> stencil(warren);
  MESH mesh(stencil);

  cin >> mesh;

  MESH::TriangleIt ti = mesh.triangleBegin();
  MESH::TriangleMr* root = (*ti)->root();
  mesh.subdivide(root, 1);

  for (int t = T0; t <= T3; t += 1) {
    MESH::TriangleMr* triangle = root->child(TriangleLabel(t));
    cout << '(';
    print(triangle);
    cout << ')';

    MESH::TriangleMr::TriangleIt tti = triangle->triangleBegin(V1);
    MESH::TriangleMr::TriangleIt tte = triangle->triangleEnd(V1);
    while (tti != tte) {
      cout << ' ';
      print(*tti);
      ++tti;
    }
    cout << endl;
    ++ti;
  }
  cout << endl;

  for (t = T0; t <= T3; t += 1) {
    MESH::TriangleMr* triangle = root->child(TriangleLabel(t));
    cout << '(';
    print(triangle);
    cout << ')';

    MESH::TriangleMr::OneRingIt toi = triangle->oneRingBegin();
    MESH::TriangleMr::OneRingIt toe = triangle->oneRingEnd();
    while (toi != toe) {
      cout << ' ';
      print(*toi);
      ++toi;
    }
    cout << endl;
    ++ti;
  }
}
