// -*- Mode: c++ -*-
// $Id: Diamond.cc,v 1.1.1.1 2000/01/06 20:46:11 eitan Exp $
// $Source: /ufs/multires/projects/Coke/Quadrature/Diamond.cc,v $

#include "ShellDOF.h"
#include "FEMTriangle.h"
#include "QuadCoeffs.h"

template <class Triangle, class Vertex>
inline void
Diamond<Triangle, Vertex>::
LoopEven( QuadCoeffs c1[], const QuadCoeffs& cnt, const bool crease ) const
{
  if( crease ){
    // if the edge is on a crease we use the usual 1D cubic spline rule for
    // the even guy (1,6,1)/8
    c1[CNT] += 6.0/8.0 * cnt; const QuadCoeffs d = 1.0/8.0 * cnt; c1[VL] += d; c1[VR] += d;
  }else{
    // else we have the standard 1 ring rule: 10/16 in the middle, 1/16 all around
    c1[CNT] += 10.0/16.0 * cnt; const QuadCoeffs d = 1.0/16.0 * cnt;
    c1[VL] += d; c1[VR] += d; c1[VBL] += d; c1[VBR] += d; c1[VTL] += d; c1[VTR] += d;
  }
}

template <class Triangle, class Vertex>
inline int
Diamond<Triangle, Vertex>::
FindCreaseNeighbors( const OneRing<Triangle,Vertex>& vn, // center vertex
		     const int vo,      // outer vertex
		     int& vl,           // first crease walking left
		     int& vr )          // first crease walking right
{
  int i;
  const int val = vn.Valence();
  // it could be that the present edge is leftmost crease
  vl = vo; vr = -1;
  // walk to the right starting at least 1 off to the right
  for( i = 1; i < val; ++i ) if( vn.Crease((vo+i)%val) ){ vr = (vo+i)%val; break; }
  assert( vr != -1 );
  // if the outer is already a crease stop immediately
  for( i = 0; i < val; ++i ) if( vn.Crease((vo+val-i)%val) ){ vl = (vo+val-i)%val; break; }
  return (vr + val - vl)%val;
}

template <class Triangle, class Vertex>
inline Real
Diamond<Triangle, Vertex>::
CornerNeighbor( const OneRing<Triangle,Vertex>& vn,
		const int vo,
		const bool cx,
		const QuadCoeffs& cn,
		Real& gamma, const Real scale ) // 
{
  // indices of left and right most creases relative to vo in ring around vn
  int vl, vr;
  const int k = FindCreaseNeighbors( vn, vo, vl, vr ); assert( k > 1 );

  const Real alpha =
    acos(( vn.v(vr)->point() - vn.c()->point() ).dir().
	 dot(( vn.v(vl)->point() - vn.c()->point() ).dir()));
  // convex or not?
  const Real theta = cx ? alpha/k : (2*Real(M_PI)-alpha)/k;
  gamma = 1.0/2.0 - (1.0/4.0)*cos(theta);

  const Real thetak = Real(M_PI)/k;
  // scale factor is s=\lambda_1^2/\lambda_3
  const Real s = sqr(1.0/2.0)/((1.0/4.0)*cos(thetak)+gamma);
  // however: for concave the bound needs to be satisfied
  assert( cx || ( (1-s) < 2.0/(2.+cos(thetak)-cos(theta)) ) );

  // index of vo with respect to this sector
  const int val = vn.Valence();
  const int vol = (vo+val-vl)%val;
  const Real a1 = scale * .5 * (sin(vol*theta)/sin(k*theta));
  const Real a2 = scale * .5 * (sin((k-vol)*theta)/sin(k*theta));
  assert( m_c.find(vn.c()) != m_c.end() );
  *m_c[vn.c()] += ((1-a1-a2)*(1-s)) * cn;
  assert( m_c.find(vn.v(vr)) != m_c.end() );
  *m_c[vn.v(vr)] += (a1*(1-s)) * cn;
  assert( m_c.find(vn.v(vl)) != m_c.end() );
  *m_c[vn.v(vl)] += (a2*(1-s)) * cn;

  return 1-s;
}

template <class Triangle, class Vertex>
inline Real
Diamond<Triangle, Vertex>::
CreaseNeighbor( const OneRing<Triangle,Vertex>& vn,
		const int vo,
		const QuadCoeffs& cn,
		Real& gamma,
		const Real scale ) //
{
  // indices of left and right most creases relative to vo in ring around vn
  int vl, vr;
  const int k = FindCreaseNeighbors( vn, vo, vl, vr ); assert( k > 1 );
  if( k == 3 ) return 0;
  
  Real thetak = Real(M_PI)/k;
  gamma = (1.0/2.) - (1.0/4.)*cos(thetak);

  // scale factor is s=\lambda_1^2/\lambda_3
  const Real s = sqr(1.0/2.)/((1.0/4.)*cos(2.*thetak)+gamma);

  const int val = vn.Valence();
  const int vol = (vo+val-vl)%val;
  // a2 has special values for left/right/center
  Real l20, l2c;
  {
    const Real xi = acos( cos(thetak) - 1.0);
    const Real a = (1.0/4.)*(1.+cos(thetak))/(3.*((1.0/2.)-(1.0/4.)*cos(thetak)));
    const Real bsig3 = (2.0/3.-a)*sin(thetak)/(cos(xi)-cos(thetak));
    const Real sig1 = sin(thetak)/(1-cos(thetak));

    l20 = -2.0/k*((a/2.+1.0/6.)*sig1+.5*bsig3);
    l2c = -2.0/k*((2.0/3.-a)*sig1-bsig3);
  }
  assert( m_c.find(vn.c()) != m_c.end() );
  *m_c[vn.c()] += ((2.0/3.+.5*scale*l2c*sin(vol*thetak))*(1-s)) * cn;
  assert( m_c.find(vn.v(vl)) != m_c.end() );
  *m_c[vn.v(vl)] += ((1.0/6.+
		      scale*(1.0/4.)*cos(vol*thetak)+
		      (1.0/2.)*scale*l20*sin(vol*thetak))*(1-s)) * cn;
  assert( m_c.find(vn.v(vr)) != m_c.end() );
  *m_c[vn.v(vr)] += ((1.0/6.-
		      scale*(1.0/4.)*cos(vol*thetak)+
		      (1.0/2.)*scale*l20*sin(vol*thetak))*(1-s)) * cn;
  for( int i = 1; i < k; ++i ){
    assert( m_c.find(vn.v((vl+i)%val)) != m_c.end() );
    *m_c[vn.v((vl+i)%val)] += sin(vol*thetak)*(scale/k*sin(i*thetak)*(1-s)) * cn;
  }
  
  return 1-s;
}

template <class Triangle, class Vertex>
inline void
Diamond<Triangle, Vertex>::
LoopOdd( const QuadCoeffs& cn,
	 const OneRing<Triangle,Vertex>& vn, const Vertex *vv,
	 const Vertex *vb, const Vertex *vt,
	 // is the edge a crease edge
	 const bool ecrease,
	 // if this edge is not a crease it may be convex
	 const bool convex ) //
{
  if( ecrease ){
    const QuadCoeffs d = .5 * cn;
    assert( m_c.find(vn.c()) != m_c.end() );
    *m_c[vn.c()] += d;
    assert( m_c.find(vv) != m_c.end() );
    *m_c[vv] += d;
  }else{
    const int vo = vn.v(vv);
    Real gamma = .375;
    const Real s = vn.c()->Corner() ? CornerNeighbor( vn, vo, convex, cn, gamma, 1 ) :
      ( vn.c()->Crease() ? CreaseNeighbor( vn, vo, cn, gamma, 1 ) : 0 );

    const QuadCoeffs d = ((1-s)/8)*cn;
    assert( m_c.find(vb) != m_c.end() );
    *m_c[vb] += d;
    assert( m_c.find(vt) != m_c.end() );
    *m_c[vt] += d;
    assert( m_c.find(vn.c()) != m_c.end() );
    *m_c[vn.c()] += ((1-s)*(.75-gamma)) * cn;
    assert( m_c.find(vv) != m_c.end() );
    *m_c[vv] += ((1-s)*gamma) * cn;
  }
}

template <class Triangle, class Vertex>
inline void
Diamond<Triangle, Vertex>::
LoopOdd( QuadCoeffs c1[], const QuadCoeffs& cn,
	 const OneRing<Triangle,Vertex>& vn, const Vertex *vv,
	 const int l, const int r, const int b, const int t,
	 // is the edge a crease edge
	 const bool ecrease,
	 // if this edge is not a crease it may be convex
	 const bool convex ) //
{
  if( ecrease ){
    // standard cubic spline boundary rule for the odd guy
    const QuadCoeffs d = .5 * cn; c1[l] += d; c1[r] += d;
  }else{
    // else the usual (3,3,1,1)/8 stencil, BUT WITH CORRECTION if near a
    // corner or crease
    const int vo = vn.v(vv);
    Real gamma = .375;
    // get the correction if needed
    const Real s = vn.c()->Corner() ? CornerNeighbor( vn, vo, convex, cn, gamma, .5 ) :
      ( vn.c()->Crease() ? CreaseNeighbor( vn, vo, cn, gamma, .5 ) : 0 );

    const QuadCoeffs d = ((1-s)/8) * cn; c1[b] += d; c1[t] += d;
    c1[l] += ((1-s)*(.75-gamma)) * cn; c1[r] += ((1-s)*gamma) * cn;
  }
}

template <class Triangle, class Vertex>
inline void
Diamond<Triangle, Vertex>::
LoopOddNoCorrection( QuadCoeffs c1[], const QuadCoeffs& cn,
		     // indices of left, right, bottom, top and center
		     const int l, const int r, const int b, const int t,
		     // is the center guy a crease vertex?
		     const bool crease ) //
{
  // totally standard
  if( crease ){
    // odd boundary rule (1,1)/2
    const QuadCoeffs d = .5 * cn; c1[l] += d; c1[r] += d;
  }else{
    // standard odd rule (3,3,1,1)/8
    const QuadCoeffs e = .125 * cn; c1[b] += e; c1[t] += e;
    const QuadCoeffs d = .375 * cn; c1[l] += d; c1[r] += d;
  }
}

template <class Triangle, class Vertex>
inline void
Diamond<Triangle, Vertex>::
LoopOddNoCorrection( const QuadCoeffs& cn,
		     const Vertex *vl, const Vertex *vr,
		     const Vertex *vb, const Vertex *vt,
		     // is the center guy a crease vertex?
		     const bool crease ) //
{
  if( crease ){
    const QuadCoeffs d = .5 * cn;
    assert( m_c.find(vl) != m_c.end() );
    *m_c[vl] += d;
    assert( m_c.find(vr) != m_c.end() );
    *m_c[vr] += d;
  }else{
    const QuadCoeffs d = .375 * cn;
    assert( m_c.find(vl) != m_c.end() );
    *m_c[vl] += d;
    assert( m_c.find(vr) != m_c.end() );
    *m_c[vr] += d;
    const QuadCoeffs e = .125 * cn;
    assert( m_c.find(vb) != m_c.end() );
    *m_c[vb] += e;
    assert( m_c.find(vt) != m_c.end() );
    *m_c[vt] += e;
  }
}


template <class Triangle, class Vertex>
inline void
Diamond<Triangle, Vertex>::
LoopEven( const OneRing<Triangle,Vertex>& r, const Vertex *vo, const QuadCoeffs& c ) //
{
  // get all the neighbors in the one ring and add them up
  if( r.c()->Corner() ){
    assert( m_c.find(r.c()) != m_c.end() );
    *m_c[r.c()] += c;
  }else if( r.c()->Crease() ){
    int vl, vr;
    const int k = FindCreaseNeighbors( r, r.v(vo), vl, vr ); assert( k >= 1 );
    assert( m_c.find(r.c()) != m_c.end() );
    *m_c[r.c()] += .75 * c;
    const QuadCoeffs d = .125 * c;
    assert( m_c.find(r.v(vl)) != m_c.end() );
    *m_c[r.v(vl)] += d;
    assert( m_c.find(r.v(vr)) != m_c.end() );
    *m_c[r.v(vr)] += d;
  }else if( r.Valence() == 6 ){
    assert( m_c.find(r.c()) != m_c.end() );
    *m_c[r.c()] += .625 * c;
    const QuadCoeffs d = .0625 * c;
    for( int i = 0; i < 6; ++i ){
      assert( m_c.find(r.v(i)) != m_c.end() );
      *m_c[r.v(i)] += d;
    }
  }else{
    const int val = r.Valence();
    QuadCoeffs d;
    if( val == 3 ){
      assert( m_c.find(r.c()) != m_c.end() );
      *m_c[r.c()] += (7.0/16.) * c;
      d = (3.0/16.) * c;
    }else{
      assert( m_c.find(r.c()) != m_c.end() );
      *m_c[r.c()] += .625 * c;
      d = (3.0/(8.*val)) * c;
    }
    for( int i = 0; i < val; ++i ){
      assert( m_c.find(r.v(i)) != m_c.end() );
      *m_c[r.v(i)] += d;
    }
  }
}

template <class Triangle, class Vertex>
Diamond<Triangle, Vertex>::
Diamond( Triangle *t, EdgeName e )
  : m_bot( t ), m_top( 0 ), m_c()
{
  // this reads slightly weird since each QuadCoeffs looks like a row in
  // this matrix, but it really is an array of columns...0sorry.
  static const QuadCoeffs stencilsfull[7] =
  { QuadCoeffs( 6.0/12.,  0.0/3.,  0.0/sqrtf(3.),-32.,  0.0/sqrtf(3.),-96.0/3.0), // CNT
    QuadCoeffs( 1.0/12., -4.0/3.,  0.0/sqrtf(3.), 16.,  0.0/sqrtf(3.),-16.0/3.0), // VL
    QuadCoeffs( 1.0/12.,  4.0/3.,  0.0/sqrtf(3.), 16.,  0.0/sqrtf(3.),-16.0/3.0), // VR
    QuadCoeffs( 1.0/12., -2.0/3., -2.0/sqrtf(3.),  0., 16.0/sqrtf(3.), 32.0/3.0), // VBL
    QuadCoeffs( 1.0/12.,  2.0/3., -2.0/sqrtf(3.),  0.,-16.0/sqrtf(3.), 32.0/3.0), // VBR
    QuadCoeffs( 1.0/12., -2.0/3.,  2.0/sqrtf(3.),  0.,-16.0/sqrtf(3.), 32.0/3.0), // VTL
    QuadCoeffs( 1.0/12.,  2.0/3.,  2.0/sqrtf(3.),  0., 16.0/sqrtf(3.), 32.0/3.0)  // VTR
  };
  static const QuadCoeffs stencilsonesided[7] =
  { QuadCoeffs(  4.0/6.,  0.,  4.0/sqrtf(3.),-32.,  0.0/sqrtf(3.),-32.0/3.0), // CNT
    QuadCoeffs(  1.0/6., -2.,  2.0/sqrtf(3.), 16.,-16.0/sqrtf(3.), 16.0/3.0), // VL
    QuadCoeffs(  1.0/6.,  2.,  2.0/sqrtf(3.), 16., 16.0/sqrtf(3.), 16.0/3.0), // VR
    QuadCoeffs(      0.,  0., -4.0/sqrtf(3.),  0., 32.0/sqrtf(3.),  0.0/3.0), // VBL
    QuadCoeffs(      0.,  0., -4.0/sqrtf(3.),  0.,-32.0/sqrtf(3.),  0.0/3.0), // VBR
    QuadCoeffs(      0.,  0.,             0.,  0.,             0.,  0.0/3.0), // VTL
    QuadCoeffs(      0.,  0.,             0.,  0.,             0.,  0.0/3.0 )  // VTR
  };

  // intially the four rings are empty
  m_r[LEFT] = m_r[RIGHT] = m_r[BOTTOM] = m_r[TOP] = 0;
  
  // names of edges for later (remember: these are vertex names, so we
  // always choose the vertex name across from the edge we mean)
  // the 4 vertices
  Vertex *vl = m_bot->v(PrevV(VertexName(e))); // left
  Vertex *vr = m_bot->v(NextV(VertexName(e))); // right
  Vertex *vb = m_bot->v(VertexName(e)); assert( e == m_bot->v(vb) ); // bottom
  m_top = m_bot->t(e);
  Vertex *vt = m_top ? m_top->v(m_top->opposite(vl,vr)) : 0; // top (if there)

  // first 2 rings get loaded
  m_r[LEFT] = &vl->Ring(); m_r[RIGHT] = &vr->Ring();

  // only if the other 2 corners are Crease or Corner do we care
  // the top and bottom rings only matter if they are part of a crease or
  // corner. In the canonical case the support of the stencils will not
  // contain these. Only if the modified rules come into play...
  if( vb->CreaseOrCorner() ) m_r[BOTTOM] = &vb->Ring(); //

  // only get the top guy if this edge is not a crease and it exists and
  // the top vertex is either crease or corner. Otherwise we don't need it.
  if( !m_bot->Crease(e) && vt && vt->CreaseOrCorner() ){
    m_r[TOP] = &vt->Ring();
  }

  const QuadCoeffs *stencils;
  // we use only a half stencil if this is a crease or boundary (!vt)
  if( m_bot->Crease(e) || !vt ) stencils = stencilsonesided;
  else stencils = stencilsfull;

  // now create coefficient storage for all members of rings
  typedef std::pair<Vertex*, QuadCoeffs*> Pvq;
  typedef std::pair<std::map<const Vertex*,QuadCoeffs*>::iterator, bool> Pib;
  QuadCoeffs *q; Pib r;
  // we begin with the coeffs storage for the immediately incident
  // vertices: vl, vr, and vb
  r = m_c.insert( Pvq( vl, q = new QuadCoeffs() ) ); assert( q ); assert( r.second );
  r = m_c.insert( Pvq( vr, q = new QuadCoeffs() ) ); assert( q ); assert( r.second );
  r = m_c.insert( Pvq( vb, q = new QuadCoeffs() ) ); assert( q ); assert( r.second );
  // vt only gets coeff if it exists
  if( vt ){
    r = m_c.insert( Pvq( vt, q = new QuadCoeffs() ) ); assert( q ); assert( r.second );
  }
  // now: for all 4 rings (if they all exist) enter as many coeffs as there
  // are vertices in these rings
  for( int i = 0; i < 4; ++i ){
    if( m_r[i] ){
      const int val = m_r[i]->Valence();
      for( int j = 0; j < val; ++j ){
	r = m_c.insert( Pvq( m_r[i]->v(j), q = new QuadCoeffs() ) ); assert( q );
	// if already present throw away coeffs
	// this is so since some members of the different rings are
	// mentioned multiple times in the rings. For example, vt and vb
	// will both be in the rings for vr and vl; exercise: convince
	// yourself that there may at times be others that are also
	// contained in more than 1 ring... (this is certainly true if the
	// rings for vt and vb have to be taken into account, but can also
	// happen in circumstances where only vl and vr have a non-trivial
	// ring; why?)
	if( !r.second ) delete q;
      }
    }
  }

  // ok. ready for adjoint subdivision...
  // we begin with coeffs for the ring after 2 levels of subdivision. Thius
  // data is contained in the precomputed stencils pointer. The first round
  // of adjoint subdivision will lift these guys into the slots c1[]
  QuadCoeffs c1[7];
  c1[CNT] = c1[VL] = c1[VR] = c1[VBL] = c1[VBR] = c1[VTL] = c1[VTR] = QuadCoeffs(); //

  //
  // level 2 to level 1
  // 
  // First the even guy in the center at level 2
  LoopEven( c1, stencils[CNT], m_bot->Crease(e) ); //
  // now the 6 odd neighbors: bottom left, bottom right
  LoopOddNoCorrection( c1, stencils[VBL], VBL, CNT, VBR, VL, false ); //
  LoopOddNoCorrection( c1, stencils[VBR], CNT, VBR, VBL, VR, false ); //
  // if exists: top left, top right
  if( m_top ){
    LoopOddNoCorrection( c1, stencils[VTL], VTL, CNT, VL, VTR, false ); //
    LoopOddNoCorrection( c1, stencils[VTR], CNT, VTR, VR, VTL, false ); //
  }
  // only the outer guys on the middle edge are candidates for something
  // more complicated
  // left and right
  LoopOdd( c1, stencils[VL], *m_r[LEFT], m_r[RIGHT]->c(),
	   VL, CNT, VBL, VTL, m_bot->Crease(e), m_bot->Convex(e) ); //
  LoopOdd( c1, stencils[VR], *m_r[RIGHT], m_r[LEFT]->c(),
	   VR, CNT, VTR, VBR, m_bot->Crease(e), m_bot->Convex(e) ); //

  //
  // level 1 to level 0
  //
  // ok. final round; first the easy guys: left and right
  LoopEven( *m_r[LEFT], vr, c1[VL] ); //
  LoopEven( *m_r[RIGHT], vl, c1[VR] ); //

  // center guy
  // now it matters whether things are on a crease or a corner
  // if neither are we do the standard thing; if both are, we don't know
  // anything better to do than the standard thing either...
  if( vl->CreaseOrCorner() == vr->CreaseOrCorner() ){
    LoopOddNoCorrection( c1[CNT], vl, vr, vb, vt, m_bot->Crease(e) ); //
  }else if( vl->CreaseOrCorner() ){
    LoopOdd( c1[CNT], *m_r[LEFT], vr, vb, vt, m_bot->Crease(e), m_bot->Convex(e) ); //
  }else if( vr->CreaseOrCorner() ){
    // if the weird guy is on the right we rotate the stencil so the
    // function call to LoopOdd() which expects the order of arguments as
    // if the weird guy was on the left will work
    LoopOdd( c1[CNT], *m_r[RIGHT], vl, vt, vb, m_bot->Crease(e), m_bot->Convex(e) ); //
  }else die();

  // ok. 4 more odd guys (maybe)
  EdgeName el = EdgeName(NextV(VertexName(e))); assert( el == m_bot->v(vr) );
  const Vertex *v = m_bot->t(vr) ? m_bot->t(vr)->v(vl,vb) : 0;
  // bottom left guy; same spiel as above in terms of special flags; this
  // time though the crease and convex flags are with respect to the bottom
  // left edge, not the edge from left to right as above (which was called "e")
  if( vl->CreaseOrCorner() == vb->CreaseOrCorner() ){
    LoopOddNoCorrection( c1[VBL], vl, vb, v, vr, m_bot->Crease(el) ); //
  }else if( vl->CreaseOrCorner() ){
    LoopOdd( c1[VBL], *m_r[LEFT], vb, v, vr, m_bot->Crease(el), m_bot->Convex(el) ); //
  }else if( vb->CreaseOrCorner() ){
    LoopOdd( c1[VBL], *m_r[BOTTOM], vl, vr, v, m_bot->Crease(el), m_bot->Convex(el) ); //
  }else die();
  
  EdgeName er = EdgeName(PrevV(VertexName(e)));
  assert( er == m_bot->v(vl) );
  v = m_bot->t(vl) ? m_bot->t(vl)->v(vr,vb) : 0;
  // one more time for bottom right just as above
  if( vr->CreaseOrCorner() == vb->CreaseOrCorner() ){
    LoopOddNoCorrection( c1[VBR], vb, vr, v, vl, m_bot->Crease(er) ); //
  }else if( vr->CreaseOrCorner() ){
    LoopOdd( c1[VBR], *m_r[RIGHT], vb, vl, v, m_bot->Crease(er), m_bot->Convex(er) ); //
  }else if( vb->CreaseOrCorner() ){
    LoopOdd( c1[VBR], *m_r[BOTTOM], vr, v, vl, m_bot->Crease(er), m_bot->Convex(er) ); //
  }else die();

  // if the top exists we'll do the same for top left and top right just as above
  if( m_top ){
    v = m_top->t(vr) ? m_top->t(vr)->v(vl,vt) : 0;
    EdgeName etl = EdgeName(m_top->v(vr));
    if( vl->CreaseOrCorner() == vt->CreaseOrCorner() ){
      LoopOddNoCorrection( c1[VTL], vl, vt, v, vr, m_top->Crease(etl) ); //
    }else if( vl->CreaseOrCorner() ){
      LoopOdd( c1[VTL], *m_r[LEFT], vt, vr, v, m_top->Crease(etl), m_top->Convex(etl) ); //
    }else if( vt->CreaseOrCorner() ){
      LoopOdd( c1[VTL], *m_r[TOP], vl, v, vr, m_top->Crease(etl), m_top->Convex(etl) ); //
    }else die();
    
    v = m_top->t(vl) ? m_top->t(vl)->v(vr,vt) : 0;
    EdgeName etr = EdgeName(m_top->v(vl));
    if( vr->CreaseOrCorner() == vt->CreaseOrCorner() ){
      LoopOddNoCorrection( c1[VTR], vt, vr, vl, v, m_top->Crease(etr) ); //
    }else if( vr->CreaseOrCorner() ){
      LoopOdd( c1[VTR], *m_r[RIGHT], vt, v, vl, m_top->Crease(etr), m_top->Convex(etr) );
    }else if( vt->CreaseOrCorner() ){
      LoopOdd( c1[VTR], *m_r[TOP], vr, vl, v, m_top->Crease(etr), m_top->Convex(etr) );
    }else die();
  }
  // done. Yow.
}

template <class Triangle, class Vertex>
Diamond<Triangle, Vertex>::
~Diamond( void )
{
  std::map<const Vertex*, QuadCoeffs*>::iterator b = m_c.begin(), e = m_c.end();
  for( ; b != e; ++b ) delete (*b).second;
}
