// -*- Mode: c++ -*-
// $Id: ThinShellReader.h,v 1.1.1.1 2000/01/06 20:46:11 eitan Exp $
// $Source: /ufs/multires/projects/Coke/Quadrature/ThinShellReader.h,v $

#ifndef	__THINSHELLREADER_H__
#define	__THINSHELLREADER_H__

// Do not change anything above this line

// include files

#include "Mesh/Labels.h"
#include "Mesh/SMF.h"

// class definition
template <class Mesh>
class ThinShellReader : public SMF::MeshReader<Mesh> {
 public:
  ThinShellReader(void) : SMF::MeshReader<Mesh>() { }

  ~ThinShellReader(void) { }

  void readMesh(istream& is, Mesh& mesh) {
    SMF::MeshReader<Mesh>::readMesh(is, mesh);

    readTags(is, mesh);
  }
 protected:
 private:
  struct MarkBoundaries : public std::unary_function<Mesh::Triangle*, void>{
    void operator()( Mesh::Triangle* t ) {
      if( t->boundary(E1) ){
	t->setCrease(E1);
	t->v2()->setCrease(); t->v2()->setBoundary();
	t->v3()->setCrease(); t->v3()->setBoundary();
      }
      if( t->boundary(E2) ){
	t->setCrease(E2);
	t->v3()->setCrease(); t->v3()->setBoundary();
	t->v1()->setCrease(); t->v1()->setBoundary();
      }
      if( t->boundary(E3) ){
	t->setCrease(E3);
	t->v1()->setCrease(); t->v1()->setBoundary();
	t->v2()->setCrease(); t->v2()->setBoundary();
      }
    }
  };

  void readTags( istream& is, Mesh& mesh ){
    Assembler<Mesh::Triangle, Mesh::Vertex>& as = *m_assembler;
    
    // set all crease tags on boundary edges and boundary vertices
    std::for_each( mesh.triangleBegin(), mesh.triangleEnd(), MarkBoundaries() );

    vector<int> new_corners; 

      // look for tag stream delimiter
    char check;
    if ( !( is >> check ) || ( check != '#' ) ) return;

    int crsize =0, ccvsize =0, cornsize =0;
    if( !( is >> crsize >> ccvsize >> cornsize ) ) return;

    // read crease tags
    // pairs of integers referring to vertices; retrieve vertex pointers and
    // walk around one of the endpoints to find the other endpoint; set
    // crease bit on that edge for both triangles incident on the edge
    // note that we simultaneously check for infered corners, which are
    // vertices with more than 2 other creases adjacent
    int i;
    for (i = 0; i < crsize; ++i) {
      int v1n = -1, v2n = -1;
      if ( !(is >> v1n >> v2n) ) return;
      Mesh::Vertex *v1 = as.getVertex(v1n), *v2 = as.getVertex(v2n);
    
      // crease counter
      int crCount = 0;
      Mesh::Vertex::TriangleIt ti = v1->triangleBegin(), te = v1->triangleEnd();
      for( ; ti != te; ++ti ) {
	Mesh::Triangle *t = *ti;
	//look for triangles to tag edges
	if ( t->contains( v2 ) ){
	  t->setCrease( t->edge(v1,v2) );
	  Mesh::Triangle *ta = t->t(v1,v2);
	  // zjw - important, tri maybe be null because hole
	  if( ta ) ta->setCrease( ta->edge(v1,v2) );
	}
	// corner check must look at all incoming edges and count # of creases
	EdgeName e1 = EdgeName(t->v(v1));
	crCount += t->Crease( NextE(e1) ) + t->Crease( PrevE(e1) );
      }
      // infer a crease or corner vertex
      crCount = ( crCount + ( v1->isBoundaryVertex() << 1 ) ) >> 1;
      if( crCount == 2 ) v1->setCrease();
      else if( crCount > 2 ){ v1->setCorner(); new_corners.push_back(v1n); }

      // we also have to check all triangles incident on v2
      // in case we just added a second or third crease to v2
      crCount = 0;
      ti = v2->triangleBegin(); te = v2->triangleEnd();
      for( ; ti != te; ++ti ) {
	Mesh::Triangle *t = *ti;
	EdgeName e1 = EdgeName(t->v(v2));
	crCount += t->Crease( NextE(e1) ) + t->Crease( PrevE(e1) );
      }
      // infer a crease or corner vertex
      crCount = ( crCount + ( v2->isBoundaryVertex() << 1 ) ) >> 1;
      if( crCount == 2 ) v2->setCrease();
      else if( crCount > 2 ){ v2->setCorner(); new_corners.push_back(v2n); }
    } 

    // read concave edge/vertex tags. (convex corners are the default and
    // thus appear below in the corner tags).  there are two integers, 
    // the first one is the corner; retrieve vertex pointers; set
    // corner tag at first vertex, then walk around it, looking for the other
    // vertex. set concave bit on that edge for both
    // adjacent triangles. verify that the edge is not of crease type
    for (i = 0; i < ccvsize; ++i) {
      int v1n = -1, v2n = -1;
      if ( !(is >> v1n >> v2n) ) return;
      Mesh::Vertex *v1 = as.getVertex(v1n), *v2 = as.getVertex(v2n);

      v1->setCorner();
      Mesh::Vertex::TriangleIt ti = v1->triangleBegin(), te = v1->triangleEnd();
      for( ; ti != te; ++ti ) {
	Mesh::Triangle *t = *ti;
	if ( t->contains(v2) ) {
	  t->setConcave( t->edge(v1,v2) );
	  Mesh::Triangle *ta = t->t( v1, v2 );
	  if( ta ) ta->setConcave( ta->edge(v1,v2) );
	  break;
	}
      }
    }

    // read all corner tags - this includes all convex corners and boundary
    // corners of valance 2.  set corner tag, then walk around it....
    // set all adjacent edges to convex unless they are crease or concave
    for (i = 0; i < cornsize; ++i) {
      int v1n = -1;
      if ( !(is >> v1n) ) return;
      Mesh::Vertex *v1 = as.getVertex(v1n);

      v1->setCorner(); 
      Mesh::Vertex::TriangleIt ti = v1->triangleBegin(), te = v1->triangleEnd();
      for( ; ti != te; ++ti ) {
	Mesh::Triangle *t = *ti;
	// set other edges as convex
	EdgeName en = EdgeName(t->v(v1));
	if ( !t->Crease( NextE(en) ) && !t->Concave( NextE(en) ) ) t->setConvex( NextE(en) );
	if ( !t->Crease( PrevE(en) ) && !t->Concave( PrevE(en) ) ) t->setConvex( PrevE(en) );
      }
    }
  
    // we also have to deal with tagging the corners we inferred from
    // reading the crease tags
    vector<int>::iterator vi = new_corners.begin(),  ve = new_corners.end();
    for( ; vi != ve; ++vi) {
      Mesh::Vertex *vc = as.getVertex(*vi);
      Mesh::Vertex::TriangleIt ti = vc->triangleBegin(), te = vc->triangleEnd();
      for( ; ti != te; ++ti ) {
	Mesh::Triangle *t = *ti;
	//set other edges as convex
	EdgeName en = EdgeName(t->v( vc ));
	if ( !t->Crease( NextE(en) ) && !t->Concave( NextE(en) ) ) t->setConvex( NextE(en) );
	if ( !t->Crease( PrevE(en) ) && !t->Concave( PrevE(en) ) ) t->setConvex( PrevE(en) );
      }
    }
  }
};

// extern defs for non-member outline functions go here
// externally visible constant defs go here

#endif	/* __THINSHELLREADER_H__ */
