// -*- Mode: c++ -*-
// $Id: lumps.cc,v 1.1.1.1 2000/01/06 20:46:13 eitan Exp $
// $Source: /ufs/multires/projects/Coke/Quadrature/test/lumps.cc,v $

#include <iostream.h>
#include <vector.h>

#include "FEMMesh.h"
#include "FEMTriangle.h"
#include "ShellDOF.h"

MESHTYPE(Mesh, FEMMesh, FEMTriangle, ShellDOF)

#if 0
class BuildMesh {
public:
  template <class Mesh>
  Mesh::Triangle *operator()(Mesh &mesh) {
    Assembler<Mesh::Triangle, Mesh::Vertex> a(mesh);
    Mesh::Triangle *centerTriangle = NULL;

    { // vertex 0
      Mesh::Vertex* v = new Mesh::Vertex();
      assert(v);
      v->point() = CVec(0,0,basisMask[0]);
      a.addVertex(v);
    }

    { // triangle 0
      Mesh::Triangle *t = new Mesh::Triangle;
      assert(t);
      t->setVertices( a.getVertex( 0 ), 
		      a.getVertex( 1 ), 
		      a.getVertex( N ));
      a.addTriangle(t);
    }

    assert(a.assembleMesh());

    // NJL: this belongs in an assembler object
    // EG: I agree! Please put it there.
    Mesh::VertexIt vi = mesh.vertexBegin();
    Mesh::VertexIt ve = mesh.vertexEnd();

    for (; vi != ve; ++vi) {
      Mesh::Vertex* vertex = *vi;
      vertex->vertex()->point(0) = vertex->point();
      vertex->vertex()->setValid(0);
    }

    return centerTriangle;
  }
};
#endif

#include <Inventor/SoInput.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoIndexedFaceSet.h>
#include <Inventor/nodes/SoIndexedLineSet.h>
#include <Inventor/nodes/SoComplexity.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoLineSet.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/actions/SoWriteAction.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoTransformSeparator.h>
#include <Inventor/nodes/SoSphere.h>


const float INF = 1e9;

class BoundingBox {
public:
  BoundingBox() : m_bmin(INF,INF,INF), m_bmax(-INF,-INF,-INF) {}

  void Clear() {
    m_bmin.x() = INF;
    m_bmin.y() = INF;
    m_bmin.z() = INF;
    m_bmax.x() = -INF;
    m_bmax.y() = -INF;
    m_bmax.z() = -INF;
  }

  void AddBox(BoundingBox &b) {
    assignMax(m_bmax, b.m_bmax);
    assignMin(m_bmin, b.m_bmin);
  }

  void AddPoint(CVec &p) {
    assignMax(m_bmax, p);
    assignMin(m_bmin, p);
  }

  bool Overlaps(BoundingBox &b) {
    if (m_bmax.x() < b.m_bmin.x()) return false;
    if (m_bmax.y() < b.m_bmin.y()) return false;
    if (m_bmax.z() < b.m_bmin.z()) return false;
    if (b.m_bmax.x() < m_bmin.x()) return false;
    if (b.m_bmax.y() < m_bmin.y()) return false;
    if (b.m_bmax.z() < m_bmin.z()) return false;
    return true;
  }

  bool Contains(BoundingBox &b) {
    if (m_bmax.x() < b.m_bmax.x()) return false;
    if (m_bmax.y() < b.m_bmax.y()) return false;
    if (m_bmax.z() < b.m_bmax.z()) return false;
    if (m_bmin.x() > b.m_bmin.x()) return false;
    if (m_bmin.y() > b.m_bmin.y()) return false;
    if (m_bmin.z() > b.m_bmin.z()) return false;
    return true;
  }

  CVec GetMaxExtent() {
    return m_bmax;
  }

  CVec GetMinExtent() {
    return m_bmin;
  }

private:
  CVec m_bmin, m_bmax;

  void assignMax(CVec &tgt, CVec &src) {
    tgt.x() = max(tgt.x(), src.x());
    tgt.y() = max(tgt.y(), src.y());
    tgt.z() = max(tgt.z(), src.z());
  }

  void assignMin(CVec &tgt, CVec &src) {
    tgt.x() = min(tgt.x(), src.x());
    tgt.y() = min(tgt.y(), src.y());
    tgt.z() = min(tgt.z(), src.z());
  }
};


static void WriteIvFile( SoSeparator* sep, const char* filename = NULL )
{
  SoWriteAction myAction;

  if( filename )
    myAction.getOutput()->openFile( filename );
  myAction.getOutput()->setBinary(FALSE);
  myAction.apply( sep );
  myAction.getOutput()->closeFile();
}

template <class T>
class hash<T *> {
public:
  size_t operator()(const T *v) const {
    return hash<char*>()((char*)v);
  }
};

void ProduceInventorFile(Mesh &mesh) {
  typedef Mesh::Vertex *VertexPtr_t;
  hash_map<Diamond<Mesh::Triangle,Mesh::Vertex> *, bool> diamondProcessed;
  Mesh::TriangleIt t, te;

  SoDB::init();

  SoSeparator *sep = new SoSeparator;
  sep->ref();

  SoComplexity *complexity = new SoComplexity;
  sep->addChild( complexity );
  complexity->type = SoComplexity::OBJECT_SPACE;
  complexity->value = 0;

  SoMaterial *surfaceMaterial = new SoMaterial;
  sep->addChild( surfaceMaterial );
  surfaceMaterial->ambientColor.setValue( 0.5, 0.5, 0.7 );
  surfaceMaterial->diffuseColor.setValue( 0.5, 0.5, 0.7 );
  surfaceMaterial->specularColor.setValue( 0.5, 0.5, 0.7 );

  SoCoordinate3* coord = new SoCoordinate3;
  coord->point.deleteValues( 0 );
  sep->addChild( coord );
  
  SoIndexedFaceSet* faceset = new SoIndexedFaceSet;
  faceset->materialIndex.deleteValues( 0 );
  faceset->coordIndex.deleteValues( 0 );
  sep->addChild( faceset );

  SoMaterial *sphereMaterial = new SoMaterial;
  sep->addChild( sphereMaterial );
  sphereMaterial->ambientColor.setValue( 1.0, 1.0, 0.1 );
  sphereMaterial->diffuseColor.setValue( 1.0, 1.0, 0.1 );
  sphereMaterial->specularColor.setValue( 1.0, 1.0, 0.1 );

  SoSeparator *sphereSep = new SoSeparator;
  sep->addChild( sphereSep );

  SoMaterial *vectorMaterial = new SoMaterial;
  sep->addChild( vectorMaterial );
  vectorMaterial->ambientColor.setValue( 0.8, 0.8, 1.0 );
  vectorMaterial->diffuseColor.setValue( 0.8, 0.8, 1.0 );
  vectorMaterial->specularColor.setValue( 0.8, 0.8, 1.0 );


  SoCoordinate3* linesetcoord = new SoCoordinate3;
  linesetcoord->point.deleteValues( 0 );
  sep->addChild( linesetcoord );

  SoLineSet* lineset = new SoLineSet;
  lineset->numVertices.deleteValues( 0 );
  lineset->vertexProperty = NULL;
  sep->addChild( lineset );

  BoundingBox box;

  for (t = mesh.triangleBegin(), te = mesh.triangleEnd();
       t != te; ++t) {
    for (int edgeNum=E1; edgeNum<=E3; ++edgeNum) {
      CVec pos = (*t)->getDiamond(EdgeLabel(edgeNum))->Evalp();
      box.AddPoint(pos);
    }
  }

  float ZExtent = box.GetMaxExtent().z() - box.GetMinExtent().z();
  float ZThreshold = box.GetMinExtent().z() + ZExtent/5.0;
  int idx = 0;

  for (t = mesh.triangleBegin(), te = mesh.triangleEnd();
       t != te; ++t) {
    for (int edgeNum=E1; edgeNum<=E3; ++edgeNum) {
      Diamond<Mesh::Triangle,Mesh::Vertex> *diamond = (*t)->getDiamond(EdgeLabel(edgeNum));

      if (diamondProcessed[diamond]) continue;
      diamondProcessed[diamond] = true;

      CVec pos = diamond->Evalp();

      //      if (pos.z() > ZThreshold) continue;

      SoTransformSeparator *sphereTSep = new SoTransformSeparator();
      SoTransform *sphereTrans = new SoTransform();
      SoSphere* sphere = new SoSphere();
      
      sphereTSep->addChild( sphereTrans );
      sphereTSep->addChild( sphere );
      sphereSep->addChild( sphereTSep );
      
      sphereTrans->translation.setValue( pos );
      sphere->radius = 0.05;

      CVec du = diamond->Evaldp1();
      CVec dv = diamond->Evaldp2();
      CVec n = du.cross(dv);
      n.normalize();

      lineset->numVertices.set1Value( idx/2, 2 );
      linesetcoord->point.set1Value( idx, pos - 0.1*n );
      ++idx;
      linesetcoord->point.set1Value( idx, pos + 0.1*n );
      ++idx;
    }
  }

  WriteIvFile( sep );
}



void
main(int , char* [])
{
  Mesh mesh;

  cin >> mesh;
  Mesh::VertexIt vb = mesh.vertexBegin(), ve = mesh.vertexEnd();
  for( ; vb != ve; ++vb ) (*vb)->InitOneRing();
  Mesh::TriangleIt tb = mesh.triangleBegin(), te = mesh.triangleEnd();
  for( ; tb != te; ++tb ) (*tb)->AttachDiamonds();
  tb = mesh.triangleBegin();
  //  for( ; tb != te; ++tb ) cout << **tb;
  ProduceInventorFile(mesh);
}






#if 0
  SoLineSet* lineset = new SoLineSet;
  SoCoordinate3* linesetcoord = new SoCoordinate3;
  linesetcoord->point.deleteValues( 0 );
  lineset->numVertices.deleteValues( 0 );
  lineset->vertexProperty = linesetcoord;
  sep->addChild( lineset );
#endif

#if 0
  for (v = mesh.vertexBegin(), ve = mesh.vertexEnd();
       v != ve; ++v) {
    int idx = coord->point.getNum();
    vIdx[*v] = idx;
    coord->point.set1Value( idx, (*v)->point() );
  }

  for (t = mesh.triangleBegin(), te = mesh.triangleEnd();
       t != te; ++t) {
    faceset->coordIndex.set1Value( faceset->coordIndex.getNum(), vIdx[(*t)->vertex(V1)] );
    faceset->coordIndex.set1Value( faceset->coordIndex.getNum(), vIdx[(*t)->vertex(V2)] );
    faceset->coordIndex.set1Value( faceset->coordIndex.getNum(), vIdx[(*t)->vertex(V3)] );
    faceset->coordIndex.set1Value( faceset->coordIndex.getNum(), -1 );
  }
#endif

#if 0
  for (v = mesh.vertexBegin(), ve = mesh.vertexEnd();
       v != ve; ++v) {
    SoTransformSeparator *sphereSep = new SoTransformSeparator();
    SoTransform *sphereTrans = new SoTransform();
    SoSphere* sphere = new SoSphere();
    assert(sphereSep);
    assert(sphereTrans);
    assert(sphere);

    sphereSep->addChild( sphereTrans );
    sphereSep->addChild( sphere );
    sep->addChild( sphereSep );

    sphereTrans->translation.setValue( (*v)->point() );
    sphere->radius = 0.03;
  }
#endif



#if 0
      linesetcoord->point.set1Value( idx, pos );
      ++idx;
      linesetcoord->point.set1Value( idx,  );
      ++idx;
#endif
