#include <assert.h>
#include <iostream.h>
#include <fstream.h>
#include <ctype.h>

#include <Inventor/SoInput.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoIndexedFaceSet.h>
#include <Inventor/nodes/SoIndexedLineSet.h>
#include <Inventor/nodes/SoCamera.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoLineSet.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/actions/SoWriteAction.h>


const int SimYELLOW = 0;
const int SimRED    = 1;
const int SimGREEN  = 2;
const int SimBLUE   = 3;
 
template <class MESH>
static void WriteIvFile(MESH &mesh, const char *filename) {
  SoWriteAction myAction;
  static bool first = true;

  assert(filename);

  SoSeparator *sep = InventorSceneGraph(mesh);

  while (!myAction.getOutput()->openFile( filename )) {
    cerr << "Error opening " << filename << " (sleeping 10 seconds)" << endl;
    sleep(10);
  }
  
  myAction.getOutput()->setBinary(TRUE);
  myAction.apply( sep );
  myAction.getOutput()->closeFile();

  if (first) {
    first = false;
    WriteIvFile(mesh, filename);
  }
}

template <class T>
class hash<T *> {
public:
  size_t operator()(const T *v) const {
    return hash<char*>()((char*)v);
  }
};

template <class MESH>
SoSeparator *InventorSceneGraph(MESH &mesh) {
  hash_map<MESH::Vertex*, int> vIdx;
  MESH::TriangleIt t, te;
  MESH::VertexIt v, ve;

  static SoSeparator *sep = NULL;
  //static SoCamera *camera = NULL;
  static SoMaterial* Cmaterial = NULL;
  static SoCoordinate3* Ccoord = NULL;
  static SoIndexedFaceSet* Cfaceset = NULL;

#if 0
  static SoMaterial* Rmaterial = NULL;
  static SoCoordinate3* Rcoord = NULL;
  static SoIndexedFaceSet* Rfaceset = NULL;
  static SoMaterial* lMaterial = NULL;
  static SoCoordinate3* linesetcoord = NULL;
  static SoLineSet* lineset = NULL;
#endif

  if (!sep) {

    SoDB::init();

    sep = new SoSeparator;
    assert(sep);
    sep->ref();

#if 0
    camera = new SoCamera;
    assert(camera);
    sep->addChild( camera );
#endif

    Cmaterial = new SoMaterial;
    assert(Cmaterial);
    Cmaterial->ambientColor.deleteValues( 0 );
    Cmaterial->diffuseColor.deleteValues( 0 );

    Cmaterial->diffuseColor.set1Value(SimYELLOW, 0.7, 0.7, 0.4); // 0 = yellow
    Cmaterial->ambientColor.set1Value(SimYELLOW, 0.7, 0.7, 0.4);

    Cmaterial->diffuseColor.set1Value(SimRED, 1.0, 0.3, 0.3); // 1 = red
    Cmaterial->ambientColor.set1Value(SimRED, 1.0, 0.3, 0.3);

    Cmaterial->diffuseColor.set1Value(SimGREEN, 0.3, 1.0, 0.3); // 2 = green
    Cmaterial->ambientColor.set1Value(SimGREEN, 0.3, 1.0, 0.3);

    Cmaterial->diffuseColor.set1Value(SimBLUE, 0.3, 0.3, 1.0); // 3 = blue
    Cmaterial->ambientColor.set1Value(SimBLUE, 0.3, 0.3, 1.0);

    sep->addChild( Cmaterial );

    SoMaterialBinding* CmaterialBinding = new SoMaterialBinding;
    CmaterialBinding->value = SoMaterialBinding::PER_FACE_INDEXED;
    sep->addChild( CmaterialBinding );

    Ccoord = new SoCoordinate3;
    assert(Ccoord);
    sep->addChild( Ccoord );
  
    Cfaceset = new SoIndexedFaceSet;
    assert(Cfaceset);
    sep->addChild( Cfaceset );

#if 0
    Rmaterial = new SoMaterial;
    assert(Rmaterial);
    Rmaterial->diffuseColor.setValue(0.0, 1.0, 0.2);
    Rmaterial->ambientColor.setValue(0.0, 1.0, 0.2);
    sep->addChild( Rmaterial );

    Rcoord = new SoCoordinate3;
    assert(Rcoord);
    sep->addChild( Rcoord );
  
    Rfaceset = new SoIndexedFaceSet;
    assert(Rfaceset);
    sep->addChild( Rfaceset );

    lMaterial = new SoMaterial;
    assert(lMaterial);
    lMaterial->diffuseColor.setValue(1.0, 0.0, 0.0);
    lMaterial->ambientColor.setValue(1.0, 0.0, 0.0);
    sep->addChild( lMaterial );

    linesetcoord = new SoCoordinate3;
    assert(linesetcoord);
    sep->addChild( linesetcoord );

    lineset = new SoLineSet;
    assert(lineset);
    lineset->vertexProperty = NULL;
    sep->addChild( lineset );
#endif
  }

  Ccoord->point.deleteValues( 0 );
  Cfaceset->coordIndex.deleteValues( 0 );
  Cfaceset->materialIndex.deleteValues( 0 );

#if 0
  Rcoord->point.deleteValues( 0 );
  Rfaceset->coordIndex.deleteValues( 0 );

  linesetcoord->point.deleteValues( 0 );
  lineset->numVertices.deleteValues( 0 );  
#endif

  for (v = mesh.vertexBegin(), ve = mesh.vertexEnd();
       v != ve; ++v) {
    int idx = Ccoord->point.getNum();
    vIdx[*v] = idx;
    Real3Vec xC = (*v)->xC();
    //Real3Vec xR = (*v)->xR();
    //    xC += 2.0 * (xC - xR);
    Ccoord->point.set1Value( idx, xC.x(), xC.y(), xC.z());
  }

  for (t = mesh.triangleBegin(), te = mesh.triangleEnd();
       t != te; ++t) {
    int color = ((*t)->GetLeafPatch() != NULL) ? (*t)->GetLeafPatch()->Color() : 0;
    Cfaceset->materialIndex.set1Value( Cfaceset->materialIndex.getNum(), color);

#if 0
    if (color) {
      cerr << (*t)->vertex(V1)->xC() << " -- ";
      cerr << (*t)->vertex(V2)->xC() << " -- ";
      cerr << (*t)->vertex(V3)->xC() << endl;
      mustWrite = true;
    }
#endif

    Cfaceset->coordIndex.set1Value( Cfaceset->coordIndex.getNum(), vIdx[(*t)->vertex(V1)] );
    Cfaceset->coordIndex.set1Value( Cfaceset->coordIndex.getNum(), vIdx[(*t)->vertex(V2)] );
    Cfaceset->coordIndex.set1Value( Cfaceset->coordIndex.getNum(), vIdx[(*t)->vertex(V3)] );
    Cfaceset->coordIndex.set1Value( Cfaceset->coordIndex.getNum(), -1 );
  }

#if 0
  for (v = mesh.vertexBegin(), ve = mesh.vertexEnd();
       v != ve; ++v) {
    int idx = Rcoord->point.getNum();
    vIdx[*v] = idx;
    Rcoord->point.set1Value( idx, (*v)->xR().x(), (*v)->xR().y(), (*v)->xR().z());
  }

  for (t = mesh.triangleBegin(), te = mesh.triangleEnd();
       t != te; ++t) {
    Rfaceset->coordIndex.set1Value( Rfaceset->coordIndex.getNum(), vIdx[(*t)->vertex(V1)] );
    Rfaceset->coordIndex.set1Value( Rfaceset->coordIndex.getNum(), vIdx[(*t)->vertex(V2)] );
    Rfaceset->coordIndex.set1Value( Rfaceset->coordIndex.getNum(), vIdx[(*t)->vertex(V3)] );
    Rfaceset->coordIndex.set1Value( Rfaceset->coordIndex.getNum(), -1 );
  }
#endif

#if 0
  int idx = 0;
  static Real maxF = 0.0;

  for (v = mesh.vertexBegin(), ve = mesh.vertexEnd();
       v != ve; ++v) {    
    Real3Vec a = (*v)->x2C();
    maxF = max(maxF, a.l2());
  }
 
  for (v = mesh.vertexBegin(), ve = mesh.vertexEnd();
       v != ve; ++v) {    
    lineset->numVertices.set1Value( idx/2, 2 );
    MESH::Vertex *neighbour = *((*v)->vertexBegin());
    Real3Vec a = (*v)->x2C();
    a /= maxF;
    assert(neighbour);
    a *= (neighbour->point() - (*v)->point()).l2() * 0.5;
    linesetcoord->point.set1Value( idx, 
				   (*v)->xC().x(),
				   (*v)->xC().y(),
				   (*v)->xC().z());
    ++idx;
    linesetcoord->point.set1Value( idx, 
				   (*v)->xC().x() + a.x(),
				   (*v)->xC().y() + a.y(),
				   (*v)->xC().z() + a.z());
    ++idx;
  }
#endif

  //  camera->viewAll(sep, 
    
  return sep;
}


#if 0
    // Move the force slightly away from the vertex
    Real3Vec perp = (a.l2() > 0.0) ? a.perp() : 0.0;
    perp.normalize();
    perp *= (neighbour->point() - (*v)->point()).l2();
#endif



