#include <math.h>

#include "Util/Real.h"



void sa3der0 (Real a3d0[3], Real a1x, Real a1y, Real a1z, Real a2x,
	      Real a2y, Real a2z, Real a3x, Real a3y, Real a3z, 
	      Real a11x, Real a11y, Real a11z, Real a22x, Real a22y,
	      Real a22z, Real a12x, Real a12y, Real a12z, Real a21x,
	      Real a21y, Real a21z)
/* director derivativ with respect to theta^0  */
{
  Real t1, t2, t7, t8, t10, t15, t23, t24, t55, t56;

  a22x = a22x;
  a22y = a22y;
  a22z = a22z;
  a12x = a12x;
  a12y = a12y;
  a12z = a12z;

  t1 = a1y*a1y;    t2 = a2z*a2z;    t7 = a1z*a1z;
  t8 = a2y*a2y;    t10 = a2x*a2x;   t15 = a1x*a1x;
  t23 = sqrt(t1*t2-2.0*a1y*a2z*a1z*a2y+t7*t8+t7*t10-2.0*a1z*a2x*a1x*a2z+t15
	     *t2+t15*t8-2.0*a1x*a2y*a1y*a2x+t1*t10);
  t24 = 1.0/t23;
  t55 = a3x*a11y*a2z-a3x*a11z*a2y+a3x*a1y*a21z-a3x*a1z*a21y+a3y*a11z*a2x-
    a3y*a11x*a2z+a3y*a1z*a21x-a3y*a1x*a21z+a3z*a11x*a2y-a3z*a11y*a2x+a3z*a1x*a21y-
    a3z*a1y*a21x;
  t56 = t55*t24;

  a3d0[0] = t24*(a11y*a2z-a11z*a2y+a1y*a21z-a1z*a21y)-t56*a3x;
  a3d0[1] = t24*(a11z*a2x-a11x*a2z+a1z*a21x-a1x*a21z)-t56*a3y;
  a3d0[2] = t24*(a11x*a2y-a11y*a2x+a1x*a21y-a1y*a21x)-t56*a3z;
  
  return;
}



void sa3der1 (Real a3d1[3], Real a1x, Real a1y, Real a1z, Real a2x,
	      Real a2y, Real a2z, Real a3x, Real a3y, Real a3z, 
	      Real a11x, Real a11y, Real a11z, Real a22x, Real a22y,
	      Real a22z, Real a12x, Real a12y, Real a12z, Real a21x,
	      Real a21y, Real a21z)
/* director derivativ with respect to theta^1  */
{
  Real t1, t2, t7, t8, t10, t15, t23, t24, t55, t56;

  a11x = a11x;
  a11y = a11y;
  a11z = a11z;

  a21x = a21x;
  a21y = a21y;
  a21z = a21z;

  t1 = a1y*a1y;    t2 = a2z*a2z;    t7 = a1z*a1z;
  t8 = a2y*a2y;    t10 = a2x*a2x;   t15 = a1x*a1x;
  t23 = sqrt(t1*t2-2.0*a1y*a2z*a1z*a2y+t7*t8+t7*t10-2.0*a1z*a2x*a1x*a2z+t15
	     *t2+t15*t8-2.0*a1x*a2y*a1y*a2x+t1*t10);
  t24 = 1.0/t23;
  t55 = a3x*a12y*a2z-a3x*a12z*a2y+a3x*a1y*a22z-a3x*a1z*a22y+a3y*a12z*a2x-
    a3y*a12x*a2z+a3y*a1z*a22x-a3y*a1x*a22z+a3z*a12x*a2y-a3z*a12y*a2x+a3z*a1x*a22y-
    a3z*a1y*a22x;
  t56 = t55*t24;

  a3d1[0] = t24*(a12y*a2z-a12z*a2y+a1y*a22z-a1z*a22y)-t56*a3x;
  a3d1[1] = t24*(a12z*a2x-a12x*a2z+a1z*a22x-a1x*a22z)-t56*a3y;
  a3d1[2] = t24*(a12x*a2y-a12y*a2x+a1x*a22y-a1y*a22x)-t56*a3z;

  return;
}


void sa3der (Real a3d[2][3], Real akovr[3][3], Real akovr2[3][3])
/* director derivativs  */
{

  sa3der0 (a3d[0], akovr[0][0], akovr[0][1], akovr[0][2], akovr[1][0],
	   akovr[1][1], akovr[1][2], akovr[2][0], akovr[2][1], akovr[2][2],
	   akovr2[0][0], akovr2[0][1], akovr2[0][2], akovr2[1][0],
	   akovr2[1][1], akovr2[1][2], akovr2[2][0], akovr2[2][1], 
	   akovr2[2][2], akovr2[2][0], akovr2[2][1], akovr2[2][2]);
 	   
  sa3der1 (a3d[1], akovr[0][0], akovr[0][1], akovr[0][2], akovr[1][0],
	   akovr[1][1], akovr[1][2], akovr[2][0], akovr[2][1], akovr[2][2],
	   akovr2[0][0], akovr2[0][1], akovr2[0][2], akovr2[1][0],
	   akovr2[1][1], akovr2[1][2], akovr2[2][0], akovr2[2][1], 
	   akovr2[2][2], akovr2[2][0], akovr2[2][1], akovr2[2][2]);

  return;
}

