#ifndef __SIMMASS_H
#define __SIMMASS_H

#include "Util/forall.h"
#include "SimTypes.h"

template <class T>
class SimMass {
public:
  DECLARE_TYPES(T)

  static void Initialize(Mesh &mesh) {
    const Real rho = Parameters::rho;

    // Zero the mask of every vertex

    forall(Mesh::VertexIt, v, mesh.vertexBegin(), mesh.vertexEnd())
      m(v) = 0;
    endforall;

    forall(Mesh::GaussPointIt, p, mesh.gaussPointBegin(), mesh.gaussPointEnd())

      forall(GaussPoint::SupportIt, s1, p->supportBegin(), p->supportEnd())
      
        Vertex *v1 = (Vertex*) s1.first;
        Real w1 = s1.second->p();

	m(v1) += rho * w1 * p->JacobianR() * p->hR() * p->W();

      endforall;
    endforall;

    // For every gauss point, for every pair of vertices "v_i" and "v_j" in the support of
    // the gauss point, compute the contribution, rho * N_i * N_j * J * h, of the pair
    // to the mass of v_i.

#if 0
    forall(Mesh::GaussPointIt, p, mesh.gaussPointBegin(), mesh.gaussPointEnd())

      forall(GaussPoint::SupportIt, s1, p->supportBegin(), p->supportEnd())
      
        Vertex *v1 = (Vertex*) s1.first;
        Real w1 = s1.second->p();

        forall(GaussPoint::SupportIt, s2, p->supportBegin(), p->supportEnd())

	  Real w2 = s2.second->p();

	  m(v1) += rho * w1 * w2 * p->JacobianR() * p->hR() * p->W();

        endforall;
      endforall;
    endforall;
#endif
  } 

  static void Update(Mesh &mesh) {
  }

  static Real m(const SimVertex<T> *v) {
    return v->massData.m_m;
  }

  static void SetM(SimVertex<T> *v, Real mass) {
    m(v) = mass;
  }

private:
  static Real &m(SimVertex<T> *v) {
    return v->massData.m_m;
  }

  Real m_m;
};

#endif










