#ifndef __SIMMESH_H
#define __SIMMESH_H

#include "Quadrature/FEMMesh.h"
#include "Util/Real.h"

#include "Util/forall.h"

template <class T>
class SimMesh : public FEMMesh<typename T::Triangle, typename T::Vertex> {
public:
  DECLARE_TYPES(T)

  Patch *rootPatch;
    
//  friend class SimAccessor<Triangle,Vertex>;

//  Parameters parameters;

  SimMesh() : rootPatch(NULL), FEMMesh<Triangle,Vertex>() {  
  }

  typedef vector<GaussPoint*> GaussPointCt;
  typedef GaussPointCt::iterator GaussPointIt;
  typedef GaussPointCt::const_iterator const_GaussPointIt;

  void InitGaussPoints() {
    forall(VertexIt, v, vertexBegin(), vertexEnd())
      v->InitOneRing();
    endforall;

    forall(TriangleIt, t, triangleBegin(), triangleEnd())

      t->AttachGaussPoints();

      if (!t->triangle(E1) || t->triangle(E1) > t) {
	m_gaussPoints.push_back(t->getGaussPoint(E1));
      }
      if (!t->triangle(E2) || t->triangle(E2) > t) {
	m_gaussPoints.push_back(t->getGaussPoint(E2));
      }
      if (!t->triangle(E3) || t->triangle(E3) > t) {
	m_gaussPoints.push_back(t->getGaussPoint(E3));
      }
    endforall;
  }

  /**
   * Returns a constant iterator over the gauss points in this mesh, positioned
   * at the start of the gauss point list.
   * @return constant iterator over gauss points in this mesh, positioned at start
   *	     of list
   */
  const_GaussPointIt gaussPointBegin(void) const {
    return (m_gaussPoints.begin());
  }

  /**
   * Returns an iterator over the gauss points in this mesh, positioned at the
   * start of the gauss point list.
   * @return iterator over gauss points in this mesh, positioned at start of list
   */
  GaussPointIt gaussPointBegin(void) {
    return (m_gaussPoints.begin());
  }

  /**
   * Returns the number of gauss points in this mesh.
   * @return number of gauss points in this mesh
   */
  int gaussPointCount(void) const {
    return (m_gaussPoints.size());
  }

  /**
   * Returns a constant iterator over the gauss points in this mesh, positioned
   * at the end of the gauss point list.
   * @return constant iterator over gauss points in this mesh, positioned at end
   *	     of list
   */
  const_GaussPointIt gaussPointEnd(void) const {
    return (m_gaussPoints.end());
  }

  /**
   * Returns an iterator over the gauss points in this mesh, positioned at the
   * end of the gauss point list.
   * @return iterator over gauss points in this mesh, positioned at end of list
   */
  GaussPointIt gaussPointEnd(void) {
    return (m_gaussPoints.end());
  }


protected:
  /** The set of gauss points in this mesh. */
  GaussPointCt m_gaussPoints;
};

#define SIMMESHTYPE(MTYPE, Real_CLASS, Real3Vec_CLASS, Real3Tensor_CLASS,\
		    Mesh_CLASS, GaussPoint_CLASS, Vertex_CLASS, Triangle_CLASS,\
		    LeafPatch_CLASS,\
		    Material_CLASS, Surface_CLASS, Mass_CLASS,\
		    Forces_CLASS, Geometry_CLASS, Integrator_CLASS,\
		    ShellIntegrator_CLASS, Parameters_CLASS)\
struct MTYPE##Types;\
typedef Mesh_CLASS<MTYPE##Types> MTYPE;\
typedef Triangle_CLASS<MTYPE##Types> MTYPE##_Triangle;\
typedef Vertex_CLASS<MTYPE##Types> MTYPE##_Vertex;\
typedef GaussPoint_CLASS<MTYPE##Types> MTYPE##_GaussPoint;\
typedef Material_CLASS<MTYPE##Types> MTYPE##_Material;\
typedef Surface_CLASS<MTYPE##Types> MTYPE##_Surface;\
typedef Mass_CLASS<MTYPE##Types> MTYPE##_Mass;\
typedef Forces_CLASS<MTYPE##Types> MTYPE##_Forces;\
typedef Geometry_CLASS<MTYPE##Types> MTYPE##_Geometry;\
typedef Integrator_CLASS<MTYPE##Types> MTYPE##_Integrator;\
typedef ShellIntegrator_CLASS<MTYPE##Types> MTYPE##_ShellIntegrator;\
typedef Parameters_CLASS<MTYPE##Types> MTYPE##_Parameters;\
struct MTYPE##Types {\
  typedef Real_CLASS           Real;\
  typedef Real3Vec_CLASS       Real3Vec;\
  typedef Real3Tensor_CLASS    Real3Tensor;\
  typedef MTYPE                Mesh;\
  typedef MTYPE##_GaussPoint   GaussPoint;\
  typedef MTYPE##_Vertex       Vertex;\
  typedef MTYPE##_Triangle     Triangle;\
  typedef LeafPatch_CLASS      LeafPatch;\
  typedef MTYPE##_Material     Material;\
  typedef MTYPE##_Surface      Surface;\
  typedef MTYPE##_Mass         Mass;\
  typedef MTYPE##_Forces       Forces;\
  typedef MTYPE##_Geometry     Geometry;\
  typedef MTYPE##_Integrator   Integrator;\
  typedef MTYPE##_ShellIntegrator   ShellIntegrator;\
  typedef MTYPE##_Parameters   Parameters;\
};\


#endif





