// -*- Mode: c++ -*-
// $Id: Vector3.h,v 1.1.1.1 2000/01/06 20:46:23 eitan Exp $
// $Source: /ufs/multires/projects/Coke/Util/Vector3.h,v $

#ifndef	__VECTOR3_H__
#define	__VECTOR3_H__

// Do not change anything above this line
#ifdef _WIN32
#undef min
#undef max
#endif

#include <stddef.h>
#include <math.h>
//#include "util.h"

class ostream;
class istream;


// column type 3 vectors
template <class Scalar>
class Vector3{
 public:
  static const Vector3 Zero;

  enum{
    X = 0,
    Y = 1,
    Z = 2
    };
  Vector3( void ) {
    m_v[X] = m_v[Y] = m_v[Z] = 0;
  }
  //  Vector3( const Scalar f ) { m_v[X] = m_v[Y] = m_v[Z] = f; }
  Vector3( const Vector3& c )
    { m_v[X] = c.m_v[X], m_v[Y] = c.m_v[Y], m_v[Z] = c.m_v[Z]; }
  Vector3( const Scalar a, const Scalar b, const Scalar c )
    { m_v[X] = a, m_v[Y] = b, m_v[Z] = c; }
  Vector3( const float* a )
    { m_v[X] = a[X], m_v[Y] = a[Y], m_v[Z] = a[Z]; }
  Vector3( const double* a )
    { m_v[X] = a[X], m_v[Y] = a[Y], m_v[Z] = a[Z]; }
  operator Scalar*( void )
    { return &m_v[X]; }
  operator const Scalar*( void ) const
    { return &m_v[X]; }
  // ~Vector3( void ) {}
  
  friend istream& operator>>( istream&, Vector3& );
  friend ostream& operator<<( ostream&, const Vector3& );
  void print( ostream& os ) const;
  bool read ( istream& is );
  
  bool operator==(const Vector3& c)const
    { return m_v[X]==c.m_v[X] && m_v[Y]==c.m_v[Y] && m_v[Z]==c.m_v[Z];}
  bool operator<(const Vector3& c)const 
    { return unsigned(this)<unsigned(&c); }
  
  Vector3& operator=( const Vector3& c )
    { m_v[X] = c.m_v[X], m_v[Y] = c.m_v[Y], m_v[Z] = c.m_v[Z]; return *this; }
  Vector3 operator+( const Vector3& c ) const
    { return Vector3( m_v[X] + c.m_v[X], m_v[Y] + c.m_v[Y], m_v[Z] + c.m_v[Z] ); }
  Vector3 operator-( const Vector3& c ) const
    { return Vector3( m_v[X] - c.m_v[X], m_v[Y] - c.m_v[Y], m_v[Z] - c.m_v[Z] ); }
  Vector3 operator*( const Scalar s ) const
    { return Vector3( s * m_v[X], s * m_v[Y], s * m_v[Z] ); }
  friend Vector3 operator*( const Scalar s, const Vector3& c )
    { return Vector3( s * c.m_v[X], s * c.m_v[Y], s * c.m_v[Z] ); }
  Vector3 operator/( const Scalar s ) const
    { return Vector3( m_v[X] / s, m_v[Y] / s, m_v[Z] / s ); }
  
  Vector3& operator+=( const Vector3& c )
    { m_v[X] += c.m_v[X], m_v[Y] += c.m_v[Y], m_v[Z] += c.m_v[Z]; return *this; }
  Vector3& operator-=( const Vector3& c )
    { m_v[X] -= c.m_v[X], m_v[Y] -= c.m_v[Y], m_v[Z] -= c.m_v[Z]; return *this; }
  Vector3& operator*=( const Scalar s )
    { m_v[X] *= s, m_v[Y] *= s, m_v[Z] *= s; return *this; }
  Vector3& operator/=( const Scalar s )
    { m_v[X] /= s, m_v[Y] /= s, m_v[Z] /= s; return *this; }
  Vector3 operator-( void ) const
    { return Vector3( -m_v[X], -m_v[Y], -m_v[Z] ); }
  
  //stupid but useful
  Vector3 operator*(const Vector3& c)const{
    return Vector3(m_v[X]*c.m_v[X], m_v[Y]*c.m_v[Y], m_v[Z]*c.m_v[Z]);
  }
  Vector3 operator/(const Vector3& c)const{
    return Vector3(m_v[X]/c.m_v[X], m_v[Y]/c.m_v[Y], m_v[Z]/c.m_v[Z]);
  }
  
  Scalar& x( void ) { return m_v[X]; }
  Scalar x( void ) const { return m_v[X]; }
  Scalar& y( void ) { return m_v[Y]; }
  Scalar y( void ) const { return m_v[Y]; }
  Scalar& z( void ) { return m_v[Z]; }
  Scalar z( void ) const { return m_v[Z]; }
  Scalar& operator() (const int i)      { return m_v[i]; }
  Scalar operator() (const int i) const { return m_v[i]; }
  
  Vector3 lerp( const Vector3& v1, const Scalar t ) const
    { return Vector3( m_v[X] + t * ( v1.m_v[X] - m_v[X] ),
		   m_v[Y] + t * ( v1.m_v[Y] - m_v[Y] ),
		   m_v[Z] + t * ( v1.m_v[Z] - m_v[Z] ) ); }

  Vector3 min( const Vector3& o ) const
  { Scalar a = ::Min( m_v[X], o.m_v[X] );
	Scalar b = ::Min( m_v[Y], o.m_v[Y] );
	Scalar c = ::Min( m_v[Z], o.m_v[Z] );
    return Vector3( a, b, c ); }
  Vector3 max( const Vector3& o ) const
  {		Scalar a = ::Max( m_v[X], o.m_v[X] );
      Scalar b = ::Max( m_v[Y], o.m_v[Y] );
      Scalar c = ::Max( m_v[Z], o.m_v[Z] );
      return Vector3( a, b, c ); }
  void bbox( Vector3& min, Vector3& max ){
    if( m_v[X] < min.m_v[X] ) min.m_v[X] = m_v[X];
    else if( m_v[X] > max.m_v[X] ) max.m_v[X] = m_v[X];
    if( m_v[Y] < min.m_v[Y] ) min.m_v[Y] = m_v[Y];
    else if( m_v[Y] > max.m_v[Y] ) max.m_v[Y] = m_v[Y];
    if( m_v[Z] < min.m_v[Z] ) min.m_v[Z] = m_v[Z];
    else if( m_v[Z] > max.m_v[Z] ) max.m_v[Z] = m_v[Z];
  }

  Scalar dot( const Vector3& c ) const
    { return m_v[X] * c.m_v[X] + m_v[Y] * c.m_v[Y] + m_v[Z] * c.m_v[Z]; }
  Scalar dot( void ) const
    { return m_v[X] * m_v[X] + m_v[Y] * m_v[Y] + m_v[Z] * m_v[Z]; }
  Vector3 cross( const Vector3& c ) const
    { return Vector3( m_v[Y] * c.m_v[Z] - m_v[Z] * c.m_v[Y],
		  m_v[Z] * c.m_v[X] - m_v[X] * c.m_v[Z],
		  m_v[X] * c.m_v[Y] - m_v[Y] * c.m_v[X] ); }
  Scalar l1( void ) const
    { Scalar a = abs( m_v[X] ); a += abs( m_v[Y] ); return a + abs( m_v[Z] ); }
  Scalar linfty( void ) const
    { Scalar a = abs( m_v[X] ); a = ::Max( a, abs( m_v[Y] ) );
      return ::Max( a, abs( m_v[Z] ) ); }
  Scalar l2( void ) const { return sqrtf( dot() ); }
  
  Vector3 dir( void ) const;

  int normalize( void )
    { Scalar mag = l2(); return (mag == 0.0) ? 0.0 : ( *this *= 1 / mag, 1 ); }
  Scalar dist( const Vector3& c ) const { return ( *this - c ).l2(); }
  int collinear( const Vector3& a, const Vector3& b ) const;
  Vector3 perp( void ) const;
  
 protected:
 private:
  Scalar	m_v[3];
};

// extern defs for non-member outline functions go here
// externally visible constant defs go here

template <class Scalar>
const Vector3<Scalar> Vector3<Scalar>::Zero = Vector3<Scalar>(0.0, 0.0, 0.0);


#include "Vector3.cc"

#endif
