// -*- Mode: c++ -*-
// $Id: util.h,v 1.1.1.1 2000/01/06 20:46:23 eitan Exp $
// $Source: /ufs/multires/projects/Coke/Util/util.h,v $

#ifndef	__UTIL_H__
#define	__UTIL_H__

#include <assert.h>
#include <function.h>
#include <iostream.h>

#include "Real.h"

#ifdef _WIN32
#define fatan2(x,y) atan2(x,y)
#endif

#ifdef _WIN32
#include <Real.h>
#define isnanf(x) (_isnan(x))
#endif

template <class T>
struct Delete : public unary_function<T*, void>{
  void operator()( T* x ) const { delete x; }
};

// Inline function definitions go here
template <class _type>
inline int
argmin( _type a, _type b, _type c )
{
  return a <= b ? ( a <= c ? 0 : 2 ) : ( b <= c ? 1 : 2 );
}

template <class _type>
inline _type
Min( _type a, _type b )
{
  return a > b ? b : a;
}

template <class _type>
inline _type
Max( _type a, _type b )
{
  return a < b ? b : a;
}

template <class _type>
inline _type
Min( _type a, _type b, _type c )
{
  _type m = Min( a, b );
  return Min( m, c );
}

template <class _type>
inline _type
Max( _type a, _type b, _type c )
{
  _type m = Max( a, b );
  return Max( m, c );
}

template <class _type>
inline _type
Min( _type a, _type b, _type c, _type d )
{
  _type m = Min( a, b );
  m = Min( m, c );
  return Min( m, d );
}

template <class _type>
inline _type
Max( _type a, _type b, _type c, _type d )
{
  _type m = Max( a, b );
  m = Max( m, c );
  return Max( m, d );
}

template <class _type>
inline int
sign( const _type f )
{
  return f < 0 ? -1 : ( f > 0 ? 1 : 0 );
}

template <class _type>
inline _type
abs( const _type f )
{
  return f < 0 ? -f : f;
}

template <class _type>
inline _type
sqr( const _type s )
{
  return s * s;
}

template <class _type>
inline _type
cube( const _type s )
{
  return s * s * s;
}

inline int
even( int i )
{
  return !( i % 2 );
}

inline int
odd( int i )
{
  return i % 2;
}

inline int
zero( const Real x, const Real eps )
{
  return ( -eps < x ) && ( x < eps );
}

#if 0
static Real __negeps_f = -1e-40f;
static Real __poseps_f =  1e-40f;
#endif 

inline int
zero( const Real x )
{
  return x == 0.0;
}

// extern defs for non-member outline functions go here
// externally visible constant defs go here

#include <time.h>
// for timing
static clock_t __timerstart, __timerend;

inline void InitTimer( void ) { clock(); }
inline void StartTimer( void ) { __timerstart = clock(); }
inline void EndTimer( void ) { __timerend = clock(); }

inline ostream&
PrintDeltaTimer( ostream& os )
{
  os << ( 1000.0f* ( Real )( __timerend - __timerstart ) / (CLOCKS_PER_SEC )) <<
    " msec";
  return os;
}

#include <assert.h>
// a special version with diagnostics
// modeled after assert.h
#ifdef __cplusplus
extern "C" {
#endif
#ifdef NDEBUG

#undef assertzero
#define assertzero(EX) ((void)0)
#define die() ((void)0)
#else

#ifdef __ANSI_CPP__
#define assertzero(EX)  ((zero(EX))?((void)0):\
			 (cerr.precision(12),\
			  cerr << "assertion on zero( " <<\
			  # EX << " failed by " <<\
			  (EX)/m_eps_f << " eps\n",\
			 __assert( # EX , __FILE__, __LINE__)))
#else
#define assertzero(EX)  ((zero(EX))?((void)0):\
			 (cerr.precision(12),\
			  cerr << "assertion on zero( " <<\
			  "EX" << " failed by " <<\
			  (EX)/m_eps_f << " eps\n",\
			 __assert( "EX", __FILE__, __LINE__)))
#endif

// objectcenter doesn't like __assert
#ifdef _WIN32
#define die()            (assert(0)) 
#else
#define die()            (__assert( "disallowed", __FILE__, __LINE__ ))
#endif

#endif /* NDEBUG */

#ifdef __cplusplus
}
#endif

#endif	/* __UTIL_H__ */
