#include "Wconfig.h"
#include "Wxf.h"
#include "Wdriver.h"
#include "Wtypes.h"
#include "Wdomain.h"
#include "Woptions.h"
#include "Wcdtypes.h"
#include "Wutil.h"
#include "Wpvm.h"
#include "Whistry.h"
#include "Wmodr.h"
#include "Wnewmarki.h"
#include "Welem.h"
#if defined (MPI_WASP) && MPI_WASP /* parallel execution with MPI */
# include "Wmpi.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <time.h>
#include <sys/stat.h> 

#include "subdiv.h"

static void 
post_banner (char *log_name)
{
  int p;
  FILE *stream = stdout;
  char *banner[]
  = {
    "# ================================================================== #",
    "#                                                                    #",
    "#  WASP --- (W)ave propagation and vibration (A)nalysi(S) (P)ackage, #",
#if defined (MPI_WASP) && MPI_WASP
    "#                                                                    #",
    "#     Parallel version for message-passing systems with MPI          #",
    "#                                                                    #",
#endif
    "#               Copyright (C) 1994-2000 Petr Krysl                   #",
    "#                                                                    #",
    "#  WASP comes with ABSOLUTELY NO WARRANTY; see the file `wasp.wrn'.  #",
    "#   This is free software, and you are welcome to redistribute it    #",
    "#          under certain conditions; see the file ``wasp.use''       #",
    "#               and the GNU General Public License                   #",
    "#                                                                    #",
    "# ================================================================== #",
  };

  if (log_name) {
    stream = fopen (log_name, "a");
    if (!stream) stream = stdout;
  }
  
  for (p = 0; p < sizeof(banner)/sizeof(banner[0]); p++)
    fprintf (stream, "%s\n", banner[p]);
  fflush (stream);
}

static void
wd_error (char *file, int line, char *msg)
{
  fprintf (stderr, "\n*** ERROR: %s (%s, l.%d)\n",
           (msg ? msg : "?"), file, line);
  exit(EXIT_FAILURE);                                                 \
}
#define ERROR(msg) wd_error (__FILE__, __LINE__, msg)

#ifdef CUSTOM_SIGNAL_HANDLER

#ifdef TEST_IBM_SIGNAL_HANDLING
/*
 * This code demonstates a working floating-point exception
 * trap handler. The handler simply identifies which
 * floating-point exceptions caused the trap and return.
 * The handler can return via either the default signal return
 * mechanism (sigreturn() in AIX version 3) or longjmp();
 * a global handshaking variable determines which. In
 * practice a working trap handler would probably use one
 * of these methods, not both.
 */

/*
 * The fp_list structure allows text descriptions
 * of each possible trap type to be tied to the mask
 * that identifies it.
 */
typedef struct
  {
  fpflag_t mask;
  char     *text;
  } fp_list_t;
/* IEEE required trap types */
fp_list_t
trap_list[] =
  {
      { FP_INVALID,      "FP_INVALID"},
      { FP_OVERFLOW,     "FP_OVERFLOW"},
      { FP_UNDERFLOW,    "FP_UNDERFLOW"},
      { FP_DIV_BY_ZERO,  "FP_DIV_BY_ZERO"},
      { FP_INEXACT,      "FP_INEXACT"}
  };
/* INEXACT detail list -- this is an IBM RISC System/6000 extension */
fp_list_t
detail_list[] =  {
  { FP_INV_SNAN,   "FP_INV_SNAN" } ,
  { FP_INV_ISI,    "FP_INV_ISI" } ,
  { FP_INV_IDI,    "FP_INV_IDI" } ,
  { FP_INV_ZDZ,    "FP_INV_ZDZ" } ,
  { FP_INV_IMZ,    "FP_INV_IMZ" } ,
  { FP_INV_CMP,    "FP_INV_CMP" } ,
  { FP_INV_SQRT,   "FP_INV_SQRT" } ,
  { FP_INV_CVI,    "FP_INV_CVI" } ,
  { FP_INV_VXSOFT, "FP_INV_VXSOFT" }
};
/*
 * the TEST_IT macro is used in main() to raise
 * an exception.
 */
#define TEST_IT(WHAT, RAISE_ARG)        \
  {                                     \
  puts(strcat("testing: ", WHAT));      \
  fp_clr_flag(FP_ALL_XCP);              \
  fp_raise_xcp(RAISE_ARG);              \
  }
/*
 * NAME: my_div
 *
 * FUNCTION:  Perform floating-point division.
 *
 */
double my_div(double x, double y)
{
  return x / y;
}
/*
 * NAME: sigfpe_handler
 *
 * FUNCTION: A trap handler that is entered when
 *           a floating-point exception occurs. The
 *           function determines what exceptions caused
 *           the trap, prints this to stdout, and returns
 *           to the process which caused the trap.
 *
 * NOTES:    This trap handler can return either via the
 *           default return mechanism or via longjmp().
 *           The global variable fpsigexit determines which.
 *
 *           When entered, all floating-point traps are
 *           disabled.
 *
 *           This sample uses printf(). This should be used
 *           with caution since since printf() of a floating-
 *           point number can cause a trap, and then
 *           another printf() of a floating-point number
 *           in the signal handler will corrupt the static
 *           buffer used for the conversion.
 *
 * OUTPUTS:  The type of exception that caused
 *           the trap.
 */
static void sigfpe_handler(int sig,
			   int code,
			   struct sigcontext *SCP)
{
  struct mstsave * state = &SCP->sc_jmpbuf.jmp_context;
  fp_sh_info_t flt_context;	/* structure for fp_sh_info() */ 
				/* call */
  int i;			/* loop counter */
  extern int fpsigexit;		/* global handshaking variable */
  extern jmp_buf jump_buffer;	/*  */


  /*
   * Determine which floating-point exceptions caused
   * the trap. fp_sh_info() is used to build the floating signal
   * handler info  structure, then the member
   * flt_context.trap can be examined. First the trap type is
   * compared for the IEEE required traps, and if the trap type
   * is an invalid operation, the detail bits are examined.
   */


  fp_sh_info(SCP, &flt_context, FP_SH_INFO_SIZE);
  for (i = 0; i < (sizeof(trap_list) / sizeof(fp_list_t)); i++)      {
    if (flt_context.trap & trap_list[i].mask)
      (void) printf("Trap caused by %s error\n", trap_list[i].text);
  }
  if (flt_context.trap & FP_INVALID)      {
    for (i = 0; i < (sizeof(detail_list) / sizeof(fp_list_t)); i++)      {
      if (flt_context.trap & detail_list[i].mask)
	(void) printf("Type of invalid op is %s\n", detail_list[i].text);
    }
  }
  /* report which trap mode was in effect */
  switch (flt_context.trap_mode) {
  case FP_TRAP_OFF:
    puts("Trapping Mode is OFF");
    break;
  case FP_TRAP_SYNC:
    puts("Trapping Mode is SYNC");
    break;
  case FP_TRAP_IMP:
    puts("Trapping Mode is IMP");
    break;
  case FP_TRAP_IMP_REC:
    puts("Trapping Mode is IMP_REC");
    break;
  default:
    puts("ERROR:  Invalid trap mode");
  }
  if (fpsigexit == LONGJUMP_EXIT)      {
    /*
     * Return via longjmp. In this instance there is no need to
     * clear any exceptions or disable traps to prevent
     * recurrence of the exception, because on return the
     * process will have the signal handler's floating-point
     * state.
     */
    longjmp(jump_buffer, JMP_FPE);
  }  else      {
    /*
     * Return via default signal handler return mechanism.
     * In this case you must take some action to prevent
     * recurrence of the trap, either by clearing the
     * exception bit in the fpscr or by disabling the trap.
     * In this case, clear the exception bit.
     * The fp_sh_set_stat routine is used to clear
     * the exception bit.
     */
    fp_sh_set_stat(SCP, (flt_context.fpscr & ((fpstat_t) ~flt_context.trap)));


      /*
       * Increment the iar of the process that caused the trap,
       * to prevent re-execution of the instruction.
       * The FP_IAR_STAT bit in flt_context.flags indicates if
       * state->iar points to an instruction that has logically
       * started. If this bit is true, state->iar points to
       * an operation that has started and will cause another
       * exception if it runs again. In this case you want to
       * continue execution and ignore the results of that
       * operation, so the iar is advanced to point to the
       * next instruction. If the bit is false, the iar already
       * points to the next instruction that must run.
       */


      if ( flt_context.flags & FP_IAR_STAT ) {
	puts("Increment IAR");
	state->iar += 4;
      }
  }
  return;
}


/*
 * NAME: main
 *
 * FUNCTION: Demonstrate the sigfpe_handler trap handler.
 *
 */
int test_fp_err_handler(void)
{
  struct sigaction response;
  struct sigaction old_response;
  extern int fpsigexit;
  extern jmp_buf jump_buffer;
  int jump_rc;
  int trap_mode;
  double arg1, arg2, r;


  /*
   * Set up for floating-point trapping. Do the following:
   *  1.  Clear any existing floating-point exception flags.
   *  2.  Set up a SIGFPE signal handler.
   *  3.  Place the process in synchronous execution mode.
   *  4.  Enable all floating-point traps.
   */
  fp_clr_flag(FP_ALL_XCP);
  (void) sigaction(SIGFPE, NULL, &old_response);
  (void) sigemptyset(&response.sa_mask);
  response.sa_flags = FALSE;
  response.sa_handler = (void (*)(int)) sigfpe_handler;
  (void) sigaction(SIGFPE, &response, NULL);
  fp_enable_all();
  /*
   * Demonstate trap handler return via default signal handler
   * return. The TEST_IT macro will raise the floating-point
   * exception type given in its second argument. Testing
   * is done in this case with precise trapping, because
   * it is supported on all platforms to date.
   */
  trap_mode = fp_trap(FP_TRAP_SYNC);
  if ((trap_mode == FP_TRAP_ERROR) ||
      (trap_mode == FP_TRAP_UNIMPL)) {
    printf("ERROR: rc from fp_trap is %d\n",
	   trap_mode);
    exit(-1);
  }


  (void) printf("Default signal handler return: \n");
  fpsigexit = SIGRETURN_EXIT;
/*
  TEST_IT("div by zero", FP_DIV_BY_ZERO);
  TEST_IT("overflow",    FP_OVERFLOW);
  TEST_IT("underflow",   FP_UNDERFLOW);
  TEST_IT("inexact",     FP_INEXACT);
  TEST_IT("signaling nan",      FP_INV_SNAN);
  TEST_IT("INF - INF",          FP_INV_ISI);
  TEST_IT("INF / INF",          FP_INV_IDI);
  TEST_IT("ZERO / ZERO",        FP_INV_ZDZ);
  TEST_IT("INF * ZERO",         FP_INV_IMZ);
  TEST_IT("invalid compare",    FP_INV_CMP);
  TEST_IT("invalid sqrt",       FP_INV_SQRT);
  TEST_IT("invalid coversion",  FP_INV_CVI);
  TEST_IT("software request",   FP_INV_VXSOFT);
*/
  /*
   * Next, use fp_trap() to determine what the
   * the fastest trapmode available is on
   * this platform.
   */
  trap_mode = fp_trap(FP_TRAP_FASTMODE);
  switch (trap_mode) {
  case FP_TRAP_SYNC:
    puts("Fast mode for this platform is PRECISE");
    break;
  case FP_TRAP_OFF:
    puts("This platform dosn't support trapping");
    break;
  case FP_TRAP_IMP:
    puts("Fast mode for this platform is IMPRECISE");
    break;
  case FP_TRAP_IMP_REC:
    puts("Fast mode for this platform is IMPRECISE RECOVERABLE");
    break;
  default:
    printf("Unexpected return code from fp_trap(FP_TRAP_FASTMODE): %d\n",
	   trap_mode);
    exit(-2);
  }


  /*
   * if this platform supports imprecise trapping, demonstate this.
   */

/*
  trap_mode = fp_trap(FP_TRAP_IMP);
  if (trap_mode != FP_TRAP_UNIMPL) {
    puts("Demonstrate imprecise FP execeptions");
    arg1 = 1.2;
    arg2 = 0.0;
    r = my_div(arg1, arg2);
    fp_flush_imprecise();
  }
  */
  /* demonstate trap handler return via longjmp().
   */

/*
  (void) printf("longjmp return: \n");
  fpsigexit = LONGJUMP_EXIT;
  jump_rc = setjmp(jump_buffer);
*/

/*  switch (jump_rc) {
  case JMP_DEFINED:
    (void) printf("setjmp has been set up; testing ...\n");
    TEST_IT("div by zero", FP_DIV_BY_ZERO);
    break;
  case JMP_FPE:
    (void) printf("back from signal handler\n"); */
      /*
       * Note that at this point the process has the floating-
       * point status inherited from the trap handler. If the
       * trap hander did not enable trapping (as the example
       * did not) then this process at this point has no traps
       * enabled.  We create a floating-point exception to
       * demonstrate that a trap does not occur, then re-enable
       * traps.
       */
/*    (void) printf("Creating overflow; should not trap\n");
    TEST_IT("Overflow", FP_OVERFLOW);
    fp_enable_all();
    break;
  default:
    (void) printf("unexpected rc from setjmp: %d\n", jump_rc);
    exit(EXIT_BAD);
  } 
*/
  exit(EXIT_GOOD);
}
#endif /* IBM_SIGNAL_HANDLING */

#endif


#ifdef CUSTOM_SIGNAL_HANDLER
#include <signal.h>
void (*old_signal_handler)(int sig);

#define FAILED_SIGNAL()                                                 \
  {                                                                     \
    ERROR ("Failed to install signal handler");                         \
  }
#define SET_HANDLER(sig, handler)                               \
  { if ((old_signal_handler = signal(sig, handler)) == SIG_ERR) \
      FAILED_SIGNAL();                                          \
  }


 /* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */


typedef void (*action_proc)(W_domain_t *);

typedef struct menu_choice_t {
  char            option;
  int             only_with_X;
  char           *descr;
  action_proc     action;
}              menu_choice_t;

static void
stop_choice(W_domain_t *d)
{
  if (d) WD_clean_up_before_exit (d);
  exit(EXIT_SUCCESS);
}


#ifdef X_XFACE
static void
xiface_interact_choice(W_domain_t *d)
{
  sprintf(W_buf(),
	  "Stopped by an interrupt.  Press ^P inside the view to continue.\n");
  WXF_loop(TRUE, W_buf());
}
#endif


static void
abort_choice(W_domain_t *d)
{
  exit(EXIT_FAILURE);
}


static void
hexvue_choice (W_domain_t *d)
{
  if (d == NULL)
    return;
  
  WOPT_SET_OPT (d->options, write_postfile_file_now, TRUE);
  WOPT_SET_OPT (d->options, postfile_type, HEXVUE_POSTPROC_FILE);
}


static void
ve3d_choice (W_domain_t *d)
{
  if (d == NULL)
    return;
  
  WOPT_SET_OPT (d->options, write_postfile_file_now, TRUE);
  WOPT_SET_OPT (d->options, postfile_type, VE3D_POSTPROC_FILE);
}


static void
tecplot_choice (W_domain_t *d)
{
  if (d == NULL)
    return;
  
  WOPT_SET_OPT (d->options, write_postfile_file_now, TRUE);
  WOPT_SET_OPT (d->options, postfile_type, TECPLOT_I_POSTPROC_FILE);
}


static void
flushhist_choice (W_domain_t *d)
{
  if (d) WD_flush_history(d);
}


static void
configure_choice (W_domain_t *d)
{
  W_elem_t *el;
  W_index_t id;

#define MXC 1024
  char line[MXC];
#define GETINP()                                \
  {                                             \
    gets(line);                                 \
  }
  
  if (d == NULL)
    return;

  fprintf(stderr, "Input element number <"); fflush(stderr);
  fflush(stdin);
  GETINP();
  if (tokens_parse_line(d->parser, line) < 1) {
    REPORT_WARNING(d->errh, "Invalid input");  fflush(stderr);
    return;
  }
  
  if (!TOKEN_IS_NUMBER(1)) {
    REPORT_WARNING(d->errh, "Expecting number"); fflush(stderr);
    return;
  } 
  
  id = TOKEN_INT(1);
  el = WD_get_elem_by_idx(d, id);
  if (el == NULL) {
    REPORT_WARNING(d->errh, "No such element"); fflush(stderr);
    return;
  }

getopt:
  fprintf(stderr, "Input configuration option (<CR> to continue)<");
  fflush(stderr);
  GETINP();
  if (tokens_parse_line(d->parser, line) < 1) {
    return;
  }
  if (!WE_configure(el, line)) {
    REPORT_WARNING(d->errh, "Configuration failed"); fflush(stderr);
  }
  goto getopt;
}


static void
elinfo_choice (W_domain_t *d)
{
  W_index_t id;
#define MXC 1024
  char line[MXC];
#define GETINP()                                \
  {                                             \
    gets(line);                                 \
  }
  
  if (d == NULL)
    return;

  fprintf(stderr, "Input element number <"); fflush(stderr);
  fflush(stdin);
  GETINP();
  if (tokens_parse_line(d->parser, line) < 1) {
    REPORT_WARNING(d->errh, "Invalid input");  fflush(stderr);
    return;
  }
  
  if (!TOKEN_IS_NUMBER(1)) {
    REPORT_WARNING(d->errh, "Expecting number"); fflush(stderr);
    return;
  } 
  
  id = TOKEN_INT(1);
  {
    char buf[L_tmpnam] = "./tmp";
    FILE *fp;

    tmpnam(buf);
    fp = fopen(buf, "w");
    fprintf(stderr, "Writing element info to %s\n", buf);
    WD_print_elem_info(d, id, W_INFO_FULL, fp);
    fclose(fp);
    fprintf(stderr, "Done writing %s\n", buf);
  }
}


static void
set_target_time_choice (W_domain_t *d)
{
#define MXC 1024
  char line[MXC];
#define GETINP()                                \
  {                                             \
    gets(line);                                 \
  }
  
  if (d == NULL)
    return;

  fprintf(stderr, "Input target time <"); fflush(stderr);
  fflush(stdin);
  GETINP();
  if (tokens_parse_line(d->parser, line) < 1) {
    REPORT_WARNING(d->errh, "Invalid input");  fflush(stderr);
    return;
  }
  
  if (!TOKEN_IS_NUMBER(1)) {
    REPORT_WARNING(d->errh, "Expecting number"); fflush(stderr);
    return;
  } 
  
  WOPT_SET_OPT (d->options, target_time, TOKEN_DBL(1));

  return;
}



static menu_choice_t choices[] = {
  { 's',  FALSE, "stop (with cleanup)",             stop_choice },
  { 'a',  FALSE, "abort (no cleanup)",              abort_choice },
  { 'c',  FALSE, "continue",                        NULL },
  { 'T',  FALSE, "set target time",                 set_target_time_choice },
  { 'h',  FALSE, "write a HEXVUE file",             hexvue_choice },
  { 'v',  FALSE, "write a VE3D file  ",             ve3d_choice },
  { 't',  FALSE, "write a TECPLOT file",            tecplot_choice },
  { 'f',  FALSE, "flush history files",             flushhist_choice },
#ifdef X_XFACE
  { 'X',   TRUE, "enter X interface",               xiface_interact_choice },
#endif
  { 'e',  FALSE, "configure element",               configure_choice },
  { 'i',  FALSE, "element info",                    elinfo_choice },
  { '\0', FALSE, NULL,                              NULL }
};



static int received_an_interrupt = FALSE;

void 
wasp_signal_handler(int sig)
{
#define DO_GLUE(A, B)  A##B
#define GLUE(A, B)     DO_GLUE(A, B)
#define DO_STRNGZ(S)   #S
#define STRNGZ(S)      DO_STRNGZ(S) 
#define SIGNLNM(S)     GLUE(STRNGZ(SIG),STRNGZ(S))
#define PRERR(NAME)    fprintf(stderr,                                  \
                               "wasp_signal_handler: signal %s\n",      \
                               SIGNLNM(NAME))
#define CASE(NAME)                                                      \
   case GLUE(SIG,NAME):                                                 \
     PRERR(NAME);                                                       \
     SET_HANDLER(SIGINT, SIG_DFL);                                      \
     if (isatty (2)) {                                                  \
       fprintf(stderr, "      ^c will interrupt the program\n");        \
       fflush(stderr);                                                  \
     } else {                                                           \
       REPORT_FATAL_ERROR (NULL, "Caught signal " SIGNLNM(NAME));       \
     }                                                                  \
     break

  switch (sig) {
  CASE(FPE);
  CASE(SEGV);
  CASE(BUS);
  CASE(ILL);
  case SIGINT:
    {
      received_an_interrupt = TRUE;
      SET_HANDLER(SIGINT, &wasp_signal_handler);
    }
    break;
  default:
    fprintf(stderr, "wasp_signal_handler: signal %d\n", sig);
    fflush(stderr);
    exit(EXIT_FAILURE);
    break;
  }

  return;
}


static int
menu (W_domain_t *d)
{
  int i, c;
  double realt, usert, syst;
  int always_true = 1;
  
  W_get_times(&realt, &usert, &syst);
  while (always_true) {
    fprintf(stderr, "Interrupt (problem time %g, user %g, sys %g).\n",
            d ? WD_time(d) : 0, usert, syst);
    i = 0;
    while (choices[i].option != '\0') {
      if ((!choices[i].only_with_X)
#         ifdef X_XFACE
	  || WOPT_OPT (d->options, X_xface)
#         endif
	  )
	fprintf(stderr, "   %c   %s\n", choices[i].option, choices[i].descr);
      i++;
    }
    fprintf(stderr,   "         Your choice? ");
    while (always_true) {
      c = fgetc(stdin);
      i = 0;
      while (choices[i].option != '\0') {
        if (choices[i].option == c
	    && ((!choices[i].only_with_X)
#               ifdef X_XFACE
		|| WOPT_OPT (d->options, X_xface)
#               endif
		))
          return i;
        i++;
      }
    }
  }
  return TRUE;
}

int
wasp_check_interrupt (W_domain_t *d)
{
  if (!received_an_interrupt) {
    return TRUE;
  } else {
    int c;
    signal(SIGINT, SIG_IGN);
    c = menu (d);
    if (choices[c].action) choices[c].action(d);
    SET_HANDLER(SIGINT, &wasp_signal_handler);
    received_an_interrupt = FALSE;
    return TRUE;
  }
}


#else
int
wasp_check_interrupt(W_domain_t *d)
{
  return TRUE;
}
#endif /* CUSTOM_SIGNAL_HANDLER */



#ifdef CUSTOM_SIGNAL_HANDLER
#include <signal.h>
void (*old_signal_handler)(int sig);

#define FAILED_SIGNAL()                                                 \
  {                                                                     \
    ERROR ("Failed to install signal handler");                         \
  }
#define SET_HANDLER(sig, handler)                               \
  { if ((old_signal_handler = signal(sig, handler)) == SIG_ERR) \
      FAILED_SIGNAL();                                          \
  }


#   ifdef IBM_SIGNAL_HANDLING
#      include <signal.h>
#      include <setjmp.h>
#      include <fpxcp.h>
#      include <fptrap.h>
#      include <stdlib.h>
#      include <stdio.h>
#      define EXIT_BAD  -1
#      define EXIT_GOOD  0

int test_fp_err_handler(void);
/*
 * Handshaking variable with the signal handler. If zero,
 * then the signal hander returns via  the default signal
 * return mechanism; if non-zero, then the signal handler
 * returns via longjmp.
 */
int fpsigexit;
#      define SIGRETURN_EXIT 0
#      define LONGJUMP_EXIT  1
jmp_buf jump_buffer;	/* jump buffer */
#      define JMP_DEFINED 0		/* setjmp rc on initial call */
#      define JMP_FPE     2		/* setjmp rc on return from */
				/* signal handler */
#   endif
#endif


static void
custom_error_handler_setup (void)
{
#ifdef IBM_SIGNAL_HANDLING
  int trap_mode;
#endif

#ifdef CUSTOM_SIGNAL_HANDLER
#   ifdef IBM_SIGNAL_HANDLING
  trap_mode = fp_trap(FP_TRAP_SYNC);
  if ((trap_mode == FP_TRAP_ERROR) ||
      (trap_mode == FP_TRAP_UNIMPL)) {
    printf("ERROR: rc from fp_trap is %d\n",
	   trap_mode);
    exit(-1);
  }
  fp_enable(TRP_UNDERFLOW|TRP_OVERFLOW|TRP_DIV_BY_ZERO); 
#   endif
#   ifdef HP_SIGNAL_HANDLING
#      define SET_MASK(mask) {fpsetmask(mask);}
  SET_MASK(FP_X_INV);
  SET_MASK(FP_X_DZ);
  SET_HANDLER(SIGFPE, &wasp_signal_handler);
  SET_HANDLER(SIGBUS, &wasp_signal_handler);
  SET_HANDLER(SIGILL, &wasp_signal_handler);
  SET_HANDLER(SIGSEGV, &wasp_signal_handler);
#   endif
#endif
}

static void 
print_usage_and_exit (int argc, char **argv)
{
  char *option, *usage;

  post_banner (NULL);

  fprintf (stderr, "usage: %s -i in_file_name + options below\n", argv[0]);
  
  if (ckit_opt_first_recognized (&option, &usage)) {
    fprintf (stderr, "%s %s\n",
             (option != NULL ? option : "?"),
             (usage  != NULL ? usage  : "?"));
  }
  while (ckit_opt_next_recognized (&option, &usage)) {
    fprintf (stderr, "%s %s\n",
             (option != NULL ? option : "?"),
             (usage  != NULL ? usage  : "?"));
  }
  exit (0);
} 


static void 
check_in_out_file_names (struct W_options_t *o)
{
  char line[MAXFILENAMECHAR];
  
  if (WOPT_OPT (o, in_file_name) == NULL)
    print_usage_and_exit (WOPT_OPT (o, argc), WOPT_OPT (o, argv));
  
  if (!FILE_ACCESSIBLE(WOPT_OPT (o, in_file_name))) {
    strcpy(line, WOPT_OPT (o, in_file_name));
    strcat(line, IN_SUFFIX);
    if (!FILE_ACCESSIBLE (line)) {
      char buffer[1024];
      
      sprintf(buffer,"\nInaccessible input file %s (or %s)\n",
	      WOPT_OPT (o, in_file_name), line);
      REPORT_FATAL_ERROR (NULL, buffer);
      exit(EXIT_FAILURE);
    } else {
      char *s;
      free(WOPT_OPT (o, in_file_name));
      DUPLICATE_STRING (s, line);
      WOPT_SET_OPT (o, in_file_name, s);
    }
  }
  
  if (WOPT_OPT (o, out_file_name) == NULL) {
    char *s;
    strcpy(line, WOPT_OPT (o, in_file_name));
    if (W_has_suffix(line, IN_SUFFIX)) 
      line[strlen(line)-strlen(IN_SUFFIX)] = '\0';
    strcat (line, OUT_SUFFIX);
    DUPLICATE_STRING (s, line);
    WOPT_SET_OPT (o, out_file_name, s);
  }
}

int 
build_xiface(int argc, char *argv[]);

static void
save_cmd_line (W_options_t *o)
{
  FILE *fp;
  tokens_parser p = tokens_new_parser ();
  fp = fopen ("WD_CMD_LINE.LOG", "a");
  if (fp) {
    int i;
    fprintf (fp, "# %s\n", ckit_get_date_and_time ());
    FOR (i, WOPT_OPT (o, argc)) {
      tokens_parse_line (p, WOPT_OPT (o, argv)[i]);
      if (tokens_total_of_tokens (p) > 1)
        fprintf (fp, "\"%s\" ", WOPT_OPT (o, argv)[i]);
      else
        fprintf (fp, "%s ", WOPT_OPT (o, argv)[i]);
    }
    fprintf (fp, "\n\n");
    fclose (fp);
  }
  tokens_delete_parser (p);
}

static W_domain_t *
wasp_init (int argc, char **argv)
{
  W_domain_t *d = NULL;
  struct W_options_t *o;

  /* set up our custom signal handler */
#ifdef CUSTOM_SIGNAL_HANDLER
  custom_error_handler_setup ();
#endif

  /* Set error handler for the G3D library (optional) */
  G3D_set_err_callback (W_g3d_error_callback);

  /* for parallel execution with MPI (explicit dynamics only)  */
#if defined (MPI_WASP) && MPI_WASP /* parallel execution with MPI */
  /* Initialize MPI; command line args are collected by main() */
  MPI_Init (&argc, &argv);
#endif /* parallel execution with MPI */

  /* get options relevant to WASP */
  o = WO_new_options ();
  if (!WO_get_options (o, argc, argv)) { /* load the options from the command line */
    ERROR ("Could not access command line options");
  }

  /* if a help summary should be produced */
  if (WOPT_OPT (o, help)) {
    print_usage_and_exit (WOPT_OPT (o, argc), WOPT_OPT (o, argv));
  } else { /* check accessibility of input/output files */
    check_in_out_file_names (o);
  }

  /* print banner */
  if (WOPT_OPT (o, verbose))
    post_banner (WOPT_OPT (o, log_name));

  /* for parallel execution with MPI (explicit dynamics only)  */
#if defined (MPI_WASP) && MPI_WASP /* parallel execution with MPI */
  if (WOPT_OPT (o, verbose)) {
    char processor[MPI_MAX_PROCESSOR_NAME];
    int len;
    MPI_Get_processor_name (processor, &len);
    sprintf (W_buf (), "Running on processor/ machine: %s", processor);
    W_log_update (W_buf ());
  }
#endif /* parallel execution with MPI */

  /* save command line contents to disk for later reference */
  save_cmd_line (o);

  /* if you want to register finite elements, materials, do it now
     because the WD_new_domain() function will want to use them
   */

  WE_register_reader (SUBDIV_ELCODE, subdiv_reader);
  
  /* now comes the WASP thing: create domain first */
  if ((d = WD_new_domain (o)) == NULL) {
    return NULL;
  }

  /* if an on-line graphic interface should be produced, do it */
#ifdef X_XFACE 
  if (WOPT_OPT (d->options, X_xface)) {
    WXF_build_interface (argc, argv, d);
    WXF_loop (TRUE, "WASP (C) P.Krysl 1994-2000");
  }
#endif

  /* at this point it is convenient to set an interrupt handler we want to use */
#ifdef CUSTOM_SIGNAL_HANDLER
  WD_set_interrupt_func (d, wasp_check_interrupt);
#endif

  return d;
}

static int
wasp_run (W_domain_t *d)
{
  { /* this is to initialize the real time */
    double real_time, usr_time, systm_time;
    W_get_times(&real_time, &usr_time, &systm_time);
  }
  
  /* if graphic interface should be used, set it up */
#ifdef X_XFACE
  if (WOPT_OPT (d->options, X_xface)) {
    WD_build_grep (d, FALSE);
    sprintf (W_buf (), "layer on %d", WX_NODE_LAYER);
    ESIHandleCmd (W_buf ());
    sprintf (W_buf (), "layer on %d", WX_ELEM_LAYER);
    ESIHandleCmd (W_buf ());
    sprintf (W_buf (), "layer off %d", WX_NATURAL_CF_LAYER);
    ESIHandleCmd (W_buf ());
    sprintf (W_buf (), "layer off %d", WX_RIGID_CF_LAYER);
    ESIHandleCmd (W_buf ());
    sprintf (W_buf (), "layer off %d", WX_NATURAL_CE_LAYER);
    ESIHandleCmd (W_buf ());
    sprintf (W_buf (), "layer off %d", WX_RIGID_CE_LAYER);
    ESIHandleCmd (W_buf ());
    sprintf (W_buf (), "layer off %d", WX_ANNOT_LAYER);
    ESIHandleCmd (W_buf ());
    WXF_loop (TRUE, "Ready to run");
  }
#endif

  /* If you want to do some more twisting of the knobs and all sorts of
     customization, do it now */
  
  
  /* now is the right time to set our signal handler */
#ifdef CUSTOM_SIGNAL_HANDLER
  SET_HANDLER(SIGINT, &wasp_signal_handler);
#endif

  /* verbose time information */
  if (WOPT_OPT (d->options, verbose)) {
    sprintf (W_buf (), "Started: %s", ckit_get_date_and_time ());
    W_log_update (d->logh, W_buf ());
  }

  /* do the WASP thing: integrate in time */
  if (WOPT_OPT (d->options, integrator) == CD_INTEGRATOR)
    WD_integrate_expl (d);
  else if (WOPT_OPT (d->options, integrator) == NEWMARK_INTEGRATOR)
    WD_integrate_impl (d);
  else {
    REPORT_FATAL_ERROR (d->errh, "Unknown integrator");
  }

  /* if WASP collected POD modes, let's write it out */
  if (d->modr && W_modr_is_in_collect_mode (d->modr)) {
    W_modr_compute_modes (d->modr);
#   ifdef X_XFACE
    if (WOPT_OPT (d->options, X_xface))
      WXF_loop (1, "Inspect modes");
    else 
#   endif
      {
        FILE *fp;
        sprintf (W_buf (), "%s.pod", WOPT_OPT (d->options, in_file_basename));
        fp = fopen (W_buf (), "w");
        if (fp) {
          W_modr_write_modr (d->modr, fp);
          fclose (fp);
        } 
      }
  }
  
  /* verbose finish time information */
  if (WOPT_OPT (d->options, verbose)) {
    sprintf (W_buf (), "Finished: %s", ckit_get_date_and_time ());
    W_log_update (d->logh, W_buf ());
  }

  /* for parallel execution with MPI */
#if defined (MPI_WASP) && MPI_WASP /* parallel execution with MPI */
  /* Initialize MPI; command line args are collected by main() */
  MPI_Finalize ();
#endif /* parallel execution with MPI */

  /* produce time statistics if desired */
#if TIMED_WASP
  if (WOPT_OPT (d->options, verbose)) {
# if defined (MPI_WASP) && MPI_WASP
    sprintf (W_buf (), "Timing statistics: effl=%g, solv=%g, ucfg=%g, fina=%g, comm=%g",
             W_timer_get_time (d->effl_timer),
             W_timer_get_time (d->solv_timer),
             W_timer_get_time (d->ucfg_timer),
             W_timer_get_time (d->fina_timer),
             W_timer_get_time (d->comm_timer));
# else
    if (d->cd_integrator) {
      sprintf (W_buf (), "Timing statistics: effl=%g, solv=%g, ucfg=%g, fina=%g",
               W_timer_get_time (d->effl_timer),
               W_timer_get_time (d->solv_timer),
               W_timer_get_time (d->ucfg_timer),
               W_timer_get_time (d->fina_timer));
    } else if (d->newmark_integrator) {
      double ass_rhs_time, ass_lhs_time, solve_time, pred_time, updt_time;
      W_newmark_get_stat (d->newmark_integrator,
                          &ass_rhs_time, &ass_lhs_time, &solve_time, &pred_time, &updt_time);
      sprintf (W_buf (), "Timing statistics:\n#  rass=%g, lass=%g, solv=%g, pred=%g, updt=%g",
               ass_rhs_time, ass_lhs_time, solve_time, pred_time, updt_time);  
    }
# endif
    W_log_update (d->logh, W_buf ());
  }
#endif

  return EXIT_SUCCESS;
}


int
main (int argc, char **argv)
{
  W_domain_t *d;
  
  if ((d = wasp_init (argc, argv)) == NULL) {
    ERROR ("No WASP domain could be created");
  }
  return wasp_run (d);
}
