// -*- Mode: c++ -*-
// $Id: ControlVertex.h,v 1.2 2000/09/16 04:35:24 eitan Exp $
// $Source: /cs/research/multires/cvsroot/Coke/Loop/ControlVertex.h,v $

#ifndef __LOOP_CONTROLVERTEX_H__
#define __LOOP_CONTROLVERTEX_H__

#include <assert.h>
#include "Mesh/PointVertex.h"
#include "Util/cvec.h"

/**
 * ControlVertex Class
 */
template <class TRIANGLEIF, class VERTEXIF, class VERTEXMR>
class ControlVertex : public PointVertex<TRIANGLEIF, VERTEXIF> {
public:
  typedef VERTEXMR VertexMr;

protected:
  /** The multiresolution vertex associated with this geometric vertex. */
  VertexMr* m_vertex;

private:
  /**
   * Assignment operator: undefined operation.
   */
  ControlVertex& operator=(const ControlVertex& instance) {
    die();
    return (*this);
  }

public:
  /**
   * Constructs an uninitialized vertex.  Initially the vertex does not
   * belong to the topology of a mesh.
   */
  ControlVertex(void) : PointVertex<Triangle, Vertex>() {
    m_vertex = new VertexMr();
    assert(m_vertex);
  }

  /**
   * Copy constructor.
   */
  ControlVertex(const ControlVertex& instance)
   : PointVertex<Triangle, Vertex>()
  {
    m_vertex = new VertexMr();
    assert(m_vertex);

    // NJL: this is a hack to circumvent lack of order control in initializers.
    // Ideally, m_vertex would be initialized before the superclass.  As this
    // doesn't seem possible, we emulate the superclass copy constructor here.
    ((VERTEXIF*)this)->setPoint(instance.point());
  }
  
  /**
   * Destructor.
   */
  ~ControlVertex(void) {
    delete (m_vertex);
  }

  /**
   * Defines the point at this vertex.
   * @param point  new point to store at this vertex
   */
  void setPoint(const CVec& point) {
    PointVertex<Triangle, Vertex>::setPoint(point);
    
    VertexMr::Accessor accessor;
    accessor.initialize(m_vertex, point);
    m_vertex->setValid(0);
  }

  /**
   * Returns the multiresolution vertex associated with this geometric vertex.
   * @return multiresolution vertex at this geometric vertex
   */
  VertexMr* vertex(void) const {
    return (m_vertex);
  }
};

#endif  /* __LOOP_CONTROLVERTEX_H__ */
