// -*- Mode: c++ -*-
// $Id: LoopTesselator.h,v 1.2 2000/09/16 04:35:24 eitan Exp $
// $Source: /cs/research/multires/cvsroot/Coke/Loop/LoopTesselator.h,v $

#ifndef __LOOP_LOOPTESSELATOR_H__
#define __LOOP_LOOPTESSELATOR_H__

#include <hash_map.h>
#include <stack.h>
#include "Loop/TesselatedMesh.h"
#include "Loop/Topology.h"
#include "Mesh/MeshTp.h"
#include "Mesh/PointVertex.h"
#include "Mesh/TriangleTp.h"

/**
 * LoopTesselator Class
 */
template <class Loop, class Accessor = Loop::VertexMr::Accessor>
class LoopTesselator {
private:
  class Tesselator_Mesh;
  class Tesselator_Triangle;
  class Tesselator_Vertex;

  class Tesselator_Mesh
   : public TesselatedMesh<Tesselator_Mesh,
                           Tesselator_Triangle,
                           Tesselator_Vertex>
  {
   public:
    Tesselator_Mesh(void)
     : TesselatedMesh<Tesselator_Mesh,
                      Tesselator_Triangle,
                      Tesselator_Vertex>() { }
    ~Tesselator_Mesh(void) { }
  };

  class Tesselator_Triangle
   : public TesselatedTriangle<Tesselator_Triangle,
                               Tesselator_Vertex,
                               Loop::Triangle,
                               Loop::TriangleMr*>
  {
   public:
    Tesselator_Triangle(Loop::Triangle* domain, Loop::TriangleMr* secondary)
     : TesselatedTriangle<Tesselator_Triangle,
                          Tesselator_Vertex,
                          Loop::Triangle,
                          Loop::TriangleMr*>(domain, secondary) { }
    ~Tesselator_Triangle(void) { }
  };

  class Tesselator_Vertex
   : public TesselatedVertex<Tesselator_Triangle,
                             Tesselator_Vertex,
                             Loop::Triangle,
                             Loop::VertexMr*>
  {
   public:
    Tesselator_Vertex(TesselatedVertexTag<Loop::Triangle>* tag,
		      Loop::VertexMr* secondary)
     : TesselatedVertex<Tesselator_Triangle,
                        Tesselator_Vertex,
                        Loop::Triangle,
                        Loop::VertexMr*>(tag, secondary) { }
    ~Tesselator_Vertex(void) { }
  };

public:
  typedef Tesselator_Mesh Mesh;
  typedef Mesh::Triangle Triangle;
  typedef Mesh::Vertex Vertex;
  typedef Assembler<Mesh> Assembler;
  typedef Loop::TriangleMr TriangleMr;
  typedef Loop::VertexMr VertexMr;
  typedef TriangleMr::Orientation Orientation;

// NJL: detail temperature hack
typedef hash_map<Vertex*, VertexMr*, Vertex::hash> VertexMap;

private:
  Assembler* m_assembler;

  Loop* m_loop;

  stack<TriangleMr*> m_stack;

  hash_map<VertexMr*, int, VertexMr::hash> m_table;

  bool m_limitFlag;

// NJL: detail temperature hack
VertexMap m_vertexMap;

public:
  LoopTesselator(bool limitFlag = true) : m_limitFlag(limitFlag) { }

  ~LoopTesselator(void) { }

  Mesh* operator()(Loop& loopMesh) {
    return (tesselate(loopMesh));
  }

// NJL: detail temperature hack
VertexMap& getVertexMap(void) { return (m_vertexMap); }

  Mesh* tesselate(Loop& loopMesh, int depth = __MAX_DEPTH) {
    assert(depth >= 0);
    Mesh* mesh = new Mesh();
    assert(mesh);
    m_assembler = new Assembler(*mesh);
    assert(m_assembler);
    m_loop = &loopMesh;

    Loop::const_TriangleIt ti = m_loop->triangleBegin();
    Loop::const_TriangleIt te = m_loop->triangleEnd();
    for (; ti != te; ++ti) {
      TriangleMr* triangle = (*ti)->root();
      m_stack.push(triangle);

      while (!m_stack.empty()) {
	triangle = m_stack.top();
	m_stack.pop();

	if (triangle->isLeaf()) {
	  tesselate(triangle, depth);
	} else if (triangle->depth() == depth) {
	  tesselate(triangle, depth);
	} else {
	  int below = triangle->depth() + 1;
	  TriangleMr* child = triangle->child(T0);
	  if (!triangle->vertex(V1)->isValid(below) ||
	      !triangle->vertex(V2)->isValid(below) ||
	      !triangle->vertex(V3)->isValid(below) ||
	      !child->vertex(V1)->isValid(below) ||
	      !child->vertex(V2)->isValid(below) ||
	      !child->vertex(V3)->isValid(below))
	  {
	    tesselate(triangle, depth);
	  } else {
	    for (int t = T0; t <= T3; t += 1) {
	      m_stack.push(triangle->child(TriangleLabel(t)));
	    }
	  }
	}
      }
    }

    m_assembler->assembleMesh();
    delete (m_assembler);
    m_loop = 0;
    m_table.clear();

    return (mesh);
  }

private:
  void tesselate(TriangleMr* triangle, int depth) {
    Loop::Triangle* domain = triangle->triangle();
    Orientation o = triangle->orientation();
    Vertex* vertices[6];

    for (int i = V1; i <= V3; i += 1) {
      vertices[i] = vertex(triangle, VertexLabel(i));
    }

    int mask = 0;
    Permutation p;
    for (int e = E1; e <= E3; e += 1) {
      TriangleMr* adjacent = triangle->adjacentTriangle(EdgeLabel(e), p);
      if (!adjacent || adjacent->isLeaf() || adjacent->depth() == depth)
	continue;

      VertexLabel v = p.vertex(OppositeVertex(EdgeLabel(e)));
      if (!adjacent->child(T0)->vertex(v)->isValid(triangle->depth() + 1))
        continue;

      vertices[e + 3] = vertex(adjacent->child(T0), v);
      mask |= 1 << e;
    }

    switch (mask) {
    case 0:
      addTriangle(domain, triangle, vertices[0], vertices[1], vertices[2], o);
      break;
      
    case 1:
      addTriangle(domain, triangle, vertices[0], vertices[3], vertices[2], o);
      addTriangle(domain, triangle, vertices[0], vertices[1], vertices[3], o);
      break;
      
    case 2:
      addTriangle(domain, triangle, vertices[0], vertices[1], vertices[4], o);
      addTriangle(domain, triangle, vertices[1], vertices[2], vertices[4], o);
      break;
      
    case 3:
      addTriangle(domain, triangle, vertices[0], vertices[1], vertices[3], o);
      addTriangle(domain, triangle, vertices[0], vertices[3], vertices[4], o);
      addTriangle(domain, triangle, vertices[2], vertices[4], vertices[3], o);
      break;
      
    case 4:
      addTriangle(domain, triangle, vertices[0], vertices[5], vertices[2], o);
      addTriangle(domain, triangle, vertices[1], vertices[2], vertices[5], o);
      break;
      
    case 5:
      addTriangle(domain, triangle, vertices[0], vertices[5], vertices[2], o);
      addTriangle(domain, triangle, vertices[1], vertices[3], vertices[5], o);
      addTriangle(domain, triangle, vertices[2], vertices[5], vertices[3], o);
      break;
      
    case 6:
      addTriangle(domain, triangle, vertices[0], vertices[5], vertices[4], o);
      addTriangle(domain, triangle, vertices[1], vertices[2], vertices[4], o);
      addTriangle(domain, triangle, vertices[1], vertices[4], vertices[5], o);
      break;
      
    case 7:
      addTriangle(domain, triangle, vertices[0], vertices[5], vertices[4], o);
      addTriangle(domain, triangle, vertices[1], vertices[3], vertices[5], o);
      addTriangle(domain, triangle, vertices[2], vertices[4], vertices[3], o);
      addTriangle(domain, triangle, vertices[3], vertices[4], vertices[5], o);
      break;
    }
  }

  void addTriangle(Loop::Triangle* domain,
		   Loop::TriangleMr* secondary,
		   Vertex* v1,
		   Vertex* v2,
		   Vertex* v3,
		   int orientation)
  {
    Triangle* triangle = new Triangle(domain, secondary);
    assert(triangle);

    if (Orientation(orientation) == TriangleMr::COUNTERCLOCKWISE)
      triangle->setVertices(v1, v2, v3);
    else
      triangle->setVertices(v1, v3, v2);

    m_assembler->addTriangle(triangle);
  }

  Vertex* vertex(TriangleMr* triangle, VertexLabel v) {
    VertexMr* vertex = triangle->vertex(v);
    if (m_table.find(vertex) != m_table.end())
      return (m_assembler->getVertex(m_table[vertex]));

    Loop::Triangle* domain = triangle->triangle();
    TesselatedVertexTag<Loop::Triangle>* tag;
    if (triangle->vertex(v)->isControlVertex()) {
      tag = new TesselatedControlVertexTag<Loop::Triangle>(domain, v);
    } else {
      CVec coords(0.0f, 0.0f, 0.0f);
      coords(v) = 1.0f;

      TriangleMr* next = triangle;
      while (!next->isRoot()) {
	switch (next->label()) {
	case T0:
	  coords = coords.x() * CVec(0.0f, 0.5f, 0.5f) +
	           coords.y() * CVec(0.5f, 0.0f, 0.5f) +
	           coords.z() * CVec(0.5f, 0.5f, 0.0f);
	  break;

	case T1:
	  coords = coords.x() * CVec(1.0f, 0.0f, 0.0f) +
	           coords.y() * CVec(0.5f, 0.0f, 0.5f) +
	           coords.z() * CVec(0.5f, 0.5f, 0.0f);
	  break;

	case T2:
	  coords = coords.x() * CVec(0.0f, 0.5f, 0.5f) +
	           coords.y() * CVec(0.0f, 1.0f, 0.0f) +
	           coords.z() * CVec(0.5f, 0.5f, 0.0f);
	  break;

	case T3:
	  coords = coords.x() * CVec(0.0f, 0.5f, 0.5f) +
	           coords.y() * CVec(0.5f, 0.0f, 0.5f) +
	           coords.z() * CVec(0.0f, 0.0f, 1.0f);
	  break;
	}

	next = next->parent();
      }

      if (coords.x() == 0.0f)
	tag = new TesselatedEdgeVertexTag<Loop::Triangle>(domain, E1, coords);
      else if (coords.y() == 0.0f)
	tag = new TesselatedEdgeVertexTag<Loop::Triangle>(domain, E2, coords);
      else if (coords.z() == 0.0f)
	tag = new TesselatedEdgeVertexTag<Loop::Triangle>(domain, E3, coords);
      else
	tag = new TesselatedInteriorVertexTag<Loop::Triangle>(domain, coords);
    }
    assert(tag);

    Vertex* newVertex = new Vertex(tag, triangle->vertex(v));
    assert(newVertex);
    if (m_limitFlag == true) {
      Accessor accessor;
      m_loop->limit(triangle, v);
      newVertex->setPoint(accessor.limit(vertex));
    } else {
      Accessor accessor;
      newVertex->setPoint(accessor(vertex, triangle->depth()));
    }

    int index = m_assembler->addVertex(newVertex);
    m_table[vertex] = index;

// NJL: detail temperature hack
m_vertexMap[newVertex] = vertex;
    return (newVertex);				
  }
};

#endif  /* __LOOP_LOOPTESSELATOR_H__ */
