// -*- Mode: c++ -*-
// $Id: WarrenStencil.h,v 1.3 2000/09/29 21:00:16 eitan Exp $
// $Source: /cs/research/multires/cvsroot/Coke/Loop/WarrenStencil.h,v $

#ifndef __LOOP_WARRENSTENCIL_H__
#define __LOOP_WARRENSTENCIL_H__

#include "Loop/LoopStencil.h"
#include "Util/cvec.h"

/**
 * WarrenStencil Class
 */
class WarrenStencil : public LoopStencil {
public:
  WarrenStencil(void) : LoopStencil() { }

  WarrenStencil(const WarrenStencil& instance)
   : LoopStencil(instance) { }

  ~WarrenStencil(void) { }

  WarrenStencil& operator=(const WarrenStencil& instance) {
    LoopStencil::operator=(instance);
    return (*this);
  }

  template <class TriangleMr, class Accessor, class Accumulator>
  void even(TriangleMr* triangle, VertexLabel v, Accessor accessor, Accumulator &result) const {
    // In 9/16 of all cases, this point is incident to an edge of the
    // triangle's parent.
    TriangleLabel t = triangle->label();
    if (!triangle->isRoot() && (t == T0 || v != ParentVertex(t))) {
      LoopStencil::even(triangle, v, accessor, result);
      return;
    }

    int valence = triangle->valence(v);
    float alpha, beta;
    if (valence == 3) {
      alpha = 7.0f / 16.0f;
      beta = 3.0f / 16.0f;
    } else {
      alpha = 5.0f / 8.0f;
      beta = 3.0f / (8.0f * (float)valence);
    }

    TriangleMr::VertexMr* vertex = triangle->vertex(v);
    int depth = triangle->depth();
    result += alpha * accessor(vertex, depth);
    
    TriangleMr::VertexIt vi = triangle->vertexBegin(v);
    TriangleMr::VertexIt ve = triangle->vertexEnd(v);
    for (int i = 0; vi != ve; i += 1, ++vi) {
      result += beta * accessor(*vi, depth);
    }
    assert(i == valence);
  }
};

#endif  /* __WARRENSTENCIL_H__ */
