// -*- Mode: c++ -*-
// $Id: Diamond.h,v 1.4 2000/08/31 22:56:11 eitan Exp $
// $Source: /cs/research/multires/cvsroot/Coke/Quadrature/Diamond.h,v $

#ifndef	__DIAMOND_H__
#define	__DIAMOND_H__

// Do not change anything above this line

// include files
#include <map>
#include "OneRing.h"
#include "Mesh/Labels.h"
#include "QuadCoeffs.h"
#include "Util/forall.h"
#include "Util/Real.h"

#include <algorithm>

// class definition
template <class Triangle, class VertexIf>
class Diamond{
 public:
  typedef VertexIf Vertex;
  typedef std::map<const Vertex*, QuadCoeffs*>::const_iterator SupportIt;

  Diamond( Triangle *t, EdgeName e );
  ~Diamond( void );

  const QuadCoeffs& Weights( const Vertex *v ) const{
    assert( m_c.find(v) != m_c.end() ); return *m_c[v];
  }
  const std::map<const Vertex*, QuadCoeffs*>& DOFs( void ) { return m_c; }

  SupportIt supportBegin() const {
    return m_c.begin();
  }

  SupportIt supportEnd() const {
    return m_c.end();
  }

  template <class VertexAttribute, class Result>
  Result EvalN(VertexAttribute Attribute, Result result) const {
    result = Zero3Vec;
    forall(SupportIt, s, supportBegin(), supportEnd())

      const Vertex *v = s.first;
      QuadCoeffs *w = s.second;

      result += w->N() * Attribute(v);
    endforall;
    return result;
  }

  template <class VertexAttribute, class Result>
  Result Evald0(VertexAttribute Attribute, Result result) const {
    result = Zero3Vec;
    forall(SupportIt, s, supportBegin(), supportEnd())

      const Vertex *v = s.first;
      QuadCoeffs *w = s.second;

      result += w->dN0() * Attribute(v);
    endforall;
    return result;
  }

  template <class VertexAttribute, class Result>
  Result Evald1(VertexAttribute Attribute, Result result) const {
    result = Zero3Vec;
    forall(SupportIt, s, supportBegin(), supportEnd())

      const Vertex *v = s.first;
      QuadCoeffs *w = s.second;

      result += w->dN1() * Attribute(v);
    endforall;
    return result;
  }

  template <class VertexAttribute, class Result>
  Result Evald00(VertexAttribute Attribute, Result result) const {
    result = Zero3Vec;
    forall(SupportIt, s, supportBegin(), supportEnd())

      const Vertex *v = s.first;
      QuadCoeffs *w = s.second;

      result += w->dN00() * Attribute(v);
    endforall;
    return result;
  }

  Real SumWeights() const {
    Real result = 0;
    forall(SupportIt, s, supportBegin(), supportEnd())

      QuadCoeffs *w = s.second;

      result += w->dN00();
    endforall;
    return result;
  }

  template <class VertexAttribute, class Result>
  Result Evald01(VertexAttribute Attribute, Result result) const {
    result = Zero3Vec;
    forall(SupportIt, s, supportBegin(), supportEnd())

      const Vertex *v = s.first;
      QuadCoeffs *w = s.second;

      result += w->dN01() * Attribute(v);
    endforall;
    return result;
  }

  template <class VertexAttribute, class Result>
  Result Evald11(VertexAttribute Attribute, Result result) const {
    result = Zero3Vec;
    forall(SupportIt, s, supportBegin(), supportEnd())

      const Vertex *v = s.first;
      QuadCoeffs *w = s.second;

      result += w->dN11() * Attribute(v);
    endforall;
    return result;
  }

  CVec Evalp( void ) const{
    CVec total(0,0,0);
    std::map<const Vertex*, QuadCoeffs*>::const_iterator b = m_c.begin(), e = m_c.end();
    for( ; b != e; ++b ) total += (*b).first->p() * (*b).second->p();
    return total;
  }
  CVec Evaldp1( void ) const{
    CVec total(0,0,0);
    std::map<const Vertex*, QuadCoeffs*>::const_iterator b = m_c.begin(), e = m_c.end();
    for( ; b != e; ++b ) total += (*b).first->p() * (*b).second->dp1();
    return total;
  }
  CVec Evaldp2( void ) const{
    CVec total(0,0,0);
    std::map<const Vertex*, QuadCoeffs*>::const_iterator b = m_c.begin(), e = m_c.end();
    for( ; b != e; ++b ) total += (*b).first->p() * (*b).second->dp2();
    return total;
  }
  CVec Evaldp11( void ) const{
    CVec total(0,0,0);
    std::map<const Vertex*, QuadCoeffs*>::const_iterator b = m_c.begin(), e = m_c.end();
    for( ; b != e; ++b ) total += (*b).first->p() * (*b).second->dp11();
    return total;
  }
  CVec Evaldp12( void ) const{
    CVec total(0,0,0);
    std::map<const Vertex*, QuadCoeffs*>::const_iterator b = m_c.begin(), e = m_c.end();
    for( ; b != e; ++b ) total += (*b).first->p() * (*b).second->dp12();
    return total;
  }
  CVec Evaldp22( void ) const{
    CVec total(0,0,0);
    std::map<const Vertex*, QuadCoeffs*>::const_iterator b = m_c.begin(), e = m_c.end();
    for( ; b != e; ++b ) total += (*b).first->p() * (*b).second->dp22();
    return total;
  }

  bool Boundary( void ) const{
    return (m_top == NULL);
  }

 protected:
 private:
  // names for the elements of a hexagon
  enum RingMembers {
    CNT = 0, // center vertex
    VL = 1,  // vertex left
    VR = 2,  // vertex right
    VBL = 3, // vertex bottom left
    VBR = 4, // vertex bottom right
    VTL = 5, // vertex top left
    VTR = 6  // vertex top right
  };
  enum DiamondCorners {
    LEFT = 0,
    RIGHT = 1,
    BOTTOM = 2,
    TOP = 3
  };
  
  // left right bottom top
  OneRing<Triangle,Vertex> *m_r[4];
  // the incident triangles
  Triangle *m_bot, *m_top;
  // the corresponding weights
  std::map<const Vertex*, QuadCoeffs*> m_c;

  inline int FindCreaseNeighbors( const OneRing<Triangle,Vertex>& vn, // center vertex
				  const int vo,      // outer vertex
				  int& vl,           // first crease walking left
				  int& vr );         // first crease walking right
  inline Real CornerNeighbor( const OneRing<Triangle,Vertex>& vn,
			       const int vo,
			       const bool cx,
			       const QuadCoeffs& cn,
			       Real& gamma, const Real scale ); 
  inline Real CreaseNeighbor( const OneRing<Triangle,Vertex>& vn,
			       const int vo,
			       const QuadCoeffs& cn,
			       Real& gamma,
			       const Real scale );
  inline void LoopOdd( QuadCoeffs c1[], const QuadCoeffs& cn,
		       const OneRing<Triangle,Vertex>& vn, const Vertex *vv,
		       const int l, const int r, const int b, const int t,
		       // is the edge a crease edge
		       const bool ecrease,
		       // if this edge is not a crease it may be convex
		       const bool convex );
  inline void LoopOdd( const QuadCoeffs& cn,
		       const OneRing<Triangle,Vertex>& vn, const Vertex *vv,
		       const Vertex *vb, const Vertex *vt,
		       // is the edge a crease edge
		       const bool ecrease,
		       // if this edge is not a crease it may be convex
		       const bool convex );
  inline void LoopOddNoCorrection( QuadCoeffs c1[], const QuadCoeffs& cn,
				   // indices of left, right, bottom, top and center
				   const int l, const int r, const int b, const int t,
				   // is the center guy a crease vertex?
				   const bool crease ); //
  inline void LoopOddNoCorrection( const QuadCoeffs& cn,
				   const Vertex *vl, const Vertex *vr,
				   const Vertex *vb, const Vertex *vt,
				   // is the center guy a crease vertex?
				   const bool crease );
  inline void LoopEven( QuadCoeffs c1[], const QuadCoeffs& cnt, const bool crease ) const;
  inline void LoopEven( const OneRing<Triangle,Vertex>& r, const Vertex *vo, const QuadCoeffs& c );
};

// extern defs for non-member outline functions go here
// externally visible constant defs go here

#include "Diamond.cc"

#endif	/* __DIAMOND_H__ */
