// -*- Mode: c++ -*-
// $Id: QuadCoeffs.h,v 1.2 2000/09/12 20:42:55 eitan Exp $
// $Source: /cs/research/multires/cvsroot/Coke/Quadrature/QuadCoeffs.h,v $

#ifndef	__QUADCOEFFS_H__
#define	__QUADCOEFFS_H__

// Do not change anything above this line

// include files

#include "Util/Real.h"

// class definition
class QuadCoeffs{
 public:
  enum {
    P = 0,
    DP1 = 1,
    DP2 = 2,
    DP11 = 3,
    DP12 = 4,
    DP22 = 5
  };
  QuadCoeffs( void )
    { m_c[P] = m_c[DP1] = m_c[DP2] = m_c[DP11] = m_c[DP12] = m_c[DP22] = 0.0; }
  QuadCoeffs( const Real p, const Real dx, const Real dy,
	      const Real dxx, const Real dxy, const Real dyy )
    { m_c[P] = p; m_c[DP1] = dx; m_c[DP2] = dy; m_c[DP11] = dxx; m_c[DP12] = dxy; m_c[DP22] = dyy; }
  QuadCoeffs( const QuadCoeffs& q )
    { m_c[P] = q.m_c[P]; m_c[DP1] = q.m_c[DP1]; m_c[DP2] = q.m_c[DP2];
      m_c[DP11] = q.m_c[DP11]; m_c[DP12] = q.m_c[DP12]; m_c[DP22] = q.m_c[DP22]; }
  Real& p( void ) { return m_c[P]; }
  Real  p( void ) const { return m_c[P]; }
  Real& dp1( void ) { return m_c[DP1]; }
  Real  dp1( void ) const { return m_c[DP1]; }
  Real& dp2( void ) { return m_c[DP2]; }
  Real  dp2( void ) const { return m_c[DP2]; }
  Real& dp11( void ) { return m_c[DP11]; }
  Real  dp11( void ) const { return m_c[DP11]; }
  Real& dp12( void ) { return m_c[DP12]; }
  Real  dp12( void ) const { return m_c[DP12]; }
  Real& dp22( void ) { return m_c[DP22]; }
  Real  dp22( void ) const { return m_c[DP22]; }

  //Real& N( void ) { return m_c[P]; }
  Real  N( void ) const { return m_c[P]; }
  //Real& dN0( void ) { return m_c[DP1]; }
  Real  dN0( void ) const { return m_c[DP1]; }
  //Real& dN1( void ) { return m_c[DP2]; }
  Real  dN1( void ) const { return m_c[DP2]; }
  //Real& dN00( void ) { return m_c[DP11]; }
  Real  dN00( void ) const { return m_c[DP11]; }
  //Real& dN01( void ) { return m_c[DP12]; }
  Real  dN01( void ) const { return m_c[DP12]; }
  //Real& dN11( void ) { return m_c[DP22]; }
  Real  dN11( void ) const { return m_c[DP22]; }

  // bitwise copy ok; QuadCoeffs& operator=( const QuadCoeffs& q );
  QuadCoeffs operator+( const QuadCoeffs& q ) const
    { return QuadCoeffs( m_c[P] + q.m_c[P],
			 m_c[DP1] + q.m_c[DP1],
			 m_c[DP2] + q.m_c[DP2],
			 m_c[DP11] + q.m_c[DP11],
			 m_c[DP12] + q.m_c[DP12],
			 m_c[DP22] + q.m_c[DP22] ); }
  QuadCoeffs operator-( const QuadCoeffs& q ) const
    { return QuadCoeffs( m_c[P] - q.m_c[P],
			 m_c[DP1] - q.m_c[DP1],
			 m_c[DP2] - q.m_c[DP2],
			 m_c[DP11] - q.m_c[DP11],
			 m_c[DP12] - q.m_c[DP12],
			 m_c[DP22] - q.m_c[DP22] ); }
  QuadCoeffs operator*( const Real s ) const
    { return QuadCoeffs( m_c[P] * s,
			 m_c[DP1] * s,
			 m_c[DP2] * s,
			 m_c[DP11] * s,
			 m_c[DP12] * s,
			 m_c[DP22] * s ); }
  friend QuadCoeffs operator*( const Real s, const QuadCoeffs& q )
    { return QuadCoeffs( q.m_c[P] * s,
			 q.m_c[DP1] * s,
			 q.m_c[DP2] * s,
			 q.m_c[DP11] * s,
			 q.m_c[DP12] * s,
			 q.m_c[DP22] * s ); }
  QuadCoeffs operator/( const Real s ) const
    { return QuadCoeffs( m_c[P] / s,
			 m_c[DP1] / s,
			 m_c[DP2] / s,
			 m_c[DP11] / s,
			 m_c[DP12] / s,
			 m_c[DP22] / s ); }
  QuadCoeffs& operator+=( const QuadCoeffs& q )
    { m_c[P] += q.m_c[P]; m_c[DP1] += q.m_c[DP1]; m_c[DP2] += q.m_c[DP2];
      m_c[DP11] += q.m_c[DP11]; m_c[DP12] += q.m_c[DP12]; m_c[DP22] += q.m_c[DP22];
      return *this; }
  QuadCoeffs& operator-=( const QuadCoeffs& q )
    { m_c[P] -= q.m_c[P]; m_c[DP1] -= q.m_c[DP1]; m_c[DP2] -= q.m_c[DP2];
      m_c[DP11] -= q.m_c[DP11]; m_c[DP12] -= q.m_c[DP12]; m_c[DP22] -= q.m_c[DP22];
      return *this; }
  QuadCoeffs& operator*=( const Real s )
    { m_c[P] *= s; m_c[DP1] *= s; m_c[DP2] *= s;
      m_c[DP11] *= s; m_c[DP12] *= s; m_c[DP22] *= s;
      return *this; }
  QuadCoeffs& operator/=( const Real s )
    { m_c[P] /= s; m_c[DP1] /= s; m_c[DP2] /= s;
      m_c[DP11] /= s; m_c[DP12] /= s; m_c[DP22] /= s;
      return *this; }
  QuadCoeffs operator-( void ) const
    { return QuadCoeffs( -m_c[P], -m_c[DP1], -m_c[DP2], -m_c[DP11], -m_c[DP12], -m_c[DP22] ); }

 protected:
 private:
  // there are 6 coefficients for pos, dx, dy, dxdx, dxdy, dydy
  Real m_c[6];
};

// extern defs for non-member outline functions go here
// externally visible constant defs go here

#endif	/* __QUADCOEFFS_H__ */
