#ifndef	__THINSHELLWRITER_H__
#define	__THINSHELLWRITER_H__

#include "Mesh/Labels.h"
#include "Mesh/SMF.h"
#include "Loop/Topology.h"

// class definition
template <class Mesh>
class ThinShellWriter : public SMF::MeshWriter<Mesh> {
public:
  ThinShellWriter(void) : SMF::MeshWriter<Mesh>() { }

  ~ThinShellWriter(void) { }

  void writeMesh(ostream& os, Mesh& mesh) {
    SMF::MeshWriter<Mesh>::writeMesh(os, mesh);

    writeTags(os, mesh);
  }

protected:

 void writeTags(ostream& os, Mesh& mesh) {
   Mesh::const_TriangleIt ti = m_mesh->triangleBegin();
   Mesh::const_TriangleIt te = m_mesh->triangleEnd();

   int count = 0;

   for (; ti != te; ++ti) {

     Mesh::Triangle *t1 = *ti;

     for (int e = E1; e <= E3; ++e) {

       Mesh::Triangle *t2 = t1->triangle(EdgeLabel(e));

       if (t2 < t1)
	 continue;

       if (!t2)
	 continue;

       if (t1->Crease(EdgeLabel(e))) {
	 ++count;
       }
     }
   }

   cerr << "Writing " << count << " creases\n";

   os << "# " << count << " 0 0" << endl;

   ti = m_mesh->triangleBegin();
   te = m_mesh->triangleEnd();
   
   for (; ti != te; ++ti) {

     Mesh::Triangle *t1 = *ti;

     for (int e = E1; e <= E3; ++e) {

       Mesh::Triangle *t2 = t1->triangle(EdgeLabel(e));

       if (t2 < t1)
	 continue;

       if (!t2)
	 continue;

       if (t1->Crease(EdgeLabel(e))) {
	 os << m_table[t1->vertex(CWVertex(VertexLabel(e)))] << ' '
	    << m_table[t1->vertex(CCWVertex(VertexLabel(e)))] << endl;

       }
     }
   }


 }
};

#endif
