// -*- Mode: c++ -*-
// $Id: planequad.cc,v 1.1.1.1 2000/01/06 20:46:13 eitan Exp $
// $Source: /cs/research/multires/cvsroot/Coke/Quadrature/test/planequad.cc,v $

#include <iostream.h>
#include <fstream.h>
#include <vector.h>

#include "FEMMesh.h"
#include "FEMTriangle.h"
#include "ShellDOF.h"

#include <Inventor/SoInput.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoIndexedFaceSet.h>
#include <Inventor/nodes/SoIndexedLineSet.h>
#include <Inventor/nodes/SoComplexity.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoLineSet.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/actions/SoWriteAction.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoTransformSeparator.h>
#include <Inventor/nodes/SoSphere.h>

#include "Loop/BoundaryStencil.h"
#include "Loop/ControlTriangle.h"
#include "Loop/ControlVertex.h"
#include "Loop/LoopMesh.h"
#include "Loop/MultiresVertex.h"
#include "Loop/WarrenStencil.h"

MESHTYPE(FEMMESH, FEMMesh, FEMTriangle, ShellDOF)

template <class T>
class hash<T *> {
public:
  size_t operator()(const T *v) const {
    return hash<char*>()((char*)v);
  }
};

/*
template <class T>
T sqr(T val) {
  return val*val;
}*/

class BuildMesh {
public:
  template <class Mesh>
  void operator()(Mesh &mesh, int N, float scale, float x_exp, float y_exp) {
    Assembler<Mesh::Triangle, Mesh::Vertex> a(mesh);
    int x, y;

    cerr << "Building mesh: N=" << N << " scale=" << scale << " (x^" << x_exp << ")(y^" << y_exp << ")\n";

    assert(N+1<50);

    Mesh::Vertex *grid[50][50];

    for (y=0; y < N+1; ++y)
      for (x=0; x < N+1; ++x) {
	if (y%2 == 0 && x == N) continue;
	double X = x;
	double Y = y;
	grid[x][y] = new Mesh::Vertex();
	if (y%2 == 0)
	  grid[x][y]->point() = CVec(X+0.5, Y*sqrt(3.0)/2.0, 0.0);
	else 
	  grid[x][y]->point() = CVec(X, Y*sqrt(3.0)/2.0, 0.0);
	grid[x][y]->point().x() *= scale;
	grid[x][y]->point().y() *= scale;
	X = grid[x][y]->point().x();
	Y = grid[x][y]->point().y();
	grid[x][y]->point().z() = pow(X,x_exp)*pow(Y,y_exp);
	a.addVertex(grid[x][y]);
      }

    for (y=1; y < N+1; y+=2)
      for (x=1; x < N; ++x) {
	Mesh::Triangle *t = new Mesh::Triangle;
	t->setVertices( (grid[x  ][y  ]),
			(grid[x-1][y-1]),
			(grid[x  ][y-1]));
	a.addTriangle(t);
      }


    for (y=2; y < N+1; y+=2)
      for (x=0; x < N; ++x) {
	Mesh::Triangle *t = new Mesh::Triangle;
	t->setVertices( (grid[x  ][y  ]),
			(grid[x  ][y-1]),
			(grid[x+1][y-1]));
	a.addTriangle(t);
      }


    for (y=1; y < N; y+=2)
      for (x=1; x < N; ++x) {
	Mesh::Triangle *t = new Mesh::Triangle;
	t->setVertices( (grid[x  ][y  ]),
			(grid[x  ][y+1]),
			(grid[x-1][y+1]));
	a.addTriangle(t);
      }

    for (y=0; y < N; y+=2)
      for (x=0; x < N; ++x) {
	Mesh::Triangle *t = new Mesh::Triangle;
	t->setVertices( (grid[x  ][y  ]),
			(grid[x+1][y+1]),
			(grid[x  ][y+1]));
	a.addTriangle(t);
      }

    assert(a.assembleMesh());
  }
};




void
main(int argc, char* argv[])
{
  FEMMESH femOutMesh;
  ofstream fout("temp.smf");

  if (argc != 5) {
    cerr << "planequad N Scale x_exp y_exp\n";
    cerr << "Constructs a mesh with NxN vertices,\n";
    cerr << "scaled by Scale, sampling the function\n";
    cerr << "(x^x_exp)*(y^y_exp)\n";
    return;
  }

  int N = atoi(argv[1]);
  float scale = atof(argv[2]);
  float x_exp = atof(argv[3]);
  float y_exp = atof(argv[4]);

  BuildMesh()(femOutMesh,N,scale,x_exp,y_exp);

  fout << femOutMesh;
  fout.close();

  ifstream fin("temp.smf");
  FEMMESH femMesh;
  fin >> femMesh;
  
  {
    FEMMESH::VertexIt vb = femMesh.vertexBegin(), ve = femMesh.vertexEnd();
    for( ; vb != ve; ++vb ) (*vb)->InitOneRing();
    FEMMESH::TriangleIt tb = femMesh.triangleBegin(), te = femMesh.triangleEnd();
    for( ; tb != te; ++tb ) (*tb)->AttachDiamonds();
    tb = femMesh.triangleBegin();
  }

  cerr << "Done initializing FEM mesh." << endl;

  for (int display=0; display<=4; ++display)
  {
    hash_map<Diamond<FEMMESH::Triangle,FEMMESH::Vertex> *, bool> diamondProcessed;
    FEMMESH::TriangleIt femT = femMesh.triangleBegin(), femTEnd = femMesh.triangleEnd();
    for(; femT != femTEnd; ++femT) {
      if ((*femT)->vertex(V1)->valence()!=6 || (*femT)->vertex(V2)->valence()!=6 || (*femT)->vertex(V3)->valence()!=6) continue;
      for (int edgeNum=E1; edgeNum<=E3; ++edgeNum) {
	Diamond<FEMMESH::Triangle, FEMMESH::Vertex> *diamond = (*femT)->getDiamond(EdgeLabel(edgeNum));
	if (diamondProcessed[diamond]) continue;
	diamondProcessed[diamond] = true;
	if ((*femT)->vertex(CCWVertex(OppositeVertex(EdgeLabel(edgeNum))))->point().y()
	    != (*femT)->vertex(CWVertex(OppositeVertex(EdgeLabel(edgeNum))))->point().y()) continue;

	CVec pos = diamond->Evalp();
	CVec dx = diamond->Evaldp1();
	CVec dy = diamond->Evaldp2();
	CVec dxdx = diamond->Evaldp11();
	CVec dydy = diamond->Evaldp22();
	CVec dxdy = diamond->Evaldp12();

	switch (display) {
	case 0:
	  cerr << "limit point: <" << pos << "> dx: <" << dx << ">\n";
	  break;
	case 1:
	  cerr << "limit point: <" << pos << "> dy: <" << dy << ">\n";
	  break;
	case 2:
	  cerr << "limit point: <" << pos << "> dxdx: <" << dxdx << ">\n";
	  break;
	case 3:
	  cerr << "limit point: <" << pos << "> dydy: <" << dydy << ">\n";
	  break;
	case 4:
	  cerr << "limit point: <" << pos << "> dxdy: <" << dxdy << ">\n";
	  break;
	}
      }
    }
  }

  cerr << "Done verifying quadrature code\n";

  //  for( ; tb != te; ++tb ) cout << **tb;
  //  ProduceInventorFile(mesh);
}





