#ifndef __SIMMATERIAL_H
#define __SIMMATERIAL_H

#include "Util/Real.h"
#include "Util/Tensor3.h"
#include "SimTypes.h"

void print33(Real a[3][3]) {
  for (int i=0; i<3; ++i) {
    for (int j=0; j<3; ++j)
      cerr << a[i][j] << "\t";
    cerr << endl;
  }
}

#include "Maple/material.c"

template <class T>
class SimMaterial {
public:
  DECLARE_TYPES(T)

  typedef Surface::e2Iter e2Iter;

  static void Initialize(Mesh &mesh) {
  }

  static void Update(Mesh &mesh) {
  }

  static Real Lambda(const GaussPoint *p) {
    return 1.0;
  }

  static CnCnTensor S(const GaussPoint *p, e2Iter e2Idx) {
    Real S[3][3];
    Real gmkovr[3][3];
    Real gmkovc[3][3];

    Real emod = Parameters::emod;
    Real nue  = Parameters::nue;

    for (int i = 0; i < 3; ++i)
      for (int j = 0; j < 3; ++j) {
	gmkovr[i][j] = p->gR(i,e2Idx). dot (p->gR(j,e2Idx));
	gmkovc[i][j] = p->gC(i,e2Idx). dot (p->gC(j,e2Idx));
      }

    stress(S, emod, nue, gmkovr, gmkovc);

    return CnCnTensor(S);
  }

  static CnCnTensor CartesianStress(const GaussPoint *p, e2Iter e2Idx) {
    CnCnTensor Stens = S(p, e2Idx);
    int i, j;

    Real gkov1[3][3];
    for (i = 0; i < 3; ++i) {
      gkov1[0][i] = p->gR(i, e2Idx).x();
      gkov1[1][i] = p->gR(i, e2Idx).y();
      gkov1[2][i] = p->gR(i, e2Idx).z();
    }

    Real Scurv[3][3];
    for (i = 0; i < 3; ++i) {
      for (j = 0; j < 3; ++j) {
	Scurv[i][j] = Stens(i)(j);
      }
    }

    Real Scart[3][3];

    svstr(Scurv, gkov1, gkov1, Scart);

    return Scart;
  }

  static CvCvTensor Strain(const GaussPoint *p, e2Iter e2Idx) {
    Real gs[3][3];
    Real gmkovr[3][3];
    Real gmkovc[3][3];

    for (int i = 0; i < 3; ++i)
      for (int j = 0; j < 3; ++j) {
	gmkovr[i][j] = p->gR(i,e2Idx). dot (p->gR(j,e2Idx));
	gmkovc[i][j] = p->gC(i,e2Idx). dot (p->gC(j,e2Idx));
      }

    greenlagrstrain(gs, gmkovr, gmkovc);

    return CvCvTensor(gs);
  }

  static CvCvTensor CartesianStrain(const GaussPoint *p, e2Iter e2Idx) {
    CvCvTensor E = Strain(p, e2Idx);
    int i, j;

    CvBasis aCv(p->gR(0, e2Idx), p->gR(1, e2Idx), p->gR(2, e2Idx));
    CnBasis aCn = RaiseIndices(aCv);

    Real gkov1[3][3];
    for (i = 0; i < 3; ++i) {
      gkov1[0][i] = aCn(i).x();
      gkov1[1][i] = aCn(i).y();
      gkov1[2][i] = aCn(i).z();
    }

    Real Ecurv[3][3];
    for (i = 0; i < 3; ++i) {
      for (j = 0; j < 3; ++j) {
	Ecurv[i][j] = E[i][j];
      }
    }

    Real Ecart[3][3];

    svstr(Ecurv, gkov1, gkov1, Ecart);

    return Ecart;
  }

private:

};

#endif





#if 0
    //cerr << "Computing S\n";
    //print33(gmkovr);
    //cerr << endl;
    //print33(gmkovc);
    stress(S, emod, nue, gmkovr, gmkovc);
    //cerr << endl;
    //cerr << emod << "\t" << nue << endl;
    //print33(S);

    Real gkov1[3][3];
    for (i = 0; i < 3; ++i) {
      gkov1[i][0] = p->gR(i).x();
      gkov1[i][1] = p->gR(i).y();
      gkov1[i][2] = p->gR(i).z();
    }
    Real oS[3][3];

    svstr(S, gkov1, gkov1, oS);

#if 0
    cerr << endl << "S" << endl;
    print33(S);
    cerr << endl;
    
    cerr << endl << "orthogonal S" << endl;
    print33(oS);
    cerr << endl;
#endif



#endif
