#include <iostream.h>

#include "SimForces.h"
#include "SimGaussPoint.h"
#include "SimGeometry.h"
#include "SimIntegrator.h"
#include "SimMass.h"
#include "SimMaterial.h"
#include "SimMesh.h"
#include "SimParameters.h"
#include "SimSurface.h"
#include "SimTriangle.h"
#include "SimVertex.h"

#include "forall.h"

SIMMESHTYPE(MESH, Real, Real3Vec, Real3Tensor,\
	    SimMesh, SimGaussPoint, SimVertex, SimTriangle,\
	    SimMaterial, SimSurface, SimMass, SimForces, SimGeometry,\
	    SimIntegrator, SimParameters)

void InitParameters() {
  MESH::Parameters::rho     = 1.0;
  MESH::Parameters::h       = 1e-5;
  MESH::Parameters::emod    = 1.0;
  MESH::Parameters::nue     = 1.0;
}

int main(int argc, char* argv[]) {
  MESH mesh;

  InitParameters();

  int count = 0;

  {
    cin >> mesh;
    MESH::VertexIt vb = mesh.vertexBegin(), ve = mesh.vertexEnd();
    for( ; vb != ve; ++vb ) (*vb)->InitOneRing();
    MESH::TriangleIt tb = mesh.triangleBegin(), te = mesh.triangleEnd();
    for( ; tb != te; ++tb ) {
      ++count;
      (*tb)->AttachGaussPoints();
    }
    mesh.InitGaussPoints();
  }

  cout << count << " \t";
  
  MESHTypes::Integrator::Initialize(mesh);
  MESHTypes::Integrator::TimeStep(mesh, 0.1, 0.5);
  MESHTypes::Forces::ComputeArea(mesh);
}






