#include <iostream.h>

#include "SimForces.h"
#include "SimGaussPoint.h"
#include "SimGeometry.h"
#include "SimIntegrator.h"
#include "SimMass.h"
#include "SimMaterial.h"
#include "SimMesh.h"
#include "SimParameters.h"
#include "SimSurface.h" 
#include "SimTriangle.h"
#include "SimVertex.h"

#include "forall.h"

#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/sensors/SoIdleSensor.h>

#include "Inventor.cc"

#undef PUT_CORNERS

SIMMESHTYPE(MESH, Real, Real3Vec, Real3Tensor,\
	    SimMesh, SimGaussPoint, SimVertex, SimTriangle,\
	    SimMaterial, SimSurface, SimMass, SimForces, SimGeometry,\
	    SimIntegrator, SimParameters)

void InitParameters() {
  MESH::Parameters::rho              = 1.0;
  MESH::Parameters::h                = 0.1/50.0;
  MESH::Parameters::thicknessSamples = 3;

  MESH::Parameters::emod             = 2.1e5;
  MESH::Parameters::nue              = 0.3;

  MESH::Parameters::predcor_gamma    = 0.7;
}

Real deltaT;

struct CallbackData {
  MESH *mesh;
  SoXtExaminerViewer *myViewer;
  int rootIdx;
  SoSeparator *root;
  int skip;
};

void IdleCallback(CallbackData *data, SoSensor *sensor) {
  MESH *mesh = data->mesh;
  //  SoXtExaminerViewer *myViewer = data->myViewer;
  SoSeparator *root = data->root;
  int rootIdx = data->rootIdx;

  sensor->schedule();

  MESHTypes::Integrator::TimeStep(*mesh, deltaT);

  root->replaceChild(rootIdx, InventorSceneGraph(*mesh, data->skip));
}

main(int argc, char **argv)
{
  MESH mesh;
  CallbackData data; 
  int skip = 1;

  InitParameters();

  if (argc < 2) 
    deltaT = 0.01;
  else
    deltaT = atof(argv[1]);

  if (argc < 3)
    skip = 1;
  else
    skip = atoi(argv[2]);

  cerr << "deltaT = " << deltaT << "   skip = " << skip << endl;
  data.skip = skip;

  int count = 0;

  {
    cin >> mesh;

#if PUT_CORNERS
    for( ; vb != ve; ++vb ) {
      if ((*vb)->point().x() == 0 || (*vb)->point().x() == 1)
	if ((*vb)->point().y() == 0 || (*vb)->point().y() == 1) {
	  (*vb)->setCorner();
	}
    }    
#endif

    mesh.InitGaussPoints();
  }

  cout << count << " \t";

  
  MESHTypes::Integrator::Initialize(mesh);

  MESHTypes::Geometry::ScaleReference(mesh, 0.1);

  MESHTypes::Geometry::ScaleCurrent(mesh, 0.1);

  MESHTypes::Geometry::PullCurrent(mesh, 0.1);

#if 0
  MESHTypes::Geometry::ScaleCurrent(mesh, 1.0 - 1e-4);
#endif

  MESHTypes::Forces::ComputeArea(mesh);

#if TIMING_TEST
  for (int iter=0; iter<skip; ++iter) {
    cerr << iter << " ";
    MESHTypes::Integrator::TimeStep(mesh, deltaT);
  }
  return 0;
#endif  

  Widget myWindow = SoXt::init(argv[0]);
  if (myWindow == NULL) exit(1);
  
  SoSeparator *root = new SoSeparator;
  root->ref();
  SoMaterial *myMaterial = new SoMaterial;
  myMaterial->diffuseColor.setValue(1.0, 1.0, 0.6);
  root->addChild(myMaterial);

  SoSeparator *r = new SoSeparator;
  root->addChild(r);

  // Set up viewer:
  SoXtExaminerViewer *myViewer =
    new SoXtExaminerViewer(myWindow);
  myViewer->setSceneGraph(root);
  myViewer->setTitle("Thin-Shell Simulator Demo");
  myViewer->show();
  myViewer->setAutoRedraw(TRUE);

  data.rootIdx = root->findChild(r);
  data.root = root; 
  data.myViewer = myViewer;
  data.mesh = &mesh;
  
  SoIdleSensor *idleSensor = new SoIdleSensor((SoSensorCB*)IdleCallback, (void*)&data);
  idleSensor->schedule();

  SoXt::show(myWindow);
  SoXt::mainLoop();
}





