#ifndef __CMDLINEPARAMS_H
#define __CMDLINEPARAMS_H

#include "Util/Real.h"
#include "stdlib.h"
#include "iostream.h"

#define MAXPARAM  (100)

struct CmdLineParams {

  enum ParamType {IntType, RealType, FlagType};

  int ParamCount;
  char *ParamNames[MAXPARAM];
  ParamType ParamTypes[MAXPARAM];
  void *ParamData[MAXPARAM];

  CmdLineParams() {
    ParamCount = 0;
  }

  void AddIntParam(char *label, int *data) {
    assert(label && data);
    int i = ParamCount++;
    assert(ParamCount < MAXPARAM);
    ParamNames[i] = label;
    ParamTypes[i] = IntType;
    ParamData[i]  = data;
  }

  void AddRealParam(char *label, Real *data) {
    assert(label && data);
    int i = ParamCount++;
    assert(ParamCount < MAXPARAM);
    ParamNames[i] = label;
    ParamTypes[i] = RealType;
    ParamData[i]  = data;
  }

  void AddFlagParam(char *label, bool *data) {
    assert(label && data);
    int i = ParamCount++;
    assert(ParamCount < MAXPARAM);
    ParamNames[i] = label;
    ParamTypes[i] = FlagType;
    ParamData[i]  = data;
  }

  void PrintParameters(ostream &os) {
    for (int i = 0; i < ParamCount; ++i) {
      switch (ParamTypes[i]) {
      case IntType:
	os << ParamNames[i] << " = " << *((int *)ParamData[i]) << endl;
	break;
      case RealType:
	os << ParamNames[i] << " = " << *((Real *)ParamData[i]) << endl;
	break;
      case FlagType:
	os << ParamNames[i] << " = " << (*((bool *)ParamData[i]) ? "true" : "false") << endl;
	break;
      default:
	cerr << "Error: Unknown parameter type! Bug in parameter handling system.\n";
	exit(1);
      }
    }
  }

  void ParseArguments(int argc, char *argv[]) {
    int i=0;
    while (++i < argc) {
      bool processed = false;
      for (int j = 0; j < ParamCount; ++j) {
	if (!strcmp(argv[i], ParamNames[j])) {
	  processed = true;
	  if (ParamTypes[j] == FlagType) {
	    *(bool *)ParamData[j] = true;
	    break;
	  }
	  if (ParamTypes[j] == IntType) {
	    if (!strcmp(argv[++i],"=")) ++i;
	    if (i >= argc) {
	      cerr << "Error: expected value for parameter \"" << ParamNames[j] << "\"" << endl;
	      exit(1);
	    }
	    char *p;
	    int data = strtol(argv[i], &p, 10);
	    if (p == argv[i]) {
	      cerr << "Error parsing value \"" << argv[i] << "\" for parameter \"" << ParamNames[j] << "\"" << endl;
	      exit(1);
	    }
	    *(int *)ParamData[j] = data;
	    break;
	  }
	  if (ParamTypes[j] == RealType) {
	    if (!strcmp(argv[++i],"=")) ++i;
	    if (i >= argc) {
	      cerr << "Error: expected value for parameter \"" << ParamNames[j] << "\"" << endl;
	      exit(1);
	    }
	    char *p;
	    Real data = strtod(argv[i], &p);
	    if (p == argv[i]) {
	      cerr << "Error parsing value \"" << argv[i] << "\" for parameter \"" << ParamNames[j] << "\"" << endl;
	      exit(1);
	    }
	    *(Real *)ParamData[j] = data;
	    break;
	  }
	}
      }
      if (!processed) {
	cerr << "Error: unknown parameter \"" << argv[i] << "\"" << endl;
	exit(1);
      }
    }
  }

};

#endif
