#ifndef __REAL_H
#define __REAL_H

typedef double Real_NativeType;

class Real {

 public:

  Real(const Real_NativeType &a) 
    : m_data(a) {}

  Real() 
    : m_data(0) {}

  Real operator +(const Real &b) const {
    return m_data + b.m_data;
  }

  Real operator -(const Real &b) const {
    return m_data - b.m_data;
  }

  Real operator *(const Real &b) const {
    return m_data * b.m_data;
  }

  Real operator /(const Real &b) const {
    return m_data / b.m_data;
  }

  Real operator +=(const Real &b) {
    return m_data += b.m_data;
  }

  Real operator -=(const Real &b) {
    return m_data -= b.m_data;
  }

  Real operator *=(const Real &b) {
    return m_data *= b.m_data;
  }

  Real operator /=(const Real &b) {
    return m_data /= b.m_data;
  }

  bool operator <(const Real &b) const {
    return m_data < b.m_data;
  }

  bool operator <=(const Real &b) const {
    return m_data <= b.m_data;
  }

  bool operator >(const Real &b) const {
    return m_data > b.m_data;
  }

  bool operator >=(const Real &b) const {
    return m_data >= b.m_data;
  }

  bool operator ==(const Real &b) const {
    return m_data == b.m_data;
  }

  Real operator +() const {
    return +m_data;
  }

  Real operator -() const {
    return -m_data;
  }

  Real_NativeType GetNativeData() const {
    return m_data;
  }

  Real_NativeType& GetNativeData() {
    return m_data;
  }

 private:
  Real_NativeType m_data;
};

Real operator +( const Real_NativeType &a,
		 const Real &b ) {
  return Real(a) + b;
}

Real operator -( const Real_NativeType &a,
		 const Real &b ) {
  return Real(a) - b;
}

Real operator *( const Real_NativeType &a,
		 const Real &b ) {
  return Real(a) * b;
}

Real operator /( const Real_NativeType &a,
		 const Real &b ) {
  return Real(a) / b;
}

#include "Util/Matrix3.h"
#include "Util/Vector3.h" 

typedef Vector3<Real> Real3Vec;
typedef Matrix3<Real> Real3Matrix;

#include "Util/Tensor3.h"

const Real3Vec Zero3Vec = Real3Vec::Zero; 
const Real3Matrix Zero3Matrix = Real3Matrix();

const Real RealLarge = 1e20;

const Real RealEPS = 0; // 1e-21; //1e-10

Real abs(const Real &x) {
  return (x >= 0.0) ? x : -x;
}

Real acos(const Real &x) {
  return acos(x.GetNativeData());
}

Real asin(const Real &x) {
  return asin(x.GetNativeData());
}

Real atan(const Real &x) {
  return atan(x.GetNativeData());
}

Real atan2(const Real &x, const Real &y) {
  return atan2(x.GetNativeData(), y.GetNativeData());
}

Real cos(const Real &x) {
  return cos(x.GetNativeData());
}

Real sin(const Real &x) {
  return sin(x.GetNativeData());
}

Real tan(const Real &x) {
  return tan(x.GetNativeData());
}

Real Sq(const Real &x) {
  return x*x;
}

Real sqrt(const Real &x) {
  return sqrt(x.GetNativeData());
}

Real Cubed(const Real &x) {
  return x*x*x;
}

ostream&
operator<<( ostream &os, const Real &c )
{
  if (abs(c) < RealEPS) {
    os << 0;
  } 
  else {
    os << c.GetNativeData();
  }

  return os;
}

istream&
operator>>( istream &is, Real &c )
{
  return is >> c.GetNativeData();
}

#include "Util/util.h"



#define _IEEE 1

#include "nan.h"


#endif


