namespace PKrysl {
extern "C" {
#define new cc_new

#include "Wconfig.h"
#include "Wdomain.h"
#include "Wnode.h"
#include "Wmats.h"
#include "Wpss.h"
#include "Wgaussp.h"
#include "Wxf.h"
#include "Wcontface.h"
#include "Whistry.h"
#include "Wnewmarki.h"
#include "Welem.h"

#undef new
}}

#undef Convex
#undef Concave

//#include <vector>

//using std::p;

using namespace PKrysl;

#include <vector>

using std::pair;

#include "subdiv.h"

#include "WASPVertex.h"
#include "WASPTriangle.h"

#include "Simulator/SimForces.h"
#include "Simulator/SimGaussPoint.h"
#include "Simulator/SimGeometry.h"
#include "Simulator/SimIntegrator.h"
#include "Simulator/SimMass.h"
#include "Simulator/SimMaterial.h"
#include "Simulator/SimMesh.h"
#include "Simulator/SimParameters.h"
#include "Simulator/SimShellIntegrator.h"
#include "Simulator/SimSurface.h" 
#include "Simulator/SimTriangle.h"
#include "Simulator/SimVertex.h"

#include "Util/forall.h"
#include "Util/random.h"
#include "Util/util.h"

#include <iostream.h>
#include <fstream.h>

class Patch {};
class LeafPatch {};

SIMMESHTYPE(Mesh,\
	    SimMesh, SimGaussPoint, WASPVertex, WASPTriangle,\
	    LeafPatch,\
	    SimMaterial, SimSurface, SimMass, SimForces, SimGeometry,\
	    SimIntegrator, SimShellIntegrator, SimParameters)

static int
elem_elcode (W_elem_t *el)
{
  return SUBDIV_ELCODE;
}

static char *
type_name(W_elem_t *el)
{
  static char buf[] = "subdiv";
  return buf;
}

static void
assemble_restoring_forces (W_elem_t *el, W_mat_state_update_form_enum update_form)
{
  Mesh &mesh = *((Mesh *)el->data);

  //cerr << "Retrieving current coordinates: ";

  forall_notypename(Mesh::VertexIt, v, mesh.vertexBegin(), mesh.vertexEnd())

    (*v).CopyFromWASPxC();

  endforall;

  //cerr << "\n\nComputing forces...\n";

  Mesh::Geometry       ::Update(mesh);
  Mesh::Surface        ::Update(mesh);
  Mesh::ShellIntegrator::Update(mesh);
  Mesh::Material       ::Update(mesh);
  Mesh::Mass           ::Update(mesh);
  Mesh::Forces         ::Update(mesh);

  //cerr << "Sending forces: ";

  forall_notypename(Mesh::VertexIt, v, mesh.vertexBegin(), mesh.vertexEnd())

    (*v).CopyToWASPForces();

  endforall;

  //cerr << endl;

  /* deformation energy */
  el->deformation_energy = 0;   /* RAW */
}

static void
assemble_inertial_forces_diag (W_elem_t *el)
{
  Mesh &mesh = *((Mesh *)el->data);

  forall_notypename(Mesh::VertexIt, v, mesh.vertexBegin(), mesh.vertexEnd())

    v->AddToWASPInertialForces();

  endforall;
}

static void
assemble_residual_forces (W_elem_t *el,
                          W_mat_state_update_form_enum update_form,
                          W_mass_type_enum mass_type)
{
  assemble_restoring_forces (el, update_form);
  assemble_inertial_forces_diag (el); /* supports only diagonal mass */
}

static W_float_t
total_mass (W_elem_t *el)
{
  Mesh &mesh = *((Mesh *)el->data);

  Real m = 0;

  forall_notypename(Mesh::VertexIt, v, mesh.vertexBegin(), mesh.vertexEnd())

    m += (*v).m();

  endforall;

  return m.GetNativeData();
}

static void
assemble_mass (W_elem_t *el)
{
  Mesh &mesh = *((Mesh *)el->data);

  forall_notypename(Mesh::VertexIt, v, mesh.vertexBegin(), mesh.vertexEnd())

    (*v).CopyToWASPm();
    
  endforall;
}


static W_float_t 
suggested_time_step (W_elem_t *el)
{
  return Mesh::Parameters::dT.GetNativeData();
}


#ifdef X_XFACE
static void
modify_elem_grep(W_elem_t *el, double displ_scale, W_bool_t immed_draw)
{
  Mesh &mesh = *((Mesh *)el->data);

  forall_notypename(Mesh::VertexIt, v, mesh.vertexBegin(), mesh.vertexEnd())

    (*v).CopyFromWASPxC();

  endforall;


  forall_notypename(Mesh::TriangleIt, t, mesh.triangleBegin(), mesh.triangleEnd())

    (*t).ModifyGeometry(displ_scale);

  endforall;
}
#endif


static W_float_t
kinetic_energy (W_elem_t *el, W_mass_type_enum mass_type)
{
  return 0.0;
}

static W_elem_methods_t methods = {
  elem_elcode,
  type_name,
  assemble_restoring_forces,
  assemble_mass,
  total_mass,
  NULL,
  suggested_time_step,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
#ifdef X_XFACE
  modify_elem_grep,
#endif
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  assemble_residual_forces,
  NULL,
  kinetic_energy
};

/* Reads element data from stream as long as there is no */
/* END_OF_BLOCK tag in the input line. Creates elements. */

extern "C" 
W_bool_t 
subdiv_reader (W_domain_t *d)
{
  const char *input_line;

  Mesh::Parameters::Initialize();

  Mesh::Parameters::Load("subdiv.params");

  Mesh::Parameters::Save(cerr);

  while ((input_line = W_get_input_line(d)) != NULL) {
    if (W_is_end_of_block (input_line))
      return TRUE;
    else {
      ifstream fin(input_line);
      assert(!fin.eof());

      Mesh &mesh = *(new Mesh);      

      fin >> mesh;

      cerr << "subdiv element: " << mesh.vertexCount() << " vertices, ";
      cerr << mesh.triangleCount() << " triangles.\n";

      Mesh::Geometry       ::Initialize(mesh);
      Mesh::Surface        ::Initialize(mesh);
      Mesh::ShellIntegrator::Initialize(mesh);
      Mesh::Material       ::Initialize(mesh);
      Mesh::Mass           ::Initialize(mesh);
      Mesh::Forces         ::Initialize(mesh);

      forall_notypename(Mesh::VertexIt, v, mesh.vertexBegin(), mesh.vertexEnd())

	(*v).Initialize(d);

      endforall;

      WD_add_element(d, -1, WE_new_element(d, &methods, &mesh) );
    }
  }
  return FALSE;
}

